/***********************************************************************
Nucleic acid alignment (nucaln) 
main file : nucaln.c 

 	Creates a nucleic acid alignment that matches an amino acid 
	sequence alignment.

  Usage: nucaln aa.aln nuc.fa out.aln
  
  Source files are:	nucaln.c
			gtsqfn.c
  			utility.c
  			Dmc.h
  			getseq2.h
  			cmp_tbl.h
			exit0.h
			exit1.h
  	
 	Naoki Sato   Feb. 22, 2000.
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.
	For Mac version, define the compiler as well.

		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"

#ifndef INTEGRATED
#ifdef MAC
extern int ccommand(char ***);
#endif
#endif

/* Prototype declaration */
static void print_usage(void);
#ifdef INTEGRATED
int seqcat(int c, char (*p)[S_2]);
#endif
void help_nucaln(void);
Seqfile *CreateNucAln(Seqfile *seqfile1,char *seq2,int frame,long start);

/* External functions */

#include "libSeq.h"

/* Global variables */
extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];
extern char univ_code[];
extern char mt_code[];

/***************************************************************/
static void print_usage(void)
/***************************************************************/
{
	help_nucaln();
	return;
}
/***************************************************************/

/***************************************************************/
void help_nucaln(void)
/***************************************************************/
{
#ifdef JAPANESE
        printf("************************* NUCALN Υإ *******************************\n");
        printf("Ȥ: nucaln aa.aln nuc.fa out.aln.\n\n");
        printf("       Υץϡߥλ󥢥饤Ȥ򸵤ˤơ\n");
	printf("       ٤ä˻󥢥饤Ȥ.\n");
        printf("       aa.aln ϡClustalΥߥλ󥢥饤ȥե.\n");
	printf("       nuc.fa ϡޥFASTAγ˻ե.\n");
	printf("       ϥեout.alnϡClustalΥ饤ȥեˤʤ.\n");
        printf("       aa.alnnuc.faˤơ̾ϴƱǤ뤳. \n");
        printf("*************************************************************************\n");
#else
        printf("***************** NUCALN HELP ***********************************\n");
        printf("Usage: nucaln aa.aln nuc.fa out.aln.\n\n");
        printf("       This program creates a nucleic acid sequence alignment\n");
	printf("       that matches exactly an amino acid sequence alignment.\n");
        printf("       aa.aln should be an amino acid sequence alignment in Clustal format.\n");
	printf("       nuc.fa should be a multi-fasta file of nucleic acid sequences.\n");
	printf("       Output file is a nucleic acid alignment file in Clustal format.\n");
        printf("       All names of sequences should be identical between aa.aln and nuc.fa. \n");
        printf("*****************************************************************\n");
#endif
	return;
}
/***************************************************************/

/***********************************************************************/
Seqfile *CreateNucAln(Seqfile *seqfile1,char *seq2,int frame,long start)
/***********************************************************************
This function returns a Seqfile of nucleic acid sequence containing
corresponding to the amino acid sequence that is part of the 
alignment file. 
************************************************************************/
{
	long i;
	char *p,*q;
	Seqfile *seq_out;
	int g='-';
	int a;
	char codon[3];
	char *codon_table;

	if(start<1 || start>strlen(seq2)){
		fprintf(stderr,"Strange number of start value %ld in CreateNucAln.\n",start);
		return NULL;
	}
	if((seq_out=copy_seq_h(seqfile1))==NULL){
		fprintf(stderr,"Out of memory.\n");
#include	"exit1.h"
	}
	if((seq_out->seq=(char*)calloc(3*seqfile1->seqlen+2,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
#include	"exit1.h"
	}
        if((codon_table=(char*)calloc(100,sizeof(char)))==NULL){
                fprintf(stderr,"Out of memory.\n");
#include        "exit1.h"
        }
	strcpy(codon_table,univ_code);

	i=frame+3*(start-1);
	q=seq_out->seq;
	for(p=seqfile1->seq;*p!='\0';p++){
		if(*p==g){
			strcpy(codon,extract_string(seq2,i+1,i+3));


/* for debug 
	printf("codon= %s\n",codon);
*/
			if((a=codon_translate(codon,codon_table))=='*'){
				*q++=seq2[i++];
				*q++=seq2[i++];
				*q++=seq2[i++];
			}
			else {
				*q++=g;
				*q++=g;
				*q++=g;
			}
/* for debug 
	printf("codon= %c\n",a);
*/
			continue;
		}
		else if(isalpha(*p)){
			strcpy(codon,extract_string(seq2,i+1,i+3));
			if(codon_translate(codon,codon_table)==*p){
				*q++=seq2[i++];
				*q++=seq2[i++];
				*q++=seq2[i++];
			}
			else {
				*q++=tolower(seq2[i++]);
				*q++=tolower(seq2[i++]);
				*q++=tolower(seq2[i++]);
			}
			continue;
		}
		else {
			fprintf(stderr,"Strange character %c in the amino acid sequence file.\n",*p);	
			return NULL;
		}
	}
	*q='\0';
	seq_out->seqlen=strlen(seq_out->seq);
	
	return seq_out;
}
/***********************************************************************/



/***************************************************************
****************** Main program *******************************/
#ifdef INTEGRATED
int nucaln(int cc, char (*p)[S_2])

#else
#ifdef MAC
main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif
#endif
/***************************************************************/
{
	char fname1[MAXLINE]="aa.aln";
	char fname2[MAXLINE]="nuc.fa";
	char fname3[MAXLINE]="out.aln";
	char *codon_table;
	int c1=0;
	int c2=0;
#ifdef JAPANESE
	char *str1="ߥλ󥢥饤ȥե̾";
	char *str2="˻ե̾";
	char *str3="ϥե̾";
#else
	char *str1="File name for aa alignment";
	char *str2="File name for nucleic acid seq. file";
	char *str3="Output file name";
#endif
	enum data_type dtype1=unknown;
	enum data_type dtype2=unknown;
	long numseq1=0L;
	long numseq2=0L;
	long i,j;
	long start;	/* start position of matching */
	int k;		/* counter for frame */
	long seqlen1;	/* sequence length */

	/* sequence variables */
	Seqfile **mseq_in1;
	Seqfile **mseq_in2;
	Seqfile **mseq_out;
	char *s;	/* temporary sequence container */
	char seqname[30];	/* temporary sequence name container */
	char *sn;	/* temporary nucleic acid sequence container */

#include	"parsarg.h"

/*********************** File names ******************/
/*
printf("argc=%d, cc=%d",argc,cc);
*/
	switch(cc){
		case 1:	
			strcpy(fname1,"file1");
			if(filename_input(fname1,str1)!=1){
#include	"exit1.h" 
			}
			strcpy(fname2,"file2");
			if(filename_input(fname2,str2)!=1){
#include	"exit1.h" 
			}
			strcpy(fname3,"outfile");
			if(filename_input(fname3,str3)!=1){
#include	"exit1.h" 
			}
			break;
		
		case 3:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			strcpy(fname3,"outfile");
			if(filename_input(fname3,str3)!=1){
#include	"exit1.h" 
			}
			break;
			
		case 4:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			strcpy(fname3,p[3]);
			break;

		default:
			print_usage();
#include        "exit1.h"
	}

/*********************** main part ****************************/

#ifdef MAC
#ifdef CODEWARRIOR
	SIOUXSetTitle("\pnucaln");
#endif
#endif
	c1=check_filetype(fname1);
	c2=check_filetype(fname2);
	if(c1==99 || c2==99){
		fprintf(stderr,"Unknown file format.\n");
		exit(1);
	}

	if(ftouch(fname3)){
		fprintf(stderr,"Error in opening output file.\n");
#include 	"exit1.h"
	}
	if(c1!=CLUSTAL_SEQ || c2!=FASTA_SEQ){
		fprintf(stderr,"File formats are not correct.\n");
#include        "exit1.h"
	}
	fprintf(stdout,"Reading aa alignment ...\n");
	if((mseq_in1=read_clustal(fname1,dtype1))==NULL){
		fprintf(stderr,"Error in reading file: %s\n",fname1);
#include        "exit1.h"
               }
	numseq1=num_seq;
	dtype1=check_data_type(mseq_in1[0]);
	if(dtype1!=protein){
		fprintf(stderr,"This is not an amino acid alignment.\n");
#include        "exit1.h"
	}

	fprintf(stdout,"Reading nucleic acid sequences...\n");
	num_seq=0L;
	if((mseq_in2=Mread_fasta(fname2,dtype2))==NULL){
                       fprintf(stderr,"Error in reading file: %s\n",fname2);
#include        "exit1.h"
	}
	numseq2=num_seq;
	dtype2=check_data_type(mseq_in2[0]);
	if(dtype2!=DNA){
		fprintf(stderr,"These are not DNA sequences.\n");
#include        "exit1.h"
	}

	if(numseq1>numseq2){
		fprintf(stderr,"Not enough number of sequences in the nucleic acid sequence file.\n");
#include        "exit1.h"
	}
		
	seqlen1=mseq_in1[0]->seqlen;

	if((s=(char*)calloc(seqlen1+2,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
#include        "exit1.h"
	}
	if((mseq_out=(Seqfile**)calloc(numseq1+2,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory.\n");
#include        "exit1.h"
	}
        if((codon_table=(char*)calloc(100,sizeof(char)))==NULL){
                fprintf(stderr,"Out of memory.\n");
#include        "exit1.h"
        }
	strcpy(codon_table,univ_code);

	for(i=0;i<numseq1;i++){

		strcpy(s,mseq_in1[i]->seq);
		strcpy(seqname,mseq_in1[i]->ID);

		if(StringClearGaps(s)){
			fprintf(stderr,"Error in clearing gaps.\n");
#include        "exit1.h"
		}
			
		for(j=0;j<numseq2;j++){
			if(!strcmp(seqname,mseq_in2[j]->ID)) break;
		}
		if(j>=numseq2){
			fprintf(stderr,"Cannot find a correct nucleic acid sequence for %s.\n",seqname);
#include        "exit1.h"
		}
		
		for(k=0;k<3;k++){
			if((sn=translate(mseq_in2[j]->seq,k+1,codon_table))==NULL){
				fprintf(stderr,"Unable to translate sequence %s frame %d.\n",seqname,k+1);
#include        "exit1.h"
			}


/* for debug 
	printf("%s \n",seqname);
	printf("%s \n",s);
	printf("j=%ld k=%d \n",j,k);
	printf("%s \n",sn);
*/

			if((start=MatchSeq(s,sn,0.95))!=0){
				if((mseq_out[i]=CreateNucAln(mseq_in1[i],mseq_in2[j]->seq,k,start))==NULL){	
					fprintf(stderr,"Unable to create nucleic sequence alignment for seq %s.\n",seqname);
#include        "exit1.h"
				}
				break;
			}
/* for debug 
	printf("start=%ld \n",start);
*/
		}
		if(k>=3){
			fprintf(stderr,"Unable to find matching position for %s.\n",seqname);
#include        "exit1.h"
		}
	}


	fprintf(stdout,"Writing sequences...\n");
	num_seq=numseq1;
	if(write_clustal(fname3,mseq_out)){
		fprintf(stderr,"Error in writing sequence.\n");
#include        "exit1.h"
	}

#ifdef VERBOUS
        fprintf(stdout,"Sequence written to file: %s\n",fname3);
#else
	putc(8,stdout);
	putc('w',stdout);
	fflush(stdout);
#endif

	printf("\nTerminated.\n");

#include "exit0.h"

}

/************************************************************
		End of file nucaln.c
************************************************************/


