/***********************************************************************
Show sites (sites) 
main file : sites.c

 	Show location of a sequence motif.
 	At first stage, input file formats are limited to
 	raw, FASTA, GenBank, and EMBL.

  Usage: sites infile outfile motif 
  
  Default outfile name is "infile"+".out"
  
  Source files are:	sites.c
			gtsqfn.c
  			utility.c
  			Dmc.h
  			getseq2.h
  			cmptbl.h
  	
 	Naoki Sato   Aug. 6, 1999.
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.
	For Mac version, define the compiler as well.
	
	Integrated in the siseq main program. Feb. 22, 2000.
	siseq_exit0 or 1 changed to exit0 or 1. Nov. 13, 1999.
	Bug fix. May 4, 2000.
	Multiple sequence file. Oct. 15, 2000.
	
		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"

#ifndef INTEGRATED
#ifdef MAC
extern int ccommand(char ***);
#endif
#endif

/* Prototype declaration */
static void print_usage(void);
long search_match(char *seq,char *motif,long position,double precision,enum data_type dtype);
long write_sites(FILE *fout,Seqfile *seq_in,enum data_type dtype1,long motlen,char *str3,double match_level,int mode);

#ifdef INTEGRATED
int sites(int c, char (*p)[S_2]);
#endif
void help_sites(void);

/* External functions. */

#include "libSeq.h"

/* Global variables */
extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];
extern long num_seq;
extern Boolean circular;

#define DEFAULT_MATCHING_LEVEL 0.75

/***************************************************************/
static void print_usage(void)
/***************************************************************/
{
	help_sites();
	return;
}
/***************************************************************/

/***************************************************************/
void help_sites(void)
/***************************************************************/
{
#ifdef JAPANESE

#ifdef UNIX
#include "EUC/sitesE.h"
#else
#ifdef MAC
#include "Mac/SJIS/sitesS.h"
#else
#include "Dos/SJIS/sitesS.h"
#endif
#endif

#else
	printf("****************************** SITES HELP *************************************\n");
        printf("Usage: sites infile outfile motif mode <matching level>.\n\n");
        printf("       This program shows location of a sequence\n");
        printf("       motif within a database entry\n");
        printf("Input file formats should be: raw, Fasta, GenBank, EMBL, SwissProt, or PRF. \n");
	printf("mode 0, short summary only; mode 1, detailed output (default).\n");
	printf("<matching level> is an option.\n");
	printf("*******************************************************************************\n");
#endif
	return;
}
/***************************************************************/


/*******************************************************************************************/
long search_match(char *seq,char *motif,long position,double precision,enum data_type dtype)
/*******************************************************************************************/
{
	long i,j;
	double score;
	long length=(long)strlen(seq);
	long motlen=(long)strlen(motif);
	char *compmot;

        if(length==0L || motlen==0L){
                fprintf(stderr,"Sequence lengh zero.\n" );
                return 0;
        }
	if(precision>1.0 || precision<0.5){
                fprintf(stderr,"Precision should be between 0.5 and 1.0.\n" );
                return 0;
        }

	/* use upper case in this subroutine. */

	for(i=0;i<length;i++) seq[i]=toupper(seq[i]);
	for(i=0;i<motlen;i++) motif[i]=toupper(motif[i]);

	/* make complementary motif */

	if((compmot=(char*)calloc(motlen+2,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		exit(1);
	}
	if(dtype!=protein){
	       	for(i=0L;i<motlen;i++){
	                if(dtype==DNA) compmot[motlen-i-1]=to_compD(motif[i]);
	                else compmot[motlen-i-1]=to_compR(motif[i]);
	        }
	}


	/* search motif */
	
	for(i=position;i<length-motlen+1;i++){
		score=0.0;
		for(j=0;j<motlen;j++){
			if(seq[i+j]==motif[j]) score+=1.0;
			else if(dtype==DNA) {
				if(motif[j]=='R'){
					if(seq[i+j]=='A' || seq[i+j]=='G') score+=1.0;
				} 
				else if(motif[j]=='Y'){
					if(seq[i+j]=='T' || seq[i+j]=='C') score+=1.0;
				} 
			}
			else if(dtype==RNA) {
				if(motif[j]=='R'){
					if(seq[i+j]=='A' || seq[i+j]=='G') score+=1.0;
				} 
				else if(motif[j]=='Y'){
					if(seq[i+j]=='U' || seq[i+j]=='C') score+=1.0;
				} 
			}
		}
		if(score>=motlen*precision) return i+1;
		if(dtype!=DNA) continue;
		score=0.0;
		for(j=0;j<motlen;j++){
			if(seq[i+j]==compmot[j]) score+=1.0;
			if(compmot[j]=='R'){
				if(seq[i+j]=='A' || seq[i+j]=='G') score+=1.0;
			} 
			else if(compmot[j]=='Y'){
				if(seq[i+j]=='T' || seq[i+j]=='C') score+=1.0;
			} 
		}
		if(score>=motlen*precision) return (-1)*(i+1);
	}
	return 0;
}
/***************************************************************/


/*************************************************************************************************************/
long write_sites(FILE *fout,Seqfile *seq_in,enum data_type dtype1,long motlen,char *str3,double match_level,int mode)
/*** mode 0, no output; mode 1, verbous output. */
/*************************************************************************************************************/
{
	long count=0L;	/* matching count */
	long i;		/* counter within loops */
	int c=0;	/* data type */
	char *pp;
	Boolean circular_bak=FALSE;
	static char str4[MINMEMSIZE];
	static char str5[MINMEMSIZE];
	long position0=0L;
	long position1=0L;
	double level=1.0; 
	long motlen1=0L;


	if(seq_in==NULL) {
		fprintf(stderr,"Sequence is empty.\n");
		return -1;
	}
				
	motlen1=strlen(str3);
	if(motlen1!=motlen) motlen=motlen1;
	if(motlen<3) {
		fprintf(stderr,"Motif too short.\n");
		return -1;
	}
	if(motlen>MAXLINE-2) {
		fprintf(stderr,"Motif too long.\n");
		return -1;
	}

	c=check_data_type(seq_in);
	if(dtype1!=c) {
		fprintf(stderr,"Data types seems mixed. Stop processing...\n");
		return -1;
	}

	circular_bak=circular;
	circular=seq_in->circular;

	for(i=0;i<motlen;i++) str3[i]=toupper(str3[i]);		/* motif converted to upper case */
	if(c==DNA){
	       	for(i=0L;i<motlen;i++){
	                if(c==DNA) str4[motlen-i-1]=to_compD(str3[i]); 	/* str4 is a complementary to motif */
/*	                else str4[motlen-i-1]=to_compR(str3[i]); */
	        }
		str4[motlen]=0;
	}

	if(mode==1){
		fprintf(fout,"  Sequence name: %s.   ",seq_in->ID);
		fprintf(fout,"    Length  : %ld. ",seq_in->seqlen);
		fprintf(fout,"    Data type: %s.\n",data_type_name[dtype1]);
		fprintf(fout,"  Key motif    : %s.",str3);
		fprintf(fout,"  Complementary: %s.\n",str4);
	}

	/* for circular */
	
	if(circular){
		printf("\nThis is a circular genome: %s.\n",seq_in->ID);
		if((seq_in->seq=(char*)realloc(seq_in->seq,(seq_in->seqlen+motlen+10)*sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory.\n");
			exit(1);
		}
		pp=extract_string(seq_in->seq,1,motlen-1); 
		strcat(seq_in->seq,pp); 		/* Here, the original seq is modified. */
		free(pp);
		seq_in->seqlen=(long)strlen(seq_in->seq);
	}

	for(level=1.0;level>=match_level-0.0000001;level-=0.1){
		if(mode==1) fprintf(fout,"\nMatching level= %4.2f\n\n",level);
		count=1L;
		while(TRUE){
			position1=search_match(seq_in->seq,str3,position0,level,c);
			if(position1==0) break;
			else {
				if(position1>0){
					pp=extract_string(seq_in->seq,position1,position1+motlen-1);
					strcpy(str5,pp);
					free(pp);
					if(mode==1) fprintf(fout,"%7ld position=%8ld,  %s\n",count,position1,str5);
				}
				else{
					position1=0-position1;
					pp=extract_string(seq_in->seq,position1,position1+motlen-1);
					strcpy(str5,pp);
					free(pp);
					if(mode==1) fprintf(fout,"%7ld position=%8ld,  %s complementary\n",count,position1,str5);
				}
				count+=1L;
			}
			position0=position1+1;
			position1=0;
		}
		position1=0L;
		position0=0L;
	}
	circular=circular_bak;
	
	return count;

}

/*******************************************************************************************************/

/***************************************************************
****************** Main program *******************************/
#ifdef INTEGRATED
int sites(int cc, char (*p)[S_2])

#else
#ifdef MAC
main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif

#endif
/***************************************************************/

{
	FILE *fin;
	FILE *fout;
	char fname1[100]="infile";
	char fname2[100]="outfile";
	int c=0;
	Boolean circular_bak=FALSE;
	char *str1="Input file name";
	char *str2="Output file name";
	char str3[MINMEMSIZE];
	long motlen=0L;
	long i=0L;
	enum data_type dtype1=unknown;
   /* sequence variables */
	Seqfile *seq_in;
	Seqfile **mseq_in;
	double level=DEFAULT_MATCHING_LEVEL;	/* matching level */
	int mode=1;	/* mode 0, no detailed output; mode 1, verbous output. */
	long total_count=0L;	/* total matching count */
	long count=0L;	/* matching count */
	long total_seqlen=0L;	/* total sequence length */
	double frequency=0.0;		

	long fileloc=0L;

#include	"parsarg.h"

/*********************** File names ******************/

	switch(cc){

		case 1:	
			strcpy(fname1,"infile");
			if(filename_input(fname1,str1)!=1){
#include	"exit1.h"	
			}
			strcpy(fname2,fname1);
			strcat(fname2,".out");
			if(filename_input(fname2,str2)!=1){
#include	"exit1.h"	
			}
			printf("Type a sequence motif.\n\n");
			scanf("%s",str3);
			break;
			
		case 3:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			printf("Type a sequence motif.\n\n");
			scanf("%s",str3);
			break;
			
		case 4:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			strcpy(str3, p[3]);
			break;
			
		case 5:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			strcpy(str3, p[3]);
			if(!strcmp(p[4],"0")) mode=0;
			break;
			
		case 6:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			strcpy(str3, p[3]);
			if(!strcmp(p[4],"0")) mode=0;
			sscanf(p[5],"%lf",&level);
			break;
			
		default:
			print_usage();
#include	"exit1.h"	
	}

/***********************Main part****************************/
		
	motlen=strlen(str3);
	if(motlen<3) {
		fprintf(stderr,"Motif too short.\n");
#include 	"exit1.h"
	}
	if(motlen>MAXLINE-2) {
		fprintf(stderr,"Motif too long.\n");
#include 	"exit1.h"
	}

	c=check_filetype(fname1);
	if(c==99){
		fprintf(stderr,"Unknown file format.\n");
#include 	"exit1.h"
	}
	printf("Reading sequence...\n");

#ifdef MAC
#ifdef CODEWARRIOR
	SIOUXSetTitle("\psites");
#endif
#endif

	if(c==FASTA_SEQ || c==CLUSTAL_SEQ){
		if(c==FASTA_SEQ) fprintf(stdout,"This is a FASTA file.\n");
		else fprintf(stdout,"This is a CLUSTAL file.\n");

		if(c==FASTA_SEQ){
			if((mseq_in=Mread_fasta(fname1,dtype1))==NULL){
				fprintf(stderr,"Error in reading file: %s\n",fname1);
#include 	"exit1.h"
                	}
		}
                else {
			if((mseq_in=read_clustal(fname1,dtype1))==NULL){
				fprintf(stderr,"Error in reading file: %s\n",fname1);
				exit(1);
			}
		}

		if(clear_gapsM(mseq_in)){
#include 	"exit1.h"
		}

		dtype1=check_data_type(mseq_in[0]);

		if(ftouch(fname2)){
			fprintf(stderr,"Error in opening output file.\n");
#include 	"exit1.h"
		}

		if((fout=fopen(fname2,"a+"))==NULL){
			fprintf(stderr,"Cannot open file: %s\n",fname2);
#include 	"exit1.h"
		}
	
		fprintf(fout,"************************************************************\n");
		fprintf(fout,"Search matching\n\n");
		fprintf(fout,"  Input file: %s.\n",fname1);
		fprintf(fout,"************************************************************\n");
		fprintf(fout,"\n");


		for(i=0;i<num_seq;i++){	

			if(mode==1 && i>0) fprintf(fout,"*************************************\n");
			if((count=write_sites(fout,mseq_in[i],dtype1,motlen,str3,level,mode))<0){ 
				fprintf(stderr,"Error in writing motif sites in file: %s\n",fname2);
#include 	"exit1.h"
			}
			if(count>1) total_count+=(count-1);
		}

		for(i=0;i<num_seq;i++){	
			total_seqlen+=mseq_in[i]->seqlen;
		}
		frequency=(double)total_count/(double)total_seqlen;

		fprintf(fout,"\n----- Summary of motif search -----\n");
		fprintf(fout,"  Total length of sequence: %ld.\n",total_seqlen);
		fprintf(fout,"  Number of match:          %ld.\n",total_count);
		fprintf(fout,"    Frequency:              %8.5f.\n",frequency);

		fprintf(fout,"************************************************************\n");
		fclose(fout);
		circular=circular_bak;

		for(i=0;i<num_seq;i++) free_seq(mseq_in[i]);

	}

	else if(c==GENBANK_SEQ || c==EMBL_SEQ || c==SW_SEQ || c==PIR_SEQ){
		if(c==GENBANK_SEQ) printf("This is a GenBank file.\n");
		else if(c==EMBL_SEQ) printf("This is an EMBL file.\n");
		else if(c==SW_SEQ) printf("This is a SwissProt file.\n");
		else if(c==PIR_SEQ) printf("This is a PIR file.\n");
		if((fin=fopen(fname1,"r"))==NULL){
			fprintf(stderr,"Cannot open file: %s\n\n",fname1);
#include 	"exit1.h"
		}

		if(ftouch(fname2)){
			fprintf(stderr,"Error in opening output file.\n");
#include 	"exit1.h"
		}

#ifdef MAC
#ifdef CODEWARRIOR
        _fcreator=CREATOR_NAME;
        _ftype=FILETYPE_NAME;
#endif
#endif

		if((fout=fopen(fname2,"a+"))==NULL){
			fprintf(stderr,"Cannot open file: %s\n",fname2);
#include 	"exit1.h"
		}
		fprintf(fout,"************************************************************\n");
		fprintf(fout,"Search matching\n\n");
		fprintf(fout,"  Input file: %s.\n",fname1);
		fprintf(fout,"************************************************************\n");
		fprintf(fout,"\n");
		fflush(fout);

		total_seqlen=0L;
		total_count=0L;

		while(TRUE){
			if(c==GENBANK_SEQ) seq_in=read_genbank(fin,&fileloc,dtype1);
			else if(c==EMBL_SEQ || c==SW_SEQ) seq_in=read_embl(fin,&fileloc,dtype1);
			else if(c==PIR_SEQ) seq_in=read_pir(fin,&fileloc,dtype1);
			if(seq_in==NULL) break;
			
			if(clear_gaps(seq_in)){
				fprintf(stderr,"Error in clearing gaps in file: %s\n",seq_in->ID);
				exit(1);
			}
	
			dtype1=check_data_type(seq_in);

			if(mode==1) fprintf(fout,"*************************************\n");
			if((count=write_sites(fout,seq_in,dtype1,motlen,str3,level,mode))<0){ 
				fprintf(stderr,"Error in writing motif sites in file: %s\n",fname2);
#include 	"exit1.h"
			}
			if(count>1) total_count+=(count-1);
			total_seqlen+=seq_in->seqlen;

			free_seq(seq_in);
		}

		printf("\n");	
		fclose(fin);
		frequency=(double)total_count/(double)total_seqlen;
		fprintf(fout,"\n----- Summary of motif search -----\n");
		fprintf(fout,"  Total length of sequence: %ld.\n",total_seqlen);
		fprintf(fout,"  Number of match:          %ld.\n",total_count);
		fprintf(fout,"    Frequency:              %8.5f.\n",frequency);
		fprintf(fout,"************************************************************\n");

		fclose(fout);
		circular=circular_bak;
	}

	else if(c==RAW_SEQ){


		if((seq_in=read_seq(fname1,c,dtype1))==NULL){
			fprintf(stderr,"Error in reading file: %s\n",fname1);
#include 	"exit1.h"
		}

		if(ftouch(fname2)){	
			fprintf(stderr,"Error in opening output file: %s.\n",fname2);
#include	"exit1.h"
		}

#ifdef MAC
#ifdef CODEWARRIOR
        _fcreator=CREATOR_NAME;
        _ftype=FILETYPE_NAME;
#endif
#endif

		if((fout=fopen(fname2,"a+"))==NULL){
			fprintf(stderr,"Cannot open file: %s\n",fname2);
#include 	"exit1.h"
		}
	
		dtype1=check_data_type(seq_in);
		circular_bak=circular;
		circular=seq_in->circular;

		printf("Data type is %s\n",data_type_name[dtype1]);

		if(strlen(seq_in->header)>50){	
			seq_in->header=extract_string(seq_in->header,1,50);
		}
	
		if(clear_gaps(seq_in)){
			fprintf(stderr,"Error in clearing gaps in file: %s\n",seq_in->ID);
#include 	"exit1.h"
		}
		printf("Gaps were cleared.\n");

		fprintf(fout,"************************************************************\n");
		fprintf(fout,"Search matching\n\n");
		fprintf(fout,"  Input file: %s.\n",fname1);
		fprintf(fout,"************************************************************\n");
		fprintf(fout,"\n");

		if((count=write_sites(fout,seq_in,dtype1,motlen,str3,level,mode))<0){ 
			fprintf(stderr,"Error in writing motif sites in file: %s\n",fname2);
#include 	"exit1.h"
		}
		if(count>0) count-=1L;
	
		frequency=(double)count/(double)seq_in->seqlen;
		fprintf(fout,"\n----- Summary of motif search -----\n");
		fprintf(fout,"  Total length of sequence: %ld.\n",seq_in->seqlen);
		fprintf(fout,"  Number of match:          %ld.\n",count);
		fprintf(fout,"    Frequency:              %8.5f.\n",frequency);
		fprintf(fout,"************************************************************\n");
		fclose(fout);
		circular=circular_bak;
	}

	else {
		fprintf(stderr,"This file is not compatible with this program: %s\n",fname2);
#include 	"exit1.h"
	}


		printf("\nTerminated.\n");

#include	"exit0.h"

}

/************************************************************
		End of file sites.c
************************************************************/

