/***********************************************************************
Split codons (splcod) 
main file : splcod.c


 	Splits the three codon positions of a CLUSTAL sequence file.

  Usage: splcod infile 
  
  Source files are:	splcod.c
  			gtsqfn.c
  			utility.c
  			Dmc.h
  			getseq2.h
  			cmptbl.h
  	
 	Naoki Sato   Feb. 2, 2000.  
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.
	For Mac version, define the compiler as well.
	
	This version is wrote as a single application, not
	as a part of the SISEQ. Do not attempt to compile
	with the '#define INTEGRATED' uncommented in the 
	Dmc.h file.

		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"

#ifndef INTEGRATED
#ifdef MAC
extern int ccommand(char ***);
#endif
#endif

/* Prototype declaration */
static void print_usage(void);
#ifdef INTEGRATED
int splcod(int c, char (*p)[S_2]);
#endif
void help_splcod(void);
static void print_welcome(void);

#include "libSeq.h"

/* Global variables */
extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];

#define SPLCOD_VERSION "1.0"
#define ALLRIGHTS "All rights reserved. Naoki Sato 2000."

 

/***************************************************************/
static void print_usage(void)
/***************************************************************/
{
	help_splcod();
	return;
}
/***************************************************************/

/***************************************************************/
void help_splcod(void)
/***************************************************************/
{
        printf("****************** SPLCOD HELP ****************************\n");
        printf("Usage: splcod infile \n");
        printf("       This program splits the three codon positions of a\n");
	printf("       CLUSTAL files. New three files will be created. \n");
        printf("***********************************************************\n");
	return;
}
/***************************************************************/

/***************************************************************/
static void print_welcome(void)
/***************************************************************/
{
	char version_string[20]=SPLCOD_VERSION;
	char allrights[80]=ALLRIGHTS;

	printf("\n");
	printf("********************************************\n");
	printf("        This is SPLCOD, a utility \n");
	printf("   for splitting a CLUSTAL alignment.\n");
	printf("            Version %s. \n",version_string);
	printf("    %s\n",allrights);
	printf("********************************************\n\n");

	return;
}
/***************************************************************/




/***************************************************************
****************** Main program *******************************/
#ifdef INTEGRATED
int splcod(int cc, char (*p)[S_2])

#else
#ifdef MAC
main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif

#endif
/***************************************************************/

{
	char fname1[100]="infile";
	char fname1r[100]="";
	char fname2[100]="outfile";
	int c=0;
	int i=0;	/* counter */
	register long k;
	char str[100];
	char tmpstr[100];
	char *str1="Input file name";
	char *str2="Output file name";
	char old_name[100];
	char new_name[100];
	Boolean interactive_mode=FALSE;
	long j,pos;	/* j is a counter for sequences, pos is a counter within a sequence. */
	long seqlen;
	enum data_type dtype1=unknown;
	Seqfile **seq_in,***seq_out;

#include	"parsarg.h"
	
/*********************** File names ******************/

	print_welcome();

	switch(cc){
		case 1:	
			strcpy(fname1,"infile");
			if(filename_input(fname1,str1)!=1){
#include	"exit1.h"
			}
			break;
		
		case 2:
			strcpy(fname1,p[1]);
			break;
			
		default:
			print_usage();
#include	"exit1.h"
	}
		
/***********************Main part****************************/

	printf("Reading sequence...\n");

#ifdef MAC
#ifdef CODEWARRIOR
	SIOUXSetTitle("\psplcod");
#endif
#endif
	c=check_filetype(fname1);
	if(c!=CLUSTAL_SEQ) {
		fprintf(stderr,"This is not a CLUSTAL file.\n");
#include 	"exit1.h"
	}

	if((seq_in=read_clustal(fname1,dtype1))==NULL){
		fprintf(stderr,"Error in reading file: %s\n",fname1);
#include 	"exit1.h"
	}

	if((seq_out=(Seqfile***)calloc(4,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Error in creating internal file\n");
#include 	"exit1.h"
	}

	for(i=0;i<3;i++){
		if((seq_out[i]=(Seqfile**)calloc(num_seq+1,sizeof(Seqfile)))==NULL){
			fprintf(stderr,"Error in creating internal file\n");
#include 	"exit1.h"
		}
	}

/******** creating three files *****************/

	for(i=0;i<3;i++){
		for(j=0;j<num_seq;j++){
			if((seq_out[i][j]=copy_seq_h(seq_in[j]))==NULL){
				fprintf(stderr,"Error in copying internal file\n");
#include 	"exit1.h"
			}
		}
	}
	
	seqlen=seq_in[0]->seqlen/3+3;
	
	for(i=0;i<3;i++){
		for(j=0;j<num_seq;j++){
			if((seq_out[i][j]->seq=(char*)calloc(seqlen,sizeof(char)))==NULL){
				fprintf(stderr,"Error in creating internal file\n");
#include 	"exit1.h"
			}
		}
	}
	
	for(pos=0;pos<seq_in[0]->seqlen;pos++){
		for(j=0;j<num_seq;j++){
			seq_out[pos%3][j]->seq[pos/3]=seq_in[j]->seq[pos];
		}
	}

	for(i=0;i<3;i++){
		for(j=0;j<num_seq;j++){
			seq_out[i][j]->seqlen=strlen(seq_out[i][j]->seq);
		}
	}

/********* write sequence **********************/

	if(tailtype(fname1,".aln")){
		strcpy(fname1r,extract_string(fname1,1,strlen(fname1)-4));
	} 
	else strcpy(fname1r,fname1);

	for(i=0;i<3;i++){
		sprintf(fname2,"%s_%d.aln",fname1r,i+1);
		if(write_clustal(fname2,seq_out[i])){
			fprintf(stderr,"Error in writing sequence.\n");
#include 	"exit1.h"
		}
	}


	for(j=0;j<num_seq;j++){
		free_seq(seq_in[j]);
	}
	free(seq_in);
	for(i=0;i<3;i++){
		for(j=0;j<num_seq;j++){
			free_seq(seq_out[i][j]);
		}
	}
	for(i=0;i<3;i++){
		free(seq_out[i]);
	}
	free(seq_out);
	
	printf("\nTerminated.\n");

#include 	"exit0.h"

}

/************************************************************
		End of file splcod.c
************************************************************/


