/***********************************************************************
Extracts database entries with keyword matching (getent).
main file : getent.c

  Usage: getent dbfile outfile AC=.... ID=... OS=... DE=...
  
  Default outfile name is "infile"+".out"
  
 	Naoki Sato   May 30, 1999.
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.
	For Mac version, define the compiler as well.

	Added parsing of quoted string to allow spaces 
	within a key word string. Aug. 25, 1999.
	Added Swiss Prot. Sept. 21, 1999.
	Bug fix. Oct. 3, 1999.
	siseq_exit0 or 1 changed to exit0 or 1. Nov. 13, 1999.
	Inclusion of Japanese messages. March 3, 2000.
	
		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"

#ifndef INTEGRATED
#ifdef MAC
extern int ccommand(char ***);
#endif
#endif

/* Prototype declaration */
static void print_usage(void);

#ifdef INTEGRATED
int getent(int c, char (*p)[S_2]);
#endif
void help_getent(void);

/* external functions */

#include "libSeq.h"

/* Global variables */
extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];

/***************************************************************/
static void print_usage(void)
/***************************************************************/
{
	help_getent();
	return;
}
/***************************************************************/

/***************************************************************/
void help_getent(void)
/***************************************************************/
{
#ifdef JAPANESE
#ifdef UNIX
#include "EUC/getentE.h"
#else
#ifdef MAC
#include "Mac/SJIS/getentS.h"
#else
#include "Dos/SJIS/getentS.h"
#endif
#endif
#else
        printf("********************* GETENT HELP *****************************\n");
        printf("Usage: getent dbfile outfile ID=... AC=... OS=... DE=...\n");
        printf("       dbfile: database file. Upto 10 ID, accession, organism,\n");
	printf("       or definition are used as keys.\n");
        printf("Wild card (#, not *) can be used at the end of ID and AC.\n");
        printf("***************************************************************\n");
#endif
	return;
}
/***************************************************************/


/***************************************************************
****************** Main program *******************************/
#ifdef INTEGRATED
int getent(int cc, char (*p)[S_2])

#else
#ifdef MAC
main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif

#endif
/***************************************************************/

{
	FILE *fin;
	FILE *fout;
	char fname1[100]="dbfile";
	char fname2[100]="outfile";
	int c=0;
        char str[MAXLINE]="";
	int i=0;
	int j=0;
	int k=0;
	int m=0;
	int n=0;
	int jmax=0;
	int kmax=0;
	int mmax=0;
	int nmax=0;
	int j1=0;
	int k1=0;
	int m1=0;
	int n1=0;
        char str3[MAXLINE]="";
        char str4[MAXLINE]="";
/*	char *str1="Input file name";
	char *str2="Output file name";
*/
	char IDstr[10][30];
	char ID[30]="";
	char ACstr[10][30];
	char OSstr[10][30];
	char DEstr[10][30];
	long fileloc;
	long fileloc_old;
	long fileloc0=0L;
	Boolean wflag=FALSE;

#include	"parsarg.h"
	
/*********************** File names ******************/
/* Always complete input is needed.
	if(cc==1){
			strcpy(fname1,"dbfile");
			if(filename_input(fname1,str1)!=1){
#include	"exit1.h"
			}
			strcpy(fname2,fname1);
			strcat(fname2,".out");
			if(filename_input(fname2,str2)!=1){
#include	"exit1.h"
			}
			 need input 
	}
	else */



	if(cc>3){
		strcpy(fname1,p[1]);
		strcpy(fname2,p[2]);
		for (i=3;i<cc && j<10 && k<10;i++){
			if(linetype(p[i],"ID=")){
				strcpy(IDstr[j++],(p[i]+3*sizeof(char)));
				for(j1=0;j1<j-1;j1++){
					if(!strcmp(IDstr[j-1],IDstr[j1])){
						j--;
						break;
					}
				} 
				
			} else if(linetype(p[i],"AC=")){
				strcpy(ACstr[k++],(p[i]+3*sizeof(char)));
				for(k1=0;k1<k-1;k1++){
					if(!strcmp(ACstr[k-1],ACstr[k1])){
						k--;
						break;
					}
				}
			}
			else if(linetype(p[i],"OS=")){
				strcpy(OSstr[m++],(p[i]+3*sizeof(char)));
				if(OSstr[m-1][0]==34 && OSstr[m-1][strlen(OSstr[m-1])-1]==34){
					erasequote(OSstr[m-1]);
				}
				for(m1=0;m1<m-1;m1++){
					if(!strcmp(OSstr[m-1],OSstr[m1])){
						m--;
						break;
					}
				}
			}
			else if(linetype(p[i],"DE=")){
				strcpy(DEstr[n++],(p[i]+3*sizeof(char)));
				if(DEstr[n-1][0]==34 && DEstr[n-1][strlen(DEstr[n-1])-1]==34){
					erasequote(DEstr[n-1]);
				}
				for(n1=0;n1<n-1;n1++){
					if(!strcmp(DEstr[n-1],DEstr[n1])){
						n--;
						break;
					}
				}
			}
			else print_usage();
		}
		jmax=j;
		kmax=k;
		mmax=m;
		nmax=n;
		if(j==0 && k==0 && m==0 && n==0){
			print_usage();
#include	"exit1.h"
		}
	}
	else{
		print_usage();
#include	"exit1.h"
	}



/*********************** Main part ****************************/

	c=check_filetype(fname1);
	if(c!=GENBANK_SEQ && c!=EMBL_SEQ && c!=SW_SEQ){
		fprintf(stderr,"This is not a GenBank, EMBL, or SwissProt file.\n");
		exit(1);
#include	"exit1.h"
	}
	printf("Reading sequence...\n");

#ifdef MAC
#ifdef CODEWARRIOR
	SIOUXSetTitle("\pgetent");
#endif
#endif

        if((fin=fopen(fname1,"r"))==NULL){
                fprintf(stderr,"Cannot open file: %s\n\n",fname1);
#include	"exit1.h"
        }

	if(ftouch(fname2)){
		fprintf(stderr,"Cannot open output file: %s.\n\n",fname2);
#include	"exit1.h"
	}

	if((fout=fopen(fname2,"a+"))==NULL){
		fprintf(stderr,"Cannot open output file: %s.\n\n",fname2);
#include	"exit1.h"
	}

/********************** main loop **********************/

	fileloc=ftell(fin);
	while(fgets(str,MAXLINE+1,fin)!=NULL){
		fileloc_old=fileloc;
		fileloc=ftell(fin);
		
		if(wflag) fprintf(fout,"%s",str);
		else{
			if(linetype(str,GB_INIT) || linetype(str,EMBL_INIT)){
				subst(str,';',' ');
				sscanf(str,"%s %s",str3,str4);
				fileloc0=fileloc_old;
				strcpy(ID,str4);
				for(j=0;j<jmax;j++){
					if(tailtype(IDstr[j],"#")){
						if(linetype(str4,extract_string(IDstr[j],1,strlen(IDstr[j])-1))){
							wflag=TRUE;
							fprintf(fout,"%s",str);
							printf("Start copying database: %s\n",ID);
							fflush(stdout);
							break;
						}
					}
					else if(!strcmp(IDstr[j],str4)) {
						wflag=TRUE;
						fprintf(fout,"%s",str);
						printf("Start copying database: %s\n",ID);
						fflush(stdout);
						break;
					}
				}
				if(wflag) continue;
			}
			else if(linetype(str,GB_AC) || linetype(str,EMBL_AC)){
				subst(str,';',' ');
				sscanf(str,"%s %s",str3,str4);
				for(k=0;k<kmax;k++){
					if(tailtype(ACstr[k],"#")){
						if(linetype(str4,extract_string(ACstr[k],1,strlen(ACstr[k])-1))){
							wflag=TRUE;
							fprintf(fout,"%s",str);
							printf("Start copying database: %s\n",ID);
							fflush(stdout);
							if(fseek(fin,fileloc0,SEEK_SET)){
								fprintf(stderr,"Error in accessing file.\n");
#include		"exit1.h"
							}
							break;
						}
					}
					else if(!strcmp(ACstr[k],getAC(str4))) {
						wflag=TRUE;
						printf("Start copying database: %s\n",ID);
						fflush(stdout);
						if(fseek(fin,fileloc0,SEEK_SET)){
							fprintf(stderr,"Error in accessing file.\n");
#include		"exit1.h"
						}
						break;
					}
				}
				if(wflag) continue;
			}
			else if(linetype(str,GB_OS) || linetype(str,EMBL_OS)){
				strcpy(str3,str_shift(str,' '));	
				for(m=0;m<mmax;m++){
					if(strstr(str3,OSstr[m])!=NULL) {
						wflag=TRUE;
						printf("Start copying database: %s\n",ID);
						fflush(stdout);
						if(fseek(fin,fileloc0,SEEK_SET)){
							fprintf(stderr,"Error in accessing file.\n");
#include		"exit1.h"
						}
						break;
					}
				}
				if(wflag) continue;
			}
			else if(linetype(str,GB_DE) || linetype(str,EMBL_DE)){
				strcpy(str3,str_shift(str,' '));	
				for(n=0;n<nmax;n++){
					if(strstr(str3,DEstr[n])!=NULL) {
						wflag=TRUE;
						printf("Start copying database: %s\n",ID);
						fflush(stdout);
						if(fseek(fin,fileloc0,SEEK_SET)){
							fprintf(stderr,"Error in accessing file.\n");
#include		"exit1.h"
						}
						break;
					}
				}
				if(wflag) continue;
			}
		}
		if(linetype(str,GB_SEQ_END) || linetype(str,EMBL_SEQ_END)){
			wflag=FALSE;
			strcpy(ID,"");
		}
	}

	fclose(fin);
	fclose(fout);
	printf("\nTerminated.\n");

#include	"exit0.h"
}

/************************************************************
		End of file getent.c
************************************************************/

