/***********************************************************************
Extract nucleic acid sequence for CDS (CDSnuc)
File name: cdncfn.c

Functions: 	exon_list
		ext_introns
		add_seq
		add_seqG
		check_se
		free_exon
		TransSplicing
		ExternalSequence
		ExternalSequence2
		getAC
		get_exon_number
			
 	Naoki Sato   Jan 8, 1998.
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.

	Added support for the type "circular". Bug fixes. Nov. 20, 1998.
	Adapted to reading a multi GB file. Jan. 09. 1999.
	Adapted to reading an EMBL file. March 15, 1999.
	Added check_se. March 13, 1999.
	Rewrite of add_seq. March 16, 1999.
	Separated exon_list from add_seq. March 18, 1999.
	Bug fix in add_seq. April 29, 1999.
	Modified exon_list to use AC. Added free_exon. May 7-8, 1999.
	Added getAC,ExternalSequence,TransSplicing. May 9, 1999.
	Modified add_seq. Catenated exons are used within a CDS. May 23, 1999.
	Dual add_seq. Old add_seq is re-used as add_seqG. May 30, 1999.
	Added ext_introns and get_exon_number. Dec. 7, 1999.
	Modify exon_list. Returns comments. Oct. 25, 2000.
	add_seq_split. add_seq is modified accordingly. May 17, 2006.
	add_seq_mutlist. June 6, 2010.
	
		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"

/* Prototype declaration */
void print_mutlist(char *fname2);
int check_se(char *start_loc,char *end_loc);
int add_seq_mutlist(char *fname2,Seqfile *seq_in,Seqfile **seq_nuc,MutList *mlist,int mutlist_num, char *codon);
int add_seq(Seqfile *seq_in,Seqfile **seq_nuc,char *start_loc,long startpos,char *end_loc,long endpos);
int add_seq_split(Seqfile *seq_in,Seqfile **seq_nuc, int n);
int ext_introns(Seqfile *seq_in,Seqfile *seq_nuc,Seqfile **seqM,char *start_loc,long startpos,char *end_loc,long endpos);
int add_seqG(Seqfile *seq_in,Seqfile **seq_nuc,char *start_loc,long startpos,char *end_loc,long endpos);
int exon_list(char *header,Exon_record **exon,char *comment);
void free_exon(Exon_record **exon,int exons);
Boolean TransSplicing(Exon_record **exon,int exons);
Boolean ExternalSequence(Exon_record **exon,int exons,char *ACstr);
Boolean ExternalSequence2(Exon_record *exon,char *ACstr);
char *getAC(char *ACstr);
int get_exon_number(char* header);


/* External functions */
#ifndef __CDNCFN
#define __CDNCFN
#endif
#include "libSeq.h"

/* Global variables */
Boolean ADDSEQG=FALSE;
extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];
extern Boolean circular;
extern Boolean SEQ_IMPORT;

char glostring[2000];
char glostr1[2000];
char glostr2[2000];
char glostr3[2000];
char glostr4[2000];
char glostr5[2000];
char glostr21[2000];
char glostr22[2000];

/* Definition of the code for the end of line (LF).*/

#define END_OF_LINE 10	/* CR will be ignored in the program. */
#define END_OF_LINE2 13		/* for DOS or Mac->UNIX converted files */

#define GB_CDS "CDS"
#define GB_GENE "/gene="
#define GB_PRODUCT "/product="
#define GB_TR "/translation="
#define GB_COMP "complement"
#define GB_JOIN "join"
/*
#define GB_CJOIN "complement\(join"
*/
/* split mode */
Boolean split_mode=FALSE;
int split_number=5;

/* mutlist mode */
Boolean mutlist_mode=FALSE;
char *code_local;
MutList *mlist_local;
static int mutlist_num_local;

/***************************************************************/
void print_mutlist(char *fname2)
/***************************************************************/
{
	int i;
	MutList *m;
	FILE *fout;

	if((fout=fopen(fname2,"w"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n\n",fname2);
		exit(1);
	}

	fprintf(fout,"pos\tNorg\tNmut\tgeneID\tNpos\tAApos");
	fprintf(fout,"\tcodon_org\tcodon_mut\tAA_org\tAA_mut\n\n");
	for(i=0;i<mutlist_num_local;i++){
		m = mlist_local + i;
		fprintf(fout,"%d\t%ld\t%c\t%c\t%s\t%ld\t%ld",i,m->pos,m->Norg,m->Nmut,m->geneID,m->Npos,m->AApos);
		fprintf(fout,"\t%s\t%s\t%c\t%c\n",m->codon_org,m->codon_mut,m->AA_org,m->AA_mut);
	}

	fclose(fout);
}
/***************************************************************/

/************************************************************************/
int check_se(char *start_loc,char *end_loc)
/************************************************************************/
{
        if((strcmp(start_loc,"S")!=0) && (strcmp(start_loc,"E")!=0) && \
                (strcmp(start_loc,"s")!=0) && (strcmp(start_loc,"e")!=0)){
                return 1;
        }
        if((strcmp(end_loc,"S")!=0) && (strcmp(end_loc,"E")!=0) && \
                (strcmp(end_loc,"s")!=0) && (strcmp(end_loc,"e")!=0)){
                return 1;
        }
        if(strcmp(start_loc,"s")==0) strcpy(start_loc,"S");
        if(strcmp(end_loc,"s")==0) strcpy(end_loc,"S");
        if(strcmp(start_loc,"e")==0) strcpy(start_loc,"E");
        if(strcmp(end_loc,"e")==0) strcpy(end_loc,"E");
        return 0;
}
/************************************************************************/

/********************************************/
void free_exon(Exon_record **exon,int exons)
/********************************************/
{
	int i;

	for(i=0;i<exons;i++) free(exon[i]);
	free(exon);

	return;	
}
/********************************************/


/************************************************************************/
Boolean TransSplicing(Exon_record **exon,int exons)
/* Returns TRUE if the exons are supposed to be trans-spliced. */
/************************************************************************/
{
	int c1,i;

	if(exons==1) return FALSE;

/* First criterion. */
	c1=exon[0]->nc;
	for(i=1;i<exons;i++){
		if(exon[i]->nc!=c1) return TRUE;	
	} 
	
/* Second criterion. */
	if(c1){	/* complementary */
		for(i=0;i<exons-1;i++){
			if(exon[i]->begin < exon[i+1]->end) return TRUE;
		}
	}
	else {
		for(i=0;i<exons-1;i++){
			if(exon[i]->end > exon[i+1]->begin) return TRUE;
		}
	}
	return FALSE;	
}


/***********************************************************************************/
Boolean ExternalSequence2(Exon_record *exon,char *ACstr)
/* Returns TRUE if the exon segments are imported from external 
sequence files. Single exon.*/
/***********************************************************************************/
{
	char AC[AC_LENGTH];

	strcpy(AC,getAC(ACstr));
	strcpy(ACstr,AC);
	strcpy(AC,"");

	strcpy(AC,getAC(exon->AC));
	if(strcmp(AC,"") && strcmp(AC,ACstr)) return TRUE;
	return FALSE;
}


/***********************************************************************************/
Boolean ExternalSequence(Exon_record **exon,int exons,char *ACstr)
/* Returns TRUE if the exon segments are imported from external 
sequence files. */
/***********************************************************************************/
{
	int i;
	char AC[AC_LENGTH];

	strcpy(AC,getAC(ACstr));
	strcpy(ACstr,AC);
	strcpy(AC,"");

	for(i=0;i<exons;i++){
		strcpy(AC,getAC(exon[i]->AC));
		if(strcmp(AC,"") && strcmp(AC,ACstr)) return TRUE;
	}
	return FALSE;
}

/*************************************************************/
char *getAC(char *ACstr)
/* returns AC part from "AC.Verion" string of GenBank 111. */
/*************************************************************/
{
	char AC[AC_LENGTH];
	char *p,*q,*r;
	
	r=AC;
	q=AC;
	for(p=ACstr;*p!='\0';p++){
		if(*p=='.') {
			*q='\0';
			return r;
		}
		else *q++=*p;
	}
	*q=*p;
	return r;
}


static int hl6=MINMEMSIZE; /* header_length */

/*****************************************************************/
int get_exon_number(char* header)
/* This function reads header line that was created from database
file (/CDS line), and returns the number of exons. */
/*****************************************************************/
{
	static char *str1;
	static char *str2;
	static char *str3;

	hl6=strlen(header);
	if((str1=(char*)realloc(str1,(hl6+2)*sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		exit(1);
	}
	if((str2=(char*)realloc(str2,(hl6+2)*sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		exit(1);
	}
	if((str3=(char*)realloc(str3,(hl6+2)*sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		exit(1);
	}

	strcpy(str1,header);
	strcpy(str2,str_shift(str1,' ')); /* str1 is gene name */
	strcpy(str3,str_shift(str2,' ')); /* str2 is exon number string */
	if(!num_check(str2)) return atoi(str2);
	else return 0;
}

/*****************************************************************************************************************/
int add_seq_mutlist(char *fname2,Seqfile *seq_in,Seqfile **seq_nuc,MutList *mlist,int mutlist_num, char *codon)
/*****************************************************************************************************************/
{
	int v;

	code_local = codon;
	mutlist_mode=TRUE;
	mlist_local = mlist;
	mutlist_num_local = mutlist_num;
	v=add_seq(seq_in,seq_nuc,"s",0, "e",0);

	print_mutlist(fname2);

	return v;
}



/*****************************************************************************************************************/
int add_seq_split(Seqfile *seq_in,Seqfile **seq_nuc,int n)
/*****************************************************************************************************************/
{
	int v;

	split_mode=TRUE;
	split_number=n;
	v=add_seq(seq_in,seq_nuc,"s",0, "e",0);
	return v;
}


static int hl1=MINMEMSIZE; /* header_length */

/*****************************************************************************************************************/
int add_seq(Seqfile *seq_in,Seqfile **seq_nuc,char *start_loc,long startpos,char *end_loc,long endpos)
/*****************************************************************************************************************/
{
	int i;
	long k;
	long j0,j1,extS,extE,realextS,realextE;
	long extS1,extE1,extS2,extE2;
	long a1,z1;
	int exons=0;
	Boolean circular_bak=FALSE; /* TRUE if circular. */
	static char *str1;
	static char *str2;
	static char *str3;
	char comment[MAXLINE];
	char *header;
	char *ID;
	char *p;
	static char ACstr[MINMEMSIZE];
	long seqlen,seqlenEX,seqlen2;
	Exon_record **exon;
	Seqfile *seq_tmp;
	Boolean comp_join=FALSE; /* flag for GB complementary join */
	Boolean error_flag=FALSE;
	Boolean errorS=FALSE;
	Boolean errorE=FALSE;

	seqlen=seq_in->seqlen;
	circular_bak=circular;
	circular=seq_in->circular;
		/* mutlist mode */
	long pos, pos_local, codon_local, triplet_position;
	char triplet[10]="   ";
	char triplet_mut[10]="   ";
	char Norg,Nmut;
	int AA_org,AA_mut;
	int j;


	if(str1==NULL || str2==NULL || str3==NULL){
		if((str1=(char*)calloc((hl1+2),sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory.\n");
			return 1;
		}
		if((str2=(char*)calloc((hl1+2),sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory.\n");
			return 1;
		}
		if((str3=(char*)calloc((hl1+2),sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory.\n");
			return 1;
		}
	}


	for(k=0;k<num_seq;k++){
		error_flag=FALSE;
		comp_join=FALSE;

		if(strcmp(seq_nuc[k]->ID,seq_in->ID)) continue;

		if(k==0 || (k>0 && hl1<(int)strlen(seq_nuc[k]->header))){
			hl1=strlen(seq_nuc[k]->header);
			if((str1=(char*)realloc(str1,(hl1+2)*sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory.\n");
				return 1;
			}
			if((str2=(char*)realloc(str2,(hl1+2)*sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory.\n");
				return 1;
			}
			if((str3=(char*)realloc(str3,(hl1+2)*sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory.\n");
				return 1;
			}
		}
		ID=seq_nuc[k]->ID;
		header=seq_nuc[k]->header;	
                strcpy(str1,seq_nuc[k]->header);
                strcpy(str2,str_shift(str1,' ')); /* str1 is gene name */
                strcpy(str3,str_shift(str2,' ')); /* str2 is exon number string */
                if(!num_check(str2)) exons=atoi(str2);
		else continue;

/**************************************************/
		if((exon=(Exon_record**)calloc(exons+1,sizeof(Exon_record)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return 1;
		}
		for(i=0;i<exons;i++){
			if((exon[i]=(Exon_record*)calloc(1,sizeof(Exon_record)))==NULL){
				fprintf(stderr,"Out of memory\n");
				return 1;
			}
		}
		if((p=strstr(str3,"AC="))!=NULL){
			subst(str3,';',' ');
			sscanf(p+3,"%s",ACstr);
		}
		strcpy(comment,"                                                    ");
		if(exon_list(seq_nuc[k]->header,exon,comment)){
			fprintf(stderr,"\nError in making exon list. Skipping %s %s.\n",ID,str1);
			continue;
		}
	/* extracting sequence */

		if(!SEQ_IMPORT){
			if(strcmp(exon[0]->AC,"") && strcmp(getAC(exon[0]->AC),ACstr)){
				fprintf(stderr,"\nSequence import from another file: %s. Not currently supported. " \
					,exon[0]->AC);
				fprintf(stderr,"Skipping %s %s.\n",seq_nuc[k]->ID,str1);
				continue;
			}
			if(strcmp(exon[exons-1]->AC,"") && strcmp(getAC(exon[exons-1]->AC),ACstr)){
				fprintf(stderr,"\nSequence import from another file: %s. Not currently supported. " \
					,exon[exons-1]->AC);
				fprintf(stderr,"Skipping %s %s.\n",seq_nuc[k]->ID,str1);
				continue;
			}
		}

/**************************************************************/

		if(SEQ_IMPORT) seqlenEX=exon_total2(exon,exons,ACstr,seqlen);
		else seqlenEX=exon_total(exon,exons,ACstr,seqlen);

		if((exon[0]->begin_ext && !exon[0]->nc) || (exon[0]->end_ext && exon[0]->nc)) errorS=TRUE; 
		else errorS=FALSE;
		if((exon[exons-1]->end_ext && !exon[0]->nc) || (exon[exons-1]->begin_ext && exon[0]->nc)) errorE=TRUE;
		else errorE=FALSE;
		if((errorS && !strcmp(start_loc,"S") && (startpos != 0)) || (errorE && !strcmp(end_loc,"E") && (endpos != 0))) {
			fprintf(stderr,"Excluded %s %s.\n",ID,str1);
			strcpy(seq_nuc[k]->seq,"*");
			seq_nuc[k]->seqlen=strlen(seq_nuc[k]->seq);
			continue;
		}
	
		extS=extE=a1=extS1=extE1=extS2=extE2=0L;

		if(!circular){
			if(exon[0]->nc) {
				if(SEQ_IMPORT && ExternalSequence2(exon[0],ACstr)){
					seqlen2=ExternalSeqlen(exon[0]->AC);
					if(seqlen2==0L){
						strcpy(seq_nuc[k]->seq,"*");
						seq_nuc[k]->seqlen=strlen(seq_nuc[k]->seq);
						continue;
					}
					realextS=seqlen2 - exon[0]->end;
				}
				else realextS=seqlen - exon[0]->end;
			}
			else realextS=exon[0]->begin - 1;
			if(exon[exons-1]->nc) realextE=exon[exons-1]->begin - 1;
			else {
				if(SEQ_IMPORT && ExternalSequence2(exon[exons-1],ACstr)){
					seqlen2=ExternalSeqlen(exon[exons-1]->AC);
					if(seqlen2==0L){
						strcpy(seq_nuc[k]->seq,"*");
						seq_nuc[k]->seqlen=strlen(seq_nuc[k]->seq);
						continue;
					}
					realextE=seqlen2 - exon[exons-1]->end;
				}
				else realextE=seqlen - exon[exons-1]->end;
			}
		}
		else{
			realextS=0L;
			realextE=0L;
		}
		if(!strcmp(start_loc,"S") && startpos<0){
			extS=0-startpos;
			if(extS>realextS && !circular) {
				extS1=extS-realextS;
				errorS=TRUE;
				extS=realextS;
			}
			if(exon[0]->nc) exon[0]->end+=extS;
			else exon[0]->begin-=extS;
		}
		if(!strcmp(end_loc,"E") && endpos>0){
			extE=endpos;
			if(extE>realextE && !circular) {
				extE1=extE-realextE;
				errorE=TRUE;
				extE=realextE;
			}
			if(exon[exons-1]->nc) exon[exons-1]->begin-=extE;
			else exon[exons-1]->end+=extE;
		}
		if(!strcmp(end_loc,"S")){
			if(endpos>seqlenEX){
				extE2=endpos-seqlenEX;
				errorE=TRUE;
			}
		}
		if(!strcmp(start_loc,"E")){
			if(seqlenEX+startpos<1){
				extS2=1-seqlenEX-startpos;
				errorS=TRUE;
			}
		}
		if((seq_tmp=join_exons2(seq_in,exon,exons,ACstr))==NULL){
			fprintf(stderr,"Error in joining exons for %s %s.\n",ID,str1);
			strcpy(seq_nuc[k]->seq,"*");
			seq_nuc[k]->seqlen=strlen(seq_nuc[k]->seq);
			continue;
		}
		else if(seq_tmp->seqlen<=1 || strlen(seq_tmp->seq)<2){
			fprintf(stderr,"Error in joining exons for %s %s.\n",ID,str1);

#ifdef VERBOUS
	printf("seq_nuc[%ld]->seq= %s.\n",k,seq_tmp->seq);
	pause_screen();
#endif

			strcpy(seq_nuc[k]->seq,"*");
			seq_nuc[k]->seqlen=strlen(seq_nuc[k]->seq);
			continue;
		}
		else seq_nuc[k]=seq_tmp;
		seq_nuc[k]->header=header;
		seq_nuc[k]->ID=ID;
		strcpy(seq_nuc[k]->AC,ACstr);
		seq_nuc[k]->seqlen=strlen(seq_nuc[k]->seq);

		z1=j0=strlen(seq_nuc[k]->seq);
		j1=j0-extS-extE;
		if(j0<0 || j1<0) {
			fprintf(stderr,"Error of chopping sides in add_seq.\n");
			strcpy(seq_nuc[k]->seq,"*");
			seq_nuc[k]->seqlen=strlen(seq_nuc[k]->seq);
			continue;
		}

		if(!strcmp(start_loc,"S") && startpos>0) a1=0L + startpos;
		if(!strcmp(start_loc,"E")) a1=j1 + extS + startpos;
		if(!strcmp(end_loc,"E") && endpos<0) z1=j1 + extS + endpos;
		if(!strcmp(end_loc,"S")) z1=extS + endpos;
		if( z1<1 || a1>j0 ) {
			fprintf(stderr,"Error of chopping sides in add_seq.\n");
			strcpy(seq_nuc[k]->seq,"*");
			seq_nuc[k]->seqlen=strlen(seq_nuc[k]->seq);
			continue;
		}
		if(pick_seq(seq_nuc[k],a1,z1)){
			fprintf(stderr,"Error in chopping sides in add_seq.\n");
			return 1;
		}
		if((errorS || errorE) && !circular) {

#ifdef VERBOUS
			printf("extS1=%ld,extE1=%ld,extS2=%ld,extE2=%ld.",extS1,extE1,extS2,extE2);
#endif

			if((seq_nuc[k]->seq=(char*)realloc(seq_nuc[k]->seq,(j0+extS1+extE1+extS2+extE2+2)*sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory\n");
				return 1;
			}
			fill_both_sides(seq_nuc[k]->seq,extS1+extS2,extE1+extE2,'-');
		}

		seq_nuc[k]->seqlen=strlen(seq_nuc[k]->seq);

/* mutlist mode */
		if(mutlist_mode){
			for(j=0;j<mutlist_num_local;j++){
				pos = mlist_local[j].pos;
				Norg = mlist_local[j].Norg;
				Nmut = mlist_local[j].Nmut;
				pos_local = 0;
				for(i=0;i<exons;i++){
					if(exon[i]->nc == 0){	/* normal */
						if(exon[i]->begin <= pos && pos <= exon[i]->end){
							pos_local += pos - exon[i]->begin + 1;
							printf("j=%d normal strand\n",j+1);
							break;
						} else pos_local += (exon[i]->end - exon[i]->begin + 1);
					} else {	/* complementary */
						if(exon[i]->begin <= pos && pos <= exon[i]->end){
							pos_local += exon[i]->end - pos + 1;
							if(Norg == '-') pos_local -= 1;
							if('A' <= Norg && Norg <= 'z') Norg = to_compD(Norg);
							if('A' <= Nmut && Nmut <= 'z') Nmut = to_compD(Nmut);
							printf("j=%d complementary strand\n",j+1);
							break;
						} else pos_local += (exon[i]->end - exon[i]->begin + 1);

					}
				}
				if(i >= exons) continue;
				codon_local = (pos_local - 1) / 3;	/* counting starts from 0 */
				strncpy(triplet,((seq_nuc[k]->seq)+(codon_local * 3)),4);
				triplet_position = pos_local - 3 * codon_local - 1;
				strcpy(triplet_mut,triplet);

	printf("triplet_position=%ld\n",triplet_position);

				if(Norg == '-'){
					if(triplet_position == 0){
						triplet_mut[triplet_position+2] = triplet_mut[triplet_position+1];
						triplet_mut[triplet_position+1] = triplet_mut[triplet_position];
						triplet_mut[triplet_position] = Nmut;
					} else if(triplet_position == 1){
						triplet_mut[triplet_position+1] = triplet_mut[triplet_position];
						triplet_mut[triplet_position] = Nmut;
					} else {
						triplet_mut[triplet_position] = Nmut;
					}
				} else if(Nmut == '-'){
					if(triplet_position == 0){
						triplet_mut[triplet_position] = triplet_mut[triplet_position+1];
						triplet_mut[triplet_position+1] = triplet_mut[triplet_position+2];
						triplet_mut[triplet_position+2] = triplet_mut[triplet_position+3];
					} else if(triplet_position == 1){
						triplet_mut[triplet_position] = triplet_mut[triplet_position+1];
						triplet_mut[triplet_position+1] = triplet_mut[triplet_position+2];
					} else {
						triplet_mut[triplet_position] = triplet_mut[triplet_position+1];
					}
				} else {
					if(triplet_mut[triplet_position] == Norg){
						triplet_mut[triplet_position] = Nmut;
					}
				}

				triplet[3] = '\0';
				triplet_mut[3] = '\0';
				if((AA_org = codon_translate(triplet,code_local))== '?'){
					AA_org = '-';
				}
				if((AA_mut = codon_translate(triplet_mut,code_local))== '?'){
					AA_mut = '-';
				}

				strcpy(mlist_local[j].codon_org,triplet);
				strcpy(mlist_local[j].codon_mut,triplet_mut);
				mlist_local[j].AA_org = AA_org;
				mlist_local[j].AA_mut = AA_mut;
				mlist_local[j].Npos = pos_local + 1;
				mlist_local[j].AApos = codon_local + 1;

				sscanf(seq_nuc[k]->header,"%s",mlist_local[j].geneID);
				fprintf(stderr,"%d: AA_org=%c, codon=%s, codon_mut=%s.\n",j+1,mlist_local[j].AA_org, triplet,triplet_mut);
				strcpy(triplet,"  ");
				strcpy(triplet_mut,"  ");
			}
		}

		free_exon(exon,exons);
		strcpy(str1,"");
		strcpy(str2,"");
		strcpy(str3,"");

		if(k%10==0) {
/*			printf(".");*/
			putc(8,stdout);
			putc('.',stdout);
			fflush(stdout);
		}
	}
	circular=circular_bak;	
	return 0;
}

static int hl5=MINMEMSIZE; /* header_length */

/********************************************************************************************************************/
int ext_introns(Seqfile *seq_in,Seqfile *seq_nuc,Seqfile **seqM,char *start_loc,long startpos,char *end_loc,long endpos)
/* Note that num_seq is used as the number of sequences in seqM. 
   Return value 1 is error, 10 is no intron. */
/********************************************************************************************************************/
{
	int i;
	long k;
	long j0,j1,extS,extE,realextS,realextE;
	long extS1,extE1,extS2,extE2;
	long a1,z1;
	int exons=0;
	int introns=0;
	Boolean circular_bak=FALSE; /* TRUE if circular. */
	static char *str1;
	static char *str2;
	static char *str3;
	char *header;
	char *ID;
	char *p;
	static char ACstr[MINMEMSIZE];
	char comment[MAXLINE]; 
	long seqlen,seqlenINT;
	Exon_record **exon;
	Boolean comp_join=FALSE; /* flag for GB complementary join */
	Boolean error_flag=FALSE;
	Boolean errorS=FALSE;
	Boolean errorE=FALSE;
	Boolean no_intron=FALSE;
	Boolean empty_seq=FALSE;
	int nc=0;	/* 1 if complementary. */

	seqlen=seq_in->seqlen;
	circular_bak=circular;
	circular=seq_in->circular;


	if(str1==NULL || str2==NULL || str3==NULL){
		if((str1=(char*)calloc((hl5+2),sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory.\n");
			return 1;
		}
		if((str2=(char*)calloc((hl5+2),sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory.\n");
			return 1;
		}
		if((str3=(char*)calloc((hl5+2),sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory.\n");
			return 1;
		}
	}


	error_flag=FALSE;
	comp_join=FALSE;

	if(strcmp(seq_nuc->ID,seq_in->ID)){ 
		no_intron=TRUE; 	/* After returning, this CDS should be skipped. */
		goto FINISH;
	}

	hl5=strlen(seq_nuc->header);
	if((str1=(char*)realloc(str1,(hl5+2)*sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return 1;
	}
	if((str2=(char*)realloc(str2,(hl5+2)*sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return 1;
	}
	if((str3=(char*)realloc(str3,(hl5+2)*sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return 1;
	}

	ID=seq_nuc->ID;
	header=seq_nuc->header;	
	strcpy(str1,seq_nuc->header);
	strcpy(str2,str_shift(str1,' ')); /* str1 is gene name */
	strcpy(str3,str_shift(str2,' ')); /* str2 is exon number string */
	if(!num_check(str2)) exons=atoi(str2);
	else{ 
		no_intron=TRUE; 	/* After returning, this CDS should be skipped. */
		goto FINISH;
	}

	if(exons==1){
		no_intron=TRUE; 	/* After returning, this CDS should be skipped. */
		goto FINISH;
	}
	else {
		introns=exons-1;
		num_seq=(long)introns;
	}

/**************************************************/
	if((exon=(Exon_record**)calloc(exons+1,sizeof(Exon_record)))==NULL){
		fprintf(stderr,"Out of memory\n");
		return 1;
	}
	for(i=0;i<exons;i++){
		if((exon[i]=(Exon_record*)calloc(1,sizeof(Exon_record)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return 1;
		}
	}
	if((p=strstr(str3,"AC="))!=NULL){
		subst(str3,';',' ');
		sscanf(p+3,"%s",ACstr);
	}
	strcpy(comment,"                                                              ");
	if(exon_list(seq_nuc->header,exon,comment)){
		fprintf(stderr,"\nError in making exon list. Skipping %s %s.\n",ID,str1);
		no_intron=TRUE;
		goto FINISH;
	}
	/* extracting sequence */

	if(strcmp(exon[0]->AC,"") && strcmp(getAC(exon[0]->AC),ACstr)){
		fprintf(stderr,"\nSequence import from another file: %s.",exon[0]->AC); 
		fprintf(stderr,"Skipping %s %s.\n",seq_nuc->ID,str1);
		no_intron=TRUE;
		goto FINISH;
	}
	if(strcmp(exon[exons-1]->AC,"") && strcmp(getAC(exon[exons-1]->AC),ACstr)){
		fprintf(stderr,"\nSequence import from another file: %s.",exon[exons-1]->AC); 
		fprintf(stderr,"Skipping %s %s.\n",seq_nuc->ID,str1);
		no_intron=TRUE;
		goto FINISH;
	}
	if(TransSplicing(exon,exons)){
		fprintf(stderr,"\nThis CDS might be trans-spliced.\n"); 
		fprintf(stderr,"Skipping %s %s.\n",seq_nuc->ID,str1);
		no_intron=TRUE;
		goto FINISH;
	}


/**************************************************************/
	nc=exon[0]->nc;
/*
	if((seqM=(Seqfile**)calloc(exons,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return 1;
	}
*/
	for(i=0;i<introns;i++){
		k=(long)i;
		empty_seq=FALSE;
		if(nc){
			seqlenINT=exon[i]->begin - exon[i+1]->end + 1;
		}
		else seqlenINT=exon[i+1]->begin - exon[i]->end +1;
		if(seqlenINT<1) {
			printf("Intron length is negative or zero. Skipping %s %s intron %ld.\n",ACstr,str1,k);
			empty_seq=TRUE;
		}
			
		extS=extE=a1=z1=extS1=extE1=extS2=extE2=0L;
		if(!circular){
			if(nc) {
				realextS=seqlen - exon[i]->begin + 1;
			}
			else realextS=exon[i]->end;
			if(nc) realextE=exon[i+1]->end;
			else {
				realextE=seqlen - exon[i+1]->begin + 1;
			}
		}
		else{
			realextS=0L;
			realextE=0L;
		}

		if(!strcmp(start_loc,"S")){
			extS=0-startpos;
			if(startpos<0){
				if(extS>realextS && !circular) {
					extS1=extS-realextS;
					errorS=TRUE;
					extS=realextS;
				}
			}
			if(nc) z1=exon[i]->begin-1+extS;
			else a1=exon[i]->end-extS+1;
		}
		if(!strcmp(end_loc,"E")){
			extE=endpos;
			if(endpos>0){
				if(extE>realextE && !circular) {
					extE1=extE-realextE;
					errorE=TRUE;
					extE=realextE;
				}
			}
			if(nc) a1=exon[i+1]->end+1-extE;
			else z1=exon[i+1]->begin-1+extE;
		}
		if(!strcmp(end_loc,"S")){
			extE=endpos;
			if(endpos>seqlenINT){
				extE2=endpos-seqlenINT;
				errorE=TRUE;
				extE=seqlenINT-1;
			}
			if(nc) a1=exon[i]->begin-1-extE;
			else z1=exon[i]->end+1+extE;
		}
		if(!strcmp(start_loc,"E")){
			extS=0-startpos;
			if(seqlenINT+startpos<1){
				extS2=1-seqlenINT-startpos;
				errorS=TRUE;
				extS=seqlenINT-1;
			}
			if(nc) z1=exon[i+1]->end+1+extS;
			else a1=exon[i+1]->begin-1-extS;
		}
		
		if(a1>=z1) {
			printf("k=%ld, a1=%ld, z1=%ld. Skipping %s %s intron %ld.\n",k,a1,z1,ACstr,str1,k);
			empty_seq=TRUE;
		}

/* for debug */
#ifdef VERBOUS
	printf("k=%ld, a1=%ld, z1=%ld.\n",k,a1,z1);
#endif
		if(!empty_seq){
			if((seqM[k]=extract_seq3(seq_in,a1,z1))==NULL){
				fprintf(stderr,"Error in extracting sequence %s %s intron %ld.\n",ACstr,str1,k);
				empty_seq=TRUE;
			}

			if(!empty_seq && nc) {
				if(comp_seq4(seqM[k])){
				fprintf(stderr,"Error in converting to complementary in %s %s intron %ld.\n",ACstr,str1,k);
				continue;
				}
			}
		}
		if(empty_seq){
			if((seqM[k]=empty_seq5())==NULL){
				fprintf(stderr,"Error in making empty sequence %s %s intron %ld.\n",ACstr,str1,k);
				return 1;
			}
		}

		strcpy(seqM[k]->ID,ID);
		strcpy(seqM[k]->AC,ACstr);
		seqM[k]->seqlen=strlen(seqM[k]->seq);
			/* header string is initialized in extract_seq3 
				with more than 50 characters, so this should be safe. */ 
		sprintf(seqM[k]->header,"%s_intron_%d",str1,i+1);

		z1=j0=strlen(seqM[k]->seq);
		j1=j0-extS-extE;
		if(j0<0 || j1<0) {
			fprintf(stderr,"Error of chopping sides in ext_introns.\n");
			strcpy(seqM[k]->seq,"*");
			seqM[k]->seqlen=strlen(seqM[k]->seq);
			continue;
		}
/*
		if(!strcmp(start_loc,"S") && startpos>0) a1=0L + startpos;
		if(!strcmp(start_loc,"E")) a1=j1 + extS + startpos;
		if(!strcmp(end_loc,"E") && endpos<0) z1=j1 + extS + endpos;
		if(!strcmp(end_loc,"S")) z1=extS + endpos;
		if( z1<1 || a1>j0 ) {
			fprintf(stderr,"Error of chopping sides in ext_introns.\n");
			strcpy(seqM[k]->seq,"*");
			seqM[k]->seqlen=strlen(seqM[k]->seq);
			continue;
		}
		if(pick_seq(seqM[k],a1,z1)){
			fprintf(stderr,"Error in chopping sides in extint.\n");
			continue;
		}
*/

		if((errorS || errorE) && !circular) {

#ifdef VERBOUS
			printf("extS1=%ld,extE1=%ld,extS2=%ld,extE2=%ld.",extS1,extE1,extS2,extE2);
#endif

			if((seqM[k]->seq=(char*)realloc(seqM[k]->seq,(j0+extS1+extE1+extS2+extE2+2)*sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory\n");
				return 1;
			}
			fill_both_sides(seqM[k]->seq,extS1+extS2,extE1+extE2,'-');
		}

			

	}

FINISH:		if(!no_intron) free_exon(exon,exons);
		strcpy(str1,"");
		strcpy(str2,"");
		strcpy(str3,"");
		if(no_intron) return 10;

	circular=circular_bak;	
	return 0;
}


/*****************************************************************************************************/
int exon_list (char *header,Exon_record **exon,char *comment)
/*****************************************************************************************************/
{
	int i;
	int exons;
	int comment_len;
	char *pp;


	char *string=&glostring[0];
	char *str1=&glostr1[0];
	char *str2=&glostr2[0];
	char *str3=&glostr3[0];
	char *str4=&glostr4[0];
	char *str5=&glostr5[0];
	char *p;	/* pointer for the location of : in str2. */
	char *str21=&glostr21[0];
	char *str22=&glostr22[0];	/* container for AC:loc. */
	long atg;
	long stop;
	Exon_record *tmp_exon;
	Boolean comp_join=FALSE; /* flag for GB complementary join */
	Boolean error_flag=FALSE;

	comment_len=strlen(comment);

	strcpy(str1,header);
	strcpy(str2,str_shift(str1,' ')); /* str1 is gene name */
	strcpy(str3,str_shift(str2,' ')); /* str2 is exon number string */
	if(!num_check(str2)) sscanf(str2,"%d",&exons); /* convert to int */
	else return 1;

	strcpy(str2,str3); /* Re-use of str2 and str3 */
	strcpy(string,str2);	/* for later use */
	strcpy(str3,"");
	strcpy(str3,str_shift(str2,' ')); 
	if(!num_check2(str2) || ((p=strchr(str2,':'))!=NULL)){	/* is a number */
		if(exons!=1){
			fprintf(stderr,"\nError 1 of exon number in seq: %s.\n",str1);
			return 1;
		}
	}
	else{
		if(!strcmp(str2,GB_COMP)){
			/* set all nc values to 1, i.e., complementary. */
			for(i=0;i<exons;i++) exon[i]->nc=1;
			strcpy(str4,str3);
			strcpy(str5,str_shift(str4,' '));
			if(!strcmp(str4,GB_JOIN)){
				if(exons<2){
					fprintf(stderr,"\nError 2 of exon number in seq: %s.\n",str1);
					return 1;
				}
				strcpy(str3,str5);
				strcpy(str4,"");
				strcpy(str5,"");
				comp_join=TRUE;
			}
			else{
				strcpy(str3,string);
			}
		}
		else if(!strcmp(str2,GB_JOIN)){ 
			if(exons<2){
				fprintf(stderr,"\nError 3 of exon number in seq: %s.\n",str1);
				return 1;
			}
		}
		else{
			fprintf(stderr,"\nUnrecognizable word: %s in exon_list.\n",str2);
			fprintf(stderr,"Error 4 of exon number in seq: %s.\n",str1);
			return 1;
		}
		strcpy(str2,str3);
		strcpy(str3,str_shift(str2,' '));
	}

	/* Now str2 indicates the start of the first exon. */

	for(i=0;i<exons;i++){
		if(num_check2(str2) && ((p=strchr(str2,':'))==NULL) ){	/* not a number */
			if(!strcmp(str2,GB_COMP)){
				exon[i]->nc=1;
				strcpy(str2,str3);
				strcpy(str3,str_shift(str2,' '));
			}	
			else{
				fprintf(stderr,"\nError 5 of exon number in seq: %s.\n",str1);
				error_flag=TRUE;
				break;	
			}
		}
		if((p=strchr(str2,':'))!=NULL){
			subst(str2,':',' ');
			sscanf(str2,"%s %s",str21,str22);
			if(strlen(str21)<AC_LENGTH) strcpy(exon[i]->AC,str21);
			else {
				fprintf(stderr,"\nError 6 of exon description in seq: %s.\n",str1);
				error_flag=TRUE;
				break;
			}
			strcpy(str2,str22);
			strcpy(str21,"");
			strcpy(str22,"");
		}
		if(*str2=='<') {
			exon[i]->begin_ext=TRUE;
			subst(str2,'<',' ');
			sscanf(str2,"%s",str21);
			strcpy(str2,str21);
			strcpy(str21,"");
		}
		if(num_check(str2)){
			fprintf(stderr,"\nError 7 of exon number in seq: %s.\n",str1);
			error_flag=TRUE;
			break;
		}
		sscanf(str2,"%ld",&atg);
		exon[i]->begin=atg;

		strcpy(str2,str3);
		strcpy(str3,str_shift(str2,' '));

		if((p=strchr(str2,':'))!=NULL){
			subst(str2,':',' ');
			sscanf(str2,"%s %s",str21,str22);
			if(strcmp(exon[i]->AC,str21)){
				fprintf(stderr,"\nError 8 of exon description in seq: %s.\n",str1);
				error_flag=TRUE;
				break;
			}
                        strcpy(str2,str22);
                        strcpy(str21,"");
                        strcpy(str22,"");
                }
		if(*str2=='>') {
			exon[i]->end_ext=TRUE;
			subst(str2,'>',' ');
			sscanf(str2,"%s",str21);
			strcpy(str2,str21);
			strcpy(str21,"");
		}
		if(!num_check(str2)) sscanf(str2,"%ld",&stop);
		else {
			fprintf(stderr,"No valid end of exon in seq: %s.\n",str1);
			error_flag=TRUE;
			break;
		}
		exon[i]->end=stop;

		if(strlen(str3)==0) break;
		strcpy(str2,str3);
		strcpy(str3,str_shift(str2,' '));
	}

	/* The remainder of str3 including comments are copied to comment. Only the 
	portion of the str2, which is short enough to be copied to comment, is
	actually copied. */

	if(strlen(str3) + strlen(str2) < comment_len) sprintf(comment,"%s %s",str2,str3);
/*	else if(strlen(str2)+2 > comment_len) strcpy(comment,"");
*/
	else {
		pp=extract_string(str3,1,comment_len-strlen(str2)-2);
		sprintf(comment,"%s %s",str2,pp);
		free(pp);
	}
	
	if(error_flag) {
		atg=0L;
		stop=0L;
	}

	/* Revert exon stack if comp_join */
	if(comp_join){
		for(i=0;i<exons/2;i++){
			tmp_exon=exon[i];
			exon[i]=exon[exons-i-1];
			exon[exons-i-1]=tmp_exon;
		}
	}

	/* if atg and stop have not been read correctly, most probably because of
	insertion of accession number of other database or corruption of file.*/
	
	if((atg==0L) && (stop==0L)){
/*		fprintf(stderr,"\nSequcence positions were not correctly read.\n"); */
		fprintf(stderr," Quiting exon_list. ");
		return 1;
	}
	
	/* At this point, exons are ordered according to its index. */

#ifdef VERBOUS
	for(i=0;i<exons;i++){
		printf("AC=%s, n/c= %d, begin=%ld, end=%ld.\n",exon[i]->AC,exon[i]->nc,exon[i]->begin,exon[i]->end);
	}
	printf("exon list was created.\n");
#endif


/*
	free(string);
	free(str1);
	free(str2);
	free(str3);
	free(str4);
	free(str5);
*/	
	return 0;
}

/************** add_seqG module for extracting genomic sequence including introns *********

add_seq and add_seqG are essentially similar in that they extracts DNA sequence 
corresponding to a given CDS or RNA gene. add_seqG has been used in the original SISEQ
package as add_seq for cdsnuc and extrna commands, but it has now been replaced by
a new add_seq module and renamed add_seqG, i.e., add_seq for genomic sequence.
The new add_seq extracts only exon sequence with flanking genomic or cDNA sequences.
The add_seqG extracts exon plus intron sequences with flanking sequences. 
Normally, add_seq might be more useful for molecular biological analysis, but
if entire genomic sequence including the intron sequence is needed, add_seqG
is best suited for this purpose. Because of various exceptional cases of errors and 
strange formats of database, the two functions might give different results, if
the flanking regions arround the initiation or termination codon is extracted.
Choose a better one depending on the results.
Currently, add_seq is used in cdsnuc and extran. add_seqG is used in cdsnucG and extranG.

*****************************************************************************************/

static int hl3=MINMEMSIZE; /* header length */

/*****************************************************************************************************************/
int add_seqG(Seqfile *seq_in,Seqfile **seq_nuc,char *start_loc,long startpos,char *end_loc,long endpos)
/*****************************************************************************************************************/
{
	int i,k;
	long j,j0;
	int exons;
	Boolean circular_bak=FALSE; /* TRUE if circular. */
	static char *str1;
	static char *str2;
	static char *str3;
	char *p;
	static char ACstr[MINMEMSIZE];
	char comment[MAXLINE];
	long atg;
	long stop;
	int atg_dir=1;
	int stop_dir=1;
	int a_dir=1;
	int z_dir=1;
	long a;
	long z;
	long seqlen;
	Seqfile *seqtmp,*seqtmp2;
	Exon_record **exon;
	Boolean comp_join=FALSE; /* flag for GB complementary join */
	Boolean error_flag=FALSE;

	seqlen=seq_in->seqlen;
	circular_bak=circular;
	circular=seq_in->circular;


	if(str1==NULL || str2==NULL || str3==NULL){
		if((str1=(char*)calloc(hl3+2,sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory.\n");
			return 1;
		}
		if((str2=(char*)calloc(hl3+2,sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory.\n");
			return 1;
		}
		if((str3=(char*)calloc(hl3+2,sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory.\n");
			return 1;
		}
	}


	for(k=0;k<num_seq;k++){
		error_flag=FALSE;
		atg=0L;
		stop=0L;
		comp_join=FALSE;

		if(strcmp(seq_nuc[k]->ID,seq_in->ID)) continue;

		if(k==0 || (k>0 && hl3<(int)strlen(seq_nuc[k]->header))){
			hl3=strlen(seq_nuc[k]->header);
			if((str1=(char*)calloc(hl3+2,sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory.\n");
				return 1;
			}
			if((str2=(char*)calloc(hl3+2,sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory.\n");
				return 1;
			}
			if((str3=(char*)calloc(hl3+2,sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory.\n");
				return 1;
			}
		}
	
                strcpy(str1,seq_nuc[k]->header);
                strcpy(str2,str_shift(str1,' ')); /* str1 is gene name */
                strcpy(str3,str_shift(str2,' ')); /* str2 is exon number string */
                if(!num_check(str2)){
			 sscanf(str2,"%d",&exons); /* convert to int */
		}
		else continue;
		
		if((exon=(Exon_record**)calloc(exons+1,sizeof(Exon_record)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return 1;
		}
		for(i=0;i<exons;i++){
			if((exon[i]=(Exon_record*)calloc(1,sizeof(Exon_record)))==NULL){
				fprintf(stderr,"Out of memory\n");
				return 1;
			}
		}
		if((p=strstr(str3,"AC="))!=NULL){
			subst(str3,';',' ');
			sscanf(p+3,"%s",ACstr);
		}
		strcpy(comment,"                                                ");
		if(exon_list(seq_nuc[k]->header,exon,comment)){
			fprintf(stderr,"\nError in making exon list. Skipping %s %s.\n",seq_nuc[k]->ID,str1);
			continue;
		}
/* for debug 
		printf("Exon list has been created for %s.\n",str1);
*/
	/* extracting sequence */
		if((seq_nuc[k]->seq=(char*)calloc(MINMEMSIZE,sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return 1;
		}

		if(strcmp(exon[0]->AC,"") && strcmp(getAC(exon[0]->AC),ACstr)){
			fprintf(stderr,"\nSequence import from another file: %s. Not currently supported. " \
				,exon[0]->AC);
			fprintf(stderr,"Skipping %s %s.\n",seq_nuc[k]->ID,str1);
			continue;
		}
		if(strcmp(exon[exons-1]->AC,"") && strcmp(getAC(exon[exons-1]->AC),ACstr)){
			fprintf(stderr,"\nSequence import from another file: %s. Not currently supported. " \
				,exon[exons-1]->AC);
			fprintf(stderr,"Skipping %s %s.\n",seq_nuc[k]->ID,str1);
			continue;
		}
		if(exon[0]->nc){
			atg=exon[0]->end;
			atg_dir=-1;
		}
		else {
			atg=exon[0]->begin;
			atg_dir=1;
		}
		if(exon[exons-1]->nc){
			stop=exon[exons-1]->begin;
			stop_dir=-1;
		}
		else{
			stop=exon[exons-1]->end;
			stop_dir=1;
		}

		if(strcmp(start_loc,"S")==0) {
			a=atg+atg_dir*startpos;
			a_dir=atg_dir;
		}
		else if(strcmp(start_loc,"E")==0) {
			a=stop+stop_dir*startpos;
			a_dir=stop_dir;
		}
		else{
			fprintf(stderr,"\nStart location is not correctly specified. Use S or E.\n");
			return 1;
		}
			
		if(strcmp(end_loc,"S")==0) {
			z=atg+atg_dir*endpos;
			z_dir=atg_dir;
		}
		else if(strcmp(end_loc,"E")==0) {
			z=stop+stop_dir*endpos;
			z_dir=stop_dir;
		}
		else{
			fprintf(stderr,"\nEnd location is not correctly specified. Use S or E.\n");
			return 1;
		}
	
		if(circular){	
			if(a>seqlen) a-=seqlen;
			if(z>seqlen) z-=seqlen;
			if(a<=0) a+=seqlen;
			if(z<=0) z+=seqlen;
		}

	/* Exceptional case for trans-splicing. */
		if(TransSplicing(exon,exons) && !strcmp(start_loc,"S") && !strcmp(end_loc,"E")){
			fprintf(stderr,"\nThis might be a trans-spliced gene: %s %s.\n",seq_nuc[k]->ID,str1);
			fprintf(stderr,"Catenated exons will be returned as a single exon sequence.\n");
			if(startpos<0){
				if(exon[0]->nc) exon[0]->end-=startpos;
				else exon[0]->begin+=startpos;
			}
			if(endpos>0){
				if(exon[exons-1]->nc) exon[exons-1]->begin-=endpos;
				else exon[exons-1]->end+=endpos;
			}
			if((seqtmp=join_exons2(seq_in,exon,exons,ACstr))==NULL){
				fprintf(stderr,"Error in joining exons for %s %s.\n",seq_nuc[k]->ID,str1);
				strcpy(seq_nuc[k]->seq,"*");
				seq_nuc[k]->seqlen=strlen(seq_nuc[k]->seq);
				free_exon(exon,exons);
				continue;
			}
			else{
				j0=strlen(seqtmp->seq);
				if(endpos<0){
					seqtmp->seq[j0+endpos]='\0';
					seqtmp->seqlen=strlen(seqtmp->seq);
				}
				if(startpos>0){
					for(j=0;j<j0-startpos;j++){
						seqtmp->seq[j]=seqtmp->seq[j+startpos];
					}
					seqtmp->seq[j0-startpos]='\0';
					seqtmp->seqlen=strlen(seqtmp->seq);
				}
			}
		}

	/* circular genome */
		else if(a_dir*(z-a)<0){
			if(circular){
				printf("\nExtraction of %s from a circular genome: %s.\n",str1,seq_nuc[k]->ID);
			}
			else printf("\nEnd of linear genome: %s for %s.\n",seq_nuc[k]->ID,str1);
			if(a_dir>0){
				if((seqtmp=extract_seq3(seq_in,a,seqlen))==NULL){
					fprintf(stderr,"\nUnable to extract nuc seq for %s.\n",str1);
					strcpy(seq_nuc[k]->seq,"*");
					seq_nuc[k]->seqlen=strlen(seq_nuc[k]->seq);
					free_exon(exon,exons);
					continue;
				}
				if((seqtmp2=extract_seq3(seq_in,1,z))==NULL){
					fprintf(stderr,"\nUnable to extract nuc seq for %s.\n",str1);
					strcpy(seq_nuc[k]->seq,"*");
					seq_nuc[k]->seqlen=strlen(seq_nuc[k]->seq);
					free_exon(exon,exons);
					continue;
				}
				j=strlen(seqtmp->seq)+strlen(seqtmp2->seq)+MINMEMSIZE;
				if((seqtmp->seq=(char*)realloc(seqtmp->seq,j*sizeof(char)))==NULL){
					fprintf(stderr,"Out of memory\n");
					return 1;
				}
				strcat(seqtmp->seq,seqtmp2->seq);
				seqtmp->seqlen=strlen(seqtmp->seq);
				free(seqtmp2);
				
			}
			else{
				if((seqtmp=extract_seq3(seq_in,z,seqlen))==NULL){
					fprintf(stderr,"\nUnable to extract nuc seq for %s.\n",str1);
					strcpy(seq_nuc[k]->seq,"*");
					seq_nuc[k]->seqlen=strlen(seq_nuc[k]->seq);
					free_exon(exon,exons);
					continue;
				}
				if((seqtmp2=extract_seq3(seq_in,1,a))==NULL){
					fprintf(stderr,"\nUnable to extract nuc seq for %s.\n",str1);
					strcpy(seq_nuc[k]->seq,"*");
					seq_nuc[k]->seqlen=strlen(seq_nuc[k]->seq);
					free_exon(exon,exons);
					continue;
				}
				j=strlen(seqtmp->seq)+strlen(seqtmp2->seq)+MINMEMSIZE;
				if((seqtmp->seq=(char*)realloc(seqtmp->seq,j*sizeof(char)))==NULL){
					fprintf(stderr,"Out of memory\n");
					return 1;
				}
				strcat(seqtmp->seq,seqtmp2->seq);
				seqtmp->seqlen=strlen(seqtmp->seq);
				free_seq(seqtmp2);
				
				if((comp_seq4(seqtmp))!=0){
					fprintf(stderr,"\nUnable to convert to comp seq for %s.\n",str1);
					free_seq(seqtmp);
					return 1;
				}
			}

		}

	/* default*/
		else {	
			if(a_dir>0){
				if((seqtmp=extract_seq3(seq_in,a,z))==NULL){
					fprintf(stderr,"\nUnable to extract nuc seq for %s.\n",str1);
					strcpy(seq_nuc[k]->seq,"*");
					seq_nuc[k]->seqlen=strlen(seq_nuc[k]->seq);
					free_exon(exon,exons);
					continue;
				}
			}
			else{
				if((seqtmp=extract_seq3(seq_in,z,a))==NULL){
					fprintf(stderr,"\nUnable to extract nuc seq for %s.\n",str1);
					strcpy(seq_nuc[k]->seq,"*");
					seq_nuc[k]->seqlen=strlen(seq_nuc[k]->seq);
					free_exon(exon,exons);
					continue;
				}
				if((comp_seq4(seqtmp))!=0){
					fprintf(stderr,"\nUnable to convert to comp seq for %s.\n",str1);
					free_seq(seqtmp);
					return 1;
				}
			}

		}
				
		if((seq_nuc[k]->seq=(char*)realloc(seq_nuc[k]->seq,(strlen(seqtmp->seq)+MINMEMSIZE)*sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return 1;
		}			
		strcpy(seq_nuc[k]->seq,seqtmp->seq);
		seq_nuc[k]->seqlen=strlen(seq_nuc[k]->seq);
		free_seq(seqtmp);
		free_exon(exon,exons);

		strcpy(str1,"");
		strcpy(str2,"");
		strcpy(str3,"");

		if(k%10==0) {
/*			printf(".");*/
			putc(8,stdout);
			putc('.',stdout);
			fflush(stdout);
		}
	}
	circular=circular_bak;	
	return 0;
}

/*****************************************************************************/


/*****************end of file cdncfn.c**************************/

