/***********************************************************************
Extracts nucleotide sequence for CDS (cdsnuc)
main file : cdsnuc.c

 	Extract nucleic acid sequences for CDS from GenBank or 
	EMBL sequence file.

  Usage: cdsnuc infile outfile start end <cdsname>
  start and end should be S xx  or E xx.
  S xx means xx nucleotides from the CDS start position.
  E xx means xx nucleotides from the CDS end position.
  cdsname (optional) specifies a single CDS to be extracted.
  
  Default outfile name is "infile"+".out"
  
  Source files are:	cdsnuc.c
 			cdncfn.c
  			gtsqfn.c
  			utility.c
  			excdfn.c
  			Dmc.h
  			getseq2.h
  	
 	Naoki Sato   Jan 5, 1998.
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.
	For Mac version, define the compiler as well.

	Modified June 7, 1998.
	Added the single CDS mode.
	Bug fix. Added integrated mode. Nov. 23, 1998.
	Adapted to reading a multi GB file. Jan. 09. 1999.
	Adapted to reading an EMBL file. March 13, 1999.
	Use of extract_cds2. March 16, 1999.
	Modified interactive input. March 19, 1999.
	Added counter to show progress of processing. April 29, 1999.
	Bug fix. May 17, 1999.
	Consecutive processing. Use of new extract_cds2. May 24, 1999. 
	Dual mode. Use of new add_seq and old add_seqG. May 30, 1999.
	Added sequence import facility. June 9, 1999.
	Verbous output suppressed. Aug. 25, 1999.
	siseq_exit0 or 1 changed to exit0 or 1. Nov. 13, 1999.
	Bug fix. May 4, 2000.
	Capability of extracting exons separately with xxxxx interleaved. May 17, 2006.
	
		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"


#ifndef INTEGRATED
#ifdef MAC
extern int ccommand(char ***);
#endif
#endif

/* Prototype declaration */
static MutList *readMutList(FILE *fin2,int *num);
static void print_usage(void);
static void help_input(void);

#ifdef INTEGRATED
int cdsnuc(int c, char (*p)[S_2]);
#endif
void help_cdsnuc(void);

/* external functions */
#include "libSeq.h"

/* Global variables */

extern int *idxmax;
extern dbidx **ix;
extern Boolean SEQ_IMPORT;

extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];
extern Boolean ADDSEQG;
extern Boolean GI_out;
extern char univ_code[];

extern FILE *fileptr;	/* defined in excdfn.c */

#ifdef JAPANESE
#ifdef UNIX
#include "EUC/cdsnucE.h"
#else
#ifdef MAC
#include "Mac/SJIS/cdsnucS.h"
#else
#include "Dos/SJIS/cdsnucS.h"
#endif
#endif
#endif

/***************************************************************/
static MutList *readMutList(FILE *fin2,int *num)
/***************************************************************/
{
	MutList *mList;
	char str1[MAXLINE+1];
	long pos;
	char Norg,Nmut;
	int count=0;
	int ins=0;
	char temp[100]="";

	if((mList=(MutList*)calloc(*num,sizeof(MutList)))==NULL){
		fprintf(stderr,"Out of memory in readMutList.\n");
/*#include	"exit1.h"*/
		return NULL;
	}

	while(fgets(str1,MAXLINE,fin2)!=NULL){
		sscanf(str1,"%s\t%c\t%c",temp,&Norg,&Nmut);
		if(strchr(temp,'.')==NULL){
			sscanf(temp,"%ld",&pos);
			ins=0;
		} else {
			sscanf(temp,"%ld.%d",&pos,&ins);
		}
		mList[count].pos = pos;
		mList[count].Npos = 0;
		mList[count].AApos = 0;
		mList[count].Norg = Norg;
		mList[count].Nmut = Nmut;
		mList[count].AA_org = '-';
		mList[count].AA_mut = '-';
		strcpy(mList[count].geneID,"none");
		

printf("%d: pos=%ld, ins=%d, Norg=%c, Nmut=%c\n",count,pos,ins,Norg,Nmut);

		count += 1;
		if(count >= *num){
			*num += 100;
			if((mList=(MutList*)realloc(mList,(*num)*sizeof(MutList)))==NULL){
				fprintf(stderr,"Out of memory in readMutList.\n");
/*#include	"exit1.h"*/
				return NULL;
			}
		}
	}
	*num = count;

	return mList;	
}
/***************************************************************/

/***************************************************************/
static void print_usage(void)
/***************************************************************/
{
	help_cdsnuc();
	return;
}

/***************************************************************/

/***************************************************************/
static void help_input(void)
/***************************************************************/
{
#ifdef JAPANESE
	me1();
#else
	printf("Type desired lengths of the 5'UTR and 3'UTR\n");
	printf("start and end should be S xx  or E xx.\n");
	printf("S xx means xx nucleotides from the CDS start position.\n");
	printf("E xx means xx nucleotides from the CDS end position.\n");
	printf("Use a space between the four values.\n");
#endif
	return;
}
/***************************************************************/

/***************************************************************/
void help_cdsnuc(void)
/***************************************************************/
{
#ifdef JAPANESE
	me2();
#else
	printf("*********************** CDSNUC HELP ***************************\n");
	printf("Usage: cdsnuc infile outfile start end <cds_name>.\n");
	printf("       start and end should be S xx  or E xx.\n");
	printf("S xx means xx nucleotides from the CDS start position.\n");
	printf("E xx means xx nucleotides from the CDS end position.\n");
	printf("<cds_name> is the name of CDS to be extracted (optional).\n");
	printf("If ADDSEQG is set true, introns are included .\n");
	printf("Partial truncated sequences are also shown if ADDSEQG is true.\n\n");
	printf("Usage: cdsnuc infile outfile splitN <cds_name>.\n");
	printf("       exons are printed individually with N consecutive x's interleaved.\n\n");
	printf("Usage: cdsnuc infile outfile mutlist=xxxxx.\n");
	printf("       mutated positions are printed.\n");
	printf("***************************************************************\n");
#endif
	return;
}
/***************************************************************/

/**************************************************************
****************** Main program *******************************/
#ifdef INTEGRATED
int cdsnuc(int cc, char (*p)[S_2])

#else
#ifdef MAC
main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif

#endif
/***************************************************************/

{
	FILE *fin;
	FILE *fin2;	/* mutlist */
	char fname1[100]="infile";
	char fname2[100]="outfile";
	int c=0;
	long counter=0L;
	char cdsname[AC_LENGTH]="";
	char *pp;
	Boolean single_mode=FALSE;
	char str[MAXLINE]="";
#ifndef INTEGRATED
	char *strenv;
	char *envstr;
#endif
	char start_loc[5];
	char end_loc[5];

#include        "filenam.h"     /* definition of char *str1 and char *str2 */

	char gname[80]="";
	long startpos;
	long endpos;
	Boolean need_input=FALSE;
	Boolean exit_flag=FALSE;
	Boolean split_mode=FALSE;
	int split_number;
	Boolean mutlist_mode=FALSE;
	char mutlist_file[100]="";
	MutList *mlist;
	int mutlist_num=100;

	long N;
	long i;

	/* sequence variables */
	Seqfile *seq_in;
	Seqfile **seq_nuc;
	long fileloc=0L;

	int indexmax=0;	/* idxmax is a pointer to indexmax, which is initialized here. */
	
#include	"parsarg.h"
#ifndef INTEGRATED
	if((strenv=getenv("ADDSEQG"))!=NULL){
		if(!strcmp(strenv,"TRUE") || !strcmp(strenv,"true") || !strcmp(strenv,"1")) ADDSEQG=TRUE;
	}
	else ADDSEQG=FALSE;
	if((strenv=getenv("SEQ_IMPORT"))!=NULL){
		if(!strcmp(strenv,"FALSE") || !strcmp(strenv,"false") || !strcmp(strenv,"0")) SEQ_IMPORT=FALSE;
	}
	else SEQ_IMPORT=TRUE;
#endif
	idxmax=&indexmax;


/*********************** File names ******************/

/*
printf("argc=%d, cc=%d",argc,cc);
*/

	switch(cc){
		case 1:	
			strcpy(fname1,"infile");
			if(filename_input(fname1,str1)!=1){
#include        "exit1.h"
			}
				
			strcpy(fname2,fname1);
			strcat(fname2,".out");
			if(filename_input(fname2,str2)!=1){
#include        "exit1.h"
			}
			need_input=TRUE;
			break;

		case 4:
		case 5:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			if(!strncmp(p[3],"split",5) && !ADDSEQG){
				split_mode=TRUE;
				if(p[3]+5=='\0') split_number=5;
				else split_number=atoi(p[3]+5);
				if(split_number==0) split_number=5;
			} else if(!strncmp(p[3],"mutlist=",8) && !ADDSEQG){
				mutlist_mode=TRUE;
				strcpy(mutlist_file,p[3]+8);
				if(!strcmp(mutlist_file,"")){
#include        "exit1.h"
				}
			} else {
				print_usage();
#include        "exit1.h"
			}
			if(cc==5){
				strcpy(cdsname,p[4]);
				single_mode=TRUE;
			}
			break;

		case 7:
		case 8:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			strcpy(start_loc,p[3]);
			startpos=parse_position(p[4]);
			strcpy(end_loc,p[5]);
			endpos=parse_position(p[6]);
			if(check_se(start_loc,end_loc) ) {
				print_usage();
#include        "exit1.h"
			}
			if(cc==8){
				strcpy(cdsname,p[7]);
				single_mode=TRUE;
			}
			break;

		default:
			print_usage();
#include        "exit1.h"
	}

/**************************** need input *************************************************/

		if(need_input){
			while(1){
				help_input();
				scanf("%s %ld %s %ld",start_loc,&startpos,end_loc,&endpos);
				if(check_se(start_loc,end_loc)) continue;
				break;
			}
			printf("What is the name of CDS? Type <exit> to exit this session.\n");
			printf("Use a command <genlist> to show all CDS names beforehand.\n");
			printf("If you input - all -, all CDS will be extracted.\n\n");
			scanf("%s",str);
			if(!strcmp(str,"exit")){
#include	"exit0.h"
			}
			else if((strcmp(str,"all")!=0)){
				single_mode=TRUE;
				if(strlen(str)>AC_LENGTH) {
					pp=extract_string(str,1,AC_LENGTH);
					strcpy(cdsname, pp);
					free(pp);
				}
				else strcpy(cdsname,str);
			}
			need_input=FALSE;
		}

/******************************************************************************************/

/*********************** Main part ****************************/

		c=check_filetype(fname1);
		if(c!=GENBANK_SEQ && c!=EMBL_SEQ){
			fprintf(stderr,"This is not a GenBank or EMBL file.\n");
			exit(1);
		}
		printf("Reading sequence...\n");

#ifdef MAC
#ifdef CODEWARRIOR
		SIOUXSetTitle("\pcdsnuc");
#endif
#endif

#ifndef INTEGRATED
		if((envstr=getenv("GI_out"))!=NULL){
			if(*envstr =='1' || envstr[0]=='t' || envstr[0]=='T'){
				GI_out = 1;
				printf("GI_out is enabled.\n");
			} else {
				GI_out = 0;
			}
		}
#else
		if(GI_out == 1) printf("GI_out has been enabled.\n");
#endif


		if((seq_nuc=(Seqfile **)calloc(1,sizeof(Seqfile)))==NULL){
			fprintf(stderr,"Out of memory\n");
			exit(1);
		}
		if((seq_in=(Seqfile *)calloc(1,sizeof(Seqfile)))==NULL){
			fprintf(stderr,"Out of memory\n");
#include	"exit1.h"
		}
		if((seq_in->ID=(char *)calloc(MINMEMSIZE,sizeof(char)))==NULL){ 
			fprintf(stderr,"Out of memory\n");
#include	"exit1.h"
		}
		if((seq_in->header=(char *)calloc(2,sizeof(char)))==NULL){ 
			fprintf(stderr,"Out of memory\n");
#include	"exit1.h"
		}

		if((fin=fopen(fname1,"r"))==NULL){
			fprintf(stderr,"Cannot open file: %s\n\n",fname1);
#include	"exit1.h"
		}
		if(mutlist_mode){
			if((fin2=fopen(mutlist_file,"r"))==NULL){
				fprintf(stderr,"Cannot open file: %s\n\n",mutlist_file);
#include	"exit1.h"
			}
			if((mlist=readMutList(fin2,&mutlist_num))==NULL){
				fprintf(stderr,"reading mutation list file\n");
#include	"exit1.h"
			}
		}
		if(ftouch(fname2)){
			fprintf(stderr,"Cannot open output file: %s\n\n",fname2);
#include	"exit1.h"
		}

		if((ix=(dbidx **)calloc(((*idxmax)+1),sizeof(ix)))==NULL){
			fprintf(stderr,"Out of memory\n");
#include	"exit1.h"
		}

		if(SEQ_IMPORT){
/**************** making index ********************/
			printf("Making index...\n");
			if(make_index(fin)){
				fprintf(stderr,"Error in making index for file: %s.\n",fname1);
#include	"exit1.h"
			}
			if(fseek(fin,fileloc,SEEK_SET)){
				fprintf(stderr,"File seek error.\n");
#include	"exit1.h"
			}
/*************END OF MAKING INDEX******************/
		}


/**************** main loop ***********************/
		
		while(!feof(fin)){
			if((seq_nuc=extract_cds2(fin,seq_in,&fileloc,c,unknown))==NULL){
				if(feof(fin)) break;
				fprintf(stderr,"Error in extracting CDS\n");
				break;
			}
			if(seq_in->seq==NULL || strlen(seq_in->seq)==0) {
				if(feof(fin)) break;
				else {
/*					printf(".");
*/
					putc(8,stdout);
					putc('/',stdout);
					fflush(stdout);
					continue;
				}
			}
#ifdef VERBOUS 
			printf("\nHeaders extracted from database: %s.\n",seq_in->ID);
#else
/*			printf("h");*/
			putc(8,stdout);
			putc('h',stdout);
			fflush(stdout);
#endif
			check_data_type(seq_in);
			if(clear_gaps(seq_in)){
				fprintf(stderr,"Error in clearing gaps in file: %s\n",seq_in->ID);
				continue;
			}
/************ adding sequences one by one. ******************/
		exit_flag=FALSE;
		N=num_seq;
		num_seq=1;
		fileptr=fin;
		for(i=0;i<N;i++){
			if(seq_nuc[i]->header!=NULL) sscanf(seq_nuc[i]->header,"%s",gname);
			else sprintf(gname,"seq.No.=%ld.",i);
#ifdef VERBOUS
			printf("Processing sequence %s %s.\n",seq_in->ID,gname);
#else
/*		printf(".");*/
			putc(8,stdout);
			putc('.',stdout);
			fflush(stdout);
#endif

			if(single_mode && strstr(seq_nuc[i]->header,cdsname)==NULL){
				free_seq(seq_nuc[i]);
				continue;
			}

			if(ADDSEQG){
				if(add_seqG(seq_in,&seq_nuc[i],start_loc,startpos,end_loc,endpos)){
					fprintf(stderr,"Error in extracting sequence.\n");
					exit_flag=TRUE;
					break;
				}
			}
			else {
				if(split_mode){
					if(add_seq_split(seq_in,&seq_nuc[i],split_number)){
						fprintf(stderr,"Error in extracting sequence.\n");
						exit_flag=TRUE;
						break;
					}
				} else if(mutlist_mode){
					if(add_seq_mutlist(fname2,seq_in,&seq_nuc[i],mlist,mutlist_num,univ_code)){
						fprintf(stderr,"Error in extracting sequence.\n");
						exit_flag=TRUE;
						break;
					}
				} else { 
					if(add_seq(seq_in,&seq_nuc[i],start_loc,startpos,end_loc,endpos)){
						fprintf(stderr,"Error in extracting sequence.\n");
						exit_flag=TRUE;
						break;
					}
				}
			}

			if(counter%20==0){
/*				printf(".");*/
				putc(8,stdout);
				putc('-',stdout);
				fflush(stdout);
			}
			else if(counter%10==0){
/*				printf(".");*/
				putc(8,stdout);
				putc('|',stdout);
				fflush(stdout);
			}
			if(mutlist_mode){
				continue;
			} else if(write_fasta(fname2,seq_nuc[i],INCLUDE_HEADER,'_')){
				fprintf(stderr,"Unable to write file.\n");
				continue;
			}
			free_seq(seq_nuc[i]);

		}
		num_seq=N;
/***********************************************************/
/*  Obsolete
			printf("\nSequences extracted.\n");

			if(counter%10==0){
				printf(".");
				fflush(stdout);
			}

			if(single_mode){	 single mode 
				printf("Extracting a single sequence: %s...\n",cdsname);
        	                if((seq_out=retrve_cds(seq_nuc,cdsname))==NULL){
        	                        fprintf(stderr,"No CDS: %s in database: %s.\n",cdsname,seq_in->ID);
					free_seq(seq_in);
					continue;
        	                }
        	                if(write_fasta(fname2,seq_out,INCLUDE_HEADER,'_')){
        	                        fprintf(stderr,"Unable to write file.\n");
        	                        continue;
                	        }
			}
	                else{ 	 multi mode 
				printf("Extracting all sequences in database: %s.\n",seq_in->ID);
				if(Mwrite(fname2,seq_nuc)){
					fprintf(stderr,"Unable to write file.\n");
					exit(1);
				}
			}
 end obsolete. */
			strcpy(seq_in->ID,"");
			strcpy(seq_in->header,"");
			if(exit_flag) break;
		}

		fclose(fin);


	printf("\nTerminated.\n");

#include	"exit0.h"

}

/************************************************************
		End of file cdsnuc.c
************************************************************/

