/***********************************************************************
Change name of clustal sequence (chname) 
main file : chname.c

 	Changes name of sequences in a FASTA or CLUSTAL sequence file.

  Usage: chname infile outfile 
  
  Default outfile name is "infile"+".out"
  
  Source files are:	chname.c
  			gtsqfn.c
  			utility.c
  			Dmc.h
  			getseq2.h
  			cmptbl.h
  	
 	Naoki Sato   Oct. 16, 1999.  
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.
	For Mac version, define the compiler as well.
	
	siseq_exit0 or 1 changed to exit0 or 1. Nov. 13, 1999.


		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"

#ifndef INTEGRATED
#ifdef MAC
extern int ccommand(char ***);
#endif
#endif

/* Prototype declaration */
static void print_usage(void);
#ifdef INTEGRATED
int chname(int c, char (*p)[S_2]);
#endif
void help_chname(void);

#include "libSeq.h"

/* Global variables */
extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];

/***************************************************************/
static void print_usage(void)
/***************************************************************/
{
	help_chname();
	return;
}
/***************************************************************/

/***************************************************************/
void help_chname(void)
/***************************************************************/
{
#ifdef JAPANESE
#ifdef UNIX
#include "EUC/chnameE.h"
#else
#ifdef MAC
#include "Mac/SJIS/chnameS.h"
#else
#include "Dos/SJIS/chnameS.h"
#endif
#endif
#else
        printf("****************** CHNAME HELP ****************************\n");
        printf("Usage: chname infile outfile [old_name new_name]\n");
        printf("       This program changes sequence name (ID) of FASTA and\n");
	printf("       CLUSTAL files. If old_name and new_name are omitted, \n");
	printf("       it opens a dialog for changing name of sequences.\n");
	printf("       Use - for an unspecified old_name.\n");
        printf("***********************************************************\n");
#endif
	return;
}
/***************************************************************/


/***************************************************************
****************** Main program *******************************/
#ifdef INTEGRATED
int chname(int cc, char (*p)[S_2])

#else
#ifdef MAC
main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif

#endif
/***************************************************************/

{
	char fname1[100]="infile";
	char fname2[100]="outfile";
	int c=0;
	register long k;
	char str[100];
	char tmpstr[100];
	char *str1="Input file name";
	char *str2="Output file name";
	char old_name[100];
	char new_name[100];
	Boolean interactive_mode=FALSE;
	long num1;
	enum data_type dtype1=unknown;
	Seqfile **seq_in;

#include	"parsarg.h"
	
/*********************** File names ******************/

	switch(cc){
		case 1:	
			strcpy(fname1,"infile");
			if(filename_input(fname1,str1)!=1){
#include	"exit1.h"
			}
			strcpy(fname2,fname1);
			strcat(fname2,".out");
			if(filename_input(fname2,str2)!=1){
#include	"exit1.h"
			}
			interactive_mode=TRUE;
			break;
		
		case 3:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			interactive_mode=TRUE;
			break;
			
		case 5:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			strcpy(old_name,p[3]);
			strcpy(new_name,p[4]);
			break;
			
		default:
			print_usage();
#include	"exit1.h"
	}
		
/***********************Main part****************************/

	printf("Reading sequence...\n");

#ifdef MAC
#ifdef CODEWARRIOR
	SIOUXSetTitle("\pchname");
#endif
#endif
	c=check_filetype(fname1);
	if(c!=CLUSTAL_SEQ && c!=FASTA_SEQ) {
		fprintf(stderr,"This is not a FASTA or CLUSTAL file.\n");
#include 	"exit1.h"
	}

	if(c==CLUSTAL_SEQ){	
		if((seq_in=read_clustal(fname1,dtype1))==NULL){
			fprintf(stderr,"Error in reading file: %s\n",fname1);
#include 	"exit1.h"
		}
	} else {
		if((seq_in=Mread_fasta(fname1,dtype1))==NULL){
			fprintf(stderr,"Error in reading file: %s\n",fname1);
#include 	"exit1.h"
		}
	}

/************* interactive mode loop ***************/

	if(interactive_mode){
		while(TRUE){
			printf("\nThis is a list of sequences in the file: %s.\n\n",fname1);
			for(k=0;k<num_seq;k++){
				strcpy(tmpstr,seq_in[k]->ID);
				strcat(tmpstr,space_line(AC_LENGTH-strlen(seq_in[k]->ID),' '));
				printf("%5ld %s\t",k+1,tmpstr);
				if(k%3==2) printf("\n");
			}
			while(TRUE){
				printf("\nType number of sequence to change name or 0 to exit: ");
				scanf("%s",str);
				if(!num_check(str)){
					sscanf(str,"%ld",&num1);
					if(num1<=num_seq) break;
				}
				printf("\nType correct number. Try again.");
			}
			if(num1==0L) break;
			num1--;
	
			while(TRUE){
				printf("\nNew name: ");
				scanf("%s",str);
				if(strlen(str)<AC_LENGTH) break;
			}
/* No need. Memory for 100 chars is allocated to ID in read_clustal. 
			if(strlen(seq_in[num1]->ID)<strlen(str)){
				if((seq_in[num1]->ID=(char*)realloc(seq_in[num1]->ID,(strlen(str)+2)*sizeof(char)))==NULL){
					fprintf(stderr,"Out of memory.\n");
					exit(1);
				}
			}
*/
			strcpy(seq_in[num1]->ID,str);
		}
/************ end of interactive loop ******************/

	} else {

/************ direct replacement *******************/
		num1=-1;
		for(k=0;k<num_seq;k++){
			if((strcmp(old_name,"-")==0 && k==0) || strcmp(old_name,seq_in[k]->ID)==0){ 
				num1=k;
				break;
			} 
		}
		if(num1>=0){
			strcpy(str,new_name);
			if(strlen(seq_in[num1]->ID)<strlen(str)){
				if((seq_in[num1]->ID=(char*)realloc(seq_in[num1]->ID,(strlen(str)+2)*sizeof(char)))==NULL){
					fprintf(stderr,"Out of memory.\n");
					exit(1);
				}
			}
			strcpy(seq_in[num1]->ID,str);
		}
	}


/********* write sequence **********************/

	if(c==CLUSTAL_SEQ){
		if(write_clustal(fname2,seq_in)){
			fprintf(stderr,"Error in writing sequence.\n");
#include 	"exit1.h"
		}
	} else {
		if(num_seq==1){
			if(write_fasta(fname2,seq_in[0],1,' ')){
				fprintf(stderr,"Error in writing sequence.\n");
#include 	"exit1.h"
			}
		} else {
			if(Mwrite(fname2,seq_in)){
				fprintf(stderr,"Error in writing sequence.\n");
#include 	"exit1.h"
			}

		}
	}
		
	for(k=0;k<num_seq;k++){
		free_seq(seq_in[k]);
	}
	
	printf("\nTerminated.\n");

#include 	"exit0.h"

}

/************************************************************
		End of file chname.c
************************************************************/


