/***********************************************************************
Extract CDS (extcds)
File name: excdfn.c

Functions: 	
		extract_cds2
		getGeneName
		estimateGeneName
		isGeneName
		Twrite
		Mfwrite
		Mwrite
		Mfwite_Dinfo
		Mwrite_Dinfo
		Mfwite_info
		Mwrite_info
		Mfwrite_infoct_cds2
		Mfwrite_info3
		Mwrite_info2
		Mwrite_info3
		write_codon_summary
		retrve_cds
		ext_exons
		show_gene_names
		show_gene_table
		join_exons2
		exon_total
		make_index

 	Naoki Sato   Jan 8, 1998.
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.
	
	Modified June 4, 1998.
	Added retrve_cds
	Added support for type "circular". Nov. 18, 1998.
	Added function ext_exons. Bug fix. Nov. 20, 1998.
	Adapted to reading a multi GB file. Jan. 08, 1999.
	Adapted to reading an EMBL file. March 15, 1999.
	extract_cds and extract_header combined. March 16, 1999.
	Use of exon_list in ext_exons. March 18, 1999.
	Added show_gene_names. March 19, 1999.
	Changed show_gene_names to int, and added fout. April 25, 1999.
	Bug fix (use of erasequote in extract_cds2). April 29, 1999.
	Modified extract_cds2 to use note as alternative gene name. April 30, 1999.
	Further bug fix in extract_cds2. May 5, 1999.
	Usability of other sequence files. Phase 4. May 10, 1999.
	Enhancement of extract_cds2. May 24-5, 1999.
	Use of copy_seq_h in join_exons. May 27, 1999.	
	join_exons replaced by join_exons2. May 28, 1999.
	Added make_index. June 3, 1999.
	Verbous output suppressed. Aug. 25, 1999.
	Bug fix. May 4, 2000.
	Added Mfwrite_info and Mwrite_info. May 11, 2000.
	Bug fix in Mfwrite. July 7, 2000.
	Modified ext_exons. Oct. 25, 2000.
	show_gene_table. Oct. 25, 2000.
	Added Mfwrite_Dinfo and Mwrite_Dinfo. Nov. 29, 2000.
	Added Mwrite2 and Mfwrite2. Jan. 13, 2002.
	Bug fix in extcds2. Aug. 13, 2002.
	Added Mwrite_info2 and Mfwrite_info2. Jan. 8, 2003.
	Added Mwrite_info3 and Mfwrite_info3. Jan. 15, 2003.
	Added estimateGeneName and isGeneName. Feb. 8, 2003.
	Added Twrite. March 23, 2003.
	Enhanced reading the product line. Jan. 8, 2007.
	Added exon_inf_mode. Jan. 17, 2007.
	Added Mwrite3 and Mfwrite3. Feb. 2, 2007.
	Adaptation to the new EMBL format. May 2, 2007.

		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"

/* Prototype declaration */

char *getGeneName(char *string);
char *estimateGeneName(char *string);
int isGeneName(char *string);
int make_index(FILE *fin);
long ExternalSeqlen(char *AC);
Seqfile **extract_cds2(FILE *fin,Seqfile *seq_in,long *loc,enum filetype c,enum data_type dtype1);
int Mfwrite3(FILE *fout,Seqfile **multi_seq,long start, long end);
int Mwrite3(char *filename, Seqfile **multi_seq,long start, long end);
int Mfwrite2(FILE *fout,Seqfile **multi_seq,long min, long max);
int Mwrite2(char *filename, Seqfile **multi_seq,long min, long max);
int Twrite(char *filename, Seqfile **multi_seq);
int Mfwrite(FILE *fout,Seqfile **multi_seq);
int Mwrite(char *filename, Seqfile **multi_seq);
int Mfwrite_Dinfo(FILE *fout,Seqfile **multi_seq,long window,long slide);
int Mwrite_Dinfo(char *filename, Seqfile **multi_seq,long window,long slide);
int Mfwrite_info(FILE *fout,Seqfile **multi_seq);
int Mwrite_info(char *filename, Seqfile **multi_seq);
int Mfwrite_info2(FILE *fout,Seqfile **multi_seq,char info_mode_opt);
int Mwrite_info2(char *filename, Seqfile **multi_seq,char info_mode_opt);
int Mfwrite_info3(FILE *fout,Seqfile **multi_seq,char *codon_table);
int Mwrite_info3(char *filename, Seqfile **multi_seq,char *codon_file);
int write_codon_summary(FILE *fout,char *codon_table,long *code_count_total,long aalen);
Seqfile *retrve_cds(Seqfile **seq_cds, char *cdsname);
Seqfile *ext_exons(Seqfile *seq_nuc);
int show_gene_names(FILE *fout,Seqfile **seq_nuc);
int show_gene_table(FILE *fout,Seqfile **seq_nuc,enum data_type dtype1);
Seqfile *join_exons2(Seqfile *seq_nuc,Exon_record **exon,int exons,char *ACstr);
long exon_total(Exon_record **exon,int exons,char *ACstr,long seqlen);
long exon_total2(Exon_record **exon,int exons,char *ACstr,long seqlen);

/* External functions */
#ifndef __EXCDFN
#define __EXCDFN
#endif
#include "libSeq.h"

/* Global variables */

extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];
extern char aaname[20][2][10];
extern Boolean circular;
extern Boolean to_stdout;
FILE *fileptr;
Boolean end_tag_found;
extern Boolean split_mode;
extern int split_number;
extern Boolean exon_inf_mode;
Boolean GI_out;

/* Definition of the code for the end of line (LF).*/

#define END_OF_LINE 10	/* CR will be ignored in the program. */
#define END_OF_LINE2 13		/* for DOS or Mac->UNIX converted files */

#define GB_tRNA "tRNA"
#define GB_rRNA "rRNA"
#define GB_snRNA "snRNA"
#define GB_scRNA "scRNA"
#define GB_miscRNA "misc_RNA"

#define GB_CDS "CDS"
#define GB_mRNA "mRNA"
#define GB_GENE "/gene="
#define GB_LOCUS_TAG "/locus_tag="
#define GB_LOCUS_TAG2 "locus_tag: "
#define GB_LOCUS_TAG3 "ORF_ID:"
#define GB_PRODUCT "/product="
#define GB_PSEUDO "/pseudo"
#define GB_TR "/translation="
#define GB_NOTE "/note="
#define GB_TRTBL "/transl_table="
#define GB_COMP "complement"
#define GB_JOIN "join"
#define GB_DB_XREF "/db_xref="
#define GB_GI_tag "GI:"
/*
#define GB_CJOIN "complement\(join"
*/

/**************************************************************/
char *getGeneName(char *string)
/* changes transfer RNA to trn.
   also recognizes rRNA  */
/**************************************************************/
{
        char str1[MINMEMSIZE]="transfer RNA";
        char *genestr;
        char gn[MINMEMSIZE];
        int i;

        genestr=gn;
		if(strstr(string,"ribosomal RNA")!=NULL){
			if(strstr(string,"16S") || strstr(string,"12S")) strcpy(genestr,"rrs");
			else if(strstr(string,"23S")) strcpy(genestr,"rrl");
			else if(strstr(string,"5S")) strcpy(genestr,"rrn5");
			else if(strstr(string,"5.8S")) strcpy(genestr,"rrn5.8");
			else strcpy(genestr,"rRNA");
			return genestr;
		}

        if(strlen(string)<13) return "";
        if(strstr(string,str1)!=NULL || strstr(string,"tRNA")){
        	strcpy(genestr,"trn");
			for(i=0;i<20;i++){
				if(strstr(string,aaname[i][0])!=NULL) {
					strcat(genestr,aaname[i][1]);
					break;
				}
			}
        	return genestr;
        } else return "";
		
}
/**************************************************************/

/**************************************************************/
char *estimateGeneName(char *string)
/* estimate and extract gene name from a string.*/
/**************************************************************/
{
	char *str1[]={"probable","hypothetical","putative","possible"};
	char *str1b[]={"Probable ","Hypothetical ","Putative ","Possible "};
	char *str2[]={" locus"," gene"};
	char *str2b[]={" homolog"," homologue"};
	char *str3[]={"match to ","identical to ","similar to ","Match to ","Identical to ","Similar to "};
	char *str4[]={"strain","plasmid","type"};
	char eststr[]="EST";
	char *orfstr[]={"ORF","orf","Orf","ORF No. ","orf No. ","Orf No. "};

	char *p,*q,*word1;
	char *genestr;
	static char gn[MINMEMSIZE];
	char word2[MINMEMSIZE]="";
	int i;

	if(string==NULL) return NULL;
        if(strlen(string)<3) return NULL;

	strcpy(gn,space_line(MINMEMSIZE-1,'\0'));
        genestr=&gn[0];
	for(i=0;i<6;i++){
		if((p=strstr(string,str3[i]))!=NULL){
			word1= p + strlen(str3[i]);
			sscanf(word1,"%s",word2);
			if(strstr(word2,eststr)!=NULL) return NULL;
			if(isGeneName(word2)){
				if(i>2) i-=3;
				sprintf(gn,"%s%s",str3[i],word2);
				return genestr;
			}
		}
	}
	for(i=0;i<3;i++){
		if((p=strstr(string,str4[i]))!=NULL) return NULL;
	}
	for(i=0;i<2;i++){
		if((p=strstr(string,str2[i]))!=NULL){
			if(p==string) continue;
			p--;
			while(p>=string){
				if(*p==' ') break;
				p--;
			}
			strcpy(word2,p+1);
			strcpy(gn,strtok(word2," "));
			if(isGeneName(genestr)) return genestr;
		}
	}
	for(i=0;i<2;i++){
		if((p=strstr(string,str2b[i]))!=NULL){
			if(p==string) continue;
			p--;
			while(p>=string){
				if(*p==' ') break;
				p--;
			}
			strcpy(word2,p+1);
			q=strtok(word2," ");
			if(q!=NULL) strcpy(gn,q);
			if(isGeneName(genestr)){
				strcat(gn,str2b[i]);
				return genestr;
			}
		}
	}
	for(i=0;i<4;i++){
		if((p=strstr(string,str1b[i]))!=NULL){
			strcpy(word2,p+strlen(str1b[i]));
			strcpy(gn,strtok(word2," "));
			if(isGeneName(genestr)){
				sprintf(word2,"%s; %s",genestr,str1[i]);
				strcpy(gn,word2);
				return genestr;
			}
		}
	}
	for(i=0;i<4;i++){
		if((p=strstr(string,str1[i]))!=NULL){
			strcpy(word2,p+strlen(str1[i]));
			if(strcmp(word2,"")){
				headchop(word2);
				strcpy(gn,strtok(word2," "));
				if(isGeneName(genestr)){
					sprintf(word2,"%s; %s",genestr,str1[i]);
					strcpy(gn,word2);
					return genestr;
				} else if(strstr(genestr,"orf")!=NULL || strstr(genestr,"ORF")!=NULL){
					sprintf(word2,"%s orf",str1[i]);
					strcpy(gn,word2);
					return genestr;
				}
			}
			if(p==string) continue;
			p--;
			while(p>=string){
				if(*p==' ') break;
				p--;
			}
			strcpy(word2,p+1);
			q=strtok(word2," ");
			if(q!=NULL){
				strcpy(gn,q);
				if(isGeneName(genestr)){
					strcat(gn," ");
					strcat(gn,str1[i]);
					return genestr;
				}
			} else {
				return NULL;
			}
		}
	}
	for(i=0;i<6;i++){
		if((p=strstr(string,orfstr[i]))!=NULL){
			p+=strlen(orfstr[i]);
			if((q=strtok(p," "))==NULL){
				strcpy(gn,"orf");
				return genestr;
			}
			sscanf(q,"%s",word2);
			subst(word2,';',' ');
			for(p=word2;*p!='\0';p++){
				if(!isalnum(*p)) return NULL;
			}
			sprintf(gn,"ORF%s",word2);
			return genestr;
		}
	}
	
	word1=strtok(string," ;");
	if(word1!=NULL){
		if(isGeneName(string)){
			strcpy(gn,string);
			return genestr;
		}
	}

	return NULL;
}
/**************************************************************/


/**************************************************************/
int isGeneName(char *string)
/* returns TRUE, if the string is judged a gene name.
The gene should be: aaa1111, aaa-111, aaa1111bb, aaa-111bb,
where a is any alphabet, 1 is any number. The length of 
the string is limited to 3-15. Hyphen and underscore is
skipped.*/
/**************************************************************/
{
	char *p;
	int count1=0;	/* first alphabets */
	Boolean flag3=FALSE;	/* third numbers */
	Boolean flag4=FALSE;	/* fourth alphabets */
	Boolean flagNG=FALSE;	/* no good */
	Boolean flagHY=FALSE;	/* hyphen etc. */

	if(strlen(string)>15 || strlen(string)<3) flagNG=TRUE;
	if(!flagNG){
		for(p=string;*p!='\0' && *p!=' ' && *p!=0x28;p++){
			if(!count1){
				if(!isalpha((int)*p)) break;
				else count1++;
			} else {
				if(*p=='-' || *p=='_' || *p=='.') {
					flagHY=TRUE;
					continue;
				}
				else if(*p=='/') {
					break;
				}
				if(!flag3){
					if(isalpha((int)*p)) {
						count1++;
						continue;
					}
					else {
						if(isdigit((int)*p)) flag3=TRUE;
						else {
							flagNG=TRUE;
							break;
						}
					}
				} else {
					if(!flag4){
						if(isdigit((int)*p)) continue;
						if(isalpha((int)*p)) flag4=TRUE;
						else {
							flagNG=TRUE;
							break;
						}
					} else {
						if(isalpha((int)*p)) continue;
						else {
							if(!isdigit((int)*p)){
								flagNG=TRUE;
							}
							break;
						}
					}
				}
			}
		}
	}
	if(count1>8 && !flagHY) flagNG=TRUE;
	else if(!flag3){
		if(count1<3) flagNG=TRUE;
		else{
			for(p=string;*p!='\0' && *p!=' ';p++){
				if(!islower((int)*p)) break;
			}
			if(*p=='\0' || *p==' ') {
				if(count1>4) flagNG=TRUE;
				else if(count1>5) flagNG=TRUE;
			}
		}
	}
	else if(flag3 && count1<2 && !flag4) flagNG=TRUE;
/*	printf("Test of gene name: %s : ",string);	*/
	if(!flagNG) {
/*		printf("yes.\n");	*/
		return TRUE;
	} else {
/*		printf("no.\n");	*/
		return FALSE;
	}
}
/**************************************************************/


int *idxmax;
dbidx **ix;
Boolean SEQ_IMPORT=TRUE;

/**************************************************************/
int make_index(FILE *fin)
/**************************************************************/
{

	static char str1[MAXLINE+1]="";
	static char str2[MAXLINE+1]="";
	static char ACstr[MAXLINE+1]="";
	long fploc=0L;

	while(fgets(str1,MAXLINE+1,fin)!=NULL){


		if(linetype(str1,GB_AC) || linetype(str1,EMBL_AC)){
			subst(str1,';',' ');
			sscanf(str1,"%s %s",str2,ACstr);
			strcpy(str2,"");
			fploc=ftell(fin);
			if((ix=(dbidx **)realloc(ix,((*idxmax)+1)*sizeof(ix)))==NULL){
				fprintf(stderr,"Out of memory\n");
				return 1;
			}
			if((ix[(*idxmax)]=(dbidx *)calloc(1,sizeof(dbidx)))==NULL){
				fprintf(stderr,"Out of memory\n");
				return 1;
			}
			strcpy(ix[(*idxmax)]->AC,getAC(ACstr));
			ix[(*idxmax)]->loc=fploc;
			++(*idxmax);

#ifdef VERBOUS
			printf("AC= %s, fploc= %ld.\n",ix[(*idxmax)-1]->AC,ix[(*idxmax)-1]->loc);
#endif
		}

	}
	return 0;
}
/**************************************************************/



/**************************************/
long ExternalSeqlen(char *AC)
/**************************************/
{
	int k;
	long fileloc=0L;
	
        for(k=0;k<(*idxmax);k++){
                if(!strcmp(ix[k]->AC,getAC(AC))){
                        fileloc=ix[k]->loc;
                        break;
                }
        }
        if(k>=(*idxmax)){
                fprintf(stderr,"Unable to find the sequence with accession: %s\n",AC);
                return 0L;
        }
        return read_dbseq(fileptr,&fileloc)->seqlen;
}

/***************************************/


static int tmplen=10;	/* length of tmpstr */
static int hl2=10;

/*********************************************************************************************************************/
Seqfile **extract_cds2(FILE *fin,Seqfile *seq_in,long *loc,enum filetype c,enum data_type dtype1)
/*
   if dtype1=protein, this function extracts CDS field,
   if dtype1=RNA, this function extracts RNA field,
   if dtype1=unknown, this function extracts only header.
   if dtype1=annotP or annotR, this function extracts only annotation.
   both GenBank and EMBL are accepted as filetype. 
   Header is: gene name, exon number, repeat of (exon begin exon end),
              accession number, product name. */
/*********************************************************************************************************************/
{
	long fileloc=*loc;
	Seqfile **multi_seq;
	static Seqfile *seqfile1;
	register long i,j;
	long offset=0L;
	long k=0;
	long memsize=MAXLINE;
#ifndef INTEGRATED
	char *str;
#endif
	static char str1[MAXLINE+1]="";
	static char str2[MAXLINE+1]="";
	static char str3[MAXLINE+1]="";
	static char str31[MAXLINE+1]="";
	static char standard[MINMEMSIZE]="";
	static char div[MINMEMSIZE]="";

	static char a1[]="                     ";
	static char a2[]="                     /";
	static char a3[]="FT                   ";
	static char a4[]="FT                   /";

	static char *tmpstr;
	static char IDstr[MINMEMSIZE]="";
	static char ACstr[MINMEMSIZE]="";
	static char OSstr[MINMEMSIZE]="";
	static char OS1[MINMEMSIZE]="";
	static char OS2[MINMEMSIZE]="";
	int b=0;	
	long seqlength=0;
	static char length_char[MINMEMSIZE]="";
	static char circular_char[MINMEMSIZE]="";
	static char bp[MINMEMSIZE]="";	
	static char genomic[MINMEMSIZE]="";	
	static char SV[MINMEMSIZE]="";	
	static char SVnum[MINMEMSIZE]="";	
	static char dtype_char[MINMEMSIZE]="";
	static char word1[MINMEMSIZE]="";
	static char *word2;		/* for buffer */
	static char *word102;	/* for mRNA. not used now. */
	static char *word202;	/* for RNA */
	static char *word302;	/* for CDS */
	static char word22[MINMEMSIZE]="";	/* temporary buffer for word2,102,202 */
	static char word3[MINMEMSIZE]="";	/* gene. Default name is orf,or RNA. */
	static char word4[MAXLINE]="";		/* product */
	static char word5[MINMEMSIZE]=""; /* note */
	static char word6[MINMEMSIZE]=""; /* temp for GI_tag */
	static char word_tmp[MAXLINE]="";
	static char locus_tag[MINMEMSIZE]="";	/* locus_tag for genome data */
	static char GI_tag[MINMEMSIZE]="";	/* GI_tag for genome data */
	
	char *pp;	/* temporary receiver for extract_string */
	
	Boolean gb_id_on=TRUE;
	Boolean gb_header_on=FALSE;
	Boolean gb_cds_on=FALSE;
	Boolean gb_pseudo=FALSE;
	Boolean gb_seq_on=FALSE;
	Boolean gb_seq_in_on=FALSE;
	Boolean write_seqfile=FALSE;
	Boolean seq_written=FALSE;
	Boolean circular_bak=FALSE;
	char gb_cjoin[]={'c','o','m','p','l','e','m','e','n','t',40,'j','o','i','n','\0'};
	Boolean geneField=FALSE;	/* /gene= field */
	Boolean annot_only=FALSE;

	long fpori;	/* for RNA */
	long fpori1=0L;

	char init_tag[MINMEMSIZE]=GB_INIT;
	char end_tag[MINMEMSIZE]=GB_SEQ_END;

	if(dtype1==annotP){
		dtype1=protein;
		annot_only=TRUE;
	} else if(dtype1==annotR){
		dtype1=RNA;
		annot_only=TRUE;
	}

#if 0
#ifndef INTEGRATED
	GI_out = 0;
	str=getenv("GI_out");
/*  printf("GI_out=%s\n",str);
*/
	if(*str =='1' || str[0]=='y' || str[0]=='Y'){
		GI_out = 1;
		printf("GI_out is set.\n");
	}
#else
	if(GI_out == 1) printf("GI_out has been enabled.\n");
#endif
#endif

	if(dtype1!=unknown && dtype1!=protein && dtype1!=RNA ){
		fprintf(stderr,"Incompatible setting of data type in extract_cds2.\n");
		return NULL;
	}
	if(dtype1==RNA) strcpy(word3,"RNA");
	else strcpy(word3,"orf");

	if(c==GENBANK_SEQ || c==GBPROT_SEQ) {
		strcpy(init_tag,GB_INIT);
		strcpy(end_tag,GB_SEQ_END);
	}
	else if(c==EMBL_SEQ) {
		strcpy(init_tag,EMBL_INIT);
		strcpy(end_tag,EMBL_SEQ_END);
	}
	else return NULL;
	
	if((multi_seq=(Seqfile**)calloc(1,sizeof(multi_seq)))==NULL){
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}	
	if(seqfile1==NULL){
		if((seqfile1=(Seqfile*)calloc(1,sizeof(Seqfile)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return NULL;
		}
	}
	circular_bak=circular;
	circular=FALSE;

/* Reading ID and description.*/

	if(seqfile1->ID==NULL){
		if((seqfile1->ID=(char *)calloc(MINMEMSIZE,sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return NULL;
		}
	}
	if(seqfile1->header==NULL){

#ifdef VERBOUS
	printf("hl2=%d",hl2);
#endif

		if((seqfile1->header=(char *)calloc(hl2+2,sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return NULL;
		}
	}
	if(seqfile1->seq==NULL){
		if((seqfile1->seq=(char *)calloc(2,sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return NULL;
		}
	}

#ifdef VERBOUS
	printf("tmplen=%d",tmplen);
#endif

	if(word302==NULL){
		if((word302=(char*)calloc(tmplen+1,sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return NULL;
		}
	}
	if(word202==NULL){
		if((word202=(char*)calloc(tmplen+1,sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return NULL;
		}
	}
	if(word102==NULL){
		if((word102=(char*)calloc(tmplen+1,sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return NULL;
		}
	}
	if(word2==NULL){
		if((word2=(char*)calloc(tmplen+1,sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return NULL;
		}
	}
	if(tmpstr==NULL){
		if((tmpstr=(char*)calloc(tmplen+1,sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return NULL;
		}
	}

	if(fseek(fin,fileloc,SEEK_SET)){
		fprintf(stderr,"File seek error.\n");
		return NULL;
	}

	while(fgets(str1,MAXLINE+1,fin)!=NULL){

		fpori=fpori1;	/* for RNA */
		fpori1=ftell(fin);

		if(write_seqfile==TRUE){
			seqfile1->seq[offset]=0;
			if(dtype1==RNA) seqfile1->seqlen=0L;
			else seqfile1->seqlen=offset;
			seqfile1->dtype=protein;
			strcpy(seqfile1->ID,IDstr);
			strcpy(seqfile1->AC,ACstr);
			strcpy(seqfile1->OS,OSstr);

			strcat(word4,"; ");
			strcat(word4,OSstr);
			if(!tailtype(OSstr,";")){	/* May 2, 2007 */
				strcat(word4,";");
			}

			if(strcmp(ACstr,"")){
				if(strlen(word4)+strlen(ACstr) < MAXLINE-20){
					strcat(word4,"; AC=");
					strcat(word4,ACstr);
					strcat(word4,"; ");
				}
			}

/**************************************/

		if(dtype1==RNA){
			if((!strcmp(word3,"") || !strcmp(word3,"RNA"))){
				strcpy(word3,getGeneName(word4));
				if(!strcmp(word3,"")) {
					if(strlen(word5)>0){
						subst(word5,' ','_');
						if(strlen(word5)>15){
							strncpy(word3,word5,15);
							if(strlen(word4)+strlen(word5) < MAXLINE-2) strcat(word4,word5);
							else {
								strncpy(word_tmp,word5,MAXLINE-strlen(word4)-2);
								strcat(word4,word_tmp);
								strcpy(word_tmp,"");
							}
						}
						else strcpy(word3,word5);
					}
					else strcpy(word3,"RNA");
				}
			}

			if(strcmp(locus_tag,"")){
				if(strstr(locus_tag,word3)!=NULL){
					strcpy(word3,locus_tag);
				} else if(strstr(word3,locus_tag)==NULL){
					tailchop(word3);
					strcat(locus_tag,"=");
					strcat(locus_tag,word3);
					strcpy(word3,locus_tag);
				}
			}
		}
		else{
			if(!geneField){
				strcpy(word_tmp,word4);
				str_shift(word_tmp,';');
/*
printf("%s : %s : ",word4,word5);
*/
				if(!strcmp(locus_tag,"")){
					if((pp=estimateGeneName(word_tmp))!=NULL){
						strcpy(word3,pp);
					} else {
						strcpy(word_tmp,word5);
						str_shift(word_tmp,';');
						if ((pp=estimateGeneName(word_tmp))!=NULL){
							strcpy(word3,pp);
						}
					}
					strcpy(word_tmp,"");
/*
printf("%s\n",word3);
*/
				}
			}
/*
			if(!strcmp(word3,"") || (!geneField && !strcmp(word3,"orf"))){
				if(strlen(word5)>0 ){
					subst(word5,' ','_');
					if(strlen(word5)>15){
						if(strlen(word4)+strlen(word5) < MAXLINE-2) strcat(word4,word5);
						else {
							strncpy(word3,word5,MAXLINE-strlen(word4)-2);
							strcat(word4,word3);
							strcpy(word3,"");
						}
						strncpy(word3,word5,15);
					}
					else strcpy(word3,word5);
				}
			}
*/

			if((pp=strchr(word3,','))!=NULL){
				*pp='\0';
				pp++;
				if(*pp==' ') pp++;
				strcpy(word_tmp,pp+1);
				strcat(word_tmp," ");
				strcat(word_tmp,word4);
				strcpy(word4,word_tmp);
				strcpy(word_tmp,"");
			}
			subst(word3,' ','_');
			
			if(strcmp(locus_tag,"")){
				if(strstr(locus_tag,word3)!=NULL){
					strcpy(word3,locus_tag);
				} else if(strstr(word3,locus_tag)==NULL){
					if(strcmp(word3,"orf")){
						if(GI_out && strcmp(GI_tag,"")){
							strcat(locus_tag,"_");
							strcat(locus_tag,GI_tag);
							strcat(locus_tag," ");
						} else {
							strcat(locus_tag,"=");
							strcat(locus_tag,word3);
						}
					}
					strcpy(word3,locus_tag);
				}
			}
		}

/********************************************/
		if(hl2 < (int)(strlen(GI_tag)+4+strlen(word3)+strlen(word2)+strlen(word4)+20)){
			hl2=strlen(GI_tag)+4+strlen(word3)+strlen(word2)+strlen(word4)+20;

#ifdef VERBOUS
			printf("hl2=%d",hl2);
#endif

			if((seqfile1->header=(char *)realloc(seqfile1->header,(hl2+2)*sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
		}

/********************************************/
			if(dtype1==RNA) sprintf(seqfile1->header,"%s %s %s", word3,word202,word4);
			else {
				sprintf(seqfile1->header,"%s %s %s", word3,word302,word4);
			}
			strcpy(word302,"");
			strcpy(word202,"");
			strcpy(word102,"");
			strcpy(word2,"");
			strcpy(locus_tag,"");
			strcpy(GI_tag,"");
/********************************************/

			write_seqfile=FALSE;
			seq_written=TRUE;

			if((multi_seq=(Seqfile**)realloc(multi_seq,(k+2)*sizeof(multi_seq)))==NULL){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			if((multi_seq[k]=(Seqfile*)calloc(1,sizeof(Seqfile)))==NULL){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			
			if((multi_seq[k]=copy_seq(seqfile1))==NULL){
				fprintf(stderr,"Unable to make multi sequence.\n");
				return NULL;
			}
			k++;
			if(k%10==0) {
				putc(8,stdout);
				if(k%20==0) putc('-',stdout);
				else putc('|',stdout);
				fflush(stdout);
			}

			strcpy(seqfile1->ID,"");
			strcpy(seqfile1->header,"");
			strcpy(seqfile1->seq,"");

/********************************/
		if(dtype1==RNA){
			if((seqfile1->seq=(char *)calloc(2,sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
		}
/*******************************/

			strcpy(str2,"");
			strcpy(word1,"");
			strcpy(word22,"");
			if(dtype1==RNA) strcpy(word3,"RNA");
			else strcpy(word3,"orf");
			strcpy(word4,"");
			strcpy(word5,"");
			strcpy(word_tmp,"");
			strcpy(word6,"");
			strcpy(locus_tag,"");
			strcpy(GI_tag,"");
			geneField=FALSE;
			gb_pseudo=FALSE;
		}
		
		if(gb_id_on==TRUE){
		
			if(linetype(str1,init_tag)){
				if(strstr(str1,"circular")!=NULL) circular=TRUE;
				if(pick_string(str1,strlen(init_tag)+1,strlen(str1))){
					fprintf(stderr,"Error in extracting string in extract_cds2.\n");
					exit(1);
				} 
				if(c==GENBANK_SEQ || c==GBPROT_SEQ){
					sscanf(str1,"%s %s %s %s",IDstr,length_char,bp,dtype_char);
					if(!strcmp(dtype_char,"circular")){
						sscanf(str1,"%s %s %s %s %s",IDstr,length_char,bp,circular_char,dtype_char);
					}
					if(!strcmp(bp,"aa")) strcpy(dtype_char,"PRT");
				}
				else if(c==EMBL_SEQ){
					subst(str1,';',' ');
					if(strstr(str1,"circular")!=NULL) {
						sscanf(str1,"%s %s %s %s %s %s %s",IDstr,standard,circular_char,dtype_char,div,length_char,bp);
					} else {
						sscanf(str1,"%s %s %s %s %s %s",IDstr,standard,dtype_char,div,length_char,bp);
					}
				/* May 2, 2007: new EMBL format */
					if(!strcmp(standard,"SV")){
						sscanf(str1,"%s %s %s %s %s %s %s %s %s.",IDstr,SV,SVnum,circular_char,dtype_char,standard,div,length_char,bp);
						if(!strcmp(dtype_char,"genomic") || !strcmp(dtype_char,"other") || !strcmp(dtype_char,"unassigned")){
							sscanf(str1,"%s %s %s %s %s %s %s %s %s %s.",IDstr,SV,SVnum,circular_char,genomic,dtype_char,standard,div,length_char,bp);
						}
						if(strstr(dtype_char,"DNA")) strcpy(dtype_char,"DNA");
						else if(strstr(dtype_char,"RNA")) strcpy(dtype_char,"RNA");
					}
				/* end */
				}
				if(strlen(IDstr)>15){
					fprintf(stderr,"\nFile format is strange. But we continue.\n");
					fprintf(stderr,"%s\n",str1);
					IDstr[15]='\0';
/*					pp=extract_string(IDstr,1,15);
					strcpy(IDstr,pp);
					free(pp);
*/
				}
				tail_string(dtype_char,3);
				if(!num_check(length_char)) sscanf(length_char,"%ld",&seqlength);
				if(c!=GBPROT_SEQ && strcmp(dtype_char,"DNA") && strcmp(dtype_char,"RNA")){
					fprintf(stderr,"\nFile format is strange. But we continue by assuming data type is DNA.\n");
					fprintf(stderr,"%s\n",str1);
					strcpy(dtype_char,"DNA");
				}
				gb_id_on=FALSE;
			}
			continue;
		}

		if(linetype(str1,GB_AC) || linetype(str1,EMBL_AC)){
			subst(str1,';',' ');
			sscanf(str1,"%s %s",str2,ACstr);
			strcpy(str2,"");
		}

		if(linetype(str1,GB_OS) || linetype(str1,EMBL_OS)){
			sscanf(str1,"%s %s %s",str2,OS1,OS2);
			subst(OS2,';',' ');
			tailchop(OS2);
			strcpy(OSstr,OS1);
			strcat(OSstr," ");
			strcat(OSstr,OS2);
		}

		if(c==GENBANK_SEQ || c==GBPROT_SEQ){
			if(linetype(str1,GB_FEATURES)) gb_header_on=TRUE;
			if(linetype(str1,GB_SEQ_INIT)){
				gb_header_on=FALSE;
				offset=0L;
				memsize=2L;
				if((seq_in->seq=(char *)calloc(memsize,sizeof(char)))==NULL){
					fprintf(stderr,"Out of memory\n");
					return NULL;
				}
				if(dtype1!=protein && seq_written) gb_seq_in_on=TRUE;
	                        continue;
	                }
		}
		else if(c==EMBL_SEQ){
			if(linetype(str1,EMBL_FHKey)) gb_header_on=TRUE;
			if(linetype(str1,EMBL_FH)) continue;
			if(linetype(str1,EMBL_SEQ_INIT)){
				gb_header_on=FALSE;
				offset=0L;
				memsize=2L;
				if((seq_in->seq=(char *)calloc(memsize,sizeof(char)))==NULL){
					fprintf(stderr,"Out of memory\n");
					return NULL;
				}
				if(dtype1!=protein && seq_written) gb_seq_in_on=TRUE;
				continue;
			}
		}


        /* for read through a multi GB file. Now obsolete. 
		This function reads a single database entry.*/

		if(linetype(str1,end_tag)){
			end_tag_found = TRUE;
			break;
		}

		if(gb_header_on==TRUE){
			if(gb_cds_on==FALSE){
				strcpy(word22,"");
				if(c==GENBANK_SEQ || c==GBPROT_SEQ) sscanf(str1,"%s %s",word1,word22);
				else if(c==EMBL_SEQ) {
					if(linetype(str1,EMBL_XX)) continue;
					sscanf(str1,"%s %s %s",str3,word1,word22);
					if(strcmp(str3,EMBL_FT)){
						fprintf(stderr,"\nNo FT tag in EMBL file:%s.\n",str3);
						return NULL;
					}
				}
				if(!strcmp(word22,"")) continue;
				if(linetype(word1,"/")) continue;
			
				if(strlen(str1)>9){
					strncpy(str31,str1+5,3);
					if(!strcmp(str31,"   ")) continue;
				}

				if((dtype1!=RNA && (!strcmp(word1,GB_CDS) || !strcmp(word1,GB_mRNA))) || \
				(dtype1==RNA && (!strcmp(word1,GB_tRNA) || !strcmp(word1,GB_rRNA) || \
				!strcmp(word1,GB_snRNA)))){

/********************************************/
	if((int)strlen(str1)>tmplen){
		tmplen=strlen(str1);
		if((tmpstr=(char*)realloc(tmpstr,(tmplen+1)*sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return NULL;
		}
		if((word302=(char*)realloc(word302,(tmplen+2)*sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return NULL;
		}
		if((word202=(char*)realloc(word202,(tmplen+2)*sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return NULL;
		}
		if((word102=(char*)realloc(word102,(tmplen+2)*sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return NULL;
		}
		if((word2=(char*)realloc(word2,(tmplen+2)*sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return NULL;
		}
	}
	strcpy(tmpstr,str1);
	tailchop(tmpstr);
/********************************************/

					if(strstr(str1,GB_JOIN)!=NULL){		/* contains join */
						if(!tailtype(str1,")")){
							while(fgets(str2,MAXLINE+1,fin)!=NULL){
									/* These are for security */
								if(!linetype(str2,a1) && !linetype(str2,a3)) break;
								if(linetype(str2,a2) || linetype(str2,a4)) break;

/**********************************************/
		if((int)(strlen(tmpstr)+strlen(str2))>tmplen){
			tmplen=strlen(tmpstr)+strlen(str2);
			if((tmpstr=(char*)realloc(tmpstr,(tmplen+2)*sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			if((word302=(char*)realloc(word302,(tmplen+2)*sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			if((word202=(char*)realloc(word202,(tmplen+2)*sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			if((word102=(char*)realloc(word102,(tmplen+2)*sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			if((word2=(char*)realloc(word2,(tmplen+2)*sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
		}
/*********************************************/

								if(c==GENBANK_SEQ || c==GBPROT_SEQ) sscanf(str2,"%s",word22);
								else if(c==EMBL_SEQ) {
									sscanf(str2,"%s %s",str3,word22);
									if(strcmp(str3,EMBL_FT)){
										fprintf(stderr,"\nNo FT tag in EMBL file:%s.\n",str3);
										return NULL;
									}
								}
								if(tailtype(word22,")")){
									subst(word22,')',' ');
									tailchop(word22);
									strcat(tmpstr," ");
									strcat(tmpstr,word22);
									break;
								}
								else{
									strcat(tmpstr," ");
									strcat(tmpstr,word22);
								}
							}
						} 
					}
	
					strcpy(word2,tmpstr+21);
					
					if(linetype(word2,GB_JOIN) || (linetype(word2,gb_cjoin))){
						b=count_char(word2,',')+1;
					}
					else b=1;
					pack_space(word2); /* for compatibility with older databases. */
					string_alnum(word2,"_:<>");
					strcpy(tmpstr,word2);
					sprintf(word2,"%d %s",b,tmpstr);
					tailchop(word2);

/********************************************************/
					/* Jan. 17, 2007 */
					if(annot_only || (!exon_inf_mode && dtype1==protein)) strcpy(word2,"");	
					if(dtype1==RNA) strcpy(word202,word2);
					else if(!strcmp(word1,GB_mRNA)) strcpy(word102,word2);
					else strcpy(word302,word2);

					if(dtype1!=RNA && !strcmp(word1,GB_mRNA)){
						strcpy(str2,"");
						strcpy(str3,"");
					}
					else gb_cds_on=TRUE;

/********************************************************/
					continue;
				}
			}
			else{
			
				if(gb_seq_on==FALSE){
					if(c==GENBANK_SEQ || c==GBPROT_SEQ){
						strcpy(str2,str1);
					}else if(c==EMBL_SEQ){
						if(linetype(str1,EMBL_XX)) continue;
						strcpy(str3,str1);
						strcpy(str2,str_shift(str3,' '));
						if(strcmp(str3,EMBL_FT)){
							fprintf(stderr,"\nNo FT tag in EMBL file.\n");
							return NULL;
						}
					}
					headchop(str2);
					tailchop(str2);

					if(linetype(str2,GB_GENE)){
						pp=str2+strlen(GB_GENE);
						strcpy(word3,pp);
						erasequote(word3);
						subst(word3,' ','_');
						geneField=TRUE;
						continue;
					}
					else if(linetype(str2,GB_PRODUCT)){
						pp=str2+strlen(GB_PRODUCT);

/* Jan. 8, 2007 */
						if(!tailtype(str1,"\"")){
							strcpy(word4,pp);
							erasequote(word4);
							while(fgets(str2,MAXLINE+1,fin)!=NULL){
								if(c==EMBL_SEQ) {
									sscanf(str2,"%s",str3);
									if(strcmp(str3,EMBL_FT)){
										fprintf(stderr,"\nNo FT tag in EMBL file:%s.\n",str3);
										return NULL;
									}
								}
								if((int)(strlen(word4)+strlen(str2))-19>MAXLINE){
									pp = str2 + MAXLINE +19 - strlen(word4);
									*pp = '\"';
									*(pp+1) = '\0';
								}
								if(tailtype(str2,"\"")){
									subst(str2,'\"',' ');
									tailchop(str2);
									strcat(word4," ");
									strcat(word4,str2+21);
									break;
								}
								else{
									strcat(word4," ");
									strcat(word4,str2+21);
								}
							}
						} else {
							strcpy(word4,pp);
							erasequote(word4);
						}

/* end Jan. 8, 2007 */

						if(dtype1==RNA && c==GENBANK_SEQ){
							gb_cds_on=FALSE;
							write_seqfile=TRUE;
						}
						if(gb_pseudo){
							gb_seq_on=TRUE;
							gb_pseudo=FALSE;
						}
						continue;
					}
					else if(linetype(str2,GB_NOTE)){
						pp=str2+strlen(GB_NOTE);
						strcpy(word5,pp);
						erasequote(word5);
						if(linetype(word5,"Citation") || linetype(word5,"unnamed")){
							strcpy(word5,"");
						} else if(linetype(word5,GB_LOCUS_TAG2)){
							strcpy(locus_tag,word5 + strlen(GB_LOCUS_TAG2));
							strcpy(word5,"");
						} else if(linetype(word5,GB_LOCUS_TAG3)){
							remove_character(word5,';');
							strcpy(locus_tag,word5 + strlen(GB_LOCUS_TAG3));
							strcpy(word5,"");
						} else if(linetype(word5,"ORF ")){
							word5[3]='_';
							remove_character(word5,';');
							strcpy(locus_tag,word5);
							strcpy(word5,"");
						} else if(linetype(word5,"ORF")){
							remove_character(word5,';');
							strcpy(locus_tag,word5);
							strcpy(word5,"");
						}
						if(gb_pseudo){
							gb_seq_on=TRUE;
							gb_pseudo=FALSE;
						}
						continue;
					}

/* Feb. 13, 2015 */
					else if(linetype(str2,GB_TRTBL)){
						if(gb_pseudo){
							gb_seq_on=TRUE;
							gb_pseudo=FALSE;
						}
					}
/* end Feb. 13, 2015 */
					else if(linetype(str2,GB_LOCUS_TAG)){
						pp=str2+strlen(GB_LOCUS_TAG);
						strcpy(locus_tag,pp);
						erasequote(locus_tag);
						continue;
					}
					/* reading GI number for protein */
					else if(linetype(str2,GB_DB_XREF)){
						pp=str2+strlen(GB_DB_XREF);
						strcpy(word6,pp);
						erasequote(word6);
						if(linetype(word6,GB_GI_tag)){
							strcpy(GI_tag,word6 + strlen(GB_GI_tag));
							strcpy(word6,"");
						}
						continue;
					}
					else if(linetype(str2,GB_PSEUDO)){
/*						gb_seq_on=TRUE;
*/
						offset=0;
						memsize=2;
						if((seqfile1->seq=(char *)calloc(memsize,sizeof(char)))==NULL){		
							fprintf(stderr,"Out of memory\n");
							return NULL;
						}
						strcpy(seqfile1->seq,"*");
						if(geneField){
							strcat(word3,"_pseudo");
							gb_seq_on=TRUE;
						} else {
							strcpy(word3,"pseudo");
							gb_pseudo=TRUE;
						}

					}

/*********************************************************************/

				else if(dtype1==RNA){
					if(!linetype(str1,"                  ") && !linetype(str1,"FT                ") && \
					!linetype(str2,"/")){ 
						if(fseek(fin, fpori,SEEK_SET)){
							fprintf(stderr,"Error in accessing file.\n");
							return NULL;
						}
						gb_cds_on=FALSE;
						write_seqfile=TRUE;
						continue; 
					}                                              
				}

				else {
					if(linetype(str2,GB_TR)){
						gb_seq_on=TRUE;
						offset=0;
						memsize=2;
						if((seqfile1->seq=(char *)calloc(memsize,sizeof(char)))==NULL){		
							fprintf(stderr,"Out of memory\n");
							return NULL;
						}
					}
				}

/********************************************************************/
				}

				if(gb_seq_on==TRUE){
					if(c==GENBANK_SEQ || c==GBPROT_SEQ) sscanf(str1,"%s",str2);
					else if(c==EMBL_SEQ){
						if(linetype(str1,EMBL_XX)) continue;
						sscanf(str1,"%s %s",str3,str2);
						if(strcmp(str3,EMBL_FT)){
							fprintf(stderr,"\nNo FT tag in EMBL file.\n");
							return NULL;
						}
					}
					if(linetype(str2,GB_TR)){
						if(pick_string(str2,strlen(GB_TR)+2,strlen(str2))){
							fprintf(stderr,"Error in extracting string in extract_cds2_2.\n");
							exit(1);
						}
					}
					if(str2[strlen(str2)-1]==34){	/* double quote */
						erasequote(str2);
						gb_seq_on=FALSE;
						gb_cds_on=FALSE;
						write_seqfile=TRUE;
					}
					
					if(dtype1==unknown) continue;
					else{	/* extract CDS */

						memsize=memsize+(10+strlen(str2))*(sizeof(char));
						if((seqfile1->seq=(char *)realloc(seqfile1->seq,memsize))==NULL){
							fprintf(stderr,"Out of memory\n");
							return NULL;
						}
						j=0;
						for(i=0;i<(long)strlen(str2);i++){
							if(islower((int)str2[i])) str2[i]=toupper(str2[i]);
							if(isupper((int)str2[i])){
								seqfile1->seq[offset+j]=str2[i];
								j++;
							}
						}
						offset=offset+j;
						continue;			
					}	
				}	
			}	
		}

		if(gb_seq_in_on){
			memsize+=(2+strlen(str1))*(sizeof(char));
			if((seq_in->seq=(char *)realloc(seq_in->seq,memsize))==NULL){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			j=0;
			for(i=0;i<(long)strlen(str1);i++){
				if(islower((int)str1[i])) str1[i]=toupper(str1[i]);
				if(isupper((int)str1[i])){
					seq_in->seq[offset+j]=str1[i];
					j++;
				}
			}
			offset=offset+j;
			continue;			
		}
		
	
	}
	if(feof(fin)) {
		return NULL;
	}
	if(gb_seq_in_on){
		seq_in->seq[offset]='\0';
		seq_in->seqlen=offset;
		if(seqlength!=offset){
			fprintf(stderr,"Sequence length is %ld in the header of file,\n",seqlength);
			fprintf(stderr,"but %ld bases were read.\n",offset);
		}
		if(!strcmp(dtype_char,"DNA")) seq_in->dtype=DNA;
		else if (!strcmp(dtype_char,"RNA")) seq_in->dtype=RNA;
		else if (!strcmp(dtype_char,"PRT")) seq_in->dtype=protein;
		else seq_in->dtype=unknown;
		seq_in->circular=circular;
	}
	strcpy(seq_in->ID,IDstr);
	strcpy(seq_in->AC,ACstr);
	strcpy(seq_in->OS,OSstr);

	if(seq_in->seqlen==0) seq_in->seqlen=seqlength;	/* for dtype1=unknown */
	else seq_in->seqlen=strlen(seq_in->seq);
	num_seq=k;
	circular=circular_bak;

	fileloc=ftell(fin);
	*loc=fileloc;

	return multi_seq;
}


/*********************************************************************/
int Mfwrite3(FILE *fout,Seqfile **multi_seq,long start,long end)
/* This subroutine write from the (start)th to the (end)th sequences.*/
/*********************************************************************/
{
	int i;

	if(end > num_seq) end = num_seq;
	for(i=start;i<end;i++){
		fwrite_fasta(fout,multi_seq[i],1,'_');	
	}
	
	return 0;
}

/********************************************************************/
int Mfwrite2(FILE *fout,Seqfile **multi_seq,long min,long max)
/* This subroutine write sequences with length limited by min and max.
If max=0, min is activated. If min=0, max is activated.
If both are zero, just sequences longer than 0 length are printed.*/
/* 159.9: negative values for min and max will result in 
mask mode. Consecutive Ns and Qs are converted to NxnnnxN and
QxnnnxQ, where nnn is the number of Ns and Qs.*/
/********************************************************************/
{
	int i;
	Boolean min_mode=FALSE;
	Boolean max_mode=FALSE;
	Boolean mask_mode=FALSE;

	if(min < 0 && max < 0) mask_mode=TRUE;
	else if(min!=0L && max==0L) min_mode=TRUE;
	else if(min==0L && max!=0L) max_mode=TRUE;

	for(i=0;i<num_seq;i++){
		if(multi_seq[i]->seqlen==0 || !strcmp(multi_seq[i]->seq,"")){
			continue;
		}
		else if(min_mode && multi_seq[i]->seqlen<min) continue;
		else if(max_mode && multi_seq[i]->seqlen>max) continue;
		else {
			if(mask_mode){
				mask_sequence(multi_seq[i]->seq,(int)(0-min),(int)(0-max));
				multi_seq[i]->seqlen = strlen(multi_seq[i]->seq);
			}
			fwrite_fasta(fout,multi_seq[i],1,'_');
		}
	}
	
	return 0;
}

/**************************************************************/
int Mfwrite(FILE *fout,Seqfile **multi_seq)
/* This subroutine does not write empty sequence. */
/**************************************************************/
{
	return Mfwrite2(fout,multi_seq,0L,0L);
}

/***********************************************************************/
int Mfwrite_Dinfo(FILE *fout,Seqfile **multi_seq,long window,long slide)
/***********************************************************************/
{
	int i;
	
	for(i=0;i<num_seq;i++){
		if(fwrite_Dinfo(fout,multi_seq[i],window,slide)){
			fprintf(stderr,"Error in Mfwrite_Dinfo. Sequence No. %d.\n",i);
			return TRUE;
		}
	}
	
	return FALSE;
}


/**************************************************************************/
int Mwrite_Dinfo(char *filename, Seqfile **multi_seq,long window,long slide)
/**************************************************************************/
{
	FILE *fout;

#ifdef MAC
#ifdef CODEWARRIOR
	_fcreator=CREATOR_NAME;
	_ftype=FILETYPE_NAME;
#endif
#endif
	if(to_stdout) fout=stdout;
	else{
		if((fout=fopen(filename,"a+"))==NULL){
			fprintf(stderr,"Cannot open file: %s\n\n",filename);
			return TRUE;
		}
	}
	
	if(Mfwrite_Dinfo(fout,multi_seq,window,slide)){
		fprintf(stderr,"Unable to write output file.\n");
		if(!to_stdout) fclose(fout);
		return TRUE;
	}
	if(!to_stdout) fclose(fout);
	return FALSE;
}

/**************************************************************/
int Mfwrite_info(FILE *fout,Seqfile **multi_seq)
/**************************************************************/
{
	int i;
	
	for(i=0;i<num_seq;i++){
		fwrite_info(fout,multi_seq[i],1,'_');	
	}
	return 0;
}


/**************************************************************/
int Mwrite_info(char *filename, Seqfile **multi_seq)
/**************************************************************/
{
	FILE *fout;

#ifdef MAC
#ifdef CODEWARRIOR
	_fcreator=CREATOR_NAME;
	_ftype=FILETYPE_NAME;
#endif
#endif
	if(to_stdout) fout=stdout;
	else{
		if((fout=fopen(filename,"a+"))==NULL){
			fprintf(stderr,"Cannot open file: %s\n\n",filename);
			return 1;
		}
	}
	
	if(Mfwrite_info(fout,multi_seq)){
		fprintf(stderr,"Unable to write output file.\n");
		if(!to_stdout) fclose(fout);
		return 1;
	}
	if(!to_stdout) fclose(fout);
	return 0;
}


/*******************************************************************/
int Mfwrite_info2(FILE *fout,Seqfile **multi_seq,char info_mode_opt)
/*******************************************************************/
{
	long i;
	
	for(i=0L;i<num_seq;i++){
		fwrite_info2(fout,multi_seq[i],info_mode_opt);	
	}
	return FALSE;
}


/************************************************************************/
int Mwrite_info2(char *filename, Seqfile **multi_seq, char info_mode_opt)
/************************************************************************/
{
	FILE *fout;

#ifdef MAC
#ifdef CODEWARRIOR
	_fcreator=CREATOR_NAME;
	_ftype=FILETYPE_NAME;
#endif
#endif
	if(to_stdout) fout=stdout;
	else{
		if((fout=fopen(filename,"a+"))==NULL){
			fprintf(stderr,"Cannot open file: %s\n\n",filename);
			return TRUE;
		}
	}

	if(info_mode_opt == '#'){
		fprintf(fout,"%ld\n",num_seq);
		return FALSE;
	}
	
	if(Mfwrite_info2(fout,multi_seq,info_mode_opt)){
		fprintf(stderr,"Unable to write output file.\n");
		if(!to_stdout) fclose(fout);
		return TRUE;
	}
	if(!to_stdout) fclose(fout);
	return FALSE;
}


/****************************************************************************/
int write_codon_summary(FILE *fout,char *codon_table,long *code_count_total,long aalen)
/****************************************************************************/
{
	int j;
	double *usage;
	long total=0L;
	char string[5]="   ";
	unsigned rc;

	if((usage=(double*)calloc(65,sizeof(double)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return TRUE;
	}

	for(rc=0;rc<23;rc++){
		total=0L;
		for(j=0;j<64;j++){
			if(residue_code_name[rc]==codon_table[j]){
				total += code_count_total[j];
			}
		}
		for(j=0;j<64;j++){
			if(residue_code_name[rc]==codon_table[j]){
				if(total==0L) usage[j]=0.0;
				else usage[j]=100*(double)code_count_total[j]/(double)total;
			}
		}
	}
		
		

	fprintf(fout,"*****************************************\n");	
        fprintf(fout,"Total codon usage\n  %ld codons in %ld sequences.\n\n",aalen,num_seq);
        for(j=0;j<64;j++){
                if(get_codon(j,string)){
                        fprintf(stderr,"Error in getting codon in write_codon_summary.\n");
                        return TRUE;
                }
                fprintf(fout,"%c  %s = %6ld, %6.2f %%.\n",\
                codon_table[j],string,code_count_total[j],100*(double)code_count_total[j]/(double)aalen);
		if(j % 4 == 3 && j < 63) fprintf(fout,"\n");
	}
	fprintf(fout,"*****************************************\n");	
        fprintf(fout,"\n");
        fflush(fout);

	fprintf(fout,"*****************************************\n");	
        fprintf(fout,"Total codon usage sorted by amino acids\n  %ld codons in %ld sequences.\n\n",aalen,num_seq);
 	for(rc=0;rc<23;rc++){
		for(j=0;j<64;j++){
			if(residue_code_name[rc]==codon_table[j]){
				if(get_codon(j,string)){
					fprintf(stderr,"Error in getting codon in write_codon_summary.\n");
					return TRUE;
				}
                		fprintf(fout,"%c  %s = %6ld, %6.2f %%, %6.2f %%.\n",\
                		codon_table[j],string,code_count_total[j],\
				100*(double)code_count_total[j]/(double)aalen,\
				usage[j]);
			}
        	}
        	if(rc!=18 && rc!=21 && rc<22) fprintf(fout,"\n");
	}
	fprintf(fout,"*****************************************\n");	
        fflush(fout);

	return FALSE;
}



/*******************************************************************/
int Mfwrite_info3(FILE *fout,Seqfile **multi_seq,char *codon_table)
/*******************************************************************/
{
	int j;
	long i;
	long *code_count;
	long *code_count_total;
	double *usage;
	long aalen=0L;

	if((code_count=(long*)calloc(65,sizeof(long)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return TRUE;
	}
	if((code_count_total=(long*)calloc(65,sizeof(long)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return TRUE;
	}
	if((usage=(double*)calloc(65,sizeof(double)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return TRUE;
	}

	for(i=0L;i<num_seq;i++){
		if(fwrite_info3(fout,multi_seq[i],codon_table,code_count)) {
			fprintf(stderr,"Error in getting code_count in Mfwrite_info3.\n");
			return TRUE;
		}
		aalen += multi_seq[i]->seqlen / 3L;

		for(j=0;j<64;j++){
			code_count_total[j] += code_count[j];
		}
		initialize_comp2(code_count,65);
	}
	
	if(write_codon_summary(fout,codon_table,code_count_total,aalen)) {
		fprintf(stderr,"Trouble in writing codon summary in Mfwrite_info3.\n");
		return TRUE;
	}
	
	return FALSE;
}


/************************************************************************/
int Mwrite_info3(char *filename, Seqfile **multi_seq, char *codon_file)
/************************************************************************/
{
	FILE *fout;
	char codon_table[100];

#ifdef MAC
#ifdef CODEWARRIOR
	_fcreator=CREATOR_NAME;
	_ftype=FILETYPE_NAME;
#endif
#endif
	if(codon_file==NULL || !strcmp(codon_file,"")) {
		fprintf(stderr,"No valid codon_file.\n");
		return 1;
	}

	strcpy(codon_table,CodonTable(codon_file));
	if(to_stdout) fout=stdout;
	else{
		if((fout=fopen(filename,"a+"))==NULL){
			fprintf(stderr,"Cannot open file: %s\n\n",filename);
			return 1;
		}
	}
	
	if(Mfwrite_info3(fout,multi_seq,codon_table)){
		fprintf(stderr,"Unable to write output file.\n");
		if(!to_stdout) fclose(fout);
		return 1;
	}
	if(!to_stdout) fclose(fout);
	return 0;
}



/**************************************************************/
int Mwrite3(char *filename, Seqfile **multi_seq, long start, long end)
/**************************************************************/
{
	FILE *fout;

#ifdef MAC
#ifdef CODEWARRIOR
	_fcreator=CREATOR_NAME;
	_ftype=FILETYPE_NAME;
#endif
#endif
	if((fout=fopen(filename,"a+"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n\n",filename);
		return 1;
	}
	
	if(Mfwrite3(fout,multi_seq,start,end)){
		fprintf(stderr,"Unable to write output file.\n");
		fclose(fout);
		return 1;
	}
	fclose(fout);
	return 0;
}


/**************************************************************/
int Mwrite2(char *filename, Seqfile **multi_seq, long min, long max)
/* 159.9: negative values for min and max will result in 
mask mode. Consecutive Ns and Qs are converted to NxnnnxN and
QxnnnxQ, where nnn is the number of Ns and Qs.*/
/**************************************************************/
{
	FILE *fout;

#ifdef MAC
#ifdef CODEWARRIOR
	_fcreator=CREATOR_NAME;
	_ftype=FILETYPE_NAME;
#endif
#endif
	if(to_stdout) fout=stdout;
	else{
		if((fout=fopen(filename,"a+"))==NULL){
			fprintf(stderr,"Cannot open file: %s\n\n",filename);
			return 1;
		}
	}
	
	if(Mfwrite2(fout,multi_seq,min,max)){
		fprintf(stderr,"Unable to write output file.\n");
		if(!to_stdout) fclose(fout);
		return 1;
	}
	if(!to_stdout) fclose(fout);
	return 0;
}



/**************************************************************/
int Mwrite(char *filename, Seqfile **multi_seq)
/**************************************************************/
{
	return Mwrite2(filename,multi_seq,0L,0L);
}


/**************************************************************/
int Twrite(char *filename, Seqfile **multi_seq)
/**************************************************************/
{
	FILE *fout;
	long i;
	char str1[MAXLINE];
	char *p;

#ifdef MAC
#ifdef CODEWARRIOR
	_fcreator=CREATOR_NAME;
	_ftype=FILETYPE_NAME;
#endif
#endif
	if((fout=fopen(filename,"a+"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n\n",filename);
		return TRUE;
	}

	for(i=0;i<num_seq;i++){
		sscanf(multi_seq[i]->header,"%s",str1);
		strcat(multi_seq[i]->ID,"_");
		strcat(multi_seq[i]->ID,str1);
		p=multi_seq[i]->header + strlen(str1) + 1;
		fprintf(fout,"%s\t%ld\t%s\n",multi_seq[i]->ID,multi_seq[i]->seqlen,p);
	}

	fclose(fout);
	return FALSE;
}


/**********************************************************/
Seqfile *retrve_cds(Seqfile **seq_cds, char *cdsname)
/**********************************************************/
{
        Seqfile *seqfile1;
        int i;
        char str1[MAXLINE]="";
	char cdsname1[MINMEMSIZE];

	if((seqfile1=(Seqfile*)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return NULL;
	}
        for(i=0;i<num_seq;i++){
                strcpy(str1,seq_cds[i]->header);
                sscanf(str1,"%s",cdsname1);
                if(!strcmp(cdsname1,cdsname)){
                        if((seqfile1=copy_seq(seq_cds[i]))==NULL){
                                fprintf(stderr,"Error in retrieving CDS\n");
                                exit(1);
                        }
                        break;
                }
        }
	if(i==num_seq) return NULL;
        return seqfile1;
}

/***********************************/
Seqfile *ext_exons(Seqfile *seq_nuc)
/**********************************/
{
	Seqfile *seq_exons;
	char string[2*MAXLINE];
	char string2[2*MAXLINE];
	char ACstr[MINMEMSIZE]="";
	char *p;
	Exon_record **exon;
	int exons=1;
	int i;
	char str1[2*MAXLINE];
	char comment[2*MAXLINE];

	strcpy(string,seq_nuc->header);
	sscanf(string,"%s %d",str1,&exons);
	strcpy(string2,string);
	subst(string2,';',' ');
	if((p=strstr(string2,"AC="))!=NULL){
		sscanf(p+3,"%s",ACstr);
	}
	else {
		fprintf(stderr,"No ACC available. Defaults to local file.\n");
	}
	
	if((exon=(Exon_record**)calloc(exons+1,sizeof(Exon_record)))==NULL){
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}
	for(i=0;i<exons;i++){
		if((exon[i]=(Exon_record*)calloc(1,sizeof(Exon_record)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return NULL;
		}
	}
	strcpy(comment,"                                           ");
	if(exon_list(seq_nuc->header,exon,comment)){
		fprintf(stderr,"\nError in extracting exons.\n");
		return NULL;
	}

	seq_exons=join_exons2(seq_nuc,exon,exons,ACstr);

	return seq_exons;
}

/*******************************************************************************/
Seqfile *join_exons2(Seqfile *seq_nuc,Exon_record **exon,int exons,char *ACstr)
/*******************************************************************************/
{
	int i=0;
	int k=0L;
	Boolean circular_bak=FALSE;
	long seqlen,seqlen2;
	Seqfile *seq_exons;
	Seqfile *seq_dbseq;
	long seqlenEX;
	char *p1,*p2;
	long fileloc;
	int counter;

	circular_bak=circular;
	circular=seq_nuc->circular;
	seqlen=seq_nuc->seqlen;

        if((seq_exons=copy_seq_h(seq_nuc))==NULL){
                fprintf(stderr,"Error in copying sequence in join_exons2.\n");
		exit(1);
        }

	if(SEQ_IMPORT) seqlenEX=exon_total2(exon,exons,ACstr,seqlen);
	else {
		seqlenEX=exon_total(exon,exons,ACstr,seqlen);
		if(seqlen+1000 < seqlenEX){
			fprintf(stderr,"Total length of exons exceeds the sequence length.\n");
			return seq_exons;
		}
	} 
	if((seq_exons->seq=(char*)realloc(seq_exons->seq,(seqlenEX+100)*sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		exit(1);
	}
	p1=seq_exons->seq;

/* Beginning of the main loop */
	for(i=0;i<exons;i++){
		if(ExternalSequence2(exon[i],ACstr)){
#ifdef VERBOUS
			printf("%d   %ld,%ld AC=%s\n",exon[i]->nc,exon[i]->begin,exon[i]->end,exon[i]->AC);
#endif
			if(SEQ_IMPORT){
#ifdef VERBOUS
				fprintf(stderr,"\nImporting.");
#else
/*			printf("i");*/
			putc(8,stdout);
			putc('i',stdout);
			fflush(stdout);
#endif
/********************************** import ************************************/
/* indentation is shifted */
	for(k=0;k<(*idxmax);k++){
		if(!strcmp(ix[k]->AC,getAC(exon[i]->AC))){
			fileloc=ix[k]->loc;
			break;
		}
	}
	if(k>=(*idxmax)){
		fprintf(stderr,"Unable to find the sequence with accession: %s\n",exon[i]->AC);
		strcpy(seq_exons->seq,"");
		seq_exons->seqlen=0L;
		return seq_exons;
	}
	seq_dbseq=read_dbseq(fileptr,&fileloc);

	seqlen2=seq_dbseq->seqlen;
/*	not compatible with circular DNA, which is normally not the case.
	if(circular && exon[i]->begin > seqlen2) exon[i]->begin -= seqlen2;
	if(circular && exon[i]->end > seqlen2) exon[i]->end -= seqlen2;
*/
	if(!circular && exon[i]->begin < 1) exon[i]->begin=1;
	if(!circular && exon[i]->end > seqlen2) exon[i]->end=seqlen2;

	if(seq_dbseq==NULL) {
		fprintf(stderr,"Unable to read the sequence with accession: %s\n",exon[i]->AC);
		strcpy(seq_exons->seq,"");
		seq_exons->seqlen=0L;
		return seq_exons;
	}
	if(exon[i]->nc){
		for(p2=seq_dbseq->seq+exon[i]->end-1;p2>=seq_dbseq->seq+exon[i]->begin-1;p1++,p2--) *p1=to_compD(*p2);
	}
	else{
		for(p2=seq_dbseq->seq+exon[i]->begin-1;p2<seq_dbseq->seq+exon[i]->end;p1++,p2++) *p1=*p2;
	}
	free_seq(seq_dbseq);

#ifdef VERBOUS
	printf("%s\n.",seq_exons->seq);
	fflush(stdout);
#endif
	continue;
/* end of shift */
/******************************************************************************/
			}
			else{
				fprintf(stderr,"\nImport from external file is not enabled.\n");
				fprintf(stderr,"Set the SEQ_IMPORT variable to true.\n");
				seq_exons->seqlen=0L;
				return seq_exons;
			}
		}
		else {
			if(circular && exon[i]->begin > seqlen) exon[i]->begin -= seqlen;
			if(circular && exon[i]->end > seqlen) exon[i]->end -= seqlen;
			if(!circular && exon[i]->begin < 1) exon[i]->begin=1;
			if(!circular && exon[i]->end > seqlen) exon[i]->end=seqlen;

#ifdef VERBOUS
			printf("%d   %ld,%ld AC=%s\n",exon[i]->nc,exon[i]->begin,exon[i]->end,exon[i]->AC);
#else
/*			printf("e");*/
			putc(8,stdout);
			putc('e',stdout);
			fflush(stdout);
#endif

			if(circular && exon[i]->begin > exon[i]->end){
				if(exon[i]->begin < seqlen-1000L || exon[i]->end > 1000L){
					fprintf(stderr,"\nError in extracting sequence from a circular genome.\n");
					seq_exons->seqlen=0L;
					return seq_exons;
				}
				if(exon[i]->nc){
					for(p2=seq_nuc->seq+exon[i]->end-1;p2>=seq_nuc->seq;p1++,p2--) *p1=to_compD(*p2);
					for(p2=seq_nuc->seq+seqlen-1;p2>=seq_nuc->seq+exon[i]->begin-1;p1++,p2--) *p1=to_compD(*p2);
				}
				else{
					for(p2=seq_nuc->seq+exon[i]->begin-1;p2<seq_nuc->seq+seqlen;p1++,p2++) *p1=*p2;
					for(p2=seq_nuc->seq;p2<seq_nuc->seq+exon[i]->end;p1++,p2++) *p1=*p2;
				}
			}
			else {
				if(exon[i]->nc){
					for(p2=seq_nuc->seq+exon[i]->end-1;p2>=seq_nuc->seq+exon[i]->begin-1;p1++,p2--) *p1=to_compD(*p2);
				}
				else{
					for(p2=seq_nuc->seq+exon[i]->begin-1;p2<seq_nuc->seq+exon[i]->end;p1++,p2++) *p1=*p2;
				}
			}
/* split mode. May 17, 2006 */
			if(split_mode && (i < exons-1)){
				*p1++ = '\n';
				for(counter=0;counter<split_number;counter++) *p1++ = 'x';
				*p1++ = '\n';
			}
		}
	}

	*p1='\0';
	seq_exons->seqlen=strlen(seq_exons->seq);

	circular=circular_bak;
	return seq_exons;

}
/*******************************************************************************/


/*********************************************************************/
long exon_total(Exon_record **exon,int exons,char *ACstr,long seqlen)
/* returns total length of exons of local file. */
/*********************************************************************/
{
	long i=0L;
	int j;

	for(j=0;j<exons;j++){
		if(strcmp(getAC(exon[j]->AC),ACstr) && strcmp(exon[j]->AC,"")) continue;
		if(exon[j]->end < exon[j]->begin) i+=(exon[j]->end+seqlen-exon[j]->begin+1);
		else i+=(exon[j]->end - exon[j]->begin+1);
	}
	return i;
}


/*********************************************************************/
long exon_total2(Exon_record **exon,int exons,char *ACstr,long seqlen)
/* returns total length of exons of local file, including 
external files. */
/*********************************************************************/
{
	long i=0L;
	int j;

	for(j=0;j<exons;j++){
		if(exon[j]->end < exon[j]->begin) i+=(exon[j]->end+seqlen-exon[j]->begin+1);
		else i+=(exon[j]->end - exon[j]->begin+1);
	}
	return i;
}


/***************************************************/
int show_gene_names(FILE *fout,Seqfile **seq_nuc)
/***************************************************/
{
	long i;
	char str1[MAXLINE]="";
	char str2[MAXLINE]="";

	if(seq_nuc==NULL){
		fprintf(stderr,"No valid sequences.\n");
		return 1;
	}

	for(i=0;i<num_seq;i++){
		if(strcmp(str2,seq_nuc[i]->ID)) {
			strcpy(str2,seq_nuc[i]->ID);
			fprintf(fout,"The genes in the file: %s are: ",str2);
		}
		else if(i!=0) fprintf(fout,", ");
		if(seq_nuc[i]->header[0]==' ') strcpy(str1,"no_info");
		else sscanf(seq_nuc[i]->header,"%s",str1);
		fprintf(fout,"%s",str1);
	}
	fprintf(fout,".\n");
	return 0;
}

/***********************************************************************/
int show_gene_table(FILE *fout,Seqfile **seq_nuc,enum data_type dtype1)
/* This subroutine prints gene list in a table format that can be
parsed in GRS program. */
/***********************************************************************/
{
	long i;
	int j=0;
	char str3[MAXLINE]="";
	char str4[MAXLINE]="";
	Boolean error_on=FALSE;
	int exons=1;
	Exon_record **exon;
	char comment[MAXLINE];
	long atg=0L;
	long stop=0L;
	long length=0L;

	if(seq_nuc==NULL){
		fprintf(stderr,"No valid sequences.\n");
		return 1;
	}

	for(i=0;i<num_seq;i++){
		sscanf(seq_nuc[i]->header,"%s",str4);

		exons=get_exon_number(seq_nuc[i]->header);

		if((exon=(Exon_record**)calloc(exons+1,sizeof(Exon_record)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return TRUE;
		}
		for(j=0;j<exons;j++){
			if((exon[j]=(Exon_record*)calloc(1,sizeof(Exon_record)))==NULL){
				fprintf(stderr,"Out of memory\n");
				return TRUE;
			}
		}
		for(j=0;j<MAXLINE-1;j++) comment[j]=' ';
		comment[MAXLINE-1]='\0';

		if(exon_list(seq_nuc[i]->header,exon,comment)){
			fprintf(stderr,"Error in making exon list in show_gene_table.\n");
			return TRUE;
		}


		for(j=0;j<exons;j++){

			if(exons==1) fprintf(fout,"%s\t",str4);	/* gene name */
			else {
				sprintf(str3,"%s_%d",str4,j+1);
				fprintf(fout,"%s\t",str3); 
			}
			if(dtype1==protein) fprintf(fout,"CDS\t");
			else if(dtype1==RNA) fprintf(fout,"RNA\t");
			else fprintf(fout,"unknown\t");
	
			if(exon[j]->nc) {
				fprintf(fout,"R\t");
				atg=exon[j]->end;
				stop=exon[j]->begin;
				length= atg - stop;
			}
			else {
				fprintf(fout,"F\t");
				atg=exon[j]->begin;
				stop=exon[j]->end;
				length= stop - atg;
			}
			fprintf(fout,"%8ld\t%8ld\t%8ld\t",atg,stop,length);

			if(exons==1) fprintf(fout,"%s\n",comment);
			else fprintf(fout,"\"Exon %d %s\"\n",j+1,comment);
			fflush(fout);


		}

		if(exon!=NULL && exons>0) free_exon(exon,exons);
		
	}

	if(error_on) {
		fprintf(stderr,"No valid sequences.\n");
		return TRUE;
	}
		
	fprintf(fout,"\n");
	return FALSE;
}

/*****************end of file excdfn.c**************************/

