/***********************************************************************
Extract CDS (extcds)
main file : extcds.c

 	Extract CDS from GenBank sequence file.

  Usage: extcds infile outfile (multi mode)
	 extcds infile outfile cds_name (single mode)
  
  Default outfile name is "infile"+".out"
  
  Source files are:	extcds.c
			gtsqfn.c
  			utility.c
  			Dmc.h
  			getseq2.h
			excdfn.c
			exit1.h
			exit0.h
  					
 	Naoki Sato   Jan 4, 1998.
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.
	For Mac version, define the compiler as well.

	Modified June 5, 1998.
	Added single mode.
	Added integrated mode. Nov. 23, 1998.
	Added reading of EMBL file. Mar. 13, 1999.
	Use of extract_cds2. March 16, 1999.
	Minor changes. April 25, 1999.
	Consecutive processing. Use of new extract_cds2. May 25, 1999.
	Verbous output suppressed. Aug. 25, 1999.
	siseq_exit0 or 1 changed to exit0 or 1. Nov. 13, 1999.
	Inclusion of Japanese messages. March 3, 2000.
	Bug fix. May 4, 2000.
	Added table mode. March 23, 2003.
	Bug fix. Jan. 17, 2007.
	Added exon_inf mode. Output of exon information is disabled normally. Jan. 17, 2007.

		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"

#ifndef INTEGRATED
#ifdef MAC
extern int ccommand(char ***);
#endif
#endif

/* Prototype declaration */
static void print_usage(void);
#ifdef INTEGRATED
int extcds(int c, char (*p)[S_2]);
#endif
void help_extcds(void);

#include "libSeq.h"

/* Global variables */
extern Boolean SEQ_IMPORT;
extern Boolean end_tag_found;
extern Boolean exon_inf_mode;
extern Boolean GI_out;

extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];
	
/***************************************************************/
static void print_usage(void)
/***************************************************************/
{
	help_extcds();
	return;
}
/***************************************************************/

/***************************************************************/
void help_extcds(void)
/***************************************************************/
{
#ifdef JAPANESE
#ifdef UNIX
#include "EUC/extcdsE.h"
#else
#ifdef MAC
#include "Mac/SJIS/extcdsS.h"
#else
#include "Dos/SJIS/extcdsS.h"
#endif
#endif
#else
	printf("******************* EXTCDS HELP *********************\n");
	printf("Usage: extcds infile outfile <optional cds_name>.\n\n");
	printf("       This program extracts CDS from\n");
	printf("       GenBank or EMBL data file.\n");
	printf("       Output is a multiple fasta file.\n");
	printf("If you add <cds_name>, only one sequence is extracted.\n");
	printf("By default, all CDS are extracted.\n");
	printf("Usage: extcds infile outfile_root t.\n");
	printf("       Table file outfile_root.p.table is also made.\n");
	printf("Usage: extcds infile outfile exoninf.\n");
	printf("       Detailed information on exons, which were shown.\n");
	printf("       in older versions of SISEQ, is normally omitted.\n");
	printf("       If exoninf option is enabled, exon information is\n");
	printf("       printed as before.\n");
	printf("*****************************************************\n");
#endif
	return;
}
/***************************************************************/


/***************************************************************
****************** Main program *******************************/
#ifdef INTEGRATED
int extcds(int cc, char (*p)[S_2])

#else
#ifdef MAC
main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif

#endif
/***************************************************************/

{
	FILE *fin;
	char fname1[100]="infile";
	char fname2[100]="outfile";
	char fname3[100]="outfile_root";
	char root[100]="outfile";	/* file name root */
	int c=0;
	int err_count=0;
	long counter=0L;
	char cdsname[AC_LENGTH]="";
	Boolean single_mode=FALSE;
	Boolean table_mode=FALSE;
/*	Boolean exon_inf_mode=FALSE;	This is made global. */
	char str[MAXLINE]="";
	long i;
#ifndef INTEGRATED
	char *envstr;
#endif

#include        "filenam.h"     /* definition of char *str1 and char *str2 */

	/* sequence variables */
	Seqfile **seq_cds;
	Seqfile *seq_out;
	Seqfile *seq_in;	/* For future enhancements. */
	long fileloc=0L;
	long fileloc_bak;

	exon_inf_mode=FALSE;	/* This is reset every time. */

#include	"parsarg.h"

/*********************** File names ******************/

	switch(cc){
		case 1:	
			strcpy(fname1,"infile");
			if(filename_input(fname1,str1)!=1){
#include	"exit1.h"
				}
			strcpy(fname2,fname1);
			strcat(fname2,".out");
			if(filename_input(fname2,str2)!=1){
#include	"exit1.h"
			}

                        printf("What is the name of CDS?\n");
                        printf("If you input - all -, all CDS will be extracted.\n");
                        scanf("%s",str);
                        if((strcmp(str,"all")!=0)){
                                single_mode=TRUE;
				if(strlen(str)>AC_LENGTH-1) {
/*					pp=extract_string(str,1,AC_LENGTH);
					strcpy(cdsname,pp);
					free(pp);
*/
					strncpy(cdsname,str,AC_LENGTH-1);
				}
				else strcpy(cdsname,str);
			}
			break;
			
		case 3:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			break;

		case 4:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			if(!strcmp(p[3],"exoninf")){
				exon_inf_mode = TRUE;
			} else if(!strcmp(p[3],"t")){
				table_mode=TRUE;
				strcat(fname2,".fa");
				strcpy(fname3,p[2]);
				strcpy(root,p[2]);
				strcat(fname3,".p.table");
			} else {
				strcpy(cdsname,p[3]);
				single_mode=TRUE;
			}
                        break;

			
		default:
			print_usage();
#include	"exit1.h"
	}


/***********************Main part****************************/
	SEQ_IMPORT=FALSE;

	c=check_filetype(fname1);
	if(c!=GENBANK_SEQ && c!=EMBL_SEQ){
		fprintf(stderr,"This is not a GenBank or EMBL file.\n");
		exit(1);
	}
	printf("Reading sequence...\n");

#ifdef MAC
#ifdef CODEWARRIOR
	SIOUXSetTitle("\pextcds");
#endif
#endif	

	if((seq_cds=(Seqfile **)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory\n");
		exit(1);
	}
	if((seq_in=(Seqfile *)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory\n");
#include        "exit1.h"
	}
	if((seq_in->ID=(char *)calloc(MINMEMSIZE,sizeof(char)))==NULL){ 
		fprintf(stderr,"Out of memory\n");
#include        "exit1.h"
	}
	if((seq_in->header=(char *)calloc(2,sizeof(char)))==NULL){ 
		fprintf(stderr,"Out of memory\n");
#include        "exit1.h"
	}

	if((fin=fopen(fname1,"r"))==NULL){
		fprintf(stderr,"Cannot open file: %s.\n\n",fname1);
#include	"exit1.h"
	}

	if(ftouch(fname2)){
               	fprintf(stderr,"Error in opening output file: %s.\n",fname2);
#include	"exit1.h"
       	}
	if(table_mode){
		if(ftouch(fname3)){
        	       	fprintf(stderr,"Error in opening output file: %s.\n",fname3);
#include	"exit1.h"
		}
	}
/******************* main loop **********************/

#ifndef INTEGRATED
	if((envstr=getenv("GI_out"))!=NULL){
		if(*envstr =='1' || envstr[0]=='t' || envstr[0]=='T'){
			GI_out = 1;
			printf("GI_out is enabled.\n");
		} else {
			GI_out = 0;
		}
	}
#else
	if(GI_out == 1) printf("GI_out has been enabled.\n");
#endif

	end_tag_found = FALSE;
	while(!feof(fin)){

	/* seq_in stores the nucleic acid sequence, which is not currently used in 
	extcds. These lines are for compatibility with the extract_cds2 function. */

		if(table_mode) fileloc_bak = fileloc;
		if((seq_cds=extract_cds2(fin,seq_in,&fileloc,c,protein))==NULL){
			if(!end_tag_found){
				fprintf(stderr,"\nEnd tag // is missing.\n");
				break;
			} else if(feof(fin)) break;
			else {
				fprintf(stderr,"Error in extracting CDS\n");
				if(err_count++>=10) break;
				else continue;
			}
		}
		if(seq_in->seq==NULL) {
			if(feof(fin)) break;
			else {
				if(err_count++>=10) break;
				else continue;
			}
		}

#ifdef VERBOUS
		printf("Headers extracted from database: %s.\n",seq_in->ID);
#else
/*		printf("h");*/
		putc(8,stdout);
		putc('h',stdout);
		fflush(stdout);
#endif

		strcpy(seq_in->ID,"");
		strcpy(seq_in->header,"");

		if(counter%10==0){
/*			printf(".");*/
			putc(8,stdout);
			putc('-',stdout);
			fflush(stdout);
		}
	
		if(single_mode){
		
		/* single mode */

			if((seq_out=retrve_cds(seq_cds,cdsname))==NULL){
				fprintf(stderr,"No CDS: %s in database: %s.\n",cdsname,seq_cds[0]->ID);
				continue;
			}

			if(write_fasta(fname2,seq_out,INCLUDE_HEADER,'_')){
				fprintf(stderr,"Unable to write file.\n");
				continue;
			}
		}
		
		else{
			if(table_mode){
				for(i=0;i<num_seq;i++) strcpy(seq_cds[i]->ID,root);
			}
		
		/* multi mode */
			if(Mwrite(fname2,seq_cds)){
				fprintf(stderr,"Unable to write file.\n");
				exit(1);
			}
		}

		if(table_mode){
			fileloc=fileloc_bak;
			if((seq_cds=extract_cds2(fin,seq_in,&fileloc,c,annotP))==NULL){
				if(feof(fin)) break;
				fprintf(stderr,"Error in extracting CDS2\n");
				if(err_count++>=10) break;
				else continue;
			}
			for(i=0;i<num_seq;i++) strcpy(seq_cds[i]->ID,root);
			if(Twrite(fname3,seq_cds)){
				fprintf(stderr,"Unable to write file %s.\n",fname3);
				exit(1);
			}
		}

	}
	fclose(fin);

	printf("\nTerminated.\n");

#include	"exit0.h"

}

/************************************************************
		End of file extcds.c
************************************************************/


