/***********************************************************************
Extracts intron sequences (extint)
main file : extint.c

 	Extract intron sequences within CDS from GenBank or 
	EMBL sequence file.

  Usage: extint infile outfile start end <cdsname>
  start and end should be S xx  or E xx.
  S xx means xx nucleotides from the intron start position.
  E xx means xx nucleotides from the intron end position.
  cdsname (optional) specifies a single CDS to be extracted.
  
  Default outfile name is "infile"+".out"
  
  Source files are:	extint.c
 			cdncfn.c
  			gtsqfn.c
  			utility.c
  			excdfn.c
  			Dmc.h
  			getseq2.h
  	
 	Naoki Sato   Dec. 8, 1999.
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.
	For Mac version, define the compiler as well.

	Inclusion of Japanese messages. March 3, 2000.
	Bug fix. May 4, 2000.

		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"


#ifndef INTEGRATED
#ifdef MAC
extern int ccommand(char ***);
#endif
#endif

/* Prototype declaration */
static void print_usage(void);
static void help_input(void);

#ifdef INTEGRATED
int extint(int c, char (*p)[S_2]);
#endif
void help_extint(void);

/* external functions */
#include "libSeq.h"

/* Global variables */

extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];

extern FILE *fileptr;	/* defined in excdfn.c */

#ifdef JAPANESE
#ifdef UNIX
#include "EUC/extintE.h"
#else
#ifdef MAC
#include "Mac/SJIS/extintS.h"
#else
#include "Dos/SJIS/extintS.h"
#endif
#endif
#endif


/***************************************************************/
static void print_usage(void)
/***************************************************************/
{
	help_extint();
	return;
}

/***************************************************************/

/***************************************************************/
static void help_input(void)
/***************************************************************/
{
#ifdef JAPANESE
	me1();
#else
	printf("Type desired lengths of the 5'exon and 3'exon\n");
	printf("start and end should be S xx  or E xx.\n");
	printf("S xx means xx nucleotides from the intron start position.\n");
	printf("E xx means xx nucleotides from the intron end position.\n");
	printf("Use a space between the four values.\n");
#endif
	return;
}
/***************************************************************/

/***************************************************************/
void help_extint(void)
/***************************************************************/
{
#ifdef JAPANESE
	me2();
#else
        printf("*********************** EXTINT HELP ********************\n");
        printf("Usage: extint infile outfile start end <cds_name>.\n\n");
        printf("       start and end should be S xx  or E xx.\n");
        printf("S xx means xx nucleotides from the intron start position.\n");
        printf("E xx means xx nucleotides from the intron end position.\n");
        printf("<cds_name> is the name of CDS to be extracted (optional).\n");
        printf("********************************************************\n");
#endif
	return;
}
/***************************************************************/

/**************************************************************
****************** Main program *******************************/
#ifdef INTEGRATED
int extint(int cc, char (*p)[S_2])

#else
#ifdef MAC
main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif

#endif
/***************************************************************/

{
	FILE *fin;
	char fname1[100]="infile";
	char fname2[100]="outfile";
	int c=0;
	long counter=0L;
        char cdsname[AC_LENGTH]="";
        Boolean single_mode=FALSE;
        char str[MAXLINE]="";
	char start_loc[5];
	char end_loc[5];
	char *pp;

#include        "filenam.h"     /* definition of char *str1 and char *str2 */

	char gname[80]="";
	long startpos;
	long endpos;
	Boolean need_input=FALSE;
	Boolean exit_flag=FALSE;
	long N;
	long i;
	int v;	/* return value from ext_introns */
	long Ntmp=0L;	/* temporary variable for the number of sequences. */

	/* sequence variables */
	Seqfile *seq_in;	/* input DNA sequence */
	Seqfile **seq_nuc;	/* headers */
	Seqfile **seqM;		/* intron sequences */
	long M=0L;	/* number of intron sequences */

	long fileloc=0L;

#include	"parsarg.h"


/*********************** File names ******************/

/*
printf("argc=%d, cc=%d",argc,cc);
*/

	switch(cc){
		case 1:	
			strcpy(fname1,"infile");
			if(filename_input(fname1,str1)!=1){
#include        "exit1.h"
			}
				
			strcpy(fname2,fname1);
			strcat(fname2,".out");
			if(filename_input(fname2,str2)!=1){
#include        "exit1.h"
			}
			need_input=TRUE;
			break;
			
		case 7:
                case 8:
			strcpy(fname1,p[1]);
                        strcpy(fname2,p[2]);
                        strcpy(start_loc,p[3]);
			startpos=parse_position(p[4]);
			strcpy(end_loc,p[5]);
			endpos=parse_position(p[6]);
                        if(check_se(start_loc,end_loc) ) {
				print_usage();
#include        "exit1.h"
			}
			if(cc==8){
				strcpy(cdsname,p[7]);
                        	single_mode=TRUE;
			}

			break;

		default:
			print_usage();
#include        "exit1.h"
	}

/**************************** need input *************************************************/

		if(need_input){
			while(1){
				help_input();
				scanf("%s %ld %s %ld",start_loc,&startpos,end_loc,&endpos);
				if(check_se(start_loc,end_loc)) continue;
				break;
			}
			printf("What is the name of CDS? Type <exit> to exit this session.\n");
			printf("Use a command <genlist> to show all CDS names beforehand.\n");
			printf("If you input - all -, introns of all CDS will be extracted.\n\n");
			scanf("%s",str);
			if(!strcmp(str,"exit")){
#include	"exit0.h"
			}
			else if((strcmp(str,"all")!=0)){
				single_mode=TRUE;
				if(strlen(str)>AC_LENGTH) {
					pp=extract_string(str,1,AC_LENGTH);
					strcpy(cdsname,pp);
					free(pp);
				}
				else strcpy(cdsname,str);
			}
			need_input=FALSE;
		}

/******************************************************************************************/

/*********************** Main part ****************************/

		c=check_filetype(fname1);
		if(c!=GENBANK_SEQ && c!=EMBL_SEQ){
			fprintf(stderr,"This is not a GenBank or EMBL file.\n");
			exit(1);
		}
		printf("Reading sequence...\n");

#ifdef MAC
#ifdef CODEWARRIOR
		SIOUXSetTitle("\pextint");
#endif
#endif

		if((seq_nuc=(Seqfile **)calloc(1,sizeof(Seqfile)))==NULL){
			fprintf(stderr,"Out of memory\n");
			exit(1);
		}
		if((seqM=(Seqfile **)calloc(1,sizeof(Seqfile)))==NULL){
			fprintf(stderr,"Out of memory\n");
			exit(1);
		}
		if((seq_in=(Seqfile *)calloc(1,sizeof(Seqfile)))==NULL){
			fprintf(stderr,"Out of memory\n");
#include	"exit1.h"
		}
		if((seq_in->ID=(char *)calloc(MINMEMSIZE,sizeof(char)))==NULL){ 
			fprintf(stderr,"Out of memory\n");
#include	"exit1.h"
		}
		if((seq_in->header=(char *)calloc(2,sizeof(char)))==NULL){ 
			fprintf(stderr,"Out of memory\n");
#include	"exit1.h"
		}

		if((fin=fopen(fname1,"r"))==NULL){
			fprintf(stderr,"Cannot open file: %s\n\n",fname1);
#include	"exit1.h"
		}

		if(ftouch(fname2)){
			fprintf(stderr,"Cannot open output file: %s\n\n",fname2);
#include	"exit1.h"
		}

/**************** main loop ***********************/
		
                while(!feof(fin)){
			if((seq_nuc=extract_cds2(fin,seq_in,&fileloc,c,unknown))==NULL){
				if(feof(fin)) break;
				fprintf(stderr,"Error in extracting CDS\n");
				break;
			}
                        if(seq_in->seq==NULL || strlen(seq_in->seq)==0) {
				if(feof(fin)) break;
				else {
/*					printf(".");
*/
					putc(8,stdout);
					putc('/',stdout);
					fflush(stdout);
					continue;
				}
			}
#ifdef VERBOUS 
			printf("\nHeaders extracted from database: %s.\n",seq_in->ID);
#else
/*			printf("h");*/
			putc(8,stdout);
			putc('h',stdout);
			fflush(stdout);
#endif
			check_data_type(seq_in);
			if(clear_gaps(seq_in)){
				fprintf(stderr,"Error in clearing gaps in file: %s\n",seq_in->ID);
				continue;
			}

/************ adding sequences one by one. ******************/
		exit_flag=FALSE;
		N=num_seq;
		num_seq=1;
		fileptr=fin;
		for(i=0;i<N;i++){
			if(seq_nuc[i]->header!=NULL) sscanf(seq_nuc[i]->header,"%s",gname);
			else sprintf(gname,"seq.No.=%ld.",i);
#ifdef VERBOUS
			printf("Processing sequence %s %s.\n",seq_in->ID,gname);
#else
			putc(8,stdout);
			putc('.',stdout);
			fflush(stdout);
#endif

			if(single_mode && strstr(seq_nuc[i]->header,cdsname)==NULL){
				free_seq(seq_nuc[i]);
				continue;
			}

			Ntmp=num_seq;
		
			M=get_exon_number(seq_nuc[i]->header);
	
			if((seqM=(Seqfile **)calloc(M+1,sizeof(Seqfile)))==NULL){
				fprintf(stderr,"Out of memory\n");
#include	"exit1.h"
			}

			v=ext_introns(seq_in,seq_nuc[i],seqM,start_loc,startpos,end_loc,endpos);
			if(v){
				if(v==10) continue;
				fprintf(stderr,"Error in extracting introns.\n");
				exit_flag=TRUE;
				break;
			}
			M=num_seq;

#ifdef VERBOUS
	printf("ID=%s, len=%ld\n",seqM[0]->ID,seqM[0]->seqlen);
#endif

			if(counter%20==0){
				putc(8,stdout);
				putc('-',stdout);
				fflush(stdout);
			}
			else if(counter%10==0){
				putc(8,stdout);
				putc('|',stdout);
				fflush(stdout);
			}


#ifdef VERBOUS
		printf("M=%ld\n",M);
#endif
      	                if(Mwrite(fname2,seqM)){
       	                        fprintf(stderr,"Unable to write file: %s.\n",fname2);
       	                        continue;
               	        }
			num_seq=Ntmp;
			free_seq(seq_nuc[i]);

		}
		num_seq=N;
/***********************************************************/
			strcpy(seq_in->ID,"");
			strcpy(seq_in->header,"");
			if(exit_flag) break;
		}

		fclose(fin);


	printf("\nTerminated.\n");

#include	"exit0.h"

}

/************************************************************
		End of file extint.c
************************************************************/

