/***********************************************************************
Extracts nucleotide sequence for RNA genes (extrna)
main file : extrna.c

 	Extract nucleic acid sequences for RNA genes from GenBank sequence file.

  Usage: extrna infile outfile start end <RNA_name>
  start and end should be S xx  or E xx.
  S xx means xx nucleotides from the start position.
  E xx means xx nucleotides from the end position.
  <RNA_name> specifies a single RNA (optional).
  
  
  Default outfile name is "infile"+".out"
  
  Source files are:	extrna.c
  			cdncfn.c
  			gtsqfn.c
  			utility.c
  			excdfn.c
  			Dmc.h
  			getseq2.h
  	
 	Naoki Sato   Jan 26, 1998.
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.
	For Mac version, define the compiler as well.

	Modified June 7, 1998.
	Added the single sequence mode.
	Bug fix. Added integrated mode. Nov. 23, 1998.
	Adapted to multi GB file. Jan. 09, 1999.
	Adapted to reading an EMBL file. March 13, 1999.
	Modified input dialog. March 19, 1999.
	Added counter to show progress of processing. April 29, 1999.
	Bug fix. May 17, 1999.
	Use of integrated extract_cds2. May 25, 1999.
	Dual mode. add_seq and add_seqg. May 30, 1999.
	Verbous output suppressed. Aug. 25, 1999.
	siseq_exit0 or 1 changed to exit0 or 1. Nov. 13, 1999.
	Inclusion of Japanese messages. March 3, 2000.
	Bug fix. May 4, 2000.
	
		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"

#ifndef INTEGRATED
#ifdef MAC
extern int ccommand(char ***);
#endif
#endif

/* Prototype declaration */
static void print_usage(void);
static void help_input(void);

#ifdef INTEGRATED
int extrna(int c, char (*p)[S_2]);
#endif
void help_extrna(void);

/* external functions */

#include "libSeq.h"

/* Global variables */
extern Boolean SEQ_IMPORT;

extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];
extern Boolean ADDSEQG;

#ifdef JAPANESE
#ifdef UNIX
#include "EUC/extrnaE.h"
#else
#ifdef MAC
#include "Mac/SJIS/extrnaS.h"
#else
#include "Dos/SJIS/extrnaS.h"
#endif
#endif
#endif



/***************************************************************/
static void print_usage(void)
/***************************************************************/
{
	help_extrna();
	return;
}
/***************************************************************/

/***************************************************************/
static void help_input(void)
/***************************************************************/
{
#ifdef JAPANESE
	me1();
#else
	printf("Type desired lengths of the 5' and 3' flanking sequences.\n");
	printf("start and end should be S xx  or E xx.\n");
	printf("S xx means xx nucleotides from the RNA start position.\n");
	printf("E xx means xx nucleotides from the RNA end position.\n");
	printf("Use a space between the four values.\n");
#endif
	return;
}

/***************************************************************/

/***************************************************************/
void help_extrna(void)
/***************************************************************/
{
#ifdef JAPANESE
	me2();
#else
        printf("******************* EXTRNA HELP *****************************\n");
        printf("Usage: extrna infile outfile start end <RNA_name>.\n\n");
        printf("       start and end should be S xx  or E xx.\n");
        printf("S xx means xx nucleotides from the CDS start position.\n");
        printf("E xx means xx nucleotides from the CDS end position.\n");
        printf("<RNA_name> is the name of RNA to be extracted (optional).\n");
	printf("By default, all sequences are extracted.\n");
	printf("If ADDSEQG is set true, introns are included.\n");
	printf("Partial truncated sequences are also shown if ADDSEQG is true.\n");
        printf("**************************************************************\n");
#endif
	return;
}
/***************************************************************/


/***************************************************************
****************** Main program *******************************/
#ifdef INTEGRATED
int extrna(int cc, char (*p)[S_2])

#else
#ifdef MAC
main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif

#endif
/***************************************************************/

{
	FILE *fin;
	char fname1[100]="infile";
	char fname2[100]="outfile";
	int c=0;
	long counter=0L;
        char cdsname[AC_LENGTH]="";	/* This is for RNA_name. */
        Boolean single_mode=FALSE;
        char str[MAXLINE]="";
	char *pp;

#ifndef INTEGRATED
	char *strenv;
#endif
	int i=0;
	int j=0;
        char str3[MAXLINE]="";
        char cdsname1[MINMEMSIZE];
	char start_loc[5];
	char end_loc[5];

#include        "filenam.h"     /* definition of char *str1 and char *str2 */   

	long int startpos;
	long int endpos;
	long fileloc=0L;
/*	enum data_type dtype1=RNA;*/
	Boolean need_input=FALSE;

	/* sequence variables */
	Seqfile *seq_in;
	Seqfile **seq_nuc;
        Seqfile **seq_out;

#include	"parsarg.h"
#ifndef	INTEGRATED
	if((strenv=getenv("ADDSEQG"))!=NULL){
		if(!strcmp(strenv,"TRUE") || !strcmp(strenv,"true") || !strcmp(strenv,"1")) ADDSEQG=TRUE;
	}
	else ADDSEQG=FALSE;
#endif	
	
/*********************** File names ******************/

	switch(cc){
		case 1:	
			strcpy(fname1,"infile");
			if(filename_input(fname1,str1)!=1){
#include	"exit1.h"
			}
			strcpy(fname2,fname1);
			strcat(fname2,".out");
			if(filename_input(fname2,str2)!=1){
#include	"exit1.h"
			}
			need_input=TRUE;
			break;
			
		
		case 7:
                case 8:
                        strcpy(fname1,p[1]);
                        strcpy(fname2,p[2]);
                        strcpy(start_loc,p[3]);
			startpos=parse_position(p[4]);
			strcpy(end_loc,p[5]);
			endpos=parse_position(p[6]);
			if(check_se(start_loc,end_loc)) {
			        print_usage();
#include	"exit1.h"
                        }
			if(cc==8){
				strcpy(cdsname,p[7]);
				single_mode=TRUE;
			}
                        break;

		default:
			print_usage();
#include	"exit1.h"
	}

/************* need input ************************************/

	if(need_input){
		while(1){
			help_input();
			scanf("%s %ld %s %ld",start_loc,&startpos,end_loc,&endpos);
			if(check_se(start_loc,end_loc)) continue;
			break;
		}
		printf("What is the name of RNA? Type <exit> to exit this session.\n");
		printf("Use a command <genlist> to show all CDS names beforehand.\n");
                printf("If you input - all -, all RNA will be extracted.\n\n");
                scanf("%s",str);
		if(!strcmp(str,"exit")){
#include	"exit1.h"
		}
                else if((strcmp(str,"all")!=0)){
                        single_mode=TRUE;
                        if(strlen(str)>AC_LENGTH) {
				pp=extract_string(str,1,AC_LENGTH);
				strcpy(cdsname,pp);
				free(pp);
			}
                        else strcpy(cdsname,str);
		}
		need_input=FALSE;
	}

/******************************************************************/

/*********************** Main part ****************************/
	SEQ_IMPORT=FALSE;

	c=check_filetype(fname1);
	if(c!=GENBANK_SEQ && c!=EMBL_SEQ){
		fprintf(stderr,"This is not a GenBank or EMBL file.\n");
		exit(1);
#include	"exit1.h"
	}
	printf("Reading sequence...\n");

#ifdef MAC
#ifdef CODEWARRIOR
	SIOUXSetTitle("\pextrna");
#endif
#endif

	if((seq_nuc=(Seqfile **)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory\n");
		exit(1);
	}

        if((fin=fopen(fname1,"r"))==NULL){
                fprintf(stderr,"Cannot open file: %s\n\n",fname1);
#include	"exit1.h"
        }
	if(ftouch(fname2)){
		fprintf(stderr,"Cannot open output file: %s.\n\n",fname2);
#include	"exit1.h"
	}

/********************** main loop **********************/

	while(!feof(fin)){
		if((seq_in=(Seqfile *)calloc(1,sizeof(Seqfile)))==NULL){
			fprintf(stderr,"Out of memory\n");
#include	"exit1.h"
		}	
		if((seq_in->ID=(char *)calloc(MINMEMSIZE,sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory\n");
#include	"exit1.h"
		}	
		if((seq_in->header=(char *)calloc(2,sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory\n");
#include	"exit1.h"
		}	

		if((seq_nuc=extract_cds2(fin,seq_in,&fileloc,c,RNA))==NULL){
			if(feof(fin)) break;
			fprintf(stderr,"Error in extracting CDS\n");
			continue;
		}
		if(seq_in->seq==NULL || strlen(seq_in->seq)==0){
			if(feof(fin)) break;
			else {
/*				printf(".");*/
				putc(8,stdout);
				putc('.',stdout);
				fflush(stdout);
				continue;
			}
		}
#ifdef VERBOUS
		printf("\nHeaders extracted from database: %s.\n",seq_in->ID);
#else
/*		printf("h");*/
		putc(8,stdout);
		putc('h',stdout);
		fflush(stdout);
#endif

		check_data_type(seq_in);
		if(clear_gaps(seq_in)){
			fprintf(stderr,"Error in clearing gaps in file: %s\n",seq_in->ID);
			free_seq(seq_in);
			continue;
		}
		if(ADDSEQG){
			if(add_seqG(seq_in,seq_nuc,start_loc,startpos,end_loc,endpos)){
				fprintf(stderr,"Error in extracting sequence.\n");
				free_seq(seq_in);
				continue;
			}
		}
		else{
			if(add_seq(seq_in,seq_nuc,start_loc,startpos,end_loc,endpos)){
				fprintf(stderr,"Error in extracting sequence.\n");
				free_seq(seq_in);
				continue;
			}
		}
		printf("\nSequences extracted.\n");

		if(counter%10==0){
			printf(".");
			fflush(stdout);
		}

		if(single_mode){

		/* single mode. This code is different from that in cdsnuc.Here, 
		all RNA genes of the same name are extracted. Useful in extracting
		multiple tRNA genes from a genome. */
                        
			printf("Extracting sequences: %s...\n",cdsname);

			if((seq_out=(Seqfile **)calloc(10,sizeof(Seqfile)))==NULL){
        	                fprintf(stderr,"Out of memory\n");
                	        exit(1);
			}
			for(i=0;i<10;i++){
                        	if((seq_out[i]=(Seqfile *)calloc(1,sizeof(Seqfile)))==NULL){
                                	fprintf(stderr,"Out of memory\n");
                                	exit(1);
                        	}
			}
		        for(i=0;i<num_seq;i++){
              		  	strcpy(str3,seq_nuc[i]->header);
                		sscanf(str3,"%s",cdsname1);
                		if(strcmp(cdsname1,cdsname)==0){
					if((seq_out[j]=copy_seq(seq_nuc[i]))==NULL){
                           	     		fprintf(stderr,"Error in retrieving RNA\n");
#include	"exit1.h"
                        		}
				j++;
                		}
				if(j>9) break;
        		}
			if(j==0){
				fprintf(stderr,"No sequence extracted.\n");
#include	"exit1.h"
			}
			num_seq=j;
			if(seq_nuc==NULL) exit(1);	
                        if((c=Mwrite(fname2,seq_out))!=0){
                                fprintf(stderr,"Unable to write file.\n");
#include	"exit1.h"
                        }

		}

                else{
                /* multi mode */
                        printf("Extracting all sequences...\n");
			if(Mwrite(fname2,seq_nuc)){
				fprintf(stderr,"Unable to write file.\n");
#include	"exit1.h"
			}
		}

		free_seq(seq_in);
	}

	fclose(fin);
	printf("\nTerminated.\n");

#include	"exit0.h"
}

/************************************************************
		End of file extrna.c
************************************************************/

