/***********************************************************************
Gene list (genlist)
main file : genlist.c

 	Show CDS names in GenBank or EMBL sequence file.

  Usage: genlist infile (output to stdout)
	 genlist infile outfile
  
  Source files are:	genlist.c
			gtsqfn.c
  			utility.c
  			Dmc.h
  			getseq2.h
			excdfn.c
  					
 	Naoki Sato   April 25, 1999.
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.
	For Mac version, define the compiler as well.

	Created. April 25, 1999.
	Added info for FASTA sequences. Oct. 14, 1999.
	siseq_exit0 or 1 changed to exit0 or 1. Nov. 13, 1999.
 	Inclusion of Japanese messages. March 3, 2000.
	Table mode. Oct. 25, 2000.
	Bug fix for parsing RNA. Feb. 8, 2003.
	
		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"


#ifndef INTEGRATED
#ifdef MAC
extern int ccommand(char ***);
#endif
#endif

/* Prototype declaration */
static void print_usage(void);
#ifdef INTEGRATED
int genlist(int c, char (*p)[S_2]);
#endif
void help_genlist(void);

#include "libSeq.h"

/* Global variables */
extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];
	
/***************************************************************/
static void print_usage(void)
/***************************************************************/
{
	help_genlist();
	return;
}
/***************************************************************/

/***************************************************************/
void help_genlist(void)
/***************************************************************/
{
#ifdef JAPANESE
#ifdef UNIX
#include "EUC/gnlsE.h"
#else
#ifdef MAC
#include "Mac/SJIS/gnlsS.h"
#else
#include "Dos/SJIS/gnlsS.h"
#endif
#endif
#else
        printf("********************** GENLIST HELP **************************\n");
        printf("Usage: genlist infile <outfile> <t>.\n\n");
        printf("       This program shows CDS names in GenBank or EMBL file.\n");
        printf("       It also shows information on FASTA sequences.\n");
	printf("  Default output is the console unless <outfile> is specified.\n");
	printf("  t indicates table output, which always needs outfile.\n");
        printf("**************************************************************\n");
#endif
	return;
}
/***************************************************************/


/***************************************************************
****************** Main program *******************************/
#ifdef INTEGRATED
int genlist(int cc, char (*p)[S_2])

#else
#ifdef MAC
main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif

#endif
/***************************************************************/

{
	FILE *fin;
	char fname1[100]="infile";
	char fname2[100]="outfile";
	int c=0;
	enum data_type dtype1=unknown;
	char type_name[MINMEMSIZE]="type_unknown";

#include        "filenam.h"     /* definition of char *str1 and char *str2 */

	FILE *fout;
	long fileloc=0L;
	long fileloc_bak=0L;
	long i;
	Boolean table_mode=FALSE;

	/* sequence variables */
	Seqfile **seq_nuc;
	Seqfile *seq_in;

#include	"parsarg.h"
				
/*********************** File names ******************/

	switch(cc){
		case 1:	
			strcpy(fname1,"infile");
			if(filename_input(fname1,str1)!=1){
#include	"exit1.h"
			}
				
			strcpy(fname2,"stdout");
			if(filename_input(fname2,str2)!=1){
#include	"exit1.h"
			}
			break;
			
		case 2:
			if(!strcmp(p[1],"help") || !strcmp(p[1],"-h")) {
			print_usage();
#include	"exit1.h"
			}
			else {
				strcpy(fname1,p[1]);
				strcpy(fname2,"stdout");
			}
			break;

                case 3:
                        strcpy(fname1,p[1]);
                        strcpy(fname2,p[2]);
                        break;
			
                case 4:
                        strcpy(fname1,p[1]);
                        strcpy(fname2,p[2]);
			if(p[3][0]=='t' || p[3][0]=='T') table_mode=TRUE;
                        break;
			
		default:
			print_usage();
#include	"exit1.h"
	}

/***********************Main part****************************/

	c=check_filetype(fname1);
	if(c!=GENBANK_SEQ && c!=EMBL_SEQ && c!=FASTA_SEQ){
		fprintf(stderr,"This is not a GenBank, EMBL, or FASTA file.\n");
		exit(1);
	}
	printf("Reading sequence...\n");

#ifdef MAC
#ifdef CODEWARRIOR
	SIOUXSetTitle("\pgenlist");
#endif
#endif	

	if(!strcmp(fname2,"stdout")) fout=stdout;
	else {
		if(ftouch(fname2)){
			fprintf(stderr,"Cannot open file: %s\n",fname2);
			fprintf(stderr,"Output to console.\n");
			fout=stdout;
		}
		else if((fout=fopen(fname2,"a+"))==NULL){
			fprintf(stderr,"Cannot open file: %s\n",fname2);
			fprintf(stderr,"Output to console.\n");
			fout=stdout;
		}
	}
	if((seq_nuc=(Seqfile **)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory\n");
		exit(1);
	}


/****************** main loop *********************/

	if(c==FASTA_SEQ){
		if(table_mode) {
			fprintf(stderr,"Table output is not available for a FASTA file.\n");
#include	"exit1.h"
		}
			
		if((seq_nuc=Mread_fasta(fname1,dtype1))==NULL){
			fprintf(stderr,"Cannot read sequence.\n");
#include	"exit1.h"
		}
		fprintf(fout,"\nThis is a list of sequences in the file: %s.\n\n",fname1);
		for(i=0;i<num_seq;i++){
			fprintf(fout,"ID=%s, %ld residues, %s.\n",seq_nuc[i]->ID,seq_nuc[i]->seqlen,\
			seq_nuc[i]->header);
		}

	} else {

	/********** begin GenBank and EMBL *****************/

	if((fin=fopen(fname1,"r"))==NULL){
		fprintf(stderr,"Cannot open file: %s.\n\n",fname1);
#include	"exit1.h"
	}
			
	while(!feof(fin)){
		
		if((seq_in=(Seqfile *)calloc(1,sizeof(Seqfile)))==NULL){
			fprintf(stderr,"Out of memory\n");
#include	"exit1.h"
		}		
		if((seq_in->ID=(char *)calloc(MINMEMSIZE,sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory\n");
#include	"exit1.h"
		}		
		if((seq_in->header=(char *)calloc(2,sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory\n");
#include	"exit1.h"
		}		

		if(table_mode){
			fileloc_bak=fileloc;
			if((seq_nuc=extract_cds2(fin,seq_in,&fileloc,c,unknown))==NULL){
				if(feof(fin)) break;
				fprintf(stderr,"Error in extracting headers.\n");
				continue;
			}
			if(seq_in->seq==NULL){
				if(feof(fin)) break;
				else continue;
			}
/*			seq_in->seqlen=(long)strlen(seq_in->seq);	*/

                        fprintf(fout,"#GTF\n");
                        fprintf(fout,"Organism: %s %s\n",seq_in->OS,seq_in->ID); 
                        fprintf(fout,"Type: %s \n",type_name);	/* to be completed by hand */
                        fprintf(fout,"Size: %ld \n",seq_in->seqlen);
                        fprintf(fout,"Contigs: 0\n");
                        fprintf(fout,"\n");
                        fprintf(fout,"definition of format:\n");
                        fprintf(fout,"\n");
                        fprintf(fout,"name\ttype\torient\tstart\tstop\tlength\tdescription\n");

			free_seq(seq_in);	/* Re-use */
			if(show_gene_table(fout,seq_nuc,protein)){
				fprintf(stderr,"Error in showing gene table.\n");
				fprintf(fout,"Error ... \n\n");
				continue;
			}

			if((seq_in=(Seqfile *)calloc(1,sizeof(Seqfile)))==NULL){
				fprintf(stderr,"Out of memory\n");
#include	"exit1.h"
			}		
			if((seq_in->ID=(char *)calloc(MINMEMSIZE,sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory\n");
#include	"exit1.h"
			}		
			if((seq_in->header=(char *)calloc(2,sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory\n");
#include	"exit1.h"
			}		
	
			fileloc=fileloc_bak;
			if((seq_nuc=extract_cds2(fin,seq_in,&fileloc,c,RNA))==NULL){
				if(feof(fin)) break;
				fprintf(stderr,"Error in extracting headers.\n");
				continue;
			}
			if(seq_in->seq==NULL){
				if(feof(fin)) break;
				else continue;
			}
			free_seq(seq_in);	/* No use */
			if(show_gene_table(fout,seq_nuc,RNA)){
				fprintf(stderr,"Error in showing gene table 2.\n");
				fprintf(fout,"Error ... \n\n");
				continue;
			}
		}

		else {
			if((seq_nuc=extract_cds2(fin,seq_in,&fileloc,c,unknown))==NULL){
				if(feof(fin)) break;
				fprintf(stderr,"Error in extracting headers.\n");
				continue;
			}
			if(seq_in->seq==NULL){
				if(feof(fin)) break;
				else continue;
			}
			free_seq(seq_in);	/* No use */
			if(show_gene_names(fout,seq_nuc)){
				fprintf(stderr,"Error in showing gene names.\n");
#include	"exit1.h"
			}
		}

	}

	} /***** end GenBank and EMBL ********/

	if(fout!=stdout) fclose(fout);

	printf("\nTerminated.\n");

#include	"exit0.h"

}

/************************************************************
		End of file genlist.c
************************************************************/


