/***********************************************************************
Get clustal sequence (getclu) 
main file : getclu.c

 	Extract a part of sequence from a clustal sequence
	alignment file.

  Usage: getclu infile outfile startpos endpos 
  
  Default outfile name is "infile"+".out"
  
  Source files are:	getclu.c
  			gtsqfn.c
  			utility.c
  			Dmc.h
  			getseq2.h
  			cmptbl.h
  	
 	Naoki Sato   July 16, 1998.  
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.
	For Mac version, define the compiler as well.

	Bugs fixed. Added integrated mode. Nov. 23, 1998. 
	Bug fix. June 13. 1999.
	siseq_exit0 or 1 changed to exit0 or 1. Nov. 13, 1999.
	Use of parse_position in input. Dec. 30, 1999.
	Inclusion of Japanese messages. March 3, 2000. 
	Expanded to include phylip format. Feb. 1, 2001.
	Use of last in input. Dec. 10, 2001.
	Options gap and inf. March 7, 2004.
	Added nexus output. May 22, 2005.
	Added artificial sequence reversal. Feb. 4, 2008.
	Option match. April 13, 2008.
	Added conversion to MEGA format. Feb. 17, 2010.
	Added stat option. May 27, 2010.
	Added ph40 option. Aug. 11, 2010.
	Added correct option. Sept. 26, 2010.
	Added mass correct option. Oct. 26, 2010.

		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"

#ifndef INTEGRATED
#ifdef MAC
extern int ccommand(char ***);
#endif
#endif

/* Prototype declaration */
static void print_usage(void);
#ifdef INTEGRATED
int getclu(int c, char (*p)[S_2]);
#endif
void help_getclu(void);

#include "libSeq.h"

/* Global variables */
extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];

/***************************************************************/
static void print_usage(void)
/***************************************************************/
{
	help_getclu();
	return;
}
/***************************************************************/

/***************************************************************/
void help_getclu(void)
/***************************************************************/
{
#ifdef JAPANESE
#ifdef UNIX
#include "EUC/getcluE.h"
#else
#ifdef MAC
#include "Mac/SJIS/getcluS.h"
#else
#include "Dos/SJIS/getcluS.h"
#endif
#endif
#else
        printf("******************** GETCLU HELP *******************************\n");
        printf("Usage: getclu infile outfile startpos endpos\n");
        printf("       This program extracts a part of \n");
        printf("       clustal sequence alignment file.\n");
        printf("  + or - can be used in startpos and endpos.\n");
        printf("       last can be used as endpos.\n");
        printf("\n");
        printf("File conversion: getclu infile outfile outputformat\n");
        printf("  outputformat can be ph, cl, nex, or meg.\n");
        printf("  ph, Phylip format. cl, Clustal format, nex, Nexus format.\n");
        printf("  meg, MEGA format.\n");
        printf("  phi, Phylip with I. ph20/40, Phylip with 20/40 chars sequence name.\n");
        printf("\n");
        printf("Sequence reversal: getclu infile outfile rev\n");
        printf("  This is a simply formal reversal of 5' and 3'\n");
        printf("\n");
        printf("Complementary sequence: getclu infile outfile comp\n");
        printf("  This is a realistic complementary alignment\n");
        printf("\n");
        printf("Sorting with keeping the species name: getclu infile outfile sort\n");
        printf("Sorting with keeping the whole name: getclu infile outfile sort_name\n");
        printf("\n");
        printf("Printing columns: getclu infile outfile site_dump\n");
        printf("Statistics: getclu infile outfile stat\n");
        printf("Statistics: getclu infile outfile stat=<threshold>\n");
		printf("       A table of alignment statistics is printed.\n");
		printf("       If threshold is given, outliers are printed using the threshold.\n");
        printf("\n");
        printf("Correct alignment: getclu infile outfile correct=<threshold> organism nuc_file offset\n");
		printf("       This option is valid only for Gclust alignments.\n");
		printf("       threshold is given as in the stat option.\n");
		printf("       offset is the length of nucleic acids before start codon.\n");
        printf("\n");
        printf("Mass correct alignment: getclu infile outfile mass_cor=<threshold> nuc_file offset\n");
		printf("       In this option, infile is a list of alignment files to be processed.\n");
		printf("       In the list file, > indicates start of processing, and # indicates comment\n");
        printf("\n");
        printf("Matching: getclu infile outfile match listfile\n");
        printf("		match1 is used to sort genes according to the list\n\n");
        printf("		match2 is used to make a list of genes per organism\n\n");
        printf("Site selection: getclu infile outfile option level/range\n");
        printf("  option can be gap or inf.\n");
        printf("  gap, sites containing gaps at a level higher than the\n");
        printf("       given level are removed.\n");
        printf("  gap1, removed sites are marked with bars.\n");
        printf("  gap2, also outputs position of conserved sites.\n");
        printf("  inf, sites containing information content at a level between\n");
        printf("       the given range in fraction of maximum value are selected.\n");
        printf("       range should be min-max without space around hyphen.\n");
        printf("       Information content varies depending on data type and\n");
        printf("       composition. Range should therefore be in relative value.\n");
        printf("****************************************************************\n");
#endif
	return;
}
/***************************************************************/


/***************************************************************
****************** Main program *******************************/
#ifdef INTEGRATED
int getclu(int cc, char (*p)[S_2])

#else
#ifdef MAC
main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif

#endif
/***************************************************************/

{
	char fname1[100]="infile";
	char fname2[100]="outfile";
	int c=0;
	register long int k;
	int lastflag=0;		/* use of last-number in startpos */

#include        "filenam.h"     /* definition of char *str1 and char *str2 */

	char str4[MINMEMSIZE];
	char str5[MINMEMSIZE];
	char out_flag[MINMEMSIZE]="";
	long startpos=0L;
	long endpos=0L;
	long i=0L;
	enum data_type dtype1=unknown;
	Seqfile **seq_in;
	Seqfile **seq_ext;

	double gap_level=0.0;
	double inf_min,inf_max;
	double thr;	/* threshold for stat */
	int option;		/* for RemoveGaps */
	char listfile[100]="listfile";

	char nuc_file[100]="nuc_file";	/* correct mode */
	Seqfile **seq_nuc;
	char org_name[100]="org_name";	
#if 1
	long num_seq_nuc;
	long num_seq_bak;
	enum data_type dtype2=unknown;
#endif
	long offset = 100L;

#include	"parsarg.h"
	
/*********************** File names ******************/

	switch(cc)
	{
		case 1:	
			strcpy(fname1,"infile");
			if(filename_input(fname1,str1)!=1){
#include	"exit1.h"
			}
			strcpy(fname2,fname1);
			strcat(fname2,".out");
			if(filename_input(fname2,str2)!=1){
#include	"exit1.h"
			}
			printf("File conversion, extraction, gap removal, selection by inf content, sequence reversal, or complementary?\n");
			printf("Type c, e, g, i, r, or p.\n");
			scanf("%s",str4);
			if(str4[0]=='c' || str4[0]=='C') {
				printf("Choose output format. ph for Phylip, cl for clustal.\n");
				scanf("%s",str4);
				strcpy(out_flag,str4);	
			} else if(str4[0]=='e' || str4[0]=='E') {
				printf("Type start position and end position\n");
				printf("Use a space between the values.\n");
				scanf("%s %s",str4,str5);
				lastflag=catch_word(str4,"last");
				startpos=parse_position(str4);
				endpos=parse_position(str5);
			} else if(str4[0]=='g' || str4[0]=='G') {
				printf("Enter maximum gap level to be allowed. 0.0 - 1.0\n");
				scanf("%lf",&gap_level);
			} else if(str4[0]=='i' || str4[0]=='I') {
				printf("Enter range of relative information content, such as 0.2-0.5.\n");
				scanf("%lf-%lf",&inf_min,&inf_max);
				printf("inf_min = %f, inf_max = %f\n",inf_min,inf_max);
			} else if(str4[0]=='r' || str4[0]=='R') {
				printf("Sequence reversal.\n");
				strcpy(out_flag,"rev");
			} else if(str4[0]=='p' || str4[0]=='P') {
				printf("Complementary sequence.\n");
				strcpy(out_flag,"comp");
			} else {
#include	"exit1.h"
			}
			break;
		case 4:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			sscanf(p[3],"%s",out_flag);
			break;
	
		case 5:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			if(!strncmp(p[3],"gap",3)){
				strcpy(out_flag,"gap");
				if(*(p[3]+3) == '1') option = 1;
				else if(*(p[3]+3) == '2') option = 2;
				else if(*(p[3]+3) == '3') option = 3;
				else option = 0;
				sscanf(p[4],"%lf",&gap_level);
				printf("gap_level = %f.\n",gap_level);
			} else if(!strcmp(p[3],"inf")){
				strcpy(out_flag,p[3]);
				sscanf(p[4],"%lf-%lf",&inf_min,&inf_max);
				printf("inf_min = %f, inf_max = %f\n",inf_min,inf_max);
			} else if(!strncmp(p[3],"match",5)){
				strcpy(out_flag,p[3]);
				strcpy(listfile,p[4]);
			} else {
				lastflag=catch_word(str4,"last");
				startpos=parse_position(p[3]);
				endpos=parse_position(p[4]);
			}
			break;
		
		case 6:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			if(!strncmp(p[3],"correct",7)){
				sscanf(p[3],"%s",out_flag);
				sscanf(p[4],"%s",nuc_file);
				sscanf(p[5],"%s",org_name);
			}else if(!strncmp(p[3],"mass_cor",8)){
				sscanf(p[3],"%s",out_flag);
				sscanf(p[4],"%s",nuc_file);
				sscanf(p[5],"%ld",&offset);
			}
			break;

		case 7:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			if(!strncmp(p[3],"correct",7)){
				sscanf(p[3],"%s",out_flag);
				sscanf(p[4],"%s",org_name);
				sscanf(p[5],"%s",nuc_file);
				sscanf(p[6],"%ld",&offset);
			}
			break;
			
		default:
			print_usage();
#include	"exit1.h"
	}
		
/***********************Main part****************************/

		if(strcmp(out_flag,"") && strcmp(out_flag,"ph") && strcmp(out_flag,"cl") && strcmp(out_flag,"nex")\
		 && strcmp(out_flag,"phi") && strcmp(out_flag,"ph20") && strcmp(out_flag,"ph40") && strcmp(out_flag,"meg")\
		 && strcmp(out_flag,"sort") && strcmp(out_flag,"gap") && strcmp(out_flag,"inf") && strcmp(out_flag,"site_dump")\
		 && strcmp(out_flag,"rev") && strcmp(out_flag,"comp") && strncmp(out_flag,"match",5) && strncmp(out_flag,"stat",4)\
		 && strncmp(out_flag,"correct",7) && strncmp(out_flag,"mass_cor",8) && strncmp(out_flag,"sort_name",9)){
			fprintf(stderr,"Use correct option.\n");
#include 	"exit1.h"
		}
	
		printf("Reading sequence...\n");

#ifdef MAC
#ifdef CODEWARRIOR
		SIOUXSetTitle("\pgetclu");
#endif
#endif
		if(strncmp(out_flag,"mass_cor",8)){
			c=check_filetype(fname1);

			if(c!=CLUSTAL_SEQ && c!=PHYLIP_SEQ) {
				fprintf(stderr,"This is not a CLUSTAL or PHYLIP file.\n");
#include 	"exit1.h"
			}
		
			if(c==CLUSTAL_SEQ){
				if((seq_in=read_clustal(fname1,dtype1))==NULL){
					fprintf(stderr,"Error in reading file: %s\n",fname1);
#include 	"exit1.h"
				}
			} else {
				if((seq_in=read_phylip(fname1,dtype1))==NULL){
					fprintf(stderr,"Error in reading file: %s\n",fname1);
#include 	"exit1.h"
				}
			}
		}

		if(strcmp(out_flag,"")){
			if(!strcmp(out_flag,"cl")){
				if(write_clustal(fname2,seq_in)!=0){
					fprintf(stderr,"Error in writing sequence.\n");
#include 	"exit1.h"
				}
			} else if(!strcmp(out_flag,"ph20")){
				if(write_phylip(fname2,seq_in,20,0)!=0){
					fprintf(stderr,"Error in writing sequence.\n");
#include 	"exit1.h"
				}
			} else if(!strcmp(out_flag,"ph40")){
				if(write_phylip(fname2,seq_in,40,0)!=0){
					fprintf(stderr,"Error in writing sequence.\n");
#include 	"exit1.h"
				}
			} else if(!strcmp(out_flag,"phi")){
				if(write_phylip(fname2,seq_in,10,1)!=0){
					fprintf(stderr,"Error in writing sequence.\n");
#include 	"exit1.h"
				}
			} else if(!strcmp(out_flag,"ph")){
				if(write_phylip(fname2,seq_in,10,0)!=0){
					fprintf(stderr,"Error in writing sequence.\n");
#include 	"exit1.h"
				}
			} else if(!strcmp(out_flag,"nex")){
				if(write_nexus(fname2,seq_in)!=0){
					fprintf(stderr,"Error in writing sequence.\n");
#include 	"exit1.h"
				}
			} else if(!strcmp(out_flag,"meg")){
				if(write_mega(fname2,fname1,seq_in)!=0){
					fprintf(stderr,"Error in writing sequence.\n");
#include 	"exit1.h"
				}
			} else {
				if(!strcmp(out_flag,"sort")){
					for(i=0;i<num_seq;i++){
						subst(seq_in[i]->ID,'_','\0');
					}
					sort_seq(seq_in);
				} else if(!strcmp(out_flag,"sort_name")){
					sort_seq(seq_in);
				} else if(!strcmp(out_flag,"stat")){
					if(alnStat(fname1,fname2,seq_in,1.0)){
						fprintf(stderr,"Error in calculating statistics.\n");
#include 	"exit1.h"
					}
					printf("\nTerminated.\n");
#include 	"exit0.h"
				} else if(!strncmp(out_flag,"stat=",5)){
					sscanf(out_flag+5,"%lf",&thr);
					if(alnStat(fname1,fname2,seq_in,thr)){
						fprintf(stderr,"Error in calculating statistics.\n");
#include 	"exit1.h"
					}
					printf("\nTerminated.\n");
#include 	"exit0.h"
				} else if(!strncmp(out_flag,"correct=",8)){
					sscanf(out_flag+8,"%lf",&thr);
#if 1	
					fprintf(stdout,"Reading nucleic acid sequences...\n");
					num_seq_bak=num_seq;
					if((seq_nuc=Mread_fasta(nuc_file,dtype2))==NULL){
						fprintf(stderr,"Error in reading file: %s\n",fname2);
#include        "exit1.h"
					}
					num_seq_nuc=num_seq;
					dtype2=check_data_type(seq_nuc[0]);
					if(dtype2!=DNA){
						fprintf(stderr,"These are not DNA sequences.\n");
#include        "exit1.h"
					}
					num_seq = num_seq_bak;
#endif
					if(alnCorrect(1,fname1,fname2,seq_in,thr,nuc_file,seq_nuc,org_name,offset)){
						fprintf(stderr,"Error in correct option.\n");
#include 	"exit1.h"
					}
					printf("\nTerminated.\n");
#include 	"exit0.h"
				} else if(!strncmp(out_flag,"mass_cor=",9)){
					sscanf(out_flag+9,"%lf",&thr);
					if(massAlnCorrect(fname1,fname2,thr,nuc_file,offset)){
						fprintf(stderr,"Error in correct option.\n");
#include 	"exit1.h"
					}
					printf("\nTerminated.\n");
#include 	"exit0.h"
				} else if(!strcmp(out_flag,"gap")){
					if(RemoveGaps(seq_in,gap_level,option)){
						fprintf(stderr,"Error in removing gaps.\n");
#include 	"exit1.h"
					}
				} else if(!strcmp(out_flag,"inf")){
					if(SelectSitesByInf(seq_in,inf_min,inf_max,dtype1)){
						fprintf(stderr,"Error in selecting sites by information content.\n");
#include 	"exit1.h"
					}
				} else if(!strcmp(out_flag,"site_dump")){
					if(DumpSites(fname2,seq_in)){
						fprintf(stderr,"Error in dumping site data.\n");
#include 	"exit1.h"
					}
					printf("\nTerminated.\n");
#include 	"exit0.h"
				} else if(!strcmp(out_flag,"rev")){
					if(to_revM(seq_in)){
						fprintf(stderr,"Error in sequence reversal.\n");
#include 	"exit1.h"
					}
				} else if(!strcmp(out_flag,"comp")){
					if(comp_seqM(seq_in)){
						fprintf(stderr,"Error in sequence reversal.\n");
#include 	"exit1.h"
					}
				} else if(!strcmp(out_flag,"match")){
					if(match_seq(seq_in,listfile,0)){
						fprintf(stderr,"Error in sequence matching.\n");
#include 	"exit1.h"
					}
				} else if(!strcmp(out_flag,"match1")){
					if(match_seq(seq_in,listfile,1)){
						fprintf(stderr,"Error in sequence matching.\n");
#include 	"exit1.h"
					}
				} else if(!strcmp(out_flag,"match2")){
					if(match_seq(seq_in,listfile,2)){
						fprintf(stderr,"Error in sequence matching.\n");
#include 	"exit1.h"
					}
					printf("\nTerminated.\n");
#include 	"exit0.h"
				}

				if(c==CLUSTAL_SEQ){
					if(write_clustal(fname2,seq_in)!=0){
						fprintf(stderr,"Error in writing sequence.\n");
#include 	"exit1.h"
					}
				}else{
					if(write_phylip(fname2,seq_in,10,0)!=0){
						fprintf(stderr,"Error in writing sequence.\n");
#include 	"exit1.h"
					}
				}
			}

			for(k=0;k<num_seq;k++){
				free_seq(seq_in[k]);
			}

			printf("\nTerminated.\n");
#include 	"exit0.h"

		}

		if((seq_ext=(Seqfile**)calloc(num_seq,sizeof(Seqfile)))==0){
			fprintf(stderr,"Out of memory\n");
#include 	"exit1.h"
		}
		for(k=0;k<num_seq;k++){
	                if((seq_ext[k]=(Seqfile*)calloc(1,sizeof(Seqfile)))==0){
       	 	                fprintf(stderr,"Out of memory\n");
#include 	"exit1.h"
        		}
		}

		if(endpos==0L) endpos=seq_in[0]->seqlen;
		if(lastflag && startpos<0L) startpos+=seq_in[0]->seqlen;

		for(k=0;k<num_seq;k++){
			seq_ext[k]=extract_seq3(seq_in[k],startpos,endpos);
		}
		
		if(c==CLUSTAL_SEQ){
			if(write_clustal(fname2,seq_ext)!=0){
				fprintf(stderr,"Error in writing sequence.\n");
#include 	"exit1.h"
			}
		} else {
			if(write_phylip(fname2,seq_ext,10,0)!=0){
				fprintf(stderr,"Error in writing sequence.\n");
#include 	"exit1.h"
			}
		}

		
		
		for(k=0;k<num_seq;k++){
			free_seq(seq_in[k]);
			free_seq(seq_ext[k]);
		}
	
		printf("\nTerminated.\n");

#include 	"exit0.h"

}

/************************************************************
		End of file getclu.c
************************************************************/

