/***********************************************************************
Extracts database entries with keyword matching (getent).
main file : getent.c

  Usage: getent dbfile outfile AC=.... ID=... OS=... DE=...
  
  Default outfile name is "infile"+".out"
  
 	Naoki Sato   May 30, 1999.
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.
	For Mac version, define the compiler as well.

	Added parsing of quoted string to allow spaces 
	within a key word string. Aug. 25, 1999.
	Added Swiss Prot. Sept. 21, 1999.
	Bug fix. Oct. 3, 1999.
	siseq_exit0 or 1 changed to exit0 or 1. Nov. 13, 1999.
	Inclusion of Japanese messages. March 3, 2000.
	Adapted to multi-FASTA file. April 6, 2000.
	Bug fix. May 4, 2000.
	Multiple ACs allowed. Oct. 14, 2000.
	Multiple IDs allowed. Mar. 2, 2001.
	Bug fix. Mar. 11, 2001.
	Number option added. Jan. 10, 2003.
	Unique sequence mode added. May 3, 2003.
	Split option added. Feb. 2, 2007.
	File option added. Feb. 3, 2007.
	List option added. April, 28, 2007.
	
		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"

#ifndef INTEGRATED
#ifdef MAC
extern int ccommand(char ***);
#endif
#endif

/* Prototype declaration */
static void print_usage(void);

#ifdef INTEGRATED
int getent(int c, char (*p)[S_2]);
#endif
void help_getent(void);

/* external functions */

#include "libSeq.h"

/* Global variables */
extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];

/***************************************************************/
static void print_usage(void)
/***************************************************************/
{
	help_getent();
	return;
}
/***************************************************************/

/***************************************************************/
void help_getent(void)
/***************************************************************/
{
#ifdef JAPANESE
#ifdef UNIX
#include "EUC/getentE.h"
#else
#ifdef MAC
#include "Mac/SJIS/getentS.h"
#else
#include "Dos/SJIS/getentS.h"
#endif
#endif
#else
	printf("********************* GETENT HELP *****************************\n");
	printf("Usage: getent dbfile outfile ID=... AC=... OS=... DE=...\n");
	printf("       dbfile: database file. Upto 10 ID, accession, organism,\n");
	printf("       or definition are used as keys.\n");
	printf("Wild card (#, not *) can be used at the end of ID and AC.\n");
	printf("       For a multi-FASTA file, only ID can be used.\n\n");
	printf("Usage: getent dbfile outfile uniID=...,...\n");
	printf("       Retrieves only the first found entry.\n\n");
	printf("Usage: getent dbfile outfile NO=...,...\n");
	printf("       Retrieves xxx-th entry to yyy-th entry.\n\n");
	printf("Usage: getent dbfile outfile File=...\n");
	printf("       Retrieves sequences listed in the file.\n\n");
	printf("Usage: getent dbfile outfile File=... Limit=xx\n");
	printf("       Retrieves sequences listed in the file, but \n");
	printf("       name length is limited to xx.\n\n");
	printf("Usage: getent dbfile outfile List=...\n");
	printf("       Retrieves sequences listed in the file, with \n");
	printf("       cutting and reversing. List should describe\n");
	printf("       seq_name ForR start end each delimited with a tab.\n\n");
	printf("Usage: getent dbfile outdir Group=...\n");
	printf("       Retrieves sequences listed in the file and save\n");
	printf("       them as Group files in the directory outdir.\n\n");
	printf("Usage: getent dbfile outfile Split=..\n");
	printf("       Splits the sequence file into designated number of files .\n\n");
	printf("***************************************************************\n");
#endif
	return;
}
/***************************************************************/


/***************************************************************
****************** Main program *******************************/
#ifdef INTEGRATED
int getent(int cc, char (*p)[S_2])

#else
#ifdef MAC
main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif

#endif
/***************************************************************/

{
	FILE *fin;
	FILE *fout;
	FILE *flist;	/* file for ID list */
	Seqfile **mseq_in;	/* for file mode and split mode */
	char fname1[100]="dbfile";
	char fname2[100]="outfile";
	char fname_out[100];	/* for split mode */
	int c=0;
	char str[4*MAXLINE]="";
	char *pp;
	int i=0;
	int j=0;
	int k=0;
	int m=0;
	int n=0;
	int jmax=0;
	int kmax=0;
	int mmax=0;
	int nmax=0;
	int j1=0;
	int k1=0;
	int m1=0;
	int n1=0;
	int slen=0;	/* string length of IDstr[j] */
	int strloc=0;	/* location of IDstring */
	int keta;
	int counter;
	int size;
	char suffix[10];
	char *pstr;	/* pointer to the ID within str. */
	char str1[2*MAXLINE]="";	/* for file mode */
	char str2[2*MAXLINE]="";	/* for file mode */
	char command[MAXLINE]="";	/* for file mode */
	char current_file[MAXLINE]="";	/* for file mode */
	char str3[4*MAXLINE]="";
	char str4[4*MAXLINE]="";
	char IDstr[10][100];
	Boolean IDflag[10]={0,0,0,0,0,0,0,0,0,0};
	char ID[100]="";
	char ACstr[10][AC_LENGTH];
	char OSstr[10][AC_LENGTH];
	char DEstr[10][AC_LENGTH];
	char filename[100];	/* name of file in the file mode. Feb. 2007 */
	int splitNum;	/* number of files to be generated in the split mode. Feb. 2007 */
	long fileloc;
	long fileloc_old;
	long fileloc0=0L;
	Boolean wflag=FALSE;
	Boolean ACflag=FALSE;
	Boolean FoundID=FALSE;
	Boolean flag1=FALSE;
	Boolean flag2=FALSE;
	enum data_type dtype1=unknown;

	char NOstr[100];
	char NOstr2[100];
	long start=0L;	/* start and end numbers of entries to be retrieved. 
				end may be last.*/
	long end=0L;
	long seq_count=0;
	Boolean num_mode=FALSE;
	Boolean unique_mode=FALSE;
	Boolean file_mode=FALSE;
	Boolean list_mode=FALSE;
	Boolean list_limit_mode=FALSE;
	int name_length=100;
	Boolean group_mode=FALSE;
	Boolean split_mode=FALSE;
	char FR[10];	/* flag for forward/reverse in List mode */
	Seqfile *temp_seq;
	Boolean long_line=FALSE;
 
#include	"parsarg.h"
	
/*********************** File names ******************/
/* Always complete input is needed. */

	if(cc > 3){
		if(cc > S_2){
			fprintf(stderr,"Too many arguments...\n");
#include "exit1.h"
		}

		strcpy(fname1,p[1]);
		strcpy(fname2,p[2]);
		for (i=3;i<cc && j<10 && k<10;i++){
			if(linetype(p[i],"ID=")){
//				strcpy(IDstr[j++],(p[i]+3*sizeof(char)));
				strcpy(IDstr[j++],(p[i]+3));
				for(j1=0;j1<j-1;j1++){
					if(!strcmp(IDstr[j-1],IDstr[j1])){
						j--;
						break;
					}
				} 
			} else if(linetype(p[i],"uniID=")){
//				strcpy(IDstr[j++],(p[i]+6*sizeof(char)));
				strcpy(IDstr[j++],p[i]+6);
				unique_mode=TRUE;
				printf("Unique mode: only a single sequence is retrieved for each ID.\n");
				for(j1=0;j1<j-1;j1++){
					if(!strcmp(IDstr[j-1],IDstr[j1])){
						j--;
						break;
					}
				} 
			} else if(linetype(p[i],"File=")){
//				strcpy(filename,(p[i]+5*sizeof(char)));
				strcpy(filename,p[i]+5);
				file_mode=TRUE;
//				break;
			} else if(file_mode && linetype(p[i],"Limit=")){
//				sscanf((p[i]+6*sizeof(char)),"%d",&name_length);
				sscanf(p[i]+6,"%d",&name_length);
				list_limit_mode=TRUE;
				break;
			} else if(linetype(p[i],"List=")){
//				strcpy(filename,(p[i]+5*sizeof(char)));
				strcpy(filename,p[i]+5);
				list_mode=TRUE;
				break;
			} else if(linetype(p[i],"Group=")){
//				strcpy(filename,(p[i]+6*sizeof(char)));
				strcpy(filename,p[i]+6);
				file_mode=TRUE;
				group_mode=TRUE;
				break;
			} else if(linetype(p[i],"Split=")){
//				strcpy(filename,(p[i]+6*sizeof(char)));
				strcpy(filename,p[i]+6);
				if(num_check(filename)){
					fprintf(stderr,"In the split mode, use number after Split=.\n");
					exit(1);
				} else {
					splitNum=atoi(filename);
					if(splitNum > 1000){
						fprintf(stderr,"The split number must be less than or equal to 1000.\n");
						exit(1);
					}
					split_mode=TRUE;
				}
				break;
			} else if(linetype(p[i],"AC=")){
//				strcpy(ACstr[k++],(p[i]+3*sizeof(char)));
				strcpy(ACstr[k++],p[i]+3);
				for(k1=0;k1<k-1;k1++){
					if(!strcmp(ACstr[k-1],ACstr[k1])){
						k--;
						break;
					}
				}
			}
			else if(linetype(p[i],"OS=")){
//				strcpy(OSstr[m++],(p[i]+3*sizeof(char)));
				strcpy(OSstr[m++],p[i]+3);
				if(OSstr[m-1][0]==34 && OSstr[m-1][strlen(OSstr[m-1])-1]==34){
					erasequote(OSstr[m-1]);
				}
				for(m1=0;m1<m-1;m1++){
					if(!strcmp(OSstr[m-1],OSstr[m1])){
						m--;
						break;
					}
				}
			}
			else if(linetype(p[i],"DE=")){
//				strcpy(DEstr[n++],(p[i]+3*sizeof(char)));
				strcpy(DEstr[n++],p[i]+3);
				if(DEstr[n-1][0]==34 && DEstr[n-1][strlen(DEstr[n-1])-1]==34){
					erasequote(DEstr[n-1]);
				}
				for(n1=0;n1<n-1;n1++){
					if(!strcmp(DEstr[n-1],DEstr[n1])){
						n--;
						break;
					}
				}
			}
			else if(linetype(p[i],"NO=")){
//				strcpy(NOstr,(p[i]+3*sizeof(char)));
				strcpy(NOstr,p[i]+3);
				strcpy(NOstr2,str_shift(NOstr,','));
				if(strcmp(NOstr,"") && !num_check(NOstr)) start=atol(NOstr);
				else {
					fprintf(stderr,"Check the number.\n");
					print_usage();
#include	"exit1.h"
				}
				if(!strcmp(NOstr2,"last")) end = -1;
				else if(strcmp(NOstr2,"") && !num_check(NOstr2)) end=atol(NOstr2);
				else {
					fprintf(stderr,"Check the number.\n");
					print_usage();
#include	"exit1.h"
				}
				num_mode=TRUE;
				break;
			}
			else print_usage();
		}
		jmax=j;
		kmax=k;
		mmax=m;
		nmax=n;
		if( !(num_mode || split_mode || file_mode || list_mode) && j==0 && k==0 && m==0 && n==0){
			print_usage();
#include	"exit1.h"
		}
	}
	else{
		print_usage();
#include	"exit1.h"
	}

/*********************** Main part ****************************/

	c=check_filetype(fname1);
	if(c!=GENBANK_SEQ && c!=GBPROT_SEQ && c!=EMBL_SEQ && c!=SW_SEQ && c!=UNIPROT_SEQ){
		if(c==FASTA_SEQ){
			fprintf(stderr,"This is a FASTA file. Only ID is recognized.\n");
		}
		else{
			fprintf(stderr,"This is not a GenBank, EMBL, SwissProt, Uniprot or FASTA file.\n");
			exit(1);
#include	"exit1.h"
		}
	}
	printf("Reading sequence...\n");

#ifdef MAC
#ifdef CODEWARRIOR
	SIOUXSetTitle("\pgetent");
#endif
#endif

	if((fin=fopen(fname1,"r"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n\n",fname1);
#include	"exit1.h"
	}

	if(!split_mode && !group_mode){
		if(ftouch(fname2)){
			fprintf(stderr,"Cannot open output file: %s.\n\n",fname2);
#include	"exit1.h"
		}

		if((fout=fopen(fname2,"a+"))==NULL){
			fprintf(stderr,"Cannot open output file: %s.\n\n",fname2);
#include	"exit1.h"
		}
	}
	if(group_mode){
		sprintf(command,"mkdir %s",fname2);
		system(command);
		strcpy(current_file,"");
	}	 

	if(file_mode || list_mode){
		if((flist=fopen(filename,"r"))==NULL){
			fprintf(stderr,"Cannot open file: %s\n\n",filename);
#include	"exit1.h"
		}
	}

/********************** main loop **********************/


/* Feb. 2007 */
	if(file_mode || list_mode || split_mode){
		if(c!=FASTA_SEQ){
			fprintf(stderr,"Only FASTA file can be processed in this mode.\n");
#include "exit1.h"
		}

		if((mseq_in=Mread_fasta(fname1,dtype1))==NULL){
			fprintf(stderr,"Error in reading file: %s\n",fname1);
			exit(1);
		}


		if(file_mode || list_mode){
			long_line = 0;
			while(fgets(str1,MAXLINE+1,flist)!=NULL){
				if(long_line == 1){
					if(str1[strlen(str1)-1]=='\n'){
						long_line = 0;
					}
					continue;
				}
				if(str1[strlen(str1)-1]=='\n'){
					long_line = 0;
				} else {
					long_line = 1;
				}
				strcpy(str2,"");
				if(file_mode) sscanf(str1,"%s",str2);
				else if(list_mode) sscanf(str1,"%s\t%s\t%ld\t%ld",str2,FR,&start,&end);	
				if(!strcmp(str2,"")){
					continue;
				} else if(!strcmp(str2,"Group")){
					if(group_mode){
						if(strcmp(current_file,"")){
							if(fout != NULL) fclose(fout);
							sprintf(command,"mv %s %s ",current_file,fname2);
							system(command);
							strcpy(current_file,"");
						}
						sscanf(str1+6,"%s",str2);
						if(*(str2+strlen(str2)-1)==':') *(str2+strlen(str2)-1)='\0';
						strcpy(current_file,"Group");
						strcat(current_file,str2);
						if((fout=fopen(current_file,"a+"))==NULL){
							fprintf(stderr,"Cannot open output file: %s.\n\n",current_file);
#include	"exit1.h"
						}
						fprintf(stdout,"%s is being processed.\n",str1);
					} else {
						fprintf(fout,"%s\n",str1);
					}
					continue;
				}

				/* file name with only numericals is skipped */
				if(!num_check(str2)){
					fprintf(stderr,"\rSkipping sequence %s.",str2);
					continue;
				}

				/* name length limitation*/
				if(!list_limit_mode){
					name_length = 100;
				}
				/* file_mode and list_mode*/
				for(i=0;i<num_seq;i++){
					if(!strncmp(str2,mseq_in[i]->ID,name_length)){
						if(list_mode){
							temp_seq = copy_seq(mseq_in[i]);
							pick_seq(temp_seq,start,end);
							if(FR[0]=='R'){
								comp_seq4(temp_seq);
							}
							if(fwrite_fasta(fout,temp_seq,1,' ')){
								fprintf(stderr,"Unable to write sequence %s.\n",str2);
								exit(1);
							}
						}else{
							if(fwrite_fasta(fout,mseq_in[i],1,' ')){
								fprintf(stderr,"Unable to write sequence %s.\n",str2);
								exit(1);
							}
						}
						break;	/* only one matching sequence is printed */
					}
				}
				if(i==num_seq){
					fprintf(stderr,"Sequence %s was not found.\n",str2);
				}
			}
			if(strcmp(current_file,"")){
				if(fout != NULL) fclose(fout);
				sprintf(command,"mv %s %s ",current_file,fname2);
				system(command);
				strcpy(current_file,"");
			}
			fclose(flist);
//			if(fout != NULL) fclose(fout);

		} else if(split_mode){
			if(splitNum > num_seq/10){
				fprintf(stderr,"Number of sequences must be more then ten times larger than the split number.\n");
				exit(1);
			}
			size = num_seq / splitNum;
			if(splitNum > 100){
				keta = 3;
			} else keta = 2;
		
			
			if(size > 10000) size = (int)(size / 10000) * 10000;
			else if(size > 1000) size = (int)(size / 1000) * 1000;
			else if(size > 100) size = (int)(size / 100) * 100;

			fprintf(stderr,"File will be split into %d files.\n",(int)(num_seq/size + 1));
			fprintf(stderr,"Each file contains %d sequences, except the last one.\n",(int)size);

			counter = 0;
			for(i = 0;i < num_seq;i += size){
				j = i + size;
				if(j > num_seq) j = num_seq;
				if(keta==3) sprintf(suffix,".%03d",counter);
				else if(keta==2) sprintf(suffix,".%02d",counter);
				strcpy(fname_out,fname2);
				strcat(fname_out,suffix);
				if(Mwrite3(fname_out,mseq_in,(long)i,(long)j)){
					fprintf(stderr,"Error in writing sequence.\n");
					exit(1);
				}
				counter += 1;
			}


		}
		fprintf(stderr,"Done.\n");
#include "exit0.h"
	}
/* end Feb. 2007 */

	for(j=0;j<jmax;j++){
		if(tailtype(IDstr[j],"#")){
			slen=strlen(IDstr[j]);
			IDstr[j][slen-1]='\0';
			IDflag[j]=1;
		}
		else IDflag[j]=0;
	}

	fileloc=ftell(fin);
	while(fgets(str,MAXLINE+1,fin)!=NULL){
		fileloc_old=fileloc;
		fileloc=ftell(fin);
		strcpy(str3,"");
		strcpy(str4,"");
		FoundID=FALSE;
		flag1=FALSE;
		flag2=FALSE; 

		if(wflag && linetype(str,FASTA_INIT)){
			wflag=FALSE;
			strcpy(ID,"");
		}
		if(wflag) {
			fprintf(fout,"%s",str);
		}
		else{
			if(linetype(str,GB_INIT) || linetype(str,EMBL_INIT) || linetype(str,FASTA_INIT)){
				seq_count += 1L;
				ACflag=FALSE;
				if(num_mode){
					if( seq_count >= start && ( seq_count <= end || end == -1 )){
						wflag=TRUE;
						fprintf(fout,"%s",str);
						printf("Start copying database No. %ld: %s\n",seq_count,ID);
						fflush(stdout);
					}
					continue;
				} else {

					for(j=0;j<jmax;j++) {
						if(!strcmp(IDstr[j],"")) continue;
						if((pstr=strstr(str,IDstr[j]))!=NULL) {
							strloc=(int)((pstr-str)/sizeof(char));
							FoundID=TRUE;
							break;
						}
					}
				}
				fileloc0=fileloc_old;

				if(linetype(str,GB_INIT) || linetype(str,EMBL_INIT)){	
					sscanf(str,"%s %s",str3,str4);
					subst(str4,';',' ');
					sscanf(str4,"%s",ID);
				}

			if(FoundID){
				FoundID=FALSE;

				if(linetype(str,FASTA_INIT)){	
				/* For FASTA files, whole string is used to detect ID */
					strcpy(str3,str+strlen(FASTA_INIT));
					tailchop(str3);
					subst(str3,'|',' ');
					subst(str3,'(',' ');
					subst(str3,')',' ');
					strloc=strloc-strlen(FASTA_INIT);
					if(strloc==0) flag1=TRUE;
					else if(str3[strloc-1]==' ') flag1=TRUE;
					slen=strlen(IDstr[j])+strloc;
					if(IDflag[j]) flag2=TRUE;
					else if(slen==strlen(str3)) flag2=TRUE;
					else if(str3[slen]==' ') flag2=TRUE;
					slen=0;
					strcpy(str3,str);
					strcpy(ID,strtok(pstr," |;:,()"));	/* get ID string */	
					strcpy(str,str3);	
					if(flag1 && flag2){
						wflag=TRUE;
						fprintf(fout,"%s",str);
						printf("Start copying database: %s\n",ID);
						fflush(stdout);
						if(unique_mode) strcpy(IDstr[j],"");
					}
				}
				else{				/* For GenBank and EMBL, only the second word is used.*/
					if(IDflag[j]){
						if(linetype(ID,IDstr[j])){
							wflag=TRUE;
							fprintf(fout,"%s",str);
							printf("Start copying database: %s\n",ID);
							fflush(stdout);
						}
					}
					else{
						if(!strcmp(IDstr[j],ID)) {
							wflag=TRUE;
							fprintf(fout,"%s",str);
							printf("Start copying database: %s\n",ID);
							fflush(stdout);
						}
					}
				}



/*
				while(TRUE){
					pp=str_shift(str3,' ');
					strcpy(ID,str3);
					strcpy(str3,pp);

					for(j=0;j<jmax;j++){
						if(IDflag[j]){
							if(linetype(ID,IDstr[j])){
								wflag=TRUE;
								fprintf(fout,"%s",str);
								printf("Start copying database: %s\n",ID);
								fflush(stdout);
								break_flag=TRUE;
								break;
							}
						}
						else{
							if(!strcmp(IDstr[j],ID)) {
								wflag=TRUE;
								fprintf(fout,"%s",str);
								printf("Start copying database: %s\n",ID);
								fflush(stdout);
								break_flag=TRUE;
								break;
							}
						}
					}
					if(break_flag){
						break_flag=FALSE;
						break;
					}
					if(strlen(str3)<=1) break;
				}
*/

				if(wflag) continue;

			}


			}
			else if(linetype(str,GB_AC) || linetype(str,EMBL_AC) || (linetype(str,"           ") && ACflag)){
			    ACflag=TRUE;
			    if(linetype(str,GB_AC) || linetype(str,"            ")) pp=extract_string(str,13,strlen(str));
			    else pp=extract_string(str,6,strlen(str));
			    strcpy(str,pp);
			    free(pp);

			    while(strlen(str)>5){

				subst(str,';',' ');
				strcpy(str3,str_shift(str,' '));
				sscanf(str,"%s",str4);
				strcpy(str,str3);

				for(k=0;k<kmax;k++){
					if(tailtype(ACstr[k],"#")){
						pp=extract_string(ACstr[k],1,strlen(ACstr[k])-1);
						if(linetype(str4,pp)){
							wflag=TRUE;
							free(pp);
							fprintf(fout,"%s",str);
							printf("Start copying database: %s\n",ID);
							fflush(stdout);
							if(fseek(fin,fileloc0,SEEK_SET)){
								fprintf(stderr,"Error in accessing file.\n");
#include		"exit1.h"
							}
							break;
						}
						else free(pp);
					}
					else if(!strcmp(ACstr[k],getAC(str4))) {
						wflag=TRUE;
						printf("Start copying database: %s\n",ID);
						fflush(stdout);
						if(fseek(fin,fileloc0,SEEK_SET)){
							fprintf(stderr,"Error in accessing file.\n");
#include		"exit1.h"
						}
						break;
					}
				}
				if(wflag) break;

			    }
			    if(wflag) continue;

			}
			else if(linetype(str,GB_OS) || linetype(str,EMBL_OS)){
				ACflag=FALSE;
				strcpy(str3,str_shift(str,' '));	
				for(m=0;m<mmax;m++){
					if(strstr(str3,OSstr[m])!=NULL) {
						wflag=TRUE;
						printf("Start copying database: %s\n",ID);
						fflush(stdout);
						if(fseek(fin,fileloc0,SEEK_SET)){
							fprintf(stderr,"Error in accessing file.\n");
#include		"exit1.h"
						}
						break;
					}
				}
				if(wflag) continue;
			}
			else if(linetype(str,GB_DE) || linetype(str,EMBL_DE)){
				ACflag=FALSE;
				strcpy(str3,str_shift(str,' '));	
				for(n=0;n<nmax;n++){
					if(strstr(str3,DEstr[n])!=NULL) {
						wflag=TRUE;
						printf("Start copying database: %s\n",ID);
						fflush(stdout);
						if(fseek(fin,fileloc0,SEEK_SET)){
							fprintf(stderr,"Error in accessing file.\n");
#include		"exit1.h"
						}
						break;
					}
				}
				if(wflag) continue;
			}
			else ACflag=FALSE;
		}
		if(linetype(str,GB_SEQ_END) || linetype(str,EMBL_SEQ_END)){
			ACflag=FALSE;
			wflag=FALSE;
			strcpy(ID,"");
		}
	}

	fclose(fin);
	fclose(fout);

	printf("\nTerminated.\n");

#include	"exit0.h"
}

/************************************************************
		End of file getent.c
************************************************************/

