/***********************************************************************
Get sequence 2 (getseq2) 
main file : getseq2.c

 	Extract a part of sequence from a database entry.
 	At first stage, input file formats are limited to
 	raw, FASTA, GenBank, and EMBL.

  Usage: getseq2 infile outfile startpos endpos n[ormal]/c[omp] datatype
  
  Default outfile name is "infile"+".out"
  
  Source files are:	getseq2.c
			gtsqfn.c
  			utility.c
  			Dmc.h
  			getseq2.h
  			cmptbl.h
  	
 	Naoki Sato   Jan 1, 1998.
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.
	For Mac version, define the compiler as well.

	Bugs fixed. Added integrated mode. Nov. 23, 1998.
	Header is shortened. Feb. 13, 1999.
	Support for circular. May 9, 1999.
	Use of parsarg.h. May 31, 1999.
	Bug fix. June 21, 1999.
	siseq_exit0 or 1 changed to exit0 or 1. Nov. 13, 1999.
	Inclusion of Japanese messages. March 3, 2000.
	Use of last-number in input. Dec. 10, 2001.
	
		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"

#ifndef INTEGRATED
#ifdef MAC
extern int ccommand(char ***);
#endif
#endif

/* Prototype declaration */
static void print_usage(void);
#ifdef INTEGRATED
int getseq2(int c, char (*p)[S_2]);
#endif
void help_getseq2(void);

/* External functions. */

#include "libSeq.h"

/* Global variables */
extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];
extern Boolean circular;


/***************************************************************/
static void print_usage(void)
/***************************************************************/
{
	help_getseq2();
	return;
}
/***************************************************************/

/***************************************************************/
void help_getseq2(void)
/***************************************************************/
{
#ifdef JAPANESE
#ifdef UNIX
#include "EUC/getseqE.h"
#else
#ifdef MAC
#include "Mac/SJIS/getseqS.h"
#else
#include "Dos/SJIS/getseqS.h"
#endif
#endif
#else
        printf("*************** GETSEQ2 HELP ***************************\n");
        printf("Usage: getseq2 infile outfile startpos endpos n/c.\n\n");
        printf("       This program extracts a part of sequence\n");
        printf("       from a database entry\n");
        printf("Input file formats should be: raw, Fasta, GenBank, EMBL,\n");
	printf("       SwissProt, PIR or PRF. \n");
	printf("  A word \"circular\" in the first line is recognized.\n");
	printf("  + or - can be used in startpos and endpos.\n");
	printf("       last can be used as endpos.\n");
        printf("********************************************************\n");
#endif
	return;
}
/***************************************************************/


/***************************************************************
****************** Main program *******************************/
#ifdef INTEGRATED
int getseq2(int cc, char (*p)[S_2])

#else
#ifdef MAC
main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif

#endif
/***************************************************************/

{
	char fname1[100]="infile";
	char fname2[100]="outfile";
	int c=0;
	Boolean circular_bak=FALSE;
	char str[10];
	int lastflag=0;		/* use of last in input of positions */

#include        "filenam.h"     /* definition of char *str1 and char *str2 */

	char str3[MINMEMSIZE];
	char str4[MINMEMSIZE];
	char str5[MINMEMSIZE];
	long startpos;
	long endpos;
	long seqlen;
	char nc[10];
	enum data_type dtype1=unknown;
   /* sequence variables */
	Seqfile *seq_in;
	Seqfile *seq_ext,*seq_ext2;

#include	"parsarg.h"

/*********************** File names ******************/

	switch(cc){

		case 1:	
			strcpy(fname1,"infile");
			if(filename_input(fname1,str1)!=1){
#include	"exit1.h"	
			}
			strcpy(fname2,fname1);
			strcat(fname2,".out");
			if(filename_input(fname2,str2)!=1){
#include	"exit1.h"	
			}
			printf("Type start position and end position\n");
			printf("Use a space between the values.\n");
			scanf("%s %s",str4,str5);
			lastflag=catch_word(str4,"last");
			startpos=parse_position(str4);
			endpos=parse_position(str5);
			printf("Normal strand or complementary strand?\n");
			printf("Type n[ormal] or c[omp].\n");
			scanf("%s",str);
			if((strcmp(str,"c")==0) || (strcmp(str,"comp")==0)) strcpy(nc,"c");
			else strcpy(nc,"n");
			break;
			
		case 5:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			lastflag=catch_word(p[3],"last");
			startpos=parse_position(p[3]);
			endpos=parse_position(p[4]);
			printf("Normal strand or complementary strand?");
			printf("Type n[ormal] or c[omp].");
			scanf("%s",str);
			if((strcmp(str,"c")==0) || (strcmp(str,"comp")==0)) strcpy(nc,"c");
			else strcpy(nc,"n");
			break;
			
		case 6:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			lastflag=catch_word(p[3],"last");
			startpos=parse_position(p[3]);
			endpos=parse_position(p[4]);
			strcpy(nc, p[5]);
			break;
			
		default:
			print_usage();
#include	"exit1.h"
	}

/***********************Main part****************************/

		c=check_filetype(fname1);
		if(c==99){
			fprintf(stderr,"Unknown file format.\n");
			exit(1);
		}
		printf("Reading sequence...\n");

#ifdef MAC
#ifdef CODEWARRIOR
		SIOUXSetTitle("\pgetseq2");
#endif
#endif

		if((seq_in=read_seq(fname1,c,dtype1))==NULL){
			fprintf(stderr,"Error in reading file: %s\n",fname1);
			exit(1);
		}
/*		print_seq(seq_in);
*/		
		if((seq_ext2=(Seqfile*)calloc(1,sizeof(Seqfile)))==NULL){
			fprintf(stderr,"Out of memory.\n");
			exit(1);
		}
		c=check_data_type(seq_in);
		circular_bak=circular;
		if(!circular) circular=seq_in->circular;
		seqlen=seq_in->seqlen;
		if(endpos==0L) endpos=seqlen;
		if(lastflag && startpos<0L) startpos+=seqlen;

		printf("Data type is %s\n",data_type_name[c]);

		if(strlen(seq_in->header)>50){	
			seq_in->header=extract_string(seq_in->header,1,50);
		}
	
		if(clear_gaps(seq_in)){
			fprintf(stderr,"Error in clearing gaps in file: %s\n",seq_in->ID);
			exit(1);
		}
		printf("Gaps were cleared.\n");

		if(circular){
			printf("\nThis is a circular genome: %s.\n",seq_in->ID);
			if(endpos > seqlen) endpos -= seqlen;
			if(startpos <= 0) startpos += seqlen;
		}
		if(circular && startpos > endpos){
			if((seq_ext=extract_seq3(seq_in,startpos,seqlen))==NULL){
				fprintf(stderr,"Error in extracting sequence.\n");
				exit(1);
			}
			if((seq_ext2=extract_seq3(seq_in,1,endpos))==NULL){
				fprintf(stderr,"Error in extracting sequence.\n");
				exit(1);
			}
			if((seq_ext->seq=(char*)realloc(seq_ext->seq,(strlen(seq_ext->seq)+strlen(seq_ext2->seq)+MINMEMSIZE)*sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory.\n");
				exit(1);
			}
			strcat(seq_ext->seq,seq_ext2->seq);
			seq_ext->seqlen=strlen(seq_ext->seq);
			sprintf(str3," Sequence added until %ld. ",endpos);
			strcat(seq_ext->header,str3);
		}

		else{
			if(startpos<=0 || endpos <= 0 || startpos > endpos){
				fprintf(stderr,"Strange positions. start=%ld, end=%ld.\n",startpos,endpos);
				exit(1);
			}
			if((seq_ext=extract_seq3(seq_in,startpos,endpos))==NULL){
				fprintf(stderr,"Error in extracting sequence.\n");
				exit(1);
			}
		}
		printf("Sequence extracted.\n");
		
/*		print_seq(seq_ext);
*/		
/*		printf("This is the value of nc:%s\n",nc);
*/
		if((strcmp(nc,"c"))==0){
			if((seq_ext->dtype==DNA) || (seq_ext->dtype==RNA)){
				printf("Converting to complementary sequence...\n");
				c=comp_seq4(seq_ext);
/*				print_seq(seq_ext);
*/
			}
			else {
				printf("No attempt to make complementary sequence.\n");
			}
		}			
		if(ftouch(fname2)){	
			fprintf(stderr,"Error in opening output file: %s.\n",fname2);
#include	"exit1.h"
		}
		if((c=write_fasta(fname2,seq_ext,INCLUDE_HEADER,' '))!=0){
			fprintf(stderr,"Error in writing sequence.\n");
#include	"exit1.h"
		}
		
		circular=circular_bak;
	
		printf("\nTerminated.\n");

#include	"exit0.h"

}

/************************************************************
		End of file getseq2.c
************************************************************/


