/******************************************************
	getseq2.h

	Naoki Sato Dec. 29, 1997
	Modified July 12, 1998
	Added EMBL defines. March 11, 1999.
	Added GB defines. May 6, 1999.
	Enhanced Exon_record. May 8, 1999.
	Enhanced Seqfile. May 23, 1999.
	Added Gene. May 23, 1999.
	Added dbidx. June 3, 1999.
	Added PIR. Aug. 31, 1999.
	Added LEADERLINE. Sept. 2, 1999.
	Added S_1, S_2. Nov. 11, 1999.
	Added SimilarityCounts. Dec. 16, 1999.
	def.h separated. Oct. 23, 2000.
	Added PHYLIP_SEQ. Jan. 31, 2001.
	Added ORF. Nov. 30, 2001.
	Added annotP and annotR in data_type. March 23, 2003.
	Added GBPROT in filetype. April 28, 2003.
	Added AC_LENGTH definition. Jan. 5, 2010.
  
  --- Do not delete this comment.---
*******************************************************/

#include "def.h"	/* various definitions. */

#define LEADERLINE 30	/* Maximum number of lines allowed to precede 
			actual sequence data in data file. */
#define MINLIMIT 300	/* Minimum limit for showing valid Orf in trnsfn.c and toprot.c */
#define AC_LENGTH 50	/* Length of accession code */

/*
#define MAX_EXONS 100 
*/

#ifndef RAW_SEQ
#define RAW_SEQ raw
#endif
#ifndef FASTA_SEQ
#define FASTA_SEQ fasta
#endif
#ifndef GENBANK_SEQ
#define GENBANK_SEQ genbank
#endif
#ifndef GBPROT_SEQ
#define GBPROT_SEQ gbprot
#endif
#ifndef EMBL_SEQ
#define EMBL_SEQ embl
#endif
#ifndef SW_SEQ
#define SW_SEQ swiss
#endif
#ifndef UNIPROT_SEQ
#define UNIPROT_SEQ uniprot
#endif
#ifndef EMBL_UK
#define EMBL_Uk embl_unknown
#endif
#ifndef PRF_SEQ
#define PRF_SEQ prf
#endif
#ifndef CLUSTAL_SEQ
#define CLUSTAL_SEQ clustal
#endif
#ifndef PHYLIP_SEQ
#define PHYLIP_SEQ phylip
#endif
#ifndef PIR_SEQ
#define PIR_SEQ pir
#endif

/* Sequence file format */
enum filetype {
	RAW_SEQ,	/* 0 */
 	FASTA_SEQ, 	/* 1 */
	GENBANK_SEQ, 	/* 2 */
	EMBL_SEQ, 	/* 3 */
	SW_SEQ, 	/* 4 */
	EMBL_UK, 	/* 5 */
	PRF_SEQ, 	/* 6 */
	CLUSTAL_SEQ, 	/* 7 */
	PIR_SEQ,	/* 8 */
	PHYLIP_SEQ, 	/* 9 */
	GBPROT_SEQ,	/* 10 */
	UNIPROT_SEQ}; 	/* 11 */
	
/* Nucleic acid or amino acid codes 
    zero(0) for symbols */
enum residue_code {
	_symbol,A,C,D,E,F,G,H,I,K,L,M,N,P,Q,R,S,T,U,V,W,X,Y};


/* Type of sequence data */
enum  data_type{
	unknown,DNA,RNA,protein,annotR,annotP};
	
typedef struct {
	long seqlen;
	int dtype;
	int circular;	/* Added.*/
	char *ID;
	char AC[AC_LENGTH];
	char OS[50];
	char dbtype[30];
	char *header;	/* currently, ACCESSION is also included in the header as AC=******. */
	char *seq;
} Seqfile;	/* Structure of description of a sequence. */

typedef struct {
	char AC[AC_LENGTH];	/* Accession number of sequence segment */
	int nc;		/* normal (0) or complementary (1) strand */
	long begin;	/* beginning position */
	Boolean begin_ext;	/* actual beginning is located upstream, ie, <begin */
	long end;	/* end position */
	Boolean end_ext;	/* actual end is located downstream, ie., >end */
} Exon_record;	/* Structure of description of exons. */

/* to be implemented.
typedef struct{
	int exons;
	Exon_record **exon;
	Exon_record **intron;
	Exon_record *utr5;
	Exon_record *utr3;
	Exon_record *TATA;
} Gene;	
 complete structure of a gene. */

typedef struct{
	char AC[AC_LENGTH];
	long loc;
} dbidx;	/* Structure for database index. */

typedef struct {
	long total;
	long match;
	long similar;
#if 0
	double total;
	double match;
	double similar;
#endif
} SimilarityCounts;     /* container of similarity scores */

typedef struct{
	int frame;
	long length;
	long open_end;
	long start;
	int start_codon;
	long stop;
	int stop_codon;
} ORF;

typedef struct{
	long pos;
	long Npos;
	long AApos;
	char Norg;
	char Nmut;
	char geneID[50];
	char codon_org[10];
	char codon_mut[10];
	char AA_org;
	char AA_mut;
} MutList;			/* SNP analysis */

/* Various definitions */

#ifndef MINMEMSIZE
#define MINMEMSIZE 100	/* minimum memory size for Seqfile */
#endif
#ifndef FASTA_INIT
#define FASTA_INIT ">"
#endif
#ifndef FASTA_INIT_CHAR
#define FASTA_INIT_CHAR '>'
#endif

#ifndef GB_INIT
#define GB_INIT "LOCUS "
#endif
#ifndef GB_AC
#define GB_AC "ACCESSION "
#endif
#ifndef GB_OS
#define GB_OS "  ORGANISM "
#endif
#ifndef GB_SEQ_INIT
#define GB_SEQ_INIT "ORIGIN"
#endif
#ifndef GB_DE
#define GB_DE "DEFINITION "
#endif
#ifndef GB_FEATURES
#define GB_FEATURES "FEATURES "
#endif
#ifndef GB_SEQ_END
#define GB_SEQ_END "//"
#endif
#ifndef GB_HEADER
#define GB_HEADER "Genetic Sequence Data Bank"
#endif

#ifndef EMBL_INIT
#define EMBL_INIT "ID "
#endif
#ifndef EMBL_AC
#define EMBL_AC "AC "
#endif
#ifndef EMBL_DE
#define EMBL_DE "DE "
#endif
#ifndef EMBL_GN
#define EMBL_GN "GN "
#endif
#ifndef EMBL_OS
#define EMBL_OS "OS "
#endif
#ifndef EMBL_OC
#define EMBL_OC "OC "
#endif
#ifndef EMBL_OG
#define EMBL_OG "OG "
#endif
#ifndef EMBL_FHKey
#define EMBL_FHKey "FH   Key"
#endif
#ifndef EMBL_FT
#define EMBL_FT "FT"
#endif
#ifndef EMBL_FH
#define EMBL_FH "FH"
#endif
#ifndef EMBL_XX
#define EMBL_XX "XX"
#endif
#ifndef EMBL_SEQ_INIT
#define EMBL_SEQ_INIT "SQ "
#endif
#ifndef EMBL_SEQ_END
#define EMBL_SEQ_END "//"
#endif
#ifndef EMBL_HEADER
#define EMBL_HEADER ""	/* not used. */
#endif
#ifndef SW_HEADER
#define SW_HEADER ""	/* not used. */
#endif

#ifndef PRF_INIT
#define PRF_INIT "CODE "
#endif
#ifndef PRF_SEQ_INIT
#define PRF_SEQ_INIT "SEQUENCE"
#endif
#ifndef PRF_DEF
#define PRF_DEF "NAME "
#endif
#ifndef PRF_FEATURES
#define PRF_FEATURES "REFERENCE "
#endif
#ifndef PRF_SEQ_END
#define PRF_SEQ_END "END"
#endif
#ifndef CLUSTAL_INIT
#define CLUSTAL_INIT "CLUSTAL"
#endif
#ifndef PIR_INIT
#define PIR_INIT "ENTRY "
#endif
#ifndef PIR_SEQ_INIT
#define PIR_SEQ_INIT "SEQUENCE"
#endif
#ifndef PIR_DEF
#define PIR_DEF "TITLE "
#endif
#ifndef PIR_AC
#define PIR_AC "ACCESSIONS "
#endif
#ifndef PIR_OS
#define PIR_OS "ORGANISM "
#endif
#ifndef PIR_SEQ_END
#define PIR_SEQ_END "///"
#endif
#ifndef PIR_HEADER
#define PIR_HEADER "P R O T E I N  S E Q U E N C E  D A T A B A S E"
#endif

#ifndef PUNCTUATION_CHAR
#define PUNCTUATION_CHAR ';'	/* Default delimiter for printing comments. */
#endif
#ifndef INCLUDE_HEADER
#define INCLUDE_HEADER TRUE
#endif

