/*****************************************************
File name : gtsqfn.c (originally getseqfunc2.c)
	
	Functions : 	read_seq
			read_fasta
			read_genbank
			err_gb_format
			err_embl_format
			read_embl
			read_prf
			read_pir
			check_filetype
			cat_seq
			cat_aln
			empty_seq5
			extract_seq3
			pick_seq
			comp_seq4
			to_compD
			to_compR
			comp_seqM
			to_rna
			to_dna
			to_rnaM
			to_dnaM
			check_data_type
			clear_gaps
			clear_gapsM
			copy_seq
			copy_seq_h
			fprint_seq
			print_seq
			free_seq
			fwrite_fasta
			write_fasta
			match_seq
			sort_seq
			qSortSeq
			partition
			read_phylip
			fwrite_phylip
			write_phylip
			read_clustal
			fwrite_clustal
			write_clustal
			parse_position
			read_dbseq
			fwrite_info
			fwrite_info2
			fwrite_info3
			write_info
			fwrite_Dinfo
			write_Dinfo
				
	Naoki Sato 29 Dec 1997.

	Integrated Mac, DOS, and UNIX version.

	Modified July 16, 1998. Minor changes of memory usage. Sept. 9, 1998.
	Added functions that cope with clustal formats.

	Added support for the type "circular" in read_genbank.
	Corrected extract_seq to indicate right numbers in ID. 
	Bugs fixed. Nov. 20-21, 1998.
	Changed output format of fwrite_clustal. Nov. 21, 1998.
	Bug fix in comp_seq3. Nov. 23, 1998.
	Error check in read_embl and read_genbank. Dec. 25, 1998.
	Allow ";" as a comment in read_fasta. Dec. 25, 1998.
	Modified read_embl. March 13, 1999.
	Added comp_seqM. April 25, 1999.
	Bug fix in comp_seq3. April 28, 1999.
	Added print_line that enables setting line length from environmental
		variable, PRINTLINE. Default is 75. April 29, 1999.
	Bug fix in check_filetype and read_embl. May 5, 1999.
	Added support for circular genome. May 9-11, 1999.
	Added parse_position. May 9, 1999.
	Added to_rna and to_dna. May 18, 1999.
	Added pick_seq. May 26, 1999.
	Added copy_seq_h, to_compD, to_compR and comp_seq4. May 27, 1999.
	Added read_dbseq. June 8, 1999.
	Bug fix in free_seq. June 21, 1999.
	Verbous output suppressed. Aug. 25, 1999.
	Added read_pir. Aug. 31, 1999.
	Use of LEADERLINE. Sept. 2, 1999.
	Changed extract_seq3. ID is kept unchanged. Oct. 4, 1999.
	ID_header separator is not inserted if header begins with a space. Oct. 4, 1999.
	Added cat_seq. Oct. 11, 1999.
	Bug fix in fwrite_clustal. Oct. 11, 1999.
	Added cat_aln and match_aln. Oct. 13, 1999.
	Included allowance of a single U in protein seq in check_data_type. Nov. 13, 1999.
	Added empty_seq5. Dec. 8, 1999.
	Bug fix. May 4, 2000.
	Added fwrite_info and write_info. May 10, 2000.
	Bug fix in c_comp. July 7, 2000.
	Added write_pretty and fwrite_pretty. Aug. 22, 2000.
	Added fwrite_Dinfo and write_Dinfo. Nov. 29, 2000.
        Non-alphabetical characters are allowed in read_genbank and read_embl.
                                Dec. 21, 2000.
	Added read_phylip and write_phylip. Jan. 31, 2001.
	Bug fix in read_clustal. Feb. 5, 2001.
	Bug fix in check_filetype. Codes for phylip added recently. Mar. 7, 2001.
	Base composition in percentage in fwrite_Dinfo. May 28, 2002.
	Enabled stdout. Jan. 7, 2003.
	Added fwrite_info2. Jan. 8, 2003.
	Added fwrite_info3. Jan. 15, 2003.
	Added sort_seq. April. 6, 2003.
	Adapted read_genbank for protein sequence. April 28, 2003.
	Modified read_embl for UniProt format. March 7, 2007.
	Further bug fix in read_embl. April 20, 2007.
	Added mask_sequence. June 29, 2007.
	Added rev_seq and to_revM. Feb. 4, 2008.
	Added match_seq. April 13, 2008.

	--- Do not delete this comment.---

******************************************************/

#include "Dmc.h"
#include "getseq2.h"
#include <math.h>

/* Prototype declaration */

int check_filetype(char *filename);
static int is_phylip(char *str1);
Seqfile *read_seq(char *filename, int seqformat1, enum data_type dtype1);
Seqfile *read_fasta(FILE *fin,enum data_type dtype1);
static void err_gb_format(void);
Seqfile *read_genbank(FILE *fin,long *loc,enum data_type dtype1);
Seqfile *read_pir(FILE *fin,long *loc,enum data_type dtype1);
static void err_embl_format(void);
Seqfile *read_embl(FILE *fin,long *loc,enum data_type dtype1);
Seqfile *read_prf(FILE *fin,enum data_type dtype1);
int cat_seq(Seqfile *seq1,Seqfile *seq2);
int cat_aln(Seqfile **seq1,Seqfile **seq2);
Seqfile *empty_seq5(void);
Seqfile *extract_seq3(Seqfile *seqfile1, long startpos, long endpos);
int pick_seq(Seqfile *seqfile1, long startpos, long endpos);
int to_compD(int s); 
int to_compR(int s); 
int rev_seq(Seqfile *seqfile1);
int to_revM(Seqfile **seqfileM);
int comp_seq4(Seqfile *seqfile1);
int comp_seqM(Seqfile **seqfileM);
int to_rna(Seqfile *seqfile1);
int to_dna(Seqfile *seqfile1);
int to_rnaM(Seqfile **seqfileM);
int to_dnaM(Seqfile **seqfileM);
enum data_type check_data_type(Seqfile *seqfile1);
Boolean clear_gaps(Seqfile *seqfile1);
Boolean clear_gapsM(Seqfile **seqfileM);
Seqfile *copy_seq(Seqfile *seqfile1);
Seqfile *copy_seq_h(Seqfile *seqfile1);
void fprint_seq(FILE *file, Seqfile *seqfile1);
int print_seq(Seqfile *seqfile1);
void free_seq(Seqfile *seqfile1);
int fwrite_Dinfo(FILE *fout, Seqfile *seqfile1,long window,long slide);
int write_Dinfo(char *filename,Seqfile *seqfile1,long window,long slide);
int fwrite_info(FILE *fout, Seqfile *seqfile1,Boolean include_header,int id_h_sep);
int fwrite_info2(FILE *fout, Seqfile *seqfile1,char info_mode_opt);
int fwrite_info3(FILE *fout, Seqfile *seqfile1,char *codon_table,long *code_count);
int write_info(char *filename,Seqfile *seqfile1,Boolean include_header,int id_h_sep);
int fwrite_fasta(FILE *fout, Seqfile *seqfile1,Boolean include_header,int id_h_sep);
int write_fasta(char *filename,Seqfile *seqfile1,Boolean include_header,int id_h_sep);
void sort_seq(Seqfile **seq);
int match_seq(Seqfile **seq,char *listfile,int mode_flag);
static int partition(Seqfile **seq, long L, long r);
static void qSortSeq(Seqfile **seq,long L,long r);
Seqfile **read_phylip(char *filename, enum data_type dtype1);
Seqfile **read_clustal(char *filename, enum data_type dtype1);
void read_clustal2(char *filename, Seqfile **seqfile1);
int fwrite_clustal(FILE *fout, Seqfile **seqfile1, char *string, int ID_len);
int write_clustal(char *filename,Seqfile **seqfile1);
int fwrite_phylip(FILE *fout, Seqfile **seqfile1, int ID_len, int I_flag);
int write_phylip(char *filename,Seqfile **seqfile1, int ID_len, int I_flag);
int fwrite_nexus(FILE *fout, Seqfile **seqfile1, int ID_len);
int write_nexus(char *filename,Seqfile **seqfile1);
int fwrite_mega(FILE *fout, char *fname,Seqfile **seqfile1);
int write_mega(char *filename,char *fname, Seqfile **seqfile1);
long parse_position(char *position_str);
Seqfile *read_dbseq(FILE *fin,long *loc);
int fwrite_pretty(FILE *fout,Seqfile *seqfile1,char *exon_file);
int write_pretty(char *fname,Seqfile *seqfile1,char *exon_file);
void mask_sequence(char *seq,int Ns, int Qs);

/* External functions */

/* utility.c */
extern char *extract_string(char *str_name, long int startpos, long int endpos);
extern Boolean linetype(char *line,char *code);
extern Boolean tailtype(char *line,char *code);
extern void tail_string(char *string,int s);
extern Boolean last_line(char *string);
extern Boolean blank_line(char *str);
extern void subst(char *string,char c1,char c2);
extern int num_check(char *string);
extern int pick_string(char *str_name, long startpos, long endpos);
extern void pack_seq(char *string,char *list);
extern void pack_space(char *string);
extern char *str_shift(char *string,char delimit);
extern int count_characters(char *string);

/* calcseq.c */
extern int c_comp(Seqfile *seq_in, long code_count[30]);
extern void initialize_comp(long code_count[30]);
extern double getProtMW(Seqfile *seqfile);
extern int c_basecomp(Seqfile *seq_in,double basecomp[4]);
extern double CountGCSkew(char *seq);
extern double CountATSkew(char *seq);
extern int codon_usage_comp(Seqfile *seq_in, long *code_count);
extern void initialize_comp2(long *code_count,unsigned dim);

/* trnsfn.c */
extern int get_codon(int s,char *string);

/* Global variables are defined here. */

#include "glovar.h"

/* for conversion of complementary bases */

#include "cmptbl.h"

/* Definition of the code for the end of line (LF).*/

#define END_OF_LINE 10	/* CR will be ignored in the program. */
#define END_OF_LINE2 13		/* for DOS or Mac->UNIX converted files */


/*********************************************/
void mask_sequence(char *seq,int Ns, int Qs)
/*********************************************/
{
	char *p,*q,*r;
	int counterN,counterQ;
	char s[20];
	int i;

	counterN=counterQ=0;
	for(p=seq;*p!='\0';p++){
		if(counterN){
			if(*p == 'N'){
				counterN += 1;
			} else {
				if(counterN >= Ns){
					sprintf(s,"x%03dxN",counterN);
		printf("x%03dx\n",counterN);
					for(i=0;i<6;i++){
						*(q+i+1) = s[i];
					}
					for(r=q+7;*p!='\0';p++){
						*r++ = *p;
					}
					*r = '\0';
					p = q+7;
				}
				counterN = 0;
			}
		} else {
			if(*p == 'N'){
				counterN = 1;
				q = p;
			} else continue;
		}
	}
	
	for(p=seq;*p!='\0';p++){
		if(counterQ){
			if(*p == 'Q'){
				counterQ += 1;
			} else {
				if(counterQ >= Qs){
					sprintf(s,"x%03dxQ",counterQ);
		printf("x%03dx\n",counterQ);
					for(i=0;i<6;i++){
						*(q+i+1) = s[i];
					}
					for(r=q+7;*p!='\0';p++){
						*r++ = *p;
					}
					*r = '\0';
					p = q+7;
				}
				counterQ = 0;
			}
		} else {
			if(*p == 'Q'){
				counterQ = 1;
				q = p;
			} else continue;
		}
	}

}
/*********************************************/


/*********************************************/
long parse_position(char *position_str)
/* returns position if position_str contains
+ or -. returns 0 if position_str is "last".*/
/*********************************************/
{
	char *p;
	char str1[MAXLINE];
	char str2[MAXLINE];
	long i,j;

	if(strlen(position_str) > MINMEMSIZE) return 0L;
	if((p=strchr(position_str+1,'+'))!=NULL){
		subst(position_str,'+',' ');
		sscanf(position_str,"%s %s",str1,str2);
		i=atol(str1);
		j=atol(str2);
		return i+j;
	}
	if((p=strchr(position_str+1,'-'))!=NULL){
		subst(position_str,'-',' ');
		sscanf(position_str,"%s %s",str1,str2);
		i=atol(str1);
		j=atol(str2);
		return i-j;
	}
	if(!strcmp(position_str,"last") || !strcmp(position_str,"LAST")) return 0L;
	if(*position_str=='+') {
		position_str++; 
		if(num_check(position_str)) return 0L;
		else return atol(position_str);
	}
	else if(*position_str=='-') {
		position_str++; 
		if(num_check(position_str)) return 0L;
		else return (-1)*atol(position_str);
	}
	else {
		if(num_check(position_str)) return 0L;
		else return atol(position_str);
	}
}


/***********************************************************************/
Seqfile *read_seq(char *filename, int seqformat1, enum data_type dtype1)
/***********************************************************************/
{
	FILE *fin;
	char *line;
	Seqfile *seqfile1;
	long int i, j, offset = 0;
	long int memsize=MINMEMSIZE;	/* for memory usage */
	long fileloc=0L;
	char str1[MAXLINE+1];
	char header_line[MAXLINE+1]="This is a raw sequence read on: ";
	char header_line2[MAXLINE+1]="Data type was automatically set to: ";
	char time_string[50];
	char data_type_string[MINMEMSIZE];
	time_t systime;
	struct tm *currtime;
	
	systime = time(NULL);
	currtime = localtime(&systime);


	if((fin=fopen(filename,"r"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n\n",filename);
		return NULL;
	}

	switch(seqformat1) {
	
		case RAW_SEQ:
			if((seqfile1=(Seqfile*)calloc(1,sizeof(Seqfile)))==0){
                		fprintf(stderr,"Out of memory\n");
                		return NULL;
        		}
	
	/* Raw file has no ID and header. We create an ID that represents the filename
	   and a header that includes the current time. */		
			if((seqfile1->ID=(char *)calloc(memsize,sizeof(char)))==0){
		                fprintf(stderr,"Out of memory\n");
        		        return NULL;
        		}

			strcpy(seqfile1->ID,filename);

			if((seqfile1->header=(char *)calloc(memsize,sizeof(char)))==0){
		                fprintf(stderr,"Out of memory\n");
		                return NULL;
		        }
			
			strcpy(time_string,asctime(currtime));
			strncat(header_line,time_string,(strlen(time_string)-1));	
			strcpy(seqfile1->header,header_line);

			if((seqfile1->seq=(char *)calloc(memsize,sizeof(char)))==0){
		                fprintf(stderr,"Out of memory\n");
		                return NULL;
		        }
			
			while(fgets(str1,MAXLINE+1,fin)!=NULL){
								
				memsize=memsize+((strlen(str1))+4)*(sizeof(char));
				if((seqfile1->seq=(char *)realloc(seqfile1->seq,memsize))==0){
					fprintf(stderr,"Out of memory\n");
					exit(1);
					}
					
				j=0;	
				for(i=0;i<(long)strlen(str1);i++){
					if(isprint((int)str1[i])){
						seqfile1->seq[offset+j]=str1[i];
						j++;
					}						
				}
				offset=offset+j;				
			}
			
			seqfile1->seq[offset]=0;
			seqfile1->seqlen=offset;	
			seqfile1->dtype=dtype1;		
			break;
		
		case FASTA_SEQ:
			if((seqfile1=read_fasta(fin,dtype1))==NULL){
				fprintf(stderr,"Error in reading sequence from file: %s\n",filename);
				return NULL;
			}
			break;
		
		case GENBANK_SEQ:
		case GBPROT_SEQ:
			if((seqfile1=read_genbank(fin,&fileloc,dtype1))==NULL){
				fprintf(stderr,"Error in reading sequence from file: %s\n",filename);
				return NULL;
			}
			break;
		
		case PIR_SEQ:
			if((seqfile1=read_pir(fin,&fileloc,dtype1))==NULL){
				fprintf(stderr,"Error in reading sequence from file: %s\n",filename);
				return NULL;
			}
			break;
		
		case EMBL_SEQ:
		case SW_SEQ:
		case EMBL_UK:
			if((seqfile1=read_embl(fin,&fileloc,dtype1))==NULL){
				fprintf(stderr,"Error in reading sequence from file: %s\n",filename);
				return NULL;
			}
			break;
		
		case PRF_SEQ:
			if((seqfile1=read_prf(fin,dtype1))==NULL){
				fprintf(stderr,"Error in reading sequence from file: %s\n",filename);
				return NULL;
			}
			break;
		
		default:
				fprintf(stderr,"Unknown sequence format.\n");
				return NULL;
			break;
	}
	
	fclose(fin);
	
	if(seqfile1->dtype==unknown){
		dtype1=check_data_type(seqfile1);
		seqfile1->dtype=dtype1;

		strcpy(data_type_string,data_type_name[dtype1]);
		line=(char*)calloc(strlen(seqfile1->header)+MINMEMSIZE,sizeof(char));
		sprintf(line,"%s\n%s%s",seqfile1->header,header_line2,data_type_string);
		seqfile1->header=(char *)realloc(seqfile1->header,(MINMEMSIZE+strlen(line))*sizeof(char));	
		strcpy(seqfile1->header,line);
		free(line);
	}
	
	return seqfile1;	

}
	
/********************************************/
int check_filetype(char *filename)
/********************************************
This function returns filetype if filetype
was recognized. It returns 99 on error.
*********************************************/
{
	FILE *fin;
	char str1[MAXLINE+1];

	int filetype1=RAW_SEQ; /* default is raw */

	char dtype_char[MINMEMSIZE];
	int i;
	
	if((fin=fopen(filename,"r"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n\n",filename);
		return 99;
	}

	for(i=0;i<LEADERLINE;i++){
		if(fgets(str1,MAXLINE+1,fin)==NULL){
			if(i==0){
				fprintf(stderr,"Empty file.\n");
				return 99;
			}
			else break;
		}
		if(linetype(str1,FASTA_INIT)){
			filetype1=FASTA_SEQ;
			break;
		}
		else if(strstr(str1,GB_HEADER)!=NULL){
			filetype1=GENBANK_SEQ;
			break;
		}
		else if(linetype(str1,GB_INIT)){
			pick_string(str1,strlen(GB_INIT)+1,strlen(str1));
			if(strstr(str1," aa ")!=NULL) strcpy(dtype_char,"PRT;");
			else if(strstr(str1,"DNA")!=NULL) strcpy(dtype_char,"DNA;");
			else if(strstr(str1,"RNA")!=NULL) strcpy(dtype_char,"RNA;");
			else if(strstr(str1,"PRT")!=NULL) strcpy(dtype_char,"PRT;"); 

			if(!strcmp(dtype_char,"PRT;")) filetype1=GBPROT_SEQ;
			else filetype1=GENBANK_SEQ;
			break;
		}
		else if(linetype(str1,PRF_INIT)){
			filetype1=PRF_SEQ;
			break;
		}
		else if(linetype(str1,PIR_INIT) || strstr(str1,PIR_HEADER)!=NULL){
			filetype1=PIR_SEQ;
			break;
		}
		else if(linetype(str1,EMBL_INIT)){
			pick_string(str1,strlen(EMBL_INIT)+1,strlen(str1));
			if(strstr(str1,"DNA")!=NULL) strcpy(dtype_char,"DNA;");
			else if(strstr(str1,"RNA")!=NULL) strcpy(dtype_char,"RNA;");
			else if(strstr(str1,"PRT")!=NULL) strcpy(dtype_char,"PRT;"); 
			else if(strstr(str1,"Reviewed")!=NULL) strcpy(dtype_char,"UniProt"); 
			else if(strstr(str1,"Unreviewed")!=NULL) strcpy(dtype_char,"UniProt"); 
			else{
				fprintf(stderr,"Unrecognizable data type in EMBL file.\n");
				filetype1=EMBL_UK;	
				break;								
			}
			if(!strcmp(dtype_char,"UniProt")){
				filetype1=UNIPROT_SEQ;
				break;
			}
			if(!strcmp(dtype_char,"PRT;")){
				filetype1=SW_SEQ;
				break;
			}
			else if(!strcmp(dtype_char,"DNA;") || !strcmp(dtype_char,"RNA;") ){ 
				filetype1=EMBL_SEQ;
				break;
			}
		}
		else if(linetype(str1,CLUSTAL_INIT)){
			filetype1=CLUSTAL_SEQ;
			break;
		}
		else if(blank_line(str1)){
			continue;
		}
		else {
			if(is_phylip(str1)){
				filetype1=PHYLIP_SEQ;
				break;
			}
		}

	}
	return filetype1;	
}


/*********************************/
static int is_phylip(char *str1)
/* option I is tolerated.
Format: number (space/tab) number (space/tab) I*/
/*********************************/
{
	char *p;
	Boolean flag1,flag2,flag3,flag4;

	flag1=flag2=flag3=flag4=FALSE;
	for(p=str1;*p!='\0' && *p!='\n';p++){
		if(!flag1){
			if(*p==' ' || *p=='\t') continue;
			else if('0' <= *p && *p <= '9') flag1 = TRUE;
			else return FALSE;
		} else if(flag1 && !flag2){
			if(*p==' ' || *p=='\t') flag2 = TRUE;
			else if('0' <= *p && *p <= '9') continue;
			else return FALSE;
		} else if(flag2 && !flag3){
			if(*p==' ' || *p=='\t') continue;
			else if('0' <= *p && *p <= '9') flag3 = TRUE;
			else return FALSE;
		} else if(flag3 && !flag4){
			if(*p==' ' || *p=='\t') flag4 = TRUE;
			else if('0' <= *p && *p <= '9') continue;
			else return FALSE;
		} else if(flag4){
			if(*p==' ' || *p=='\t') continue;
			else if(*p == 'I') return TRUE;
			else return FALSE;
		} 
	}
	if(flag3) return TRUE;
	else return FALSE;

			
/*
	strcpy(str4,str1);
	subst(str4,'\t',' ');
	p=str_shift(str4,' ');
	pack_space(str4);
	fprintf(stderr,"str1= %s\n",str4);
	if(num_check(str4)) return FALSE;

	strcpy(str2,p);
	p=str_shift(str2,' ');
	pack_space(str2);
	fprintf(stderr,"str2= %s\n",str2);
	if(num_check(str2)) return FALSE;

	strcpy(str3,p);
	p=str_shift(str3,' ');
	if(str3[0]=='I' || str3[0]=='i'){
		return TRUE;
	}
	return FALSE;
*/
}


/********************************************************************/
Seqfile *read_fasta(FILE *fin,enum data_type dtype1)
/********************************************************************/
{
	Seqfile *seqfile1;
	register long int i, j;
	long int offset = 0;
	long int memsize=MAXLINE;
	char str1[MAXLINE+1];
	char str2[MAXLINE+1];
	char IDstr[MAXLINE+1];
	int b;
	char *pp;

/* Flags for gb are also used here. */
	Boolean gb_id_on=TRUE;
	Boolean gb_header_on=FALSE;
	Boolean gb_seq_on=FALSE;
	

	if((seqfile1=(Seqfile*)calloc(1,sizeof(Seqfile)))==0){
                fprintf(stderr,"Out of memory\n");
                return NULL;
        }

/* Reading ID and saving the remaining part of the ID line in str2.*/
	if((seqfile1->ID=(char *)calloc(MINMEMSIZE,sizeof(char)))==0){
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}
	if((seqfile1->header=(char *)calloc(MAXLINE+MINMEMSIZE,sizeof(char)))==0){
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}
	if((seqfile1->seq=(char *)calloc(memsize,sizeof(char)))==0){		
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}

/* Start of main loop */

	while(fgets(str1,MAXLINE+1,fin)!=NULL){

		if(gb_id_on==TRUE){

			if(linetype(str1,FASTA_INIT)){

				gb_id_on=FALSE;
				if(strlen(seqfile1->ID)!=0){
					if(strlen(seqfile1->seq)!=0){
						fprintf(stderr,"This is a multi-fasta sequence file. Only the first sequence was read.\n");
						return seqfile1;
					}
					else{
						fprintf(stderr,"File format is strange. Reading aborted.\n");
						return NULL;
					}
				}
	/* If this is the first signal... */			
				pick_string(str1,strlen(FASTA_INIT)+1,strlen(str1));
				sscanf(str1,"%s",IDstr);
	/* IDstr is limited to 30 characters. */			
				if((strlen(IDstr)<AC_LENGTH)) strcpy(seqfile1->ID,IDstr);
				else seqfile1->ID=extract_string(IDstr,1,AC_LENGTH);
	/* The remaining part of str1 is copied to str2 for further manipulation */
	/* This code must be revised. 2006 */	
				pp=extract_string(str1,strlen(seqfile1->ID)+1,strlen(str1));
				strcpy(str2,pp);
				free(pp);
				gb_header_on=TRUE;
	/* Setting str2 to the header field. */
				strcpy(seqfile1->header,str2);
				if(str2[strlen(str2)-1]==END_OF_LINE){				
					gb_header_on=FALSE;
					gb_seq_on=TRUE;
				}
				continue;
			}
			else continue;
		}
		
		else if(gb_header_on==TRUE){

			b=strlen(seqfile1->header)+MAXLINE+MINMEMSIZE;
			if((seqfile1->header=(char *)realloc(seqfile1->header,b*sizeof(char)))==0){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}	
			strcat(seqfile1->header,str1);
			
			if(last_line(str1)){
				gb_header_on=FALSE;
				gb_seq_on=TRUE;
			continue;
			}
			
		}
/* Reading sequence */
		else if(gb_seq_on==TRUE){
		
			if(linetype(str1,GB_SEQ_END)) break;
			else if(linetype(str1,FASTA_INIT)){
				fprintf(stderr,"This seems to be a multi-fasta file. Stop reading.\n");
				break;				
			}
			else if(linetype(str1,";")){
				b=strlen(seqfile1->header)+MAXLINE+MINMEMSIZE;
				if((seqfile1->header=(char *)realloc(seqfile1->header,b*sizeof(char)))==0){
					fprintf(stderr,"Out of memory\n");
					return NULL;
				}
				strcat(seqfile1->header,str1);
			}
			else{
				memsize=memsize+((strlen(str1))+4)*(sizeof(char));
				if((seqfile1->seq=(char *)realloc(seqfile1->seq,memsize))==0){
					fprintf(stderr,"Out of memory\n");
					return NULL;
				}
		
				j=0;	
				for(i=0;i<(long)strlen(str1);i++){
					if(isprint((int)str1[i])){
						seqfile1->seq[offset+j]=str1[i];
						j++;
					}						
				}
				offset=offset+j;
			}		
		}
	}
			
	seqfile1->seq[offset]=0;
	seqfile1->seqlen=offset;
	seqfile1->dtype=dtype1;
	return seqfile1;
}


/********************************************************************/
static void err_gb_format(void)
/*******************************************************************/
{
	fprintf(stderr,"File format is strange. Reading might be aborted.\n\n");
	fprintf(stderr,"GenBank format should be:\n");
	fprintf(stderr,"LOCUS       IDstring   length bp    type          division    date\n");
	fprintf(stderr,"This is an example:\n");
	fprintf(stderr,"LOCUS       APSTCAA       787 bp    DNA             BCT       03-NOV-1993\n\n");
	fprintf(stderr,"GenBank protein format should be:\n");
	fprintf(stderr,"LOCUS       AAN75219       436 aa                linear   PLN 02-DEC-2002\n\n");
	return;
}

/********************************************************************/
Seqfile *read_genbank(FILE *fin,long *loc,enum data_type dtype1)
/* Now reads genbank protein */
/********************************************************************/
{

	Seqfile *seqfile1;
	register long i, j;
	long offset = 0;
	long memsize=MAXLINE;	/* for memory usage */
	char str1[MAXLINE+1];
	char IDstr[MINMEMSIZE];
	int b;
	Boolean circular_bak=FALSE; 
	long fileloc=*loc;
/*gb*/
	long seqlength=0;
	char bp[MINMEMSIZE];
	char dtype_char[MINMEMSIZE];
	char length_char[MINMEMSIZE];
	Boolean gb_id_on=TRUE;
	Boolean gb_header_on=FALSE;
	Boolean gb_seq_on=FALSE;
/* end gb */

	if((seqfile1=(Seqfile*)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}
	circular_bak=circular;

/* Reading ID and description.*/
	if((seqfile1->ID=(char *)calloc(MINMEMSIZE,sizeof(char)))==NULL){	/* gb */
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}
	if((seqfile1->header=(char *)calloc(MAXLINE+MINMEMSIZE,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}
	if((seqfile1->seq=(char *)calloc(memsize,sizeof(char)))==NULL){		
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}

	if(fseek(fin,fileloc,SEEK_SET)){
		fprintf(stderr,"File seek error.\n");
		return NULL;
	}

	while(fgets(str1,MAXLINE+1,fin)!=NULL){
		if(gb_id_on==TRUE){
			if(linetype(str1,GB_INIT)){	/*gb*/
				pick_string(str1,strlen(GB_INIT)+1,strlen(str1));
				if(strstr(str1,"circular")!=NULL) circular=TRUE;
				sscanf(str1,"%s %s %s %s",IDstr,length_char,bp,dtype_char);
				if((strlen(IDstr)<=12)) strcpy(seqfile1->ID,IDstr);
				else{
					fprintf(stderr,"%s",str1);
					err_gb_format();
					strncpy(seqfile1->ID,IDstr,12);
				}
/* Normally, circular comes after DNA, but in some cases, only circular is stated without saying DNA. */
				if(strcmp(dtype_char,"circular")==0) strcpy(dtype_char,"DNA");
				if(strlen(dtype_char)<28) strcpy(seqfile1->dbtype,dtype_char);
				tail_string(dtype_char,3);
				if(!strcmp(bp,"aa")) {
					if(dtype1==DNA || dtype1==RNA){
						fprintf(stderr,"%s",str1);
						err_gb_format();
						fprintf(stderr,"But we try to continue.\n\n");
					}
					strcpy(dtype_char,"PRT");
				} else if(!strcmp(bp,"bp")) {
					if(dtype1==protein || (strcmp(dtype_char,"DNA") && strcmp(dtype_char,"RNA"))){
						fprintf(stderr,"%s",str1);
						err_gb_format();
						fprintf(stderr,"But we try to continue.\n\n");
					}
					strcpy(dtype_char,"DNA");
				}
				if(!num_check(length_char)) sscanf(length_char,"%ld",&seqlength);
				gb_id_on=FALSE;
			}
			continue;
		}

		if(linetype(str1,GB_DE)) {
			gb_header_on=TRUE;
			pick_string(str1,strlen(GB_DE)+1,strlen(str1));
		}
			
		if(linetype(str1,GB_FEATURES)) gb_header_on=FALSE;

		if(linetype(str1,GB_SEQ_INIT)){
			gb_seq_on=TRUE;
			gb_header_on=FALSE;
			continue;
		}
		
		if(linetype(str1,GB_SEQ_END)){
			gb_seq_on=FALSE;
			break;
		}
		
		if(gb_header_on==TRUE){
			b=strlen(seqfile1->header)+strlen(str1)+MINMEMSIZE;
			if((seqfile1->header=(char *)realloc(seqfile1->header,b*sizeof(char)))==0){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			if(str1[strlen(str1)-1]==END_OF_LINE){
				str1[strlen(str1)-1]=PUNCTUATION_CHAR;
			}
			strcat(seqfile1->header,str1);
			continue;
		}
/* Reading sequence */
		
		if(gb_seq_on==TRUE){
			memsize=memsize+(strlen(str1))*(sizeof(char));
			if((seqfile1->seq=(char *)realloc(seqfile1->seq,memsize))==0){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			
			j=0;
			for(i=0;i<(long)strlen(str1);i++){
				if(islower((int)str1[i])) str1[i]=toupper(str1[i]);
/*				if(isupper((int)str1[i])){
					seqfile1->seq[offset+j]=str1[i];
					j++;
	 			}
 gaps and other characters are now allowed */
				if(!isdigit((int)str1[i]) && str1[i]!=' ' && !iscntrl(str1[i])){
					seqfile1->seq[offset+j]=str1[i];
					j++;
				}
/*** Dec. 21, 2000 ***/

			}
			offset=offset+j;				
		}
	}
			
	seqfile1->seq[offset]=0;
	
	seqfile1->seqlen=offset;
	if(seqlength!=offset){
		fprintf(stderr,"Sequence length is %ld in the header of file,\n",seqlength);
		fprintf(stderr,"but %ld bases were read.\n",offset);
	}
	if(seqfile1->seqlen==0) return NULL;
	if(circular) seqfile1->circular=TRUE;
	circular=circular_bak;

	if(strcmp(dtype_char,"DNA")==0) seqfile1->dtype=DNA;
	else if(strcmp(dtype_char,"RNA")==0) {
		dtype1=check_data_type(seqfile1);
		if(dtype1==DNA) seqfile1->dtype=dtype1;
	} else if(strcmp(dtype_char,"PRT")==0){
		dtype1=protein;
		seqfile1->dtype=dtype1;
	} else seqfile1->dtype=dtype1;

	fileloc=ftell(fin);
	*loc=fileloc;	
	return seqfile1;
}


/**********************************************************/
Seqfile *read_pir(FILE *fin,long *loc,enum data_type dtype1)
/**********************************************************/
{
	Seqfile *seqfile1;
	register long i, j;
	long offset = 0;
	long memsize=MAXLINE;	/* for memory usage */
	char str1[MAXLINE+1];
	char IDstr[MINMEMSIZE];
	int b;	
	long fileloc=*loc;
	Boolean gb_id_on=TRUE;
	Boolean gb_header_on=FALSE;
	Boolean gb_seq_on=FALSE;

	seqfile1=(Seqfile*)calloc(1,sizeof(Seqfile));

/* Reading ID and description.*/
	if((seqfile1->ID=(char *)calloc(MINMEMSIZE,sizeof(char)))==0){
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}
	if((seqfile1->header=(char *)calloc(MAXLINE+MINMEMSIZE,sizeof(char)))==0){
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}
	if((seqfile1->seq=(char *)calloc(memsize,sizeof(char)))==0){		
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}

	if(fseek(fin,fileloc,SEEK_SET)){
		fprintf(stderr,"File seek error.\n");
		return NULL;
	}

	while(fgets(str1,MAXLINE+1,fin)!=NULL){
		if(gb_id_on==TRUE){
			if(linetype(str1,PIR_INIT)){	
				pick_string(str1,strlen(PIR_INIT)+1,strlen(str1));
				sscanf(str1,"%s",IDstr);
				if((strlen(IDstr)<12)) strcpy(seqfile1->ID,IDstr);
				else{
					fprintf(stderr,"File format is strange. But we continue.\n");
					fprintf(stderr,"%s",str1);
					strncpy(seqfile1->ID,IDstr,12);
				}
				gb_id_on=FALSE;
			}
			continue;
		}

		if(linetype(str1,PIR_DEF)) {
			gb_header_on=TRUE;
			pick_string(str1,strlen(PIR_DEF)+1,strlen(str1));
		}
			
		if(linetype(str1,PIR_SEQ_INIT)){
			gb_seq_on=TRUE;
			gb_header_on=FALSE;
			continue;
		}
		
		if(linetype(str1,PIR_SEQ_END)){
			gb_seq_on=FALSE;
			break;
		}
		
		if(gb_header_on==TRUE){
			b=strlen(seqfile1->header)+strlen(str1)+MINMEMSIZE;
			if((seqfile1->header=(char *)realloc(seqfile1->header,b*sizeof(char)))==0){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			if(str1[strlen(str1)-1]==END_OF_LINE){
				str1[strlen(str1)-1]=PUNCTUATION_CHAR;
			}
			strcat(seqfile1->header,str1);
			continue;
		}
/* Reading sequence */
		
		if(gb_seq_on==TRUE){
			memsize=memsize+(strlen(str1))*(sizeof(char));
			if((seqfile1->seq=(char *)realloc(seqfile1->seq,memsize))==0){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			
			j=0;
			for(i=0;i<(long)strlen(str1);i++){
				if(islower((int)str1[i])) str1[i]=toupper(str1[i]);
/*				if(isupper((int)str1[i])){
					seqfile1->seq[offset+j]=str1[i];
					j++;
				}
 gaps and other characters are now allowed */
				if(!isdigit((int)str1[i]) && str1[i]!=' ' && !iscntrl(str1[i])){
					seqfile1->seq[offset+j]=str1[i];
					j++;
				}
/*** Dec. 21, 2000 ***/

			}
			offset=offset+j;				
		}
	}
			
	seqfile1->seqlen=offset;
	if(seqfile1->seqlen==0 || !strcmp(seqfile1->ID,"")) return NULL;
	seqfile1->seq[offset]=0;
	seqfile1->dtype=protein;
	dtype1=protein;

	fileloc=ftell(fin);
	*loc=fileloc;	
	return seqfile1;
}


/********************************************************************/
Seqfile *read_prf(FILE *fin,enum data_type dtype1)
/********************************************************************/
{
	Seqfile *seqfile1;
	register long int i, j;
	long int offset = 0;
	long int memsize=MAXLINE;	/* for memory usage */
	char str1[MAXLINE+1];
	char IDstr[MINMEMSIZE];
	int b;	
	Boolean gb_id_on=TRUE;
	Boolean gb_header_on=FALSE;
	Boolean gb_seq_on=FALSE;

	seqfile1=(Seqfile*)calloc(1,sizeof(Seqfile));

/* Reading ID and description.*/
	if((seqfile1->ID=(char *)calloc(MINMEMSIZE,sizeof(char)))==0){
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}
	if((seqfile1->header=(char *)calloc(MAXLINE+MINMEMSIZE,sizeof(char)))==0){
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}
	if((seqfile1->seq=(char *)calloc(memsize,sizeof(char)))==0){		
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}

	while(fgets(str1,MAXLINE+1,fin)!=NULL){

		if(gb_id_on==TRUE){
		
			if(linetype(str1,PRF_INIT)){	
				pick_string(str1,strlen(PRF_INIT)+1,strlen(str1));
				sscanf(str1,"%s",IDstr);
				if((strlen(IDstr)<12)) strcpy(seqfile1->ID,IDstr);
				else{
					fprintf(stderr,"File format is strange. But we continue.\n");
					fprintf(stderr,"%s",str1);
					strncpy(seqfile1->ID,IDstr,12);
				}
				gb_id_on=FALSE;
			}
			continue;
		}

		if(linetype(str1,PRF_DEF)) {
			gb_header_on=TRUE;
			pick_string(str1,strlen(PRF_DEF)+1,strlen(str1));
		}
			
/*		if(linetype(str1,PRF_FEATURES)) gb_header_on=FALSE;
*/
		if(linetype(str1,PRF_SEQ_INIT)){
			gb_seq_on=TRUE;
			gb_header_on=FALSE;
			continue;
		}
		
		if(linetype(str1,PRF_SEQ_END)){
			gb_seq_on=FALSE;
			break;
		}
		
		if(gb_header_on==TRUE){
			b=strlen(seqfile1->header)+strlen(str1)+MINMEMSIZE;
			if((seqfile1->header=(char *)realloc(seqfile1->header,b*sizeof(char)))==0){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			if(str1[strlen(str1)-1]==END_OF_LINE){
				str1[strlen(str1)-1]=PUNCTUATION_CHAR;
			}
			strcat(seqfile1->header,str1);
			continue;
		}
/* Reading sequence */
		
		if(gb_seq_on==TRUE){
			memsize=memsize+(strlen(str1))*(sizeof(char));
			if((seqfile1->seq=(char *)realloc(seqfile1->seq,memsize))==0){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			
			j=0;
			for(i=0;i<(long)strlen(str1);i++){
				if(islower((int)str1[i])) str1[i]=toupper(str1[i]);
/*				if(isupper((int)str1[i])){
					seqfile1->seq[offset+j]=str1[i];
					j++;
				}
 gaps and other characters are now allowed */
				if(!isdigit((int)str1[i]) && str1[i]!=' ' && !iscntrl(str1[i])){
					seqfile1->seq[offset+j]=str1[i];
					j++;
				}
/*** Dec. 21, 2000 ***/

			}
			offset=offset+j;				
		}
	}
			
	seqfile1->seq[offset]=0;
	
	seqfile1->seqlen=offset;
	seqfile1->dtype=protein;
	dtype1=protein;
	return seqfile1;
}


/********************************************************************/
static void err_embl_format(void)
/* added UniProt. March 7, 2007. */
/*******************************************************************/
{
	fprintf(stderr,"File format is strange. Reading might be aborted.\n\n");
	fprintf(stderr,"EMBL format should be:\n");
	fprintf(stderr,"ID IDstring STANDARD; DNA(RNA); division; length BP\n\n");
	fprintf(stderr,"Swiss Prot format should be:\n");
	fprintf(stderr,"ID IDstring STANDARD; PRT; length AA\n\n");
	fprintf(stderr,"UniProt format should be:\n");
	fprintf(stderr,"ID IDstring Reviewed; length AA.\n\n");
	return;
}

/********************************************************************/
Seqfile *read_embl(FILE *fin,long *loc,enum data_type dtype1)
/********************************************************************/
{
	Seqfile *seqfile1;
	register long int i, j;
	long int offset = 0;
	long int memsize=MAXLINE;	/* for memory usage */
	char str1[MAXLINE+1];
	char IDstr[MINMEMSIZE];
	int b;
	Boolean circular_bak=FALSE;
	long fileloc=*loc;
/*embl*/
	char standard[MINMEMSIZE]="";	/* no use */
	char div[MINMEMSIZE];
	char SV[MINMEMSIZE];
	char SVnum[MINMEMSIZE];
	char genomic[MINMEMSIZE];
	long seqlength=0;
	char bp_aa[MINMEMSIZE];	/* no use */
	char dtype_char[MINMEMSIZE]="";
	char circular_char[MINMEMSIZE]="";
	char length_char[MINMEMSIZE]="";
	Boolean embl_id_on=TRUE;
	Boolean embl_header_on=FALSE;
	Boolean embl_seq_on=FALSE;
/* end embl */

	seqfile1=(Seqfile*)calloc(1,sizeof(Seqfile));
	circular_bak=circular;

/* Reading ID.*/
	if((seqfile1->ID=(char *)calloc(MINMEMSIZE,sizeof(char)))==0){	/* gb */
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}
	if((seqfile1->header=(char *)calloc(MAXLINE+MINMEMSIZE,sizeof(char)))==0){
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}
	if((seqfile1->seq=(char *)calloc(memsize,sizeof(char)))==0){		
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}

	if(fseek(fin,fileloc,SEEK_SET)){
		fprintf(stderr,"File seek error.\n");
		return NULL;
	}
	while(fgets(str1,MAXLINE+1,fin)!=NULL){

		if(embl_id_on==TRUE){
		
			if(linetype(str1,EMBL_INIT)){
				pick_string(str1,strlen(EMBL_INIT)+1,strlen(str1));
				if(strstr(str1,"circular")!=NULL) circular=TRUE;
				subst(str1,';',' ');
				sscanf(str1,"%s %s %s",IDstr,standard,dtype_char);
				if(!strcmp(dtype_char,"circular")){
					sscanf(str1,"%s %s %s %s",IDstr,standard,circular_char,dtype_char);
				}
			/* May 2, 2007: new EMBL format */
				if(!strcmp(standard,"SV")){
					sscanf(str1,"%s %s %s %s %s %s %s %s %s.",IDstr,SV,SVnum,circular_char,dtype_char,standard,div,length_char,bp_aa);
					if(!strcmp(dtype_char,"genomic") || !strcmp(dtype_char,"other") || !strcmp(dtype_char,"unassigned")){
						sscanf(str1,"%s %s %s %s %s %s %s %s %s %s.",IDstr,SV,SVnum,circular_char,genomic,dtype_char,standard,div,length_char,bp_aa);
					}
					if(strstr(dtype_char,"DNA")) dtype1=DNA;
					else if(strstr(dtype_char,"RNA")) dtype1=RNA;
			/* end */
				}else if(!strcmp(standard,"Reviewed") || !strcmp(standard,"Unreviewed")){
					dtype1=protein;
					strcpy(dtype_char,"PRT");
					subst(str1,'.',' ');
					sscanf(str1,"%s %s %s %s",IDstr,standard,length_char,bp_aa);	
				} else {
		
					if(strlen(dtype_char)<28) strcpy(seqfile1->dbtype,dtype_char);
					tail_string(dtype_char,3);
					if(!strcmp(dtype_char,"PRT")){
						if((dtype1==DNA) || (dtype1==RNA)){
							fprintf(stderr,"This is a protein sequence.\n");
							err_embl_format();
							return NULL;
						}
						sscanf(str1,"%s %s %s %s %s",IDstr,standard,dtype_char,length_char,bp_aa);	
					}
					else if(!strcmp(dtype_char,"DNA") || !strcmp(dtype_char,"RNA")){
						if(strcmp(circular_char,"circular")){	/* not circular */
							sscanf(str1,"%s %s %s %s %s %s",IDstr,standard,dtype_char,div,length_char,bp_aa);
						}
						else {	/* circular */
							sscanf(str1,"%s %s %s %s %s %s %s",IDstr,standard,circular_char,dtype_char,div,length_char,bp_aa);
						}
					}
					else if(!strcmp(dtype_char,"XXX")){
						sscanf(str1,"%s %s %s %s %s %s",IDstr,standard,dtype_char,div,length_char,bp_aa);
						strcpy(dtype_char,"unknown");
						fprintf(stderr,"\nStrange ID line, but we continue.\n");
						fprintf(stderr,"%s\n",str1);
					}
					else {
						strcpy(dtype_char,"unknown");
						err_embl_format();
						fprintf(stderr,"\nStrange ID line, but we continue.\n");
						fprintf(stderr,"%s\n",str1);
					}
				}

				if(!num_check(length_char)) sscanf(length_char,"%ld",&seqlength);

				if((strlen(IDstr)<13)) strcpy(seqfile1->ID,IDstr);
				else{
					fprintf(stderr,"\n%s\n",str1);
					err_embl_format();
					fprintf(stderr,"But we continue.\n");
				}					
				embl_id_on=FALSE;
			}
			continue;
		}

/* Setting flags depending on the linetype */
		if(linetype(str1,EMBL_AC)) embl_header_on=TRUE;			
		else if(linetype(str1,EMBL_DE)) {
			embl_header_on=TRUE;
			pick_string(str1,strlen(EMBL_DE)+3,strlen(str1));
		}
		else if(linetype(str1,EMBL_GN)) {
			embl_header_on=TRUE;
			pick_string(str1,strlen(EMBL_GN)+3,strlen(str1));
		}
		else if(linetype(str1,EMBL_OS)) {
			embl_header_on=TRUE;
			pick_string(str1,strlen(EMBL_OS)+3,strlen(str1));
		}
		else if(linetype(str1,EMBL_OC)) {
			embl_header_on=TRUE;
			pick_string(str1,strlen(EMBL_OC)+3,strlen(str1));
		}
		else if(linetype(str1,EMBL_OG)) {
			embl_header_on=TRUE;
			pick_string(str1,strlen(EMBL_OG)+3,strlen(str1));
		}

		else if(linetype(str1,EMBL_SEQ_INIT)){
			embl_seq_on=TRUE;
			embl_header_on=FALSE;
			continue;
		}
		
		else if(linetype(str1,EMBL_SEQ_END)){
			embl_seq_on=FALSE;
			break;
		}
		
		else embl_header_on=FALSE;
			
/* Reading header */		
		if(embl_header_on==TRUE){
			b=strlen(seqfile1->header)+strlen(str1)+MINMEMSIZE;
			if((seqfile1->header=(char *)realloc(seqfile1->header,b*sizeof(char)))==0){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			if(str1[strlen(str1)-1]==END_OF_LINE){
				str1[strlen(str1)-1]=PUNCTUATION_CHAR;
			}
			strcat(seqfile1->header,str1);
			continue;
		}
/* Reading sequence */
		
		if(embl_seq_on==TRUE){
			memsize=memsize+(strlen(str1))*(sizeof(char));
			if((seqfile1->seq=(char *)realloc(seqfile1->seq,memsize))==0){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			
			j=0;
			for(i=0;i<(long)strlen(str1);i++){
				if(islower((int)str1[i])) str1[i]=toupper(str1[i]);
/*				if(isupper((int)str1[i])){
					seqfile1->seq[offset+j]=str1[i];
					j++;
				}
 gaps and other characters are now allowed */
				if(!isdigit((int)str1[i]) && str1[i]!=' ' && !iscntrl(str1[i])){
					seqfile1->seq[offset+j]=str1[i];
					j++;
				}
/*** Dec. 21, 2000 ***/

			}
			offset=offset+j;				
		}
	}
			
	seqfile1->seq[offset]=0;
	
	seqfile1->seqlen=offset;
	if(seqlength!=offset){
		fprintf(stderr,"Sequence length is %ld in the header of file,\n",seqlength);
		fprintf(stderr,"but %ld bases were read.\n",offset);
	}
	if(seqfile1->seqlen==0) return NULL;
	if(circular) seqfile1->circular=TRUE;
	circular=circular_bak;

	if(strcmp(dtype_char,"DNA;")==0) seqfile1->dtype=DNA;
	else if(strcmp(dtype_char,"RNA;")==0) {
		dtype1=check_data_type(seqfile1);
		if(dtype1==DNA) seqfile1->dtype=dtype1;
	}
	else if(strcmp(dtype_char,"PRT;")==0) seqfile1->dtype=protein;
	else seqfile1->dtype=dtype1;
	
	fileloc=ftell(fin);
	*loc=fileloc;	
	return seqfile1;
}

/********************************************************************/
enum data_type check_data_type(Seqfile *seqfile1)
/********************************************************************/
{
	register long int i=0;
	register long int j=0;

	int s=0;
	long int code_count[30]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	long int total_resnum=0;
	long int total_charnum=0;
	long int DNA_res_count=0;
	long int RNA_res_count=0;
	long int ambiguous_count=0;
	enum data_type code_number;
	int e_code=0;
	

	for(i=0;i<(seqfile1->seqlen);i++){
		s=seqfile1->seq[i];

/*
		if(islower(s=(seqfile1->seq[i]))){
			s=toupper(s);
		}
		
		switch(s){
			case 'A':
				++code_count[A];
				break;
			case 'C':
				++code_count[C];
				break;
			case 'D':
				++code_count[D];
				break;
			case 'E':
				++code_count[E];
				break;
			case 'F':
				++code_count[F];
				break;
			case 'G':
				++code_count[G];
				break;
			case 'H':
				++code_count[H];
				break;
			case 'I':
				++code_count[I];
				break;
			case 'K':
				++code_count[K];
				break;
			case 'L':
				++code_count[L];
				break;
			case 'M':
				++code_count[M];
				break;
			case 'N':
				++code_count[N];
				break;
			case 'P':
				++code_count[P];
				break;
			case 'Q':
				++code_count[Q];
				break;
			case 'R':
				++code_count[R];
				break;
			case 'S':
				++code_count[S];
				break;
			case 'T':
				++code_count[T];
				break;
			case 'U':
				++code_count[U];
				break;
			case 'V':
				++code_count[V];
				break;
			case 'W':
				++code_count[W];
				break;
			case 'X':
				++code_count[X];
				break;
			case 'Y':
				++code_count[Y];
				break;
			default:
				if(isprint(s)){
					++code_count[0];	
				}
				break;
		}
*/
		
		if(isprint(s)){
			++j;
		}
		
	}
	e_code=c_comp(seqfile1,code_count);
	if(e_code){
		if(e_code==2){
			code_number=unknown;
			return code_number;
		}
		fprintf(stderr,"This is not a valid sequence in c_comp.\n");
		exit(1);
	}
	
	total_charnum=j;
	total_resnum=j-code_count[0];

	DNA_res_count=code_count[A]+code_count[C]+code_count[G]+code_count[T];
	RNA_res_count=code_count[A]+code_count[C]+code_count[G]+code_count[U];
	ambiguous_count=code_count[N]+code_count[R]+code_count[Y];
	
	if(DNA_res_count == total_resnum) code_number=DNA;

	else if(RNA_res_count == total_resnum) code_number=RNA;
	
	else if((RNA_res_count+ambiguous_count) >= (0.95 * total_resnum)) code_number=RNA;

	else if((DNA_res_count+ambiguous_count) >= (0.95 * total_resnum)) code_number=DNA;
		
	else if(code_count[U]<=1){	/* U will be a code for selenocystein */
		code_number=protein;
	}
	else{
		code_number=unknown;
	}
	return code_number;
}

/********************************************************************/
Boolean clear_gaps(Seqfile *seqfile1)
/* returns a sequence without numbers, spaces and marks. */
/********************************************************************/
{
	register long i=0;
	register long j=0;
	int s;
	
	while((s=seqfile1->seq[i])!=0){
		if(isalpha(s)){
			if(islower(s)) s=toupper(s);
			seqfile1->seq[j]=s;
			++j;
		}
		++i;
	}
	if(j>seqfile1->seqlen) {
		fprintf(stderr,"Error in sequence length.\n%ld,%ld",j,seqfile1->seqlen);
		return 1;
	}
	
	seqfile1->seq[j]=0;
	seqfile1->seqlen=j;
	return 0;
}


/*****************************************************/
Boolean clear_gapsM(Seqfile **seqfileM)
/*****************************************************/
{
	long N=num_seq;
	long i;

	for(i=0;i<N;i++){
		if(clear_gaps(seqfileM[i])){
			fprintf(stderr,"Error in clearing gaps in sequence: %s.\n",seqfileM[i]->ID);
			return 1;
		}
		if(i%10==0){
/*			printf(".");*/
			putc(8,stdout);
			putc('.',stdout);
			fflush(stdout);
		}
	}
	printf("\n");
	fflush(stdout);

	return 0;
}

/********************************************/
int cat_seq(Seqfile *seq1,Seqfile *seq2)
/* This function adds sequence from seq2 to 
the end of the sequence seq1.
Note that seq1 is modified by this function.*/ 
/********************************************/
{
	long nt=seq1->seqlen+seq2->seqlen;

	if(seq2->seqlen==0L) return 0;

	if((seq1->seq=(char*)realloc(seq1->seq,(nt+2)*sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return 1;
	}
	strcat(seq1->seq,seq2->seq);
	seq1->seqlen=strlen(seq1->seq);
	return 0;
}
/********************************************/

/********************************************/
int cat_aln(Seqfile **seq1,Seqfile **seq2)
/* This function adds alignment 2 to the end of
alignment 1. IDs are checked. Only the first 10 
characters of the ID strings need to match.*/
/********************************************/
{
	long n;
	int IDlen1;
	int IDlen2;
	char ID1[12];
	char ID2[12];

	for(n=0;n<num_seq;n++){
		IDlen1=strlen(seq1[n]->ID);
		IDlen2=strlen(seq2[n]->ID);
		if(IDlen1>10) {
			strncpy(ID1,seq1[n]->ID,10);
		}
		else strcpy(ID1,seq1[n]->ID);
		if(IDlen2>10) {
			strncpy(ID2,seq2[n]->ID,10);
		}
		else strcpy(ID2,seq2[n]->ID);
		if(strcmp(ID1,ID2)!=0){
			fprintf(stderr,"Sequence IDs do not match.\n");
			return 1;
		}
		if(cat_seq(seq1[n],seq2[n])){
			fprintf(stderr,"Out of memory.\n");
			return 1;
		}
	}
	return 0;
}

/********************************************/


/*****************************************************************************/
int pick_seq(Seqfile *seqfile1, long startpos, long endpos)
/* This function returns an extracted sequence. Therefore, the
original sequence is modified by this function. */
/*****************************************************************************/
{
	
/* Checking the positions */
        if((startpos>seqfile1->seqlen) || (endpos<startpos)){
                fprintf(stderr,"Error of extraction positions: %ld, %ld\n",startpos, endpos);
                return 1;
        }
/* if endpos is larger than the actual sequence length, we will set the
end position at the end of the sequence. This is practically necessary
in sequence manipulation. */
        if(endpos>seqfile1->seqlen) endpos=seqfile1->seqlen;
        if(startpos<1) startpos=1;

	if(pick_string(seqfile1->seq,startpos,endpos)){
		fprintf(stderr,"Error in picking up sequence.\n");
		return 1;
	}
	seqfile1->seqlen = strlen(seqfile1->seq);
	return 0;

}
/*****************************************************************************/

/********************************/
Seqfile *empty_seq5(void)
/********************************/
{
	Seqfile *seqfile2;

	if((seqfile2=(Seqfile*)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return NULL;
	}

	if((seqfile2->ID=(char *)calloc(MINMEMSIZE,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return NULL;
	}
	if((seqfile2->header=(char *)calloc(MINMEMSIZE,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return NULL;
	}
	if((seqfile2->seq=(char *)calloc(2,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return NULL;
	}

	seqfile2->dtype=unknown;
	seqfile2->circular=FALSE;
	strcpy(seqfile2->AC,"");

/* Setting sequence length */	
	seqfile2->seqlen=1;

	return seqfile2;
}



/*****************************************************************************/
Seqfile *extract_seq3(Seqfile *seqfile1, long int startpos, long int endpos)
/*****************************************************************************/
{
	Seqfile *seqfile2;

	if((seqfile2=(Seqfile*)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return NULL;
	}

/* Checking the positions */
        if((startpos>seqfile1->seqlen) || (endpos<startpos)){
                fprintf(stderr,"Error of extraction positions: %ld, %ld\n",startpos, endpos);
                return NULL;
        }
/* if endpos is larger than the actual sequence length, we will set the
end position at the end of the sequence. This is practically necessary
in sequence manipulation. */
        if(endpos>seqfile1->seqlen) endpos=seqfile1->seqlen;
        if(startpos<1) startpos=1;

/* Making ID  
	if((seqfile2->ID=(char *)calloc(strlen(seqfile1->ID)+30,sizeof(char)))==NULL){
 Obsolete */
	if((seqfile2->ID=(char *)calloc(strlen(seqfile1->ID)+2,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return NULL;
	}
/*
	sprintf(seqfile2->ID,"%sEx%ld_%ld",seqfile1->ID,startpos,endpos);	
 Obsolete. ID is now kept unchanged. Oct. 4, 1999.*/
	strcpy(seqfile2->ID,seqfile1->ID);	

/* Setting data type */	
	seqfile2->dtype=seqfile1->dtype;

/* Making header */	
	if((seqfile2->header=(char*)calloc(strlen(seqfile1->header)+50,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return NULL;
	}		
	sprintf(seqfile2->header,"%s_Extracted from %ld to %ld; ",seqfile1->header,startpos,endpos);	

/* Setting sequence */	
	seqfile2->seq=extract_string(seqfile1->seq,startpos,endpos);

/* Setting sequence length */	
	seqfile2->seqlen=strlen(seqfile2->seq);

	return seqfile2;
}


/********************************/
int to_compD(int s)
/* returns complementary residue.
DNA sequence.*/
/********************************/
{
	int j;

	if(islower(s)) s=toupper(s);
	
	for(j=0;j<16;j++) {
		if(s==n2c[0][j]) break;
	}
	if(j==16) j=0;
	
	return n2c[1][j];
}
/********************************/

/********************************/
int to_compR(int s)
/* returns complementary residue.
RNA sequence.*/
/********************************/
{
	int j;

	if(islower(s)) s=toupper(s);
	
	for(j=0;j<16;j++) {
		if(s==n2c[0][j]) break;
	}
	if(j==16) j=0;
	
	return n2c[2][j];
}
/********************************/


/******************************************************/
int rev_seq(Seqfile *seqfile1)
/* returns reversed sequence. Feb. 4, 2008 */ 
/******************************************************/
{
	long i=0;
	long length=seqfile1->seqlen;
	int s1,s2;
	enum data_type dtype;
	int pos;
	char *p;

	dtype=check_data_type(seqfile1);
	if(seqfile1->seq[length]!=0){
		fprintf(stderr,"Error in sequence file: %s.\n",seqfile1->ID);
		return 1;
	}
	
	while(i<(length-1-i)){
	
		s1=seqfile1->seq[i];
		s2=seqfile1->seq[length-i-1];
		seqfile1->seq[i]=s2;
		seqfile1->seq[length-i-1]=s1;
		++i;
	}

	if(tailtype(seqfile1->ID,"_R")){
		pos = strlen(seqfile1->ID) - 2;
		seqfile1->ID[pos]='\0';
		p = strstr(seqfile1->header,"Artificial reversal");
		if(p != NULL){
			*p = '\0';
		}
	} else {
		if((seqfile1->ID=(char *)realloc(seqfile1->ID,(strlen(seqfile1->ID)+4)*sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory.\n");
			return 1;
		}
		if((seqfile1->header=(char *)realloc(seqfile1->header,\
		(strlen(seqfile1->header)+strlen(seqfile1->ID)+35)*sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory.\n");
			return 1;
		}
		
		pos = strlen(seqfile1->header);
		if(pos <= 2){
			sprintf(seqfile1->header," Artificial reversal of: %s.",seqfile1->ID);
		} else {
			sprintf(seqfile1->header,"%s; Artificial reversal of: %s.",seqfile1->header,seqfile1->ID);
		}	
		strcat(seqfile1->ID,"_R");	
	}

	return 0;
}

/******************************************************/
int comp_seq4(Seqfile *seqfile1)
/* returns complementary sequence. 
New and simple algorithm. May 27, 1999.*/
/******************************************************/
{
	long i=0;
	long length=seqfile1->seqlen;
	int s1,s2;
/*	int residue_name;
	char *line;
	char *p;
	char original_ID[MAXLINE]="";
*/
	enum data_type dtype;

	dtype=check_data_type(seqfile1);
	if(dtype!=DNA && dtype!=RNA){
		fprintf(stderr,"This is not a nucleic acid sequence: %s.\n",seqfile1->ID);
		return 1;
	}
	if(seqfile1->seq[length]!=0){
		fprintf(stderr,"Error in sequence file: %s.\n",seqfile1->ID);
		return 1;
	}
	
	while(i<(length-1-i)){
	
		s1=seqfile1->seq[i];
		s2=seqfile1->seq[length-i-1];
		if(dtype==DNA) seqfile1->seq[i]=to_compD(s2);
		else seqfile1->seq[i]=to_compR(s2);

		if(dtype==DNA) seqfile1->seq[length-i-1]=to_compD(s1);
		else seqfile1->seq[length-i-1]=to_compR(s1);
		++i;
	}

	/* At this stage, the center remains unchanged.
	The following is to manage the center position. */

	if((length%2)==1){
		i=(length-1)/2;
		s1=seqfile1->seq[i];
		if(dtype==DNA) seqfile1->seq[i]=to_compD(s1);
		else seqfile1->seq[i]=to_compR(s1);
	}

	if((seqfile1->ID=(char *)realloc(seqfile1->ID,(strlen(seqfile1->ID)+4)*sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return 1;
	}
	if((seqfile1->header=(char *)realloc(seqfile1->header,\
	(strlen(seqfile1->header)+strlen(seqfile1->ID)+25)*sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return 1;
	}		
	sprintf(seqfile1->header,"%s;Complementary for: %s.",seqfile1->header,seqfile1->ID);	
	strcat(seqfile1->ID,"_C");	
	return 0;
}

/*****************************************************/
int comp_seqM(Seqfile **seqfileM)
/*****************************************************/
{
	long N=num_seq;
	long i;

	for(i=0;i<N;i++){
		if(comp_seq4(seqfileM[i])){
			fprintf(stderr,"Error in converting to complementary in sequence: %s.\n",seqfileM[i]->ID);
			return 1;
		}
		if(i%10==0){
/*			printf(".");*/
			putc(8,stdout);
			putc('.',stdout);
			fflush(stdout);
		}
	}
	printf("\n");
	fflush(stdout);

	return 0;
}

/********************************************************************/
int to_rna(Seqfile *seqfile1)
/* conversion to RNA sequence. Originally in toRNA.c,
which is no longer used. */
/********************************************************************/
{
	register long int i=0;
	int s;
	
	if(seqfile1->dtype!=DNA && seqfile1->dtype!=RNA) {
		fprintf(stderr,"This is not nucleic acid sequence.\n");
		return 1;
	}
	while((s=seqfile1->seq[i])!=0){
		if(isalpha(s)){
			if(islower(s)) s=toupper(s);
			if(s=='T') s='U';
			seqfile1->seq[i]=s;
		}
		++i;
	}
	seqfile1->dtype=RNA;
	if((seqfile1->ID=(char*)realloc(seqfile1->ID,strlen(seqfile1->ID)+6))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return 1;
	}
	strcat(seqfile1->ID,"_RNA");
	if((seqfile1->header=(char*)realloc(seqfile1->header,strlen(seqfile1->header)+21))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return 1;
	}
	strcat(seqfile1->header,"; Converted to RNA.");
	
	return 0;
}

/********************************************************************/
int to_dna(Seqfile *seqfile1)
/* conversion to DNA sequence.*/ 
/********************************************************************/
{
	register long int i=0;
	int s;

	if(seqfile1->dtype!=DNA && seqfile1->dtype!=RNA) {
		fprintf(stderr,"This is not nucleic acid sequence.\n");
		return 1;
	}

	while((s=seqfile1->seq[i])!=0){
		if(isalpha(s)){
			if(islower(s)) s=toupper(s);
			if(s=='U') s='T';
			seqfile1->seq[i]=s;
		}
		++i;
	}
	seqfile1->dtype=DNA;
	if((seqfile1->ID=(char*)realloc(seqfile1->ID,strlen(seqfile1->ID)+6))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return 1;
	}
	strcat(seqfile1->ID,"_DNA");
	if((seqfile1->header=(char*)realloc(seqfile1->header,strlen(seqfile1->header)+21))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return 1;
	}
	strcat(seqfile1->header,"; Converted to DNA.");
	
	return 0;
}

/*****************************************************/
int to_revM(Seqfile **seqfileM)
/*****************************************************/
{
	long N=num_seq;
	long i;

	for(i=0;i<N;i++){
		if(rev_seq(seqfileM[i])){
			fprintf(stderr,"Error in reverting sequence: %s.\n",seqfileM[i]->ID);
			return 1;
		}
		if(i%10==0){
			putc(8,stdout);
			putc('.',stdout);
			fflush(stdout);
		}
	}
	printf("\n");
	fflush(stdout);

	return 0;
}


/*****************************************************/
int to_rnaM(Seqfile **seqfileM)
/*****************************************************/
{
	long N=num_seq;
	long i;

	for(i=0;i<N;i++){
		if(to_rna(seqfileM[i])){
			fprintf(stderr,"Error in converting to complementary in sequence: %s.\n",seqfileM[i]->ID);
			return 1;
		}
		if(i%10==0){
/*			printf(".");*/
			putc(8,stdout);
			putc('.',stdout);
			fflush(stdout);
		}
	}
	printf("\n");
	fflush(stdout);

	return 0;
}


/*****************************************************/
int to_dnaM(Seqfile **seqfileM)
/*****************************************************/
{
	long N=num_seq;
	long i;

	for(i=0;i<N;i++){
		if(to_dna(seqfileM[i])){
			fprintf(stderr,"Error in converting to complementary in sequence: %s.\n",seqfileM[i]->ID);
			return 1;
		}
		if(i%10==0){
/*			printf(".");*/
			putc(8,stdout);
			putc('.',stdout);
			fflush(stdout);
		}
	}
	printf("\n");
	fflush(stdout);

	return 0;
}


/******************************************************/
Seqfile *copy_seq_h(Seqfile *seqfile1)
/******************************************************/
{
	Seqfile *seqfile2;

	if((seqfile2=(Seqfile*)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return NULL;
	}
	if((seqfile2->ID=(char *)calloc(strlen(seqfile1->ID)+2,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return NULL;
	}
	if((seqfile2->header=(char *)calloc(strlen(seqfile1->header)+2,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return NULL;
	}		
	if((seqfile2->seq=(char *)calloc(2,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return NULL;
	}
	
	strcpy(seqfile2->ID,seqfile1->ID);
	strcpy(seqfile2->AC,seqfile1->AC);
	strcpy(seqfile2->OS,seqfile1->OS);
	strcpy(seqfile2->header,seqfile1->header);
	seqfile2->seqlen=seqfile1->seqlen;
	seqfile2->dtype=seqfile1->dtype;
	seqfile2->circular=seqfile1->circular;

	return seqfile2;
}

/*******************************************************/


/******************************************************/
Seqfile *copy_seq(Seqfile *seqfile1)
/******************************************************/
{
	Seqfile *seqfile2;

	if((seqfile2=(Seqfile*)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return NULL;
	}
	if((seqfile2=copy_seq_h(seqfile1))==NULL){
		fprintf(stderr,"Error in copy_seq_h.\n");
		return NULL;
	}
	if((seqfile2->seq=(char *)calloc(strlen(seqfile1->seq)+2,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return NULL;
	}		
	strcpy(seqfile2->seq,seqfile1->seq);

	return seqfile2;
}

/*******************************************************/
void fprint_seq(FILE *file, Seqfile *seqfile1)
/*******************************************************/
{
	int c;

	fprintf(file,"%s\n",seqfile1->ID);
	fprintf(file,"%s\n",seqfile1->header);
	c=seqfile1->dtype;
	fprintf(file,"Data type is %s\n",data_type_name[c]);
	fprintf(file,"%ld\n",seqfile1->seqlen);
	fprintf(file,"%s\n",seqfile1->seq);
	return;
}

/*******************************************************/
int print_seq(Seqfile *seqfile1)
/*******************************************************/
{
	if(seqfile1==NULL){
		fprintf(stderr,"Empty sequence.\n");
		return 1;
	}
	
	fprint_seq(stdout,seqfile1);
	
	return 0;
}

/*******************************************************/
void free_seq(Seqfile *seqfile1)
/*******************************************************/
{
	if(seqfile1->ID!=NULL) free(seqfile1->ID);
	if(seqfile1->header!=NULL) free(seqfile1->header);
	if(seqfile1->seq!=NULL) free(seqfile1->seq);
	if(seqfile1!=NULL) free(seqfile1);
	return;
}


/**********************************************************************************/
int fwrite_Dinfo(FILE *fout, Seqfile *seqfile1,long window,long slide)
/* Cumulative skew values added. Jan 31 2002. */
/* Base composition is now in percentage. May 28, 2002. */
/* Z-curve method is implemented. Feb. 15, 2010.
   Zhang, R. and Zhang, C.-T. (2005) Archaea 1: 335-346. 
   Variable slide is used as an averaging window. */
/**********************************************************************************/
{
	long i;
	int j;
	double bcomp[4];
	enum data_type dtype1;
	long len=seqfile1->seqlen;
	long imax;
	long nuc_start=1L;
	long nuc_end=window;
	long nuc_end_Z=slide;	/* Z-curve */
	Seqfile *seq_tmp;
	double gcskew,atskew,gc,at,t,c,a,g;
	long n;
	double Nd,imaxd; 

	long N;	/* sum of n */
	double nT,nC,nA,nG,nGC,nAT,nGCs,nATs;	/* sum of t,c,a,g,gc,at,GCskew,ATskew */
	double nT2,nC2,nA2,nG2,nGC2,nAT2,nGCs2,nATs2;	/* sum of squares */

	double dT,dC,dA,dG,dGC,dAT,dGCs,dATs;	/* deviation */
	double tT,tC,tA,tG,tGC,tAT;	/* total value */

	long zT,zC,zA,zG;
	long x,y,z,atd,gcd;	/* variables for Z-curve method. atd = (x+y)/2, gcd = (x-y)/2. */

	if(seqfile1==NULL){
		fprintf(stderr,"Empty sequence.\n");
		return TRUE;
	}

	fprintf(fout,"DNA info for %s\n",seqfile1->ID);

/********************* start of printing info **********************/
	dtype1=seqfile1->dtype;
	if(dtype1!=DNA){
		fprintf(stderr,"This is not a DNA sequence. Skipping...\n");
		return TRUE;
	}
	imax=(long)(len/slide);
	if(window>len) imax=0;
	N=0L;
	nT=nC=nA=nG=nGC=nAT=nGCs=nATs=0.0;
	nT2=nC2=nA2=nG2=nGC2=nAT2=nGCs2=nATs2=0.0;
	dT=dC=dA=dG=dGC=dAT=dGCs=dATs=0.0;
	tT=tC=tA=tG=tGC=tAT=0.0;
	
	zT=zC=zA=zG=0L;
	x = y = z = atd = gcd = 0L;

	fprintf(fout,"   Window size: %ld; Slide size: %ld.\n\n",window,slide);
	fprintf(fout,"    From \t    To  \tLength\t   T \t   C \t   A  \t   G \t  GC \t  AT  \tGC skew\tAT skew\tcumGCskew\tcumATskew\t  x  \t  y  \t  z  \t  atd \t  gcd \n\n");

	for(i=0;i<imax+1;i++){

		if(nuc_start>len) break;
		if(nuc_end>len) {
			nuc_end=len;
			if(i<imax) imax=i;
		}
		if(nuc_end_Z>len) {
			nuc_end_Z=len;
//			if(i<imax) imax=i;
		}
	
		seq_tmp=extract_seq3(seqfile1,nuc_start,nuc_end);

		for(j=0;j<4;j++) bcomp[j]=0.0;
		if(c_basecomp(seq_tmp,bcomp)) {
			fprintf(stderr,"Error in calculating residue composition in fwrite_Dinfo.\n");
			fprintf(stderr,"The data are filled with zero, and then continue.\n");
		}
		n=nuc_end-nuc_start+1;
		t=bcomp[0];
		c=bcomp[1];
		a=bcomp[2];
		g=bcomp[3];
		gc=g+c;
		at=a+t;
		gcskew=CountGCSkew(seq_tmp->seq);
		atskew=CountATSkew(seq_tmp->seq);
		if(gcskew>1.1 || gcskew<-1.1 || atskew>1.1 || atskew<-1.1){
			fprintf(stderr,"Error in calculating skew values in fwrite_Dinfo.\n");
			if(gcskew > 998.0 && atskew > 998.0){
				fprintf(stderr,"But we continue.\n");
			} else {
#include        "exit1.h"
			}
		}

		N += n;
		nT += t * (double)n;
		nC += c * (double)n;
		nA += a * (double)n;
		nG += g * (double)n;
		nGC += gc * (double)n;
		nAT += at * (double)n;
		nGCs += gcskew * (double)n;
		nATs += atskew * (double)n;

		nT2 += t * t * (double)n;
		nC2 += c * c * (double)n;
		nA2 += a * a * (double)n;
		nG2 += g * g * (double)n;
		nGC2 += gc * gc * (double)n;
		nAT2 += at * at * (double)n;
		nGCs2 += gcskew * gcskew * (double)n;
		nATs2 += atskew * atskew * (double)n;

		tT += t * (double)n;
		tC += c * (double)n;
		tA += a * (double)n;
		tG += g * (double)n;


/* Z-curve */
		free_seq(seq_tmp);
		seq_tmp=extract_seq3(seqfile1,nuc_start,nuc_end_Z);

		for(j=0;j<4;j++) bcomp[j]=0.0;
		if(c_basecomp(seq_tmp,bcomp)) {
			fprintf(stderr,"Error in calculating residue composition for Z-curve in fwrite_Dinfo.\n");
			fprintf(stderr,"The data are filled with zero, and then continue.\n");
		}
		n = nuc_end_Z - nuc_start + 1;
		t = bcomp[0];
		c = bcomp[1];
		a = bcomp[2];
		g = bcomp[3];

		zT = (long)(t * (double)n);
		zC = (long)(c * (double)n);
		zA = (long)(a * (double)n);
		zG = (long)(g * (double)n);

		x += (zA + zG - zC - zT);
		y += (zA + zC - zG - zT);
		z += (zA + zT - zC - zG);
		atd = (x + y)/2;
		gcd = (x - y)/2;

/* end Z-curve */
	


		fprintf(fout,"%8ld\t%8ld\t%6ld\t%5.2f\t%5.2f\t%5.2f\t%5.2f\t%5.2f\t%5.2f\t%6.3f\t%6.3f\t%6.3f\t%6.3f\t%ld\t%ld\t%ld\t%ld\t%ld\n",\
			 nuc_start,nuc_end,n,100*t,100*c,100*a,100*g,100*gc,100*at,gcskew,atskew,nGCs/n,nATs/n,x,y,z,atd,gcd);

		free_seq(seq_tmp);
		nuc_start += slide;
		nuc_end += slide;
		nuc_end_Z += slide;
	
	}
	
	Nd=(double)N;
	imaxd=(double)(N - window);

	dT=sqrt((nT2 - (nT*nT)/Nd)/imaxd);
	dC=sqrt((nC2 - (nC*nC)/Nd)/imaxd);
	dA=sqrt((nA2 - (nA*nA)/Nd)/imaxd);
	dG=sqrt((nG2 - (nG*nG)/Nd)/imaxd);

	dGC=sqrt((nGC2 - (nGC*nGC)/Nd)/imaxd);
	dAT=sqrt((nAT2 - (nAT*nAT)/Nd)/imaxd);
	dGCs=sqrt((nGCs2 - (nGCs*nGCs)/Nd)/imaxd);
	dATs=sqrt((nATs2 - (nATs*nATs)/Nd)/imaxd);

	tT /= N;
	tC /= N;
	tA /= N;
	tG /= N;
	
	for(j=0;j<4;j++) bcomp[j]=0.0;
	if(c_basecomp(seqfile1,bcomp)) {
		fprintf(stderr,"Error in calculating residue composition in fwrite_Dinfo.\n");
#include        "exit1.h"
	}

	tGC=tG+tC;
	tAT=tA+tT;

	fprintf(fout,"\n");

	if(imax>0){
	    fprintf(fout,"\t Deviation        \t\t%5.2f\t%5.2f\t%5.2f\t%5.2f\t%5.2f\t%5.2f\t%6.3f\t%6.3f\n",\
			100*dT,100*dC,100*dA,100*dG,100*dGC,100*dAT,dGCs,dATs);

	    fprintf(fout,"    Total accumulated\t    %8ld\t%5.2f\t%5.2f\t%5.2f\t%5.2f\t%5.2f\t%5.2f\n",\
			N,100*tT,100*tC,100*tA,100*tG,100*tGC,100*tAT);
	}

	fprintf(fout,"\t Total              \t\t%5.2f\t%5.2f\t%5.2f\t%5.2f\n",\
			100*bcomp[0],100*bcomp[1],100*bcomp[2],100*bcomp[3]);

	fprintf(fout,"\n");
	fprintf(fout,"\n");
	fflush(fout);	
	return FALSE;
}

/**********************************************************************************/

/**********************************************************************************/
int write_Dinfo(char *filename,Seqfile *seqfile1,long window,long slide)
/**********************************************************************************/
{
	FILE *fout;
	int c;

	if(seqfile1==NULL){
		fprintf(stderr,"Empty sequence.\n");
		return 1;
	}
	
#ifdef MAC
#ifdef CODEWARRIOR
	_fcreator=CREATOR_NAME;
	_ftype=FILETYPE_NAME;
#endif
#endif

	if((fout=fopen(filename,"a+"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n",filename);
		return 1;
	}
	
	if((c=fwrite_Dinfo(fout,seqfile1,window,slide))!=0){
		fprintf(stderr,"Trouble in writing info: %s\n",filename);
		fclose(fout);
		return c;
	}
#ifdef VERBOUS
	fprintf(stdout,"DNA info written to file: %s\n",filename);
#else
	putc(8,stdout);
	putc('w',stdout);
	fflush(stdout);
#endif

	fclose(fout);
	return 0;
}

/**********************************************************************************/
int fwrite_info3(FILE *fout,Seqfile *seqfile1,char *codon_table,long *code_count)
/* prints codon usage if fout is defined. Always returns code_count. */
/**********************************************************************************/
{
	int i;
	enum data_type dtype1;
	long len=seqfile1->seqlen;
	char string[5]="   ";
	long aalen;

	if(seqfile1==NULL){
		fprintf(stderr,"Empty sequence.\n");
		return TRUE;
	}

	if(code_count==NULL){
		fprintf(stderr,"Empty code_count in fwrite_info3.\n");
		return TRUE;
	}

/********************* start of printing info **********************/
	dtype1=seqfile1->dtype;
	if(dtype1!=DNA) {
		fprintf(stderr,"Sequence should be DNA in fwrite_info3.\n");
		return TRUE;
	}
	initialize_comp2(code_count,65);
	if(codon_usage_comp(seqfile1,code_count)) {
		fprintf(stderr,"Error in calculating codon_usage in fwrite_info3.\n");
		return TRUE;
	}

	if(fout==NULL || fout==stdout) return FALSE;

	fprintf(fout,"%s: %ld nt\n\n",seqfile1->ID,len);
	aalen=len/3L;
	for(i=0;i<64;i++){
		if(get_codon(i,string)){
			fprintf(stderr,"Error in getting codon in fwrite_info3.\n");
			return TRUE;
		}
		fprintf(fout,"%c  %s = %6ld, %6.2f %%.\n",\
		codon_table[i],string,code_count[i],100*(double)code_count[i]/(double)aalen);
	}
	fprintf(fout,"\n");	
	fflush(fout);

	return FALSE;
}



/*******************************************************************/
int fwrite_info2(FILE *fout, Seqfile *seqfile1,char info_mode_opt)
/*******************************************************************/
{
	long len=seqfile1->seqlen;

	if(seqfile1==NULL){
		fprintf(stderr,"Empty sequence.\n");
		return 1;
	}

	switch(info_mode_opt){
		case 'l':
			fprintf(fout,"%ld\n",len);
			break;
		case 's':
			fprintf(fout,"%s\n",seqfile1->seq);
			break;
		case 'n':
		default:
			fprintf(fout,"%s\n",seqfile1->ID);
			break;
	}
	fflush(fout);	
	return 0;
}


/**********************************************************************************/
int fwrite_info(FILE *fout,Seqfile *seqfile1,Boolean include_header,int id_h_sep)
/**********************************************************************************/
{
	long i;
	int s, t;
	long code_count[30];
	enum data_type dtype1;
	long len=seqfile1->seqlen;

	if(seqfile1==NULL){
		fprintf(stderr,"Empty sequence.\n");
		return 1;
	}

	fprintf(fout,"%s%s",FASTA_INIT,seqfile1->ID);

	if(include_header){
		s=seqfile1->header[0];	
		if(s!=' ' && s!='\0' && s!=END_OF_LINE && s!=END_OF_LINE2){
			fputc(id_h_sep,fout);
		}
		for(i=0;((s=seqfile1->header[i])!=0);++i){
			if(s==PUNCTUATION_CHAR){
				fputc('\n',fout);
				continue;
			}
			if(isprint(s)) {
				if((t=fputc(s,fout))==EOF){
					fprintf(stderr,"Writing error.\n");
					return 2;
				}
			}
			else if(s==END_OF_LINE && strlen(seqfile1->header)>1) fputc(PUNCTUATION_CHAR,fout);
		}				
	}
	fputc('\n',fout);

/********************* start of printing info **********************/
	dtype1=seqfile1->dtype;

	initialize_comp(code_count);
	if(c_comp(seqfile1,code_count)) {
		fprintf(stderr,"Error in calculating residue composition in fwrite_info.\n");
#include        "exit1.h"
	}

	fprintf(fout,"   Data type: %s.\n",data_type_name[dtype1]);
	fprintf(fout,"   Sequence length: %ld.\n\n",len);
	if(dtype1==DNA || dtype1==RNA){
		fprintf(fout,"   A = %6ld, %6.2f %%.\n",code_count[A],100*(double)code_count[A]/(double)len);
		fprintf(fout,"   C = %6ld, %6.2f %%.\n",code_count[C],100*(double)code_count[C]/(double)len);
		fprintf(fout,"   G = %6ld, %6.2f %%.\n",code_count[G],100*(double)code_count[G]/(double)len);
		if(dtype1==DNA){
			fprintf(fout,"   T = %6ld, %6.2f %%.\n",code_count[T],100*(double)code_count[T]/(double)len);
		}
		else{
			fprintf(fout,"   U = %6ld, %6.2f %%.\n",code_count[U],100*(double)code_count[U]/(double)len);
		}
	}
	else {
		fprintf(fout,"   A = %6ld, %6.2f %%.\n",code_count[A],100*(double)code_count[A]/(double)len);
		fprintf(fout,"   C = %6ld, %6.2f %%.\n",code_count[C],100*(double)code_count[C]/(double)len);
		fprintf(fout,"   D = %6ld, %6.2f %%.\n",code_count[D],100*(double)code_count[D]/(double)len);
		fprintf(fout,"   E = %6ld, %6.2f %%.\n",code_count[E],100*(double)code_count[E]/(double)len);
		fprintf(fout,"   F = %6ld, %6.2f %%.\n",code_count[F],100*(double)code_count[F]/(double)len);
		fprintf(fout,"   G = %6ld, %6.2f %%.\n",code_count[G],100*(double)code_count[G]/(double)len);
		fprintf(fout,"   H = %6ld, %6.2f %%.\n",code_count[H],100*(double)code_count[H]/(double)len);
		fprintf(fout,"   I = %6ld, %6.2f %%.\n",code_count[I],100*(double)code_count[I]/(double)len);
		fprintf(fout,"   K = %6ld, %6.2f %%.\n",code_count[K],100*(double)code_count[K]/(double)len);
		fprintf(fout,"   L = %6ld, %6.2f %%.\n",code_count[L],100*(double)code_count[L]/(double)len);
		fprintf(fout,"   M = %6ld, %6.2f %%.\n",code_count[M],100*(double)code_count[M]/(double)len);
		fprintf(fout,"   N = %6ld, %6.2f %%.\n",code_count[N],100*(double)code_count[N]/(double)len);
		fprintf(fout,"   P = %6ld, %6.2f %%.\n",code_count[P],100*(double)code_count[P]/(double)len);
		fprintf(fout,"   Q = %6ld, %6.2f %%.\n",code_count[Q],100*(double)code_count[Q]/(double)len);
		fprintf(fout,"   R = %6ld, %6.2f %%.\n",code_count[R],100*(double)code_count[R]/(double)len);
		fprintf(fout,"   S = %6ld, %6.2f %%.\n",code_count[S],100*(double)code_count[S]/(double)len);
		fprintf(fout,"   T = %6ld, %6.2f %%.\n",code_count[T],100*(double)code_count[T]/(double)len);
		fprintf(fout,"   V = %6ld, %6.2f %%.\n",code_count[V],100*(double)code_count[V]/(double)len);
		fprintf(fout,"   W = %6ld, %6.2f %%.\n",code_count[W],100*(double)code_count[W]/(double)len);
		fprintf(fout,"   Y = %6ld, %6.2f %%.\n",code_count[Y],100*(double)code_count[Y]/(double)len);
		fprintf(fout,"\n");	
                fprintf(fout,"   Mol. Wt. = %10.2f \n", getProtMW(seqfile1));
 
	}	

	fputc('\n',fout);
	fputc('\n',fout);
	fflush(fout);	
	return 0;
}



/*************************************************************************************/
int write_info(char *filename,Seqfile *seqfile1,Boolean include_header,int id_h_sep)
/*************************************************************************************/
{
	FILE *fout;
	int c;

	if(seqfile1==NULL){
		fprintf(stderr,"Empty sequence.\n");
		return 1;
	}
	
#ifdef MAC
#ifdef CODEWARRIOR
	_fcreator=CREATOR_NAME;
	_ftype=FILETYPE_NAME;
#endif
#endif

	if((fout=fopen(filename,"a+"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n",filename);
		return 1;
	}
	
	if((c=fwrite_info(fout,seqfile1,include_header,id_h_sep))!=0){
		fprintf(stderr,"Trouble in writing info: %s\n",filename);
		fclose(fout);
		return c;
	}
#ifdef VERBOUS
	fprintf(stdout,"Sequence info written to file: %s\n",filename);
#else
	putc(8,stdout);
	putc('w',stdout);
	fflush(stdout);
#endif

	fclose(fout);
	return 0;
}


/**********************************************************************************/
int fwrite_fasta(FILE *fout,Seqfile *seqfile1,Boolean include_header,int id_h_sep)
/**********************************************************************************/
{
	register long i;
	register long j;
	int s, t;

#ifndef INTEGRATED
	char *str=getenv("PRINTLINE");

	if(str!=NULL){
/*		printf("PRINTLINE is %s.\n",str);
*/
		if(!num_check(str)) print_line=atoi(str);
	}
#ifdef VERBOUS
	printf("Line length is %d characters.\n",print_line);
#endif

#endif

	if(seqfile1==NULL){
		fprintf(stderr,"Empty sequence.\n");
		return 1;
	}
	
	if((t=fputc(FASTA_INIT_CHAR,fout))==EOF){
		fprintf(stderr,"Writing error.\n");
		return 2;
	}
	
	for(i=0;((s=seqfile1->ID[i])!=0);++i){
		if(isprint(s)) {
		/* Writing error is always checked. If fputc returns EOF,
		   there is a problem in writing file. */
			if((t=fputc(s,fout))==EOF){
				fprintf(stderr,"Writing error.\n");
				return 2;
			}
		}
	}

/* needs revision */

	if(include_header){
		s=seqfile1->header[0];	
		if(s!=' ' && s!='\0' && s!=END_OF_LINE && s!=END_OF_LINE2) fputc(id_h_sep,fout);	/* Oct. 4, 1999 */
		for(i=0;((s=seqfile1->header[i])!=0);++i){
			if(isprint(s)) {
				if((t=fputc(s,fout))==EOF){
					fprintf(stderr,"Writing error.\n");
					return 2;
				}
			}
			else if(s==END_OF_LINE && strlen(seqfile1->header)>1) fputc(PUNCTUATION_CHAR,fout);
		}				
	}
	fputc('\n',fout);
	
	j=0;
	for(i=0;((s=seqfile1->seq[i])!='\0');++i){
		if(isprint(s)) {
			if((t=fputc(s,fout))==EOF){
				fprintf(stderr,"Writing error.\n");
				return 2;
			}
			++j;
		}
		if((i+1)%print_line==0){
			fputc('\n',fout);
		} else if(s=='\n'){
			fputc(s,fout);
			seqfile1->seq[i]='-';
			++j;
		}
	}
	
	/*This is another type of error. Maybe a strange character in the sequence.*/
	if(j != seqfile1->seqlen){
		fprintf(stderr,"Error in sequence length: j=%ld seqlen=%ld.\n",j,seqfile1->seqlen);
		return 3;
	}
	
	fputc('\n',fout);
	fputc('\n',fout);
	fflush(fout);	
	return 0;
}


/*************************************************************************************/
int write_fasta(char *filename,Seqfile *seqfile1,Boolean include_header,int id_h_sep)
/*************************************************************************************/
{
	FILE *fout;
	int c;

	if(seqfile1==NULL){
		fprintf(stderr,"Empty sequence.\n");
		return 1;
	}
	
#ifdef MAC
#ifdef CODEWARRIOR
	_fcreator=CREATOR_NAME;
	_ftype=FILETYPE_NAME;
#endif
#endif

	if((fout=fopen(filename,"a+"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n",filename);
		return 1;
	}
	
	if((c=fwrite_fasta(fout,seqfile1,include_header,id_h_sep))!=0){
		fprintf(stderr,"Trouble in writing file: %s\n",filename);
		fclose(fout);
		return c;
	}
#ifdef VERBOUS
	fprintf(stdout,"Sequence written to file: %s\n",filename);
#else
/*	printf("w");*/
	putc(8,stdout);
	putc('w',stdout);
	fflush(stdout);
#endif

	fclose(fout);
	return 0;
	

}
	
/*************************************************************************************/
Seqfile **read_phylip(char *filename, enum data_type dtype1)
/* This is a function to read Phylip file in interleaved format. Jan. 31, 2001.*/
/*************************************************************************************/
{
	Seqfile **seqfile1;
	register long k=0L;
	long offset = 0L;
	long n;		/* number of sequences in the first line */
	long s;		/* nuumber of sites in the first line */
	int linelength=0;
	long int memsize=MAXLINE;	/* for memory usage */
	char str1[MAXLINE+1];
	char str2[MAXLINE+1];		/* The third argument in the first line of Phylip file. 
					This should be always I or i, interleaved format. */
	char IDstr[MINMEMSIZE];
	char seqstr[MAXLINE];
	char *p;	/* temporary character string. */
	Boolean clustal_on=FALSE;	/* This flag is used although this is not a clustal file. */
	Boolean first_stack=TRUE;
	FILE *fin;


	if((seqfile1=(Seqfile**)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return NULL;
	}
	if((fin=fopen(filename,"r"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n\n",filename);
		return NULL;
	}

	while(fgets(str1,MAXLINE+1,fin)!=NULL){
	
	if(count_characters(str1)<1) {
		if(first_stack==TRUE && k>1){
			if(k!=n){
				fprintf(stderr,"Number of sequences is not correct.\n");
				return NULL;
			}
			num_seq=k;
			k=0;
			offset=linelength;
			first_stack=FALSE;
		}
		continue;
	}

	if(clustal_on==FALSE) {
		sscanf(str1,"%ld %ld %s",&n,&s,str2);
		if(n<=1 || s<=10) {
			fprintf(stderr,"Strange sequence alignment.\n");
			return NULL;
		}
		if(str2[0]!='I' && str2[0]!='i'){
			fprintf(stderr,"Specify I in the first line in a Phylip interleaved file .\n");
			return NULL;
		}

		if((seqfile1=(Seqfile**)calloc(1,sizeof(Seqfile)))==0){
			fprintf(stderr,"Out of memory\n");
			return NULL;
		}
		clustal_on=TRUE;
		continue;
	}
	else {
		if(first_stack==TRUE) {
			subst(str1,'\t',' ');
			p=str_shift(str1,' ');
			if(strlen(str1)>20) {
				fprintf(stderr,"Too long ID name.\n");
				return NULL;
			}
			else strcpy(IDstr,str1);
			strcpy(str1,p);
			if(k>0) {
				if(strcmp(seqfile1[0]->ID,IDstr)==0) {
					if(k==1) {
						fprintf(stderr,"Only one sequence.\n");
						return NULL;
					}
				}
			}
			

			/* Memory allocation for the k-th sequence */

			if((seqfile1=(Seqfile**)realloc(seqfile1,(k+2)*sizeof(Seqfile)))==0){
                                fprintf(stderr,"Out of memory\n");
                                return NULL;
                        }

			if((seqfile1[k]=(Seqfile *)calloc(1,sizeof(Seqfile)))==0){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			if((seqfile1[k]->ID=(char *)calloc(MINMEMSIZE,sizeof(char)))==0){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			if((seqfile1[k]->header=(char *)calloc(MINMEMSIZE,sizeof(char)))==0){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			if((seqfile1[k]->seq=(char *)calloc(MAXLINE,sizeof(char)))==0){		
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			strcpy(seqfile1[k]->ID,IDstr);
		}
		else {
			if(k>=num_seq) {
				k=0;
				offset=offset+linelength;
			}

			/* Memory allocation for the k-th sequence */

			memsize=strlen(seqfile1[k]->seq)+MAXLINE;
			if((seqfile1[k]->seq=(char *)realloc(seqfile1[k]->seq,memsize*sizeof(char)))==0){		
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}	
		}

		strcpy(seqstr,str1);
		strcpy(str1,"");
		pack_seq(seqstr,"-*");

		if(k>0) {
			if((int)strlen(seqstr)!=linelength) {
				fprintf(stderr,"Line length is not identical.\n");
				fprintf(stderr,"linelength= %d, seqstr= %d\n",linelength,(int)strlen(seqstr));
				fprintf(stderr,"seqstr: %s\n",seqstr);
				return NULL;
			}
		}

		strcat(seqfile1[k]->seq,seqstr);

		if(k==0) linelength=strlen(seqstr);

		k++;
	}

	}	/* end while */

	if(k!=num_seq) {
		fprintf(stderr,"Sequence might not be complete.\n");
		fprintf(stderr,"k= %ld, num_seq= %ld.\n",k,num_seq);
	}
	
	offset=offset+linelength;
	if(offset!=s){
		fprintf(stderr,"Error in sequence length.\n");
		fprintf(stderr,"In the first line, it is %ld, while it is in fact %ld.\n",s,offset);
		if(k!=num_seq) return NULL;
	
	}

	for(k=0;k<num_seq;k++){
		seqfile1[k]->seq[offset]=0;	
		seqfile1[k]->seqlen=offset;
	}

	fclose(fin);
	
	if(dtype1==unknown){
		dtype1=check_data_type(seqfile1[0]);
		for(k=0;k<num_seq;k++){
			seqfile1[k]->dtype=dtype1;
		}
	}
	
	return seqfile1;
}


/*************************************************************************************/
int fwrite_phylip(FILE *fout, Seqfile **seqfile1, int ID_len, int I_flag)
/*************************************************************************************/
{
	register long int i,j,k;
	int s, t;
	int line_length=50;
	int block_size=10;
	Boolean flag_first_block=TRUE;

	if(seqfile1==NULL){
		fprintf(stderr,"Empty sequence.\n");
		return 1;
	}
	if(I_flag ==1) fprintf(fout,"%ld\t%ld\tI\n",num_seq,seqfile1[0]->seqlen);
	else fprintf(fout,"%ld\t%ld\n",num_seq,seqfile1[0]->seqlen);

	for(j=0;j<(long)strlen(seqfile1[0]->seq);j+=line_length){
		
		for(k=0;k<num_seq;++k){

			if(flag_first_block){
				for(i=0;i<ID_len+1;++i){
					if(i>=ID_len) s=' ';
					else if(i>=(long)strlen(seqfile1[k]->ID)) s=' ';
					else if((s=seqfile1[k]->ID[i])==0) s=' ';

					if(isprint(s)) {
					/* Writing error is always checked. If fputc returns EOF,
					   there is a problem in writing file. */
						if((t=fputc(s,fout))==EOF){
							fprintf(stderr,"Writing error.\n");
							return 2;
						}
					}
				}
			}
			else{
				fprintf(fout,"           ");
			}

			for(i=j;((s=seqfile1[k]->seq[i])!=0);++i){
				if(i>=(j+line_length)) 	break;
				if(isprint(s)) {
					if((t=fputc(s,fout))==EOF){
						fprintf(stderr,"Writing error.\n");
						return 2;
					}
				}
				if(i%block_size==9 && i<j+line_length-1) fputc(' ',fout);
			}
			fputc('\n',fout);

		}

        fputc('\n',fout);
	flag_first_block=FALSE;
	}	
/*   fputc('\n',fout);*/
	return 0;

}


/*************************************************************************************/
int write_phylip(char *filename,Seqfile **seqfile1,int ID_len, int I_flag)
/*************************************************************************************/
{
	FILE *fout;
	int c;

	if(seqfile1==NULL){
		fprintf(stderr,"Empty sequence.\n");
		return 1;
	}
	
#ifdef MAC
#ifdef CODEWARRIOR
	_fcreator=CREATOR_NAME;
	_ftype=FILETYPE_NAME;
#endif
#endif

	if((fout=fopen(filename,"w"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n",filename);
		return 1;
	}
	
	if((c=fwrite_phylip(fout,seqfile1,ID_len,I_flag))!=0){
		fprintf(stderr,"Trouble in writing file: %s\n",filename);
		fclose(fout);
		return c;
	}
#ifdef VERBOUS
	fprintf(stdout,"Sequence written to file: %s\n",filename);
#else
	putc(8,stdout);
	putc('w',stdout);
	fflush(stdout);
#endif

	fclose(fout);
	return FALSE;
}


/*************************************************************************************/
int fwrite_mega(FILE *fout, char *fname,Seqfile **seqfile1)
/* Note that this command generates a file with Windows-type line endings. */
/*************************************************************************************/
{
	long int i,j,k;
	int s, t;
	int datatype;
	char str1[20];
	int line_length=60;

	if(seqfile1==NULL){
		fprintf(stderr,"Empty sequence.\n");
		return 1;
	}
	datatype = seqfile1[0]->dtype;
	if(datatype==DNA) strcpy(str1,"dna");
	else if(datatype==RNA) strcpy(str1,"rna");
	else if(datatype==protein) strcpy(str1,"protein");
	else strcpy(str1,"unknown");	/* This does not make sense. */

	fprintf(fout,"#Mega\n");
	fprintf(fout,"Title: %s\r\n",fname);
	fprintf(fout,"\r\n");

	for(k=0;k<num_seq;++k){
		if(k > 0) fprintf(fout,"\r\n");
		subst(seqfile1[k]->ID,'=','_');		/* 130.52 */
		fprintf(fout,"#%s\r\n",seqfile1[k]->ID);
		j=0;
		for(i=0;((s=seqfile1[k]->seq[i])!=0);++i){
			if(isprint(s)) {
				if((t=fputc(s,fout))==EOF){
					fprintf(stderr,"Writing error.\n");
					return 2;
				}
				++j;
			}
			if((i+1)%line_length==0){
				fputc('\r',fout);
				fputc('\n',fout);
			} 
		}
		
		if(j!=seqfile1[k]->seqlen){
			fprintf(stderr,"Error in sequence length.\n");
			return 3;
		}
		if(k < num_seq-1) fprintf(fout,"\r\n");
	}
	fputc('\r',fout);
	fputc('\n',fout);

	return 0;
}


/*************************************************************************************/
int write_mega(char *filename,char *fname, Seqfile **seqfile1)
/*************************************************************************************/
{
	FILE *fout;
	int c;
/*	int i;	*/

	if(seqfile1==NULL){
		fprintf(stderr,"Empty sequence.\n");
		return 1;
	}
	
#ifdef MAC
#ifdef CODEWARRIOR
	_fcreator=CREATOR_NAME;
	_ftype=FILETYPE_NAME;
#endif
#endif

	if((fout=fopen(filename,"w"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n",filename);
		return 1;
	}

	fprintf(stdout,"Writing a file with Windows-type line endings.\n");
	if((c=fwrite_mega(fout,fname,seqfile1))!=0){
		fprintf(stderr,"Trouble in writing file: %s\n",filename);
		fclose(fout);
		return c;
	}
#ifdef VERBOUS
	fprintf(stdout,"Sequence written to file: %s\n",filename);
	fprintf(stdout,"Note that this file has Windows-type line endings.\n");
#else
	putc(8,stdout);
	putc('w',stdout);
	fflush(stdout);
#endif

	fclose(fout);
	return FALSE;
}


/*************************************************************************************/
int fwrite_nexus(FILE *fout, Seqfile **seqfile1, int ID_len)
/*************************************************************************************/
{
	long int i,j,k;
	int s, t;
	int datatype;
	char str1[20];
	int line_length=50;

#ifndef INTEGRATED
	char *str=getenv("PRINTLINE");

	if(str!=NULL){
		if(!num_check(str)) print_line=atoi(str);
	}
#ifdef VERBOUS
	printf("Line length is %d characters.\n",print_line);
#endif

#endif

	line_length = print_line - print_line % 10 + 10;

	if(seqfile1==NULL){
		fprintf(stderr,"Empty sequence.\n");
		return 1;
	}
	datatype = seqfile1[0]->dtype;
	if(datatype==DNA) strcpy(str1,"dna");
	else if(datatype==RNA) strcpy(str1,"rna");
	else if(datatype==protein) strcpy(str1,"protein");
	else strcpy(str1,"unknown");	/* This does not make sense. */

	fprintf(fout,"#NEXUS\n");
	fprintf(fout,"begin data;\n");
	fprintf(fout,"   dimensions ntax=%ld nchar=%ld;\n",num_seq,seqfile1[0]->seqlen);	
	fprintf(fout,"   format datatype=%s interleave=yes gap=-;\n",str1);
	fprintf(fout,"   matrix\n");

	for(j=0;j<(long)strlen(seqfile1[0]->seq);j+=line_length){

		for(k=0;k<num_seq;++k){
				subst(seqfile1[k]->ID,'=','_');		/* 130.52 */
				for(i=0;i<ID_len+1;++i){
					if(i>=ID_len) s=' ';
					else if(i>=(long)strlen(seqfile1[k]->ID)) s=' ';
					else if((s=seqfile1[k]->ID[i])==0) s=' ';

					if(isprint(s)) {
					/* Writing error is always checked. If fputc returns EOF,
					   there is a problem in writing file. */
						if((t=fputc(s,fout))==EOF){
							fprintf(stderr,"Writing error.\n");
							return 2;
						}
					}
				}

			for(i=j;((s=seqfile1[k]->seq[i])!=0);++i){
				if(i>=(j+line_length)) 	break;
				if(isprint(s)) {
					if((t=fputc(s,fout))==EOF){
						fprintf(stderr,"Writing error.\n");
						return 2;
					}
				}
			}
			fputc('\n',fout);

		}
		fputc('\n',fout);

	}

	fprintf(fout,";\n");
	fprintf(fout,"end;\n");

	return 0;
}


/*************************************************************************************/
int write_nexus(char *filename,Seqfile **seqfile1)
/*************************************************************************************/
{
	FILE *fout;
	int c;
	int maxIDlen=0;
	int i;

	if(seqfile1==NULL){
		fprintf(stderr,"Empty sequence.\n");
		return 1;
	}
	
#ifdef MAC
#ifdef CODEWARRIOR
	_fcreator=CREATOR_NAME;
	_ftype=FILETYPE_NAME;
#endif
#endif

	if((fout=fopen(filename,"w"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n",filename);
		return 1;
	}

	for(i=0;i<num_seq;i++){
		if(maxIDlen < strlen(seqfile1[i]->ID)) maxIDlen = strlen(seqfile1[i]->ID);
	}
	if(maxIDlen < 10) maxIDlen = 10;
	if((c=fwrite_nexus(fout,seqfile1,maxIDlen))!=0){
		fprintf(stderr,"Trouble in writing file: %s\n",filename);
		fclose(fout);
		return c;
	}
#ifdef VERBOUS
	fprintf(stdout,"Sequence written to file: %s\n",filename);
#else
	putc(8,stdout);
	putc('w',stdout);
	fflush(stdout);
#endif

	fclose(fout);
	return FALSE;
}


/*************************************************************************************/
void sort_seq(Seqfile **seq)
/*************************************************************************************/
{
	long num=num_seq - 1L;

	qSortSeq(seq,0,num);

}
/*************************************************************************************/


/*************************************************************************************/
static void qSortSeq(Seqfile **seq,long L,long r)
/*************************************************************************************/
{
        long v;

        if(L>=r) return;

        v=partition(seq,L,r);
        
        qSortSeq(seq,L,v-1);

        qSortSeq(seq,v+1,r);

        return;
}
/*************************************************************************************/

/*************************************************************************************/
static int partition(Seqfile **seq, long L, long r)
/*************************************************************************************/
{
        long i,j;
	char pivot[MINMEMSIZE];
        Seqfile *tmp;

        i=L-1;
        j=r;
        strcpy(pivot,seq[r]->ID);

        while(TRUE){
                while(strcmp(seq[++i]->ID,pivot) < 0);
                while(i < --j && strcmp(pivot,seq[j]->ID) < 0);
                if(i >= j) break;
                tmp = seq[i];
                seq[i] = seq[j];
                seq[j] = tmp;
        }
        tmp = seq[i];
        seq[i] = seq[r];
        seq[r] = tmp;

        return i;
}
/*************************************************************************************/


/*************************************************************************************/
int match_seq(Seqfile **seq,char *listfile,int mode_flag)
/* In this function, sequences are arranged according to the list in listfile.*/
/* mode_flag=0 normal. mode_flag=2 a list of genes/organism. */
/* mode_flag=1 sequences are sorted according to the list.*/
/*************************************************************************************/
{
	FILE *fin;
	char name[100];
	char name1[100];
	char str1[MAXLINE+1];
	Seqfile *temp;
	long i,j,i0,k;
	int count;
	char *name_i, *name_j;
	Boolean flag;
	char *p, *q;
	char *seq1, *seq2;
	long seqlen;

	if((fin=fopen(listfile,"r"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n\n",listfile);
		return TRUE;
	}
	i0 = 0;
	while(fgets(str1,MAXLINE+1,fin)!=NULL){
		if(!strcmp(str1,"")) continue;
		count = 0;
		sscanf(str1,"%s",name);
		strcpy(name1,name);
		if(mode_flag!=1) strcat(name1,"_");
		for(i=i0;i<num_seq;i++){
			if(linetype(seq[i]->ID,name1)){
				count += 1;
				if(i != i0){
					temp = seq[i0];
					seq[i0] = seq[i];
					seq[i] = temp;
				}
				i0 += 1;
				continue;
			}
		}
		if(mode_flag == 2){
			printf("%s\t%d\n",name,count);
		} else {
			if(count > 1){
				printf("%d sequences for %s\n",count,name);
			} else if(count == 0){
				printf("No sequence for %s\n",name);
			}
		}
	}
	fclose(fin);
	num_seq = i0;
	if(mode_flag == 2) return FALSE;

	seqlen = seq[0]->seqlen;
	if((seq1=(char*)calloc(seqlen+2,sizeof(char)))==0){
		fprintf(stderr,"Out of memory\n");
		return TRUE;
	}
	if((seq2=(char*)calloc(seqlen+2,sizeof(char)))==0){
		fprintf(stderr,"Out of memory\n");
		return TRUE;
	}
	
	i0 = 0;
	for(i=i0;i<num_seq;i++){
		strcpy(str1,seq[i]->ID);
		name_i = strtok(str1,"_");
		for(j=i+1;j<num_seq;j++){
			strcpy(str1,seq[j]->ID);
			name_j = strtok(str1,"_");
			flag = FALSE;
			if(!strcmp(name_i,name_j)){
				strcpy(seq1,seq[i]->seq);
				strcpy(seq2,seq[j]->seq);
				pack_seq(seq1," ");
				pack_seq(seq2," ");
				if(strlen(str1) == strlen(seq2)){
					for(p=seq1,q=seq2;*p!='\0' && *q!='\0';p++,q++){
						if(*p != *q) break;
					}
					if(*p == *q) flag = TRUE;
				}
			}
			if(flag){
				printf("Sequence %s is being removed\n",name_j);
				for(k=j;k<num_seq-1;k++){
					seq[k] = seq[k+1];
				}
				num_seq -= 1;
			}
		}
	}

	return FALSE;
}
/*************************************************************************************/


/*************************************************************************************/
void read_clustal2(char *filename, Seqfile **seqfile1)
/*************************************************************************************/
{
	long i=0;
	long k=0;
	long offset = 0;
	int linelength=0;
	int filetype1=0;
/*	long memsize=MAXLINE;	for memory usage */
	char str1[MAXLINE+1];
	char IDstr[MINMEMSIZE];
	char seqstr[MAXLINE];
	Boolean clustal_on=FALSE;
	Boolean first_stack=TRUE;
	Boolean wait_for_second_stack=FALSE;
	FILE *fin;
	enum data_type dtype1;

	if((fin=fopen(filename,"r"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n\n",filename);
		return;
	}

	while(fgets(str1,MAXLINE+1,fin)!=NULL){
	
#ifdef VERBOUS  
	printf("%s \n",str1);
	printf("k= %ld, i= %ld \n",k,i);
#endif

		if(strlen(str1)<=10 || linetype(str1,"     ")) {
			if(first_stack==TRUE && k>1){
				num_seq=k;
				k=0;
				offset=linelength;
				first_stack=FALSE;
				wait_for_second_stack=TRUE;
			}
			continue;
		}
	
		if(!first_stack && wait_for_second_stack) wait_for_second_stack=FALSE;

		if(clustal_on==FALSE) {
			if(linetype(str1,CLUSTAL_INIT)) filetype1=CLUSTAL_SEQ;
			else {
				fprintf(stderr,"Unknown file format.\n");
				return;
			}
			clustal_on=TRUE;
			continue;
		} else {
		
			if(first_stack==TRUE) {
				sscanf(str1,"%s %s",IDstr,seqstr);	
				if(strlen(IDstr)>AC_LENGTH) {
					fprintf(stderr,"Long ID name is truncated.\n");
				}
				if(k>0) {
					if(strcmp(seqfile1[0]->ID,IDstr)==0) {
						if(k==1) {
							fprintf(stderr,"Only one sequence.\n");
							return;
						}
					}
				}
				if(strlen(IDstr)>AC_LENGTH) strncpy(seqfile1[k]->ID,IDstr,AC_LENGTH);
				else strcpy(seqfile1[k]->ID,IDstr);
			} else {
				if(k >= num_seq) {
					k=0;
					offset += linelength;
				}
				sscanf(str1,"%s %s",IDstr,seqstr);	

/* for debug  
			printf("str1= %s, IDstr= %s, seqfile1[k]ID= %s\n",str1,IDstr,seqfile1[k]->ID);
*/
			
				if(strcmp(seqfile1[k]->ID,IDstr)!=0) {
					fprintf(stderr,"Wrong ID name in %ld -th sequence.\n", k+1);
					return;
				}
			}
	
			if(k > 0) {
				if((int)strlen(seqstr)!=linelength) {
					fprintf(stderr,"Line length is not identical.\n");
					return;
				}
			}
			for(i=0;i<(long)strlen(seqstr);i++){
				if(isalpha((int)seqstr[i]) || seqstr[i]=='-'){
					seqfile1[k]->seq[offset+i]=seqstr[i];
				}
				else {
					fprintf(stderr,"Strange character in sequence.\n");
					return;
				}
			}
			if(k==0) linelength=strlen(seqstr);

			k++;
		}

	}	/* end while */

	if(wait_for_second_stack){
		k=num_seq;
		linelength=0;
	}else if(first_stack){
		num_seq=k;
	}

	if(k!=num_seq) {
		fprintf(stderr,"Sequence is not complete.\n");
		return;
	}
	
	offset=offset+linelength;

	for(k=0;k<num_seq;k++){
		seqfile1[k]->seq[offset]='\0';	
		seqfile1[k]->seqlen=offset;
	}

	fclose(fin);
	
	/* each sequence may be of a different type, such as RNA or DNA */
	for(k=0;k<num_seq;k++){
		dtype1=check_data_type(seqfile1[k]);
		seqfile1[k]->dtype=dtype1;
	}
	
	return;
}

/*************************************************************************************/
Seqfile **read_clustal(char *filename, enum data_type dtype1)
/*************************************************************************************/
{
	Seqfile **seqfile1;
	register long int i=0;
	register long int k=0;
	long int offset = 0;
	int linelength=0;
	int filetype1=0;
	long int memsize=MAXLINE;	/* for memory usage */
	char str1[MAXLINE+1];
	char IDstr[MINMEMSIZE];
	char seqstr[MAXLINE];
	Boolean clustal_on=FALSE;
	Boolean first_stack=TRUE;
	Boolean wait_for_second_stack=FALSE;
	FILE *fin;


	if((seqfile1=(Seqfile**)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return NULL;
	}
	if((fin=fopen(filename,"r"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n\n",filename);
		return NULL;
	}

	while(fgets(str1,MAXLINE+1,fin)!=NULL){
	
#ifdef VERBOUS  
	printf("%s \n",str1);
	printf("k= %ld, i= %ld \n",k,i);
#endif

	if(strlen(str1)<=10 || linetype(str1,"     ")) {
		if(first_stack==TRUE && k>1){
			num_seq=k;
			k=0;
			offset=linelength;
			first_stack=FALSE;
			wait_for_second_stack=TRUE;
		}
		continue;
	}
	
	if(!first_stack && wait_for_second_stack) wait_for_second_stack=FALSE;

	if(clustal_on==FALSE) {

		if(linetype(str1,CLUSTAL_INIT)) filetype1=CLUSTAL_SEQ;
		else {
			fprintf(stderr,"Unknown file format.\n");
			return NULL;
		}

		if((seqfile1=(Seqfile**)calloc(1,sizeof(Seqfile)))==0){
			fprintf(stderr,"Out of memory\n");
			return NULL;
		}
		clustal_on=TRUE;
		continue;
	} else {
		
		if(first_stack==TRUE) {

			sscanf(str1,"%s %s",IDstr,seqstr);	
			if(strlen(IDstr)>AC_LENGTH) {
				fprintf(stderr,"Long ID name is truncated.\n");
/*				return NULL;	*/
			}
			if(k>0) {
				if(strcmp(seqfile1[0]->ID,IDstr)==0) {
					if(k==1) {
						fprintf(stderr,"Only one sequence.\n");
						return NULL;
					}
				}
			}
			

			/* Memory allocation for the k-th sequence */

			if((seqfile1=(Seqfile**)realloc(seqfile1,(k+2)*sizeof(Seqfile)))==0){
                                fprintf(stderr,"Out of memory\n");
                                return NULL;
                        }

			if((seqfile1[k]=(Seqfile *)calloc(1,sizeof(Seqfile)))==0){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			if((seqfile1[k]->ID=(char *)calloc(MINMEMSIZE,sizeof(char)))==0){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			if((seqfile1[k]->header=(char *)calloc(MINMEMSIZE,sizeof(char)))==0){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			if((seqfile1[k]->seq=(char *)calloc(MAXLINE,sizeof(char)))==0){		
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			if(strlen(IDstr)>AC_LENGTH) strncpy(seqfile1[k]->ID,IDstr,AC_LENGTH);
			else strcpy(seqfile1[k]->ID,IDstr);
		} else {
			if(k>=num_seq) {
				k=0;
				offset=offset+linelength;
			}

			sscanf(str1,"%s %s",IDstr,seqstr);	

/* for debug  
			printf("str1= %s, IDstr= %s, seqfile1[k]ID= %s\n",str1,IDstr,seqfile1[k]->ID);
*/
			
			if(strcmp(seqfile1[k]->ID,IDstr)!=0) {
				fprintf(stderr,"Wrong ID name in %ld -th sequence.\n", k+1);
				return NULL;
			}

			/* Memory allocation for the k-th sequence */

			memsize=strlen(seqfile1[k]->seq)+MAXLINE;
			if((seqfile1[k]->seq=(char *)realloc(seqfile1[k]->seq,memsize*sizeof(char)))==0){		
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}	
		}

		if(k>0) {
			if((int)strlen(seqstr)!=linelength) {
				fprintf(stderr,"Line length is not identical.\n");
				return NULL;
			}
		}
		for(i=0;i<(long)strlen(seqstr);i++){
			if(isalpha((int)seqstr[i]) || seqstr[i]=='-'){
				seqfile1[k]->seq[offset+i]=seqstr[i];
			}
			else {
				fprintf(stderr,"Strange character in sequence.\n");
				return NULL;
			}
		}
		

		if(k==0) linelength=strlen(seqstr);

		k++;
	}
	}	/* end while */

	if(wait_for_second_stack){
		k=num_seq;
		linelength=0;
	}
	else if(first_stack){
		num_seq=k;
	}

	if(k!=num_seq) {
		fprintf(stderr,"Sequence is not complete.\n");
		return NULL;
	}
	
	offset=offset+linelength;

	for(k=0;k<num_seq;k++){
		seqfile1[k]->seq[offset]=0;	
		seqfile1[k]->seqlen=offset;
	}

	fclose(fin);
	
	/* each sequence may be of a different type, such as RNA or DNA */
	if(dtype1==unknown){
		for(k=0;k<num_seq;k++){
			dtype1=check_data_type(seqfile1[k]);
			seqfile1[k]->dtype=dtype1;
		}
	}
	
	return seqfile1;

}


/*************************************************************************************/
int fwrite_clustal(FILE *fout, Seqfile **seqfile1, char *string, int ID_len)
/*************************************************************************************/
{
	register long int i,j,k;
	int s, t;
	int clustal_line_length=60;
	char clustal_header[MAXLINE]="CLUSTAL W MULTIPLE SEQUENCE ALIGNMENT (Output from SISEQ)";

	if(strlen(string)!=0) strcpy(clustal_header,string);

	if(seqfile1==NULL){
		fprintf(stderr,"Empty sequence.\n");
		return 1;
	}
	
	for(i=0;((s=clustal_header[i])!=0);++i){
		if(isprint(s)) {
			if((t=fputc(s,fout))==EOF){
				fprintf(stderr,"Writing error.\n");
				return 2;
			}
		}
	}		

	fputc('\n',fout);
	fputc('\n',fout);
	fputc('\n',fout);

	for(j=0;j<(long)strlen(seqfile1[0]->seq);j+=clustal_line_length){
		
		for(k=0;k<num_seq;++k){
			for(i=0;i<ID_len+5;++i){
				if(i>=ID_len) s=' ';
				else if(i>=(long)strlen(seqfile1[k]->ID)) s=' ';
				else if((s=seqfile1[k]->ID[i])==0) s=' ';

				if(isprint(s)) {
				/* Writing error is always checked. If fputc returns EOF,
				   there is a problem in writing file. */
					if((t=fputc(s,fout))==EOF){
						fprintf(stderr,"Writing error.\n");
						return 2;
					}
				}
			}

			for(i=j;((s=seqfile1[k]->seq[i])!=0);++i){
				if(i>=(j+clustal_line_length)) 	break;
				if(isprint(s)) {
					if((t=fputc(s,fout))==EOF){
						fprintf(stderr,"Writing error.\n");
						return 2;
					}
				}
			}
			fputc('\n',fout);

		}

        fputc('\n',fout);
        fputc('\n',fout);

	}	
	
	
        fputc('\n',fout);
        fputc('\n',fout);
	return 0;

}


/*************************************************************************************/
int write_clustal(char *filename,Seqfile **seqfile1)
/*************************************************************************************/
{
	FILE *fout;
	int c;

	if(seqfile1==NULL){
		fprintf(stderr,"Empty sequence.\n");
		return 1;
	}
	
#ifdef MAC
#ifdef CODEWARRIOR
	_fcreator=CREATOR_NAME;
	_ftype=FILETYPE_NAME;
#endif
#endif

	if((fout=fopen(filename,"w"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n",filename);
		return 1;
	}
	
	if((c=fwrite_clustal(fout,seqfile1,"",30))!=0){
		fprintf(stderr,"Trouble in writing file: %s\n",filename);
		fclose(fout);
		return c;
	}
#ifdef VERBOUS
	fprintf(stdout,"Sequence written to file: %s\n",filename);
#else
/*	printf("w");*/
	putc(8,stdout);
	putc('w',stdout);
	fflush(stdout);
#endif

	fclose(fout);
	return 0;
	

}

/********************************************************************/
Seqfile *read_dbseq(FILE *fin,long *loc)
/* This function is called from join_exons2 to join external 
sequences. */
/********************************************************************/
{
	Seqfile *seqfile1;
	register long int i, j;
	char str1[MAXLINE+1];
	long int offset = 0;
	long int memsize=2L;	/* for memory usage */
	long fileloc=*loc;
	long fileloc_bak;
	Boolean gb_seq_on=FALSE;

	fileloc_bak=ftell(fin);

	if((seqfile1=(Seqfile*)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}

	if((seqfile1->ID=(char *)calloc(AC_LENGTH+2,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}
	if((seqfile1->header=(char *)calloc(2,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}
	if((seqfile1->seq=(char *)calloc(memsize,sizeof(char)))==NULL){		
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}

	if(fseek(fin,fileloc,SEEK_SET)){
		fprintf(stderr,"File seek error.\n");
		return NULL;
	}

	while(fgets(str1,MAXLINE,fin)!=NULL){
		if(linetype(str1,GB_SEQ_INIT) || linetype(str1,EMBL_SEQ_INIT)){
			gb_seq_on=TRUE;
			continue;
		}
		
		if(linetype(str1,GB_SEQ_END) || linetype(str1,EMBL_SEQ_END)){
			gb_seq_on=FALSE;
			break;
		}
		
/* Reading sequence */
		
		if(gb_seq_on==TRUE){
			memsize=memsize+(strlen(str1))*(sizeof(char));
			if((seqfile1->seq=(char *)realloc(seqfile1->seq,memsize))==0){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			j=0;
			for(i=0;i<(long)strlen(str1);i++){
				if(islower((int)str1[i])) str1[i]=toupper(str1[i]);
				if(isupper((int)str1[i])){
					seqfile1->seq[offset+j]=str1[i];
					j++;
				}
			}
			offset=offset+j;				
		}
	}
			
	seqfile1->seq[offset]=0;
	seqfile1->seqlen=offset;
	if(seqfile1->seqlen==0) return NULL;
	else seqfile1->dtype=DNA;
	
	if(fseek(fin,fileloc_bak,SEEK_SET)){
		fprintf(stderr,"File seek error.\n");
	}

	return seqfile1;
}


/******************************************************
		 End of file gtsqfn.c 
******************************************************/

