/*****************************************************
Sequence correlation analysis functions
Program name : seqcor
File name : mrdfa.c 
	
	Functions :	Mread_fasta
	
	Naoki Sato, Sept. 6, 1998.
	Made compatible with comment lines beginning with ";", Dec. 28, 1998.
	Allows comment lines with ";" before sequence lines. Jan. 25, 1999. 
	Console output of dots during the processing. April 29. 1999.
	Bug fix. May 4, 2000.
	Accept circular. Oct. 18, 2000.
	Bug fix. May 25, 2001.

	--- Do not delete this comment.---

******************************************************/
#include "Dmc.h"
#include "getseq2.h"

/* Prototype declaration */

Seqfile **Mread_fasta(char *filename, enum data_type dtype1);

/* External functions */
#ifndef __MRDFA
#define __MRDFA
#endif
#include "libSeq.h"

/* Global variables */
extern Boolean circular;

/* Definition of the code for the end of line (LF).*/

#define END_OF_LINE 10  /* CR will be ignored in the program. */
#define FASTA_COMMENT ";"

/*************************************************************************/
Seqfile **Mread_fasta(char *filename, enum data_type dtype1)
/*************************************************************************/
{
	FILE *fin;
	Seqfile **seqfileM;
	long memsize=5;
	long seqfile_size;
	register long int i = 0;
	register long int j = 0;
	long ii = 0;
	long offset = 0;
	char str1[MAXLINE+1];
	char str2[MAXLINE+1];
	char IDstr[MAXLINE+1];
	int b;
	char *pp;

/* Flags for gb are also used here. */
	Boolean gb_header_on=FALSE;
	Boolean gb_seq_on=FALSE;

	if(dtype1!=DNA && dtype1!=RNA && dtype1!=protein) dtype1=unknown;

	if((fin=fopen(filename,"r"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n\n",filename);
		return NULL;
	}
        if((seqfileM=(Seqfile**)calloc(1,sizeof(Seqfile)))==0){
		fprintf(stderr,"Out of memory\n");
		return NULL;
	}

	/* Reading loop */

	while(fgets(str1,MAXLINE+1,fin)!=NULL){

/* compatibility with the comment lines biginning with ";" */
		if((gb_header_on==TRUE) || (linetype(str1,FASTA_COMMENT) && (gb_seq_on==TRUE)) ){

/* Header may be omitted to reduce memory usage.*/

			b=strlen(seqfileM[i]->header)+MAXLINE+MINMEMSIZE;
			if((seqfileM[i]->header=(char *)realloc(seqfileM[i]->header,b*sizeof(char)))==0){
				fprintf(stderr,"Out of memory\n");
				return NULL;
			}
			strcat(seqfileM[i]->header,str1);
/***/
			if(last_line(str1)){
				gb_header_on=FALSE;
				gb_seq_on=TRUE;
				continue;	/* could be deleted */
			}
		}
		else {
			if(!gb_header_on && !gb_seq_on && linetype(str1,FASTA_COMMENT)) continue;

			if(linetype(str1,FASTA_INIT)){

				if(gb_seq_on==TRUE){
					seqfileM[i]->seq[offset]=0;
					seqfileM[i]->seqlen=offset;
					if(dtype1==unknown)  dtype1=check_data_type(seqfileM[i]);
					seqfileM[i]->dtype=dtype1;

					++i;
					offset=0;
                                        gb_seq_on = FALSE;
					if(i%100==0){
						fprintf(stderr,".");
						fflush(stderr);
					}
				}

	 		/* Initialize a new sequence */
			
				seqfile_size=(i + 2)*sizeof(Seqfile); 	
				if((seqfileM=(Seqfile**)realloc(seqfileM,seqfile_size))==0){
					fprintf(stderr,"Out of memory\n");
					return NULL;
				}
				if((seqfileM[i]=(Seqfile*)calloc(1,sizeof(Seqfile)))==0){
					fprintf(stderr,"Out of memory\n");
					return NULL;
				}
				if((seqfileM[i]->ID=(char *)calloc(32,sizeof(char)))==0){
					fprintf(stderr,"Out of memory\n");
					return NULL;
				}
/* Header may be omitted. */
				if((seqfileM[i]->header=(char *)calloc(MAXLINE+MINMEMSIZE,sizeof(char)))==0){
					fprintf(stderr,"Out of memory\n");
					return NULL;
				}
/***/
				if((seqfileM[i]->seq=(char *)calloc(1,sizeof(char)))==0){
					fprintf(stderr,"Out of memory\n");
					return NULL;
				}
				memsize=5;

				/* Reading IDstring */

				pick_string(str1,strlen(FASTA_INIT)+1,strlen(str1));
				sscanf(str1,"%s",IDstr);

        		/* IDstr is limited to 30 characters. */
       			/* The remaining part of str1 is copied to str2 for further manipulation */
			/* May 3, 2007 */
				if(strlen(IDstr) > AC_LENGTH){
					for(pp=IDstr+AC_LENGTH;pp>IDstr;pp--){
						if(*pp=='='){
							break;
						}
					}
					if(*pp == '='){
						*pp = '\0';
						strcpy(seqfileM[i]->ID,IDstr);
						/* The remaining part of IDstr after = is discarded. */
						strcpy(str1,str1 + strlen(IDstr) + 1);
					} else {
						strncpy(seqfileM[i]->ID,IDstr,AC_LENGTH);
						seqfileM[i]->ID[AC_LENGTH]='\0';
						strcpy(str2, str1 + strlen(seqfileM[i]->ID));
					}
				} else {
					strcpy(seqfileM[i]->ID,IDstr);
					strcpy(str2, str1 + strlen(seqfileM[i]->ID));
				}
			/* end */
				/* Setting str2 to the header field. */
				strcpy(seqfileM[i]->header,str2);
				gb_header_on=TRUE;

				/* setting circular if defined **/
				if(strstr(str1,"circular")!=NULL) seqfileM[i]->circular=TRUE;

				if(str2[strlen(str2)-1]==END_OF_LINE){
					gb_header_on=FALSE;
					gb_seq_on=TRUE;
				}
			}

			/* Reading sequence */

			else if(gb_seq_on){
				memsize=memsize+((strlen(str1))+4)*(sizeof(char));
				if((seqfileM[i]->seq=(char *)realloc(seqfileM[i]->seq,memsize))==NULL){
					fprintf(stderr,"Out of memory\n");
					return NULL;
				}

				j=0;
				for(ii=0;ii<(long)strlen(str1);ii++){
					if(isprint((int)str1[ii]) && !(isspace((int)str1[ii]))){
						seqfileM[i]->seq[offset+j]=str1[ii];
						j++;
					}
				}
				offset=offset+j;
			}
			else {
				continue;
			}
		}
	}

	/* end of loop */

        /* Completing the last sequence */

	seqfileM[i]->seq[offset]=0;
	seqfileM[i]->seqlen=offset;
	seqfileM[i]->dtype=dtype1;
	fprintf(stderr,".\n");
	fflush(stderr);

	num_seq=i+1;

	fclose(fin);
	
	for(i=0;i<num_seq;++i){
		if(seqfileM[i]->dtype==unknown){
			dtype1=check_data_type(seqfileM[i]);
			seqfileM[i]->dtype=dtype1;
		}			
	}
	return seqfileM;	
}

/********************* end of file mrdfa.c *************************************/

