/*****************************************************
File name : msanal.c 

	MS analysis
	
	Naoki Sato March 7, 2007.

	Integrated Mac, DOS, and UNIX version.

	--- Do not delete this comment.---

******************************************************/

#include "Dmc.h"
#include "getseq2.h"
#include <math.h>

/* Definition of structure */
typedef struct{
	char ID[MINMEMSIZE];
	long start;
	long end;
	double obs;
	double calc;
	double diff;
	char seq[MINMEMSIZE];
	int category;	/* 1: C>, 2: >C, 3: N>, 4: >N. */
	int weight;
}MSlist;

typedef struct{
	char ID[MINMEMSIZE];
	long len;
	double coverage1;
	double coverage2;
	int number;
	MSlist *mslist;
}MS;

/* Prototype declaration */

int MSanal(char *msdef, char *outfile, Seqfile **mseq);
static double getProtMW0(char *protseq);
static MSlist* MSrealloc(char *title,MSlist *mslist1,int old,int new);

/* External functions */
#ifndef __MSANAL_
#define __MSANAL
#endif

#include "libSeq.h"

/* Global variables are defined here. */

extern char aaname[20][2][10];
extern char *data_type_name[];
extern long num_seq;
extern double aawt0[];
extern double H2O0;

MS *ms1;		/* structure of proteins with significant MS signals */
int ms_num=100;		/* number of proteins with significant MS signals */
Boolean carboxymethylation=FALSE;
Boolean methionineoxidation=FALSE;
double carboxymethyl=58.00548126;
double oxymethionine=15.99491563;


/*********************************************************/
static MSlist* MSrealloc(char *title,MSlist *mslist1,int old,int new)
/*********************************************************/
{	
	int i2;

	if(old < 0 || new < 0){
		fprintf(stderr,"Strange value in MSrealloc.\n");
		return NULL;
	}
	if(old >= new) return mslist1;
	if((mslist1=(MSlist*)realloc(mslist1,(new+1)*sizeof(MSlist)))==NULL){
		fprintf(stderr,"Memory allocation error in MSrealloc for %s.\n",title);
		return NULL;
	}
/*
	fprintf(stderr,"mslist is being reallocated for %s. old size=%d, new size=%d.\n",title,old,new);
*/
	for(i2=old;i2<new;i2++){
		strcpy(mslist1[i2].ID,"");
		mslist1[i2].start = 0L;
		mslist1[i2].end = 0L;
		mslist1[i2].obs = 0.0;
		mslist1[i2].calc = 0.0;
		mslist1[i2].diff = 0.0;
		strcpy(mslist1[i2].seq,"");
		mslist1[i2].category = 0;
		mslist1[i2].weight = 0;
	}
	return mslist1;	
}
/*********************************************************/

/*********************************************************/
int MSanal(char *msdef, char *outfile, Seqfile **mseq)
/*********************************************************/
{	
	FILE *fin, *fout;
	char str1[MAXLINE+2];
	char title[100]="";
	char Nterm[20]="";
	char Cterm[20]="";
	char Cterm_bak[20]="";
	double lowlim=1000.0;
	double upplim=3500.0;
	int num_peaks=1;
	int old_number;
	double *peak;
	Boolean first_line=TRUE;
	Boolean peak_input=FALSE;
	int counter=0;
	int i,i1,i2,i3;
	long j,total;
	char *p,*q,*r;
	char protseq[200];
	double mw;
	double allowance=0.2;
	double displacement=0.0;
	double mincoverage2=0.05;
	int minmatch = 1;
	int match;
	double delta=-1.01;
	int num_fragments;
	int flag = 0;
	int output=0;	/* output format. 0:selected 1:all */
	char head1[100];
	MSlist *mslist0;
	int num_items;
	int increment;

	int max_i1,max_i2;

	if((fin=fopen(msdef,"r"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n\n",msdef);
		return TRUE;
	}
	while(fgets(str1,MAXLINE+1,fin)!=NULL){
		if(str1[0]=='#'){
			continue;
		}
		if(!strcmp(str1,"\n")){
			continue;
		}
		if(peak_input){
			sscanf(str1,"%lf",&peak[counter++]);
			if(counter >= num_peaks) peak_input=FALSE;
			continue;
		}
		if(first_line){
			strncpy(title,str1,99);
			first_line = FALSE;
			continue;
		}
		if(strstr(str1,"Nterm")!=NULL){
			sscanf(str1,"%s\t",Nterm);
			first_line=FALSE;
		}else if(strstr(str1,"Cterm")!=NULL){
			sscanf(str1,"%s\t",Cterm);
		}else if(strstr(str1,"Carboxymethylation")!=NULL){
			if(str1[0]=='1') carboxymethylation=TRUE;
		}else if(strstr(str1,"Methionine oxidation")!=NULL){
			if(str1[0]=='1') methionineoxidation=TRUE;
		}else if(strstr(str1,"Lower")!=NULL){
			sscanf(str1,"%lf",&lowlim);
		}else if(strstr(str1,"Upper")!=NULL){
			sscanf(str1,"%lf",&upplim);
		}else if(strstr(str1,"Allowance")!=NULL){
			sscanf(str1,"%lf",&allowance);
		}else if(strstr(str1,"Displacement")!=NULL){
			sscanf(str1,"%lf",&displacement);
		}else if(strstr(str1,"Output")!=NULL){
			sscanf(str1,"%d",&output);
		}else if(strstr(str1,"MinCoverage2")!=NULL){
			sscanf(str1,"%lf",&mincoverage2);
		}else if(strstr(str1,"MinMatch")!=NULL){
			sscanf(str1,"%d",&minmatch);
		}else if(strstr(str1,"Number of fragments")!=NULL){
			sscanf(str1,"%d",&num_fragments);
		}else if(strstr(str1,"Number of peaks")!=NULL){
			peak_input=TRUE;
			sscanf(str1,"%d",&num_peaks);
			if((peak=(double*)calloc(num_peaks+1,sizeof(double)))==NULL){
				fprintf(stderr,"Memory allocation error in MSanal.\n");
				exit(1);
			}
		}else if(strstr(str1,"//END")!=NULL){
			break;
		} else {
			continue;
		}
	}
	fclose(fin);

	if((fout=fopen(outfile,"w"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n\n",outfile);
		return TRUE;
	}
	fprintf(fout,"%s",title);
	fprintf(fout,"%s\tNterm\n",Nterm);	
	fprintf(fout,"%s\tCterm\n",Cterm);
	if(carboxymethylation) fprintf(fout,"1");
	else fprintf(fout,"0");
	fprintf(fout,"\tCarboxymethylation\n");
	if(methionineoxidation) fprintf(fout,"1");
	else fprintf(fout,"0");
	fprintf(fout,"\tMethionine oxidation\n");
	fprintf(fout,"%9.4f\tLower limit\n",lowlim);
	fprintf(fout,"%9.4f\tUpper limit\n",upplim);
	fprintf(fout,"%9.4f\tAllowance\n",allowance);
	fprintf(fout,"%9.4f\tDisplacement\n",displacement);
	fprintf(fout,"%d\tOutput (0: selected, 1: all)\n",output);
	fprintf(fout,"%9.4f\tMinCoverage2\n",mincoverage2);
	fprintf(fout,"%d\tMinMatch (minimum number of completely matched peaks)\n",minmatch);
	fprintf(fout,"%d\tNumber of fragments\n",num_fragments);
	fprintf(fout,"%d\tNumber of peaks\n",num_peaks);
	fprintf(fout,"m/z (obs)\tm/z (corr)\n");
	for(i=0;i<num_peaks;i++){
		fprintf(fout,"%9.4f\t%9.4f\n",peak[i],peak[i]+delta);
		peak[i] += delta;
	}
	fprintf(fout,"//END of msdef file\n\n");

	if((ms1=(MS*)calloc(ms_num+1,sizeof(MS)))==NULL){
		fprintf(stderr,"Memory allocation error in MSanal 1.\n");
		return TRUE;
	}
	
/*	num_items = 20 + 3 * num_peaks;	*/
	num_items = 3 * num_peaks;
	if(num_peaks > 10) increment = 30;
	else increment = 3 * num_peaks;

	for(i=0;i<ms_num;i++){
		ms1[i].number = num_items;
		if((ms1[i].mslist=(MSlist*)calloc(ms1[i].number+1,sizeof(MSlist)))==NULL){
			fprintf(stderr,"Memory allocation error in MSanal 2.\n");
			return TRUE;
		}
	}
	if(num_fragments < 20 * num_peaks) num_fragments = 20 * num_peaks;
	if((mslist0=(MSlist*)calloc(num_fragments+1,sizeof(MSlist)))==NULL){
		fprintf(stderr,"Memory allocation error in MSanal 2.1.\n");
		return TRUE;
	}
	strcpy(Cterm_bak,Cterm);

	max_i1=max_i2=0;

	fprintf(fout,"ID     \tlength\t match \tcoverage 2\tcoverage 1\tstart - end\t MW  \t  m/z \t diff  \tsequence      \t category \t weight\n\n");
	
	i1=0;
	for(j=0;j<num_seq;j++){


		fprintf(stderr,"\r%ld",j);

		strcpy(ms1[i1].ID,"");
		ms1[i1].len = 0L;
		ms1[i1].coverage1 = 0.0;
		ms1[i1].coverage2 = 0.0;
		old_number = ms1[i1].number;
		ms1[i1].number = num_items;
		if((ms1[i1].mslist=MSrealloc("new",ms1[i1].mslist,old_number,ms1[i1].number))==NULL){
/*			if((ms1[i1].mslist=(MSlist*)realloc(ms1[i1].mslist,(ms1[i1].number+1)*sizeof(MSlist)))==NULL){
				fprintf(stderr,"Memory allocation error in MSanal 3.\n");
*/
			return TRUE;
		}

		for(i2=0;i2<ms1[i1].number;i2++){
			strcpy(ms1[i1].mslist[i2].ID,"");
			ms1[i1].mslist[i2].start = 0L;
			ms1[i1].mslist[i2].end = 0L;
			ms1[i1].mslist[i2].obs = 0.0;
			ms1[i1].mslist[i2].calc = 0.0;
			ms1[i1].mslist[i2].diff = 0.0;
			strcpy(ms1[i1].mslist[i2].seq,"");
			ms1[i1].mslist[i2].category = 0;
			ms1[i1].mslist[i2].weight = 0;
		}

		i2=0;
		for(q=Cterm;*q!='\0' && *q!='0';q++){
			if(*q=='0') break;
			for(p=mseq[j]->seq;*p!='\0';p++){
				if(*p==*q){
					if((long)(p - mseq[j]->seq) < 5) flag = 1;
					else if((long)(p - mseq[j]->seq) > mseq[j]->seqlen +5) flag = 2;
					else flag = 0;
					for(r=p;r >= mseq[j]->seq;r--){
						if(flag == 1) break;
						if(r==p) continue;
						if((int)(p-r+1) > 100) break;
						strncpy(protseq,r,(int)(p-r+1));
						*(protseq+(p-r+1)) = '\0';
						if((mw = getProtMW0(protseq))==0.0) continue;
						mw += displacement;
						if(mw < lowlim) continue;
						else if(mw > upplim) break;
						else{
							for(i=0;i<num_peaks;i++){
								if(fabs(peak[i] - mw) < allowance){
									strcpy(ms1[i1].mslist[i2].ID,mseq[j]->ID);
									ms1[i1].mslist[i2].start = (long)(r - mseq[j]->seq + 1);
									ms1[i1].mslist[i2].end = (long)(p - mseq[j]->seq + 1);
									ms1[i1].mslist[i2].obs = peak[i];
									ms1[i1].mslist[i2].calc = mw;
									ms1[i1].mslist[i2].diff = (peak[i] - mw);
									strcpy(ms1[i1].mslist[i2].seq,protseq);
									ms1[i1].mslist[i2].category = 2;
									ms1[i1].mslist[i2].weight = 1;
									i2++;
									if(i2 >= ms1[i1].number) {
										if(i2 > num_fragments){
											fprintf(stderr,"Too many hits. Change parameters to get fewer hits.\n");
											return TRUE;
										}else{
											if((ms1[i1].mslist=MSrealloc(mseq[j]->ID,ms1[i1].mslist,ms1[i1].number,ms1[i1].number+increment))==NULL) return TRUE;
											ms1[i1].number += increment;
										}
									}
/*
									fprintf(fout,"%s %ld-%ld\t%9.4f\t%9.4f\t%9.4f\t%s [>C]\n",mseq[j]->ID,(long)(r - mseq[j]->seq + 1),\
									(long)(p - mseq[j]->seq + 1), mw, peak[i],(peak[i] - mw),protseq);
*/
									break;
								}
							}
						}
					}
					for(r=p+1;*r!='\0';r++){
						if(flag == 2 || *r=='\0') break;
						if((int)(r-p) > 100) break;
						strncpy(protseq,(p+1),(int)(r-p));
						*(protseq+(r-p)) = '\0';
						if((mw = getProtMW0(protseq))==0.0) continue;
						mw += displacement;
						if(mw < lowlim) continue;
						else if(mw > upplim) break;
						else{
							for(i=0;i<num_peaks;i++){
								if(fabs(peak[i] - mw) < allowance){
									strcpy(ms1[i1].mslist[i2].ID,mseq[j]->ID);
									ms1[i1].mslist[i2].start = (long)(p - mseq[j]->seq + 2);
									ms1[i1].mslist[i2].end = (long)(r - mseq[j]->seq + 1);
									ms1[i1].mslist[i2].obs = peak[i];
									ms1[i1].mslist[i2].calc = mw;
									ms1[i1].mslist[i2].diff = (peak[i] - mw);
									strcpy(ms1[i1].mslist[i2].seq,protseq);
									ms1[i1].mslist[i2].category = 1;
									ms1[i1].mslist[i2].weight = 1;
									i2++;
									if(i2 >= ms1[i1].number) {
										if(i2 > num_fragments){
											fprintf(stderr,"Too many hits. Change parameters to get fewer hits.\n");
											return TRUE;
										}else{
											if((ms1[i1].mslist=MSrealloc(mseq[j]->ID,ms1[i1].mslist,ms1[i1].number,ms1[i1].number+increment))==NULL) return TRUE;
											ms1[i1].number += increment;
										}
									}
/*
									fprintf(fout,"%s %ld-%ld\t%9.4f\t%9.4f\t%9.4f\t%s [C>]\n",mseq[j]->ID,(long)(p - mseq[j]->seq + 2),\
									(long)(r - mseq[j]->seq + 1), mw, peak[i],(peak[i] - mw),protseq);
*/
									break;
								}
							}
						}
					}
				}
			}
		}

		for(q=Nterm;*q!='\0' && *q!='0';q++){
			if(*q=='0') break;
			for(p=mseq[j]->seq;*p!='\0';p++){
				if(*p==*q){
					if((long)(p - mseq[j]->seq) < 5) flag = 1;
					else if((long)(p - mseq[j]->seq) > mseq[j]->seqlen +5) flag = 2;
					else flag = 0;
					for(r=p;r>=mseq[j]->seq;r--){
						if(flag == 1) break;
						if(r==p) continue;
						if((int)(p-r) > 100) break;
						strncpy(protseq,r,(int)(p-r));
						*(protseq+(p-r)) = '\0';
						if((mw = getProtMW0(protseq))==0.0) continue;
						mw += displacement;
						if(mw < lowlim) continue;
						else if(mw > upplim) break;
						else{
							for(i=0;i<num_peaks;i++){
								if(fabs(peak[i] - mw) < allowance){
									strcpy(ms1[i1].mslist[i2].ID,mseq[j]->ID);
									ms1[i1].mslist[i2].start = (long)(r - mseq[j]->seq + 1);
									ms1[i1].mslist[i2].end = (long)(p - mseq[j]->seq);
									ms1[i1].mslist[i2].obs = peak[i];
									ms1[i1].mslist[i2].calc = mw;
									ms1[i1].mslist[i2].diff = (peak[i] - mw);
									strcpy(ms1[i1].mslist[i2].seq,protseq);
									ms1[i1].mslist[i2].category = 8;
									ms1[i1].mslist[i2].weight = 1;
									i2++;
									if(i2 >= ms1[i1].number) {
										if(i2 > num_fragments){
											fprintf(stderr,"Too many hits. Change parameters to get fewer hits.\n");
											return TRUE;
										}else{
											if((ms1[i1].mslist=MSrealloc(mseq[j]->ID,ms1[i1].mslist,ms1[i1].number,ms1[i1].number+increment))==NULL) return TRUE;
											ms1[i1].number += increment;
										}
									}
/*
									fprintf(fout,"%s %ld-%ld\t%9.4f\t%9.4f\t%9.4f\t%s [>N]\n",mseq[j]->ID,(long)(r - mseq[j]->seq + 1),\
									(long)(p - mseq[j]->seq), mw, peak[i],(peak[i] - mw),protseq);
*/
									break;
								}
							}
						}
					}

					for(r=p+1;*r!='\0';r++){
						if(flag == 2 || *r=='\0') break;
						if((int)(r-p+1) > 100) break;
						strncpy(protseq,p,(int)(r-p+1));
						*(protseq+(r-p+1)) = '\0';
						if((mw = getProtMW0(protseq))==0.0) continue;
						mw += displacement;
						if(mw < lowlim) continue;
						else if(mw > upplim) break;
						else{
							for(i=0;i<num_peaks;i++){
								if(fabs(peak[i] - mw) < allowance){
									strcpy(ms1[i1].mslist[i2].ID,mseq[j]->ID);
									ms1[i1].mslist[i2].start = (long)(p - mseq[j]->seq + 1);
									ms1[i1].mslist[i2].end = (long)(r - mseq[j]->seq + 1);
									ms1[i1].mslist[i2].obs = peak[i];
									ms1[i1].mslist[i2].calc = mw;
									ms1[i1].mslist[i2].diff = (peak[i] - mw);
									strcpy(ms1[i1].mslist[i2].seq,protseq);
									ms1[i1].mslist[i2].category = 4;
									ms1[i1].mslist[i2].weight = 1;
									i2++;
									if(i2 >= ms1[i1].number) {
										if(i2 > num_fragments){
											fprintf(stderr,"Too many hits. Change parameters to get fewer hits.\n");
											return TRUE;
										}else{
											if((ms1[i1].mslist=MSrealloc(mseq[j]->ID,ms1[i1].mslist,ms1[i1].number,ms1[i1].number+increment))==NULL) return TRUE;
											ms1[i1].number += increment;
										}
									}
/*
									fprintf(fout,"%s %ld-%ld\t%9.4f\t%9.4f\t%9.4f\t%s [N>]\n",mseq[j]->ID,(long)(p - mseq[j]->seq + 1),\
									(long)(r - mseq[j]->seq + 1), mw, peak[i],(peak[i] - mw),protseq);
*/
									break;
								}
							}
						}
					}
				}
			}
		}

		if(i2 > max_i2) max_i2 = i2;

/*
		for(i2=0;i2<ms1[i1].number;i2++){
printf("i1=%d, i2=%d, ID=%s, start=%ld, end=%ld,calc=%9.4f, weight=%d\n",\
	i1,i2, ms1[i1].mslist[i2].ID,ms1[i1].mslist[i2].start,ms1[i1].mslist[i2].end,ms1[i1].mslist[i2].calc,ms1[i1].mslist[i2].weight);
		}
printf("\n");
*/
	
		/* sorting */
		for(i2=0;i2<ms1[i1].number;i2++){
			if(ms1[i1].mslist[i2].weight == 0) continue;
			for(i3=i2+1;i3<ms1[i1].number;i3++){
				if(ms1[i1].mslist[i3].weight == 0) continue;
				if(ms1[i1].mslist[i2].start == ms1[i1].mslist[i3].start){	
					if(ms1[i1].mslist[i2].end == ms1[i1].mslist[i3].end){	
						ms1[i1].mslist[i2].category += ms1[i1].mslist[i3].category;
						ms1[i1].mslist[i2].weight += ms1[i1].mslist[i3].weight;
						ms1[i1].mslist[i3].weight = 0;	/* this is used as a flag */
					}
				}
			}
		}
/*
		for(i2=0;i2<ms1[i1].number;i2++){
printf("i1=%d, i2=%d, ID=%s, start=%ld, end=%ld,calc=%9.4f, weight=%d\n",\
	i1,i2, ms1[i1].mslist[i2].ID,ms1[i1].mslist[i2].start,ms1[i1].mslist[i2].end,ms1[i1].mslist[i2].calc,ms1[i1].mslist[i2].weight);
		}
*/

		/* making copy of mslist */
		if(i2 > num_fragments) {
			fprintf(stderr,"Too many matches. i2=%d.\n",i2);
			return TRUE;
		}
		for(i2=0;i2<ms1[i1].number;i2++){
			mslist0[i2].start = ms1[i1].mslist[i2].start;
			mslist0[i2].end = ms1[i1].mslist[i2].end;
			mslist0[i2].weight = ms1[i1].mslist[i2].weight;
			mslist0[i2].category = ms1[i1].mslist[i2].category;
		}
/*
printf("mslist0\n");
		for(i2=0;i2<ms1[i1].number;i2++){
printf("i1=%d, i2=%d, ID=%s, start=%ld, end=%ld,calc=%9.4f, weight=%d\n",\
	i1,i2, mslist0[i2].ID,mslist0[i2].start,mslist0[i2].end,mslist0[i2].calc,mslist0[i2].weight);
}
printf("\n");
*/
		/* coverage with weight=2 */
		for(i2=0;i2<ms1[i1].number;i2++){
			if(mslist0[i2].weight >= 2){
				for(i3=i2+1;i3<ms1[i1].number;i3++){
					if(mslist0[i3].weight >= 2){
						if(mslist0[i2].start >= mslist0[i3].start && mslist0[i2].start <= mslist0[i3].end){
							if(mslist0[i2].end >= mslist0[i3].end) {
								mslist0[i3].end = mslist0[i2].end;
							}
							mslist0[i2].weight = 0;
						} else if(mslist0[i2].end >= mslist0[i3].start && mslist0[i2].end <= mslist0[i3].end){
							if(mslist0[i2].start <= mslist0[i3].start) {
								mslist0[i3].start = mslist0[i2].start;
							}
							mslist0[i2].weight = 0;
						} else if(mslist0[i2].start < mslist0[i3].start && mslist0[i2].end > mslist0[i3].end){
							mslist0[i3].weight = 0;
							break;
						}
						if(mslist0[i2].weight == 0) break;
					}
				}
				if(mslist0[i2].weight == 0) continue;
			}
		}



/*
printf("mslist0\n");
		for(i2=0;i2<ms1[i1].number;i2++){
printf("i1=%d, i2=%d, ID=%s, start=%ld, end=%ld,calc=%9.4f, weight=%d\n",\
	i1,i2, mslist0[i2].ID,mslist0[i2].start,mslist0[i2].end,mslist0[i2].calc,mslist0[i2].weight);
}
printf("\n");
*/

		total = 0;
		match = 0;
		for(i2=0;i2<ms1[i1].number;i2++){
			if(mslist0[i2].weight >= 2){
				total += (mslist0[i2].end - mslist0[i2].start + 1);
				match += 1;
			}
		}
/*
printf("total=%ld\n",total);
*/
		if(total > 0){
			ms1[i1].coverage2 = (double)total / (double)(mseq[j]->seqlen);
			if(output==0 && (mincoverage2 > ms1[i1].coverage2 || match < minmatch)) continue;
			ms1[i1].len = mseq[j]->seqlen;
			strcpy(ms1[i1].ID, mseq[j]->ID);
		} else if(output == 0){
			continue;
		}
/*
printf("coverage=%9.4f\n",ms1[i1].coverage2);
*/

		/* coverage with weight=1 */
		for(i2=0;i2<ms1[i1].number;i2++){
			if(mslist0[i2].weight != 0){
				for(i3=i2+1;i3<ms1[i1].number;i3++){
					if(mslist0[i3].weight != 0){
						if(mslist0[i2].start >= mslist0[i3].start && mslist0[i2].start <= mslist0[i3].end){
							if(mslist0[i2].end >= mslist0[i3].end) {
								mslist0[i3].end = mslist0[i2].end;
							}
							mslist0[i2].weight = 0;
						} else if(mslist0[i2].end >= mslist0[i3].start && mslist0[i2].end <= mslist0[i3].end){
							if(mslist0[i2].start <= mslist0[i3].start) {
								mslist0[i3].start = mslist0[i2].start;
							}
							mslist0[i2].weight = 0;
						} else if(mslist0[i2].start < mslist0[i3].start && mslist0[i2].end > mslist0[i3].end){
							mslist0[i3].weight = 0;
						}
						if(mslist0[i2].weight == 0) break;
					}
				}
				if(mslist0[i2].weight == 0) continue;
			}
		}
		total = 0;
		for(i2=0;i2<ms1[i1].number;i2++){
			if(mslist0[i2].weight != 0){
				total += (mslist0[i2].end - mslist0[i2].start + 1);
			}
		}
		if(total > 0){
			ms1[i1].coverage1 = (double)total / (double)(mseq[j]->seqlen);
		}


printf("\nNumber %d: %s coverage2=%9.4f, coverage1=%9.4f\n",i1,ms1[i1].ID,ms1[i1].coverage2,ms1[i1].coverage1);

		if(i1 > max_i1) max_i1 = i1;

		/* printing */

/*		sprintf(head1,"%s\t%ld\t%9.4f\t%9.4f\t",ms1[i1].ID,ms1[i1].len,ms1[i1].coverage2,ms1[i1].coverage1);
*/
		sprintf(head1,"%s\t%ld\t%d\t%9.4f\t%9.4f\t",mseq[j]->ID,mseq[j]->seqlen,match,ms1[i1].coverage2,ms1[i1].coverage1);

		for(i2=0;i2<ms1[i1].number;i2++){
			if(ms1[i1].mslist[i2].weight != 0){
				fprintf(fout,"%s",head1);
				fprintf(fout,"%ld-%ld\t%9.4f\t%9.4f\t%9.4f\t%s\t%d\t%d\n",ms1[i1].mslist[i2].start,ms1[i1].mslist[i2].end,\
					ms1[i1].mslist[i2].calc,ms1[i1].mslist[i2].obs,ms1[i1].mslist[i2].diff,\
					ms1[i1].mslist[i2].seq,ms1[i1].mslist[i2].category,ms1[i1].mslist[i2].weight);
			}
		}		

		/* if the current data are retained, i1 is increased for the next */
		if(ms1[i1].len > 0){
			i1++;
			if(i1 >= ms_num){
				fprintf(stderr,"ms_num is being increased from %d to %d.\n",ms_num,ms_num+10);
				ms_num += 10;
				if((ms1=(MS*)realloc(ms1,((ms_num+1)*sizeof(MS))))==NULL){
					fprintf(stderr,"Memory allocation error in MSanal 4.\n");
					return TRUE;
				}
				for(i=ms_num-10;i<ms_num;i++){
					ms1[i].number = num_items;
					if((ms1[i].mslist=(MSlist*)calloc(ms1[i].number+1,sizeof(MSlist)))==NULL){
						fprintf(stderr,"Memory allocation error in MSanal 5.\n");
						return TRUE;
					}
				}
			}
		}
	}

	fprintf(stderr,"max_i1=%d, max_i2=%d\n",max_i1,max_i2);
	fprintf(stderr,"size of mslist0=%ld\n",sizeof(*mslist0));

	fprintf(fout,"\n");

	fclose(fout);
	return FALSE;
}
/*********************************************************/


/***********************************************************/
static double getProtMW0(char *protseq)
/* This function returns monoisotopic protein molecular weight. 
  On error, it returns 0.0. */
/***********************************************************/
{
	double mw=0.0;
	char aares;
	int i;
	char *p;	/* pointer to aares */

	if(protseq==NULL) return 0.0;
	if(strlen(protseq) < 2) return 0.0;

	for(p=protseq;*p!='\0';p++){
		if(islower(*p)) aares=toupper(*p);
		else if(isupper(*p)) aares=*p;
		else return 0.0;

		for(i=0;i<21;i++){
			if(aaname[i][1][0]==aares) break;
		}
		if(i>=0 && i<21){
			mw += aawt0[i];
		}
		else return 0.0;
		if(aares=='M' && methionineoxidation) mw += oxymethionine;
		else if(aares=='C' && carboxymethylation) mw += carboxymethyl;
	}

	mw = mw - H2O0*(strlen(protseq)-1);	
	return mw;
}
/***********************************************************/

/* end of file msanal.c */
