/***********************************************************************
Non coding region extractor (noncod) 
main file : noncod.c

	Extract non-coding regions from annotated database files
	in the GenBank or EMBL formats.

  Usage: noncod infile outfile flag
  
  Source files are:	noncod.c
			libSeq.a
			Dmc.h
			getseq2.h
			cmptbl.h
			glovar.h
			libSeq.h
	
 	Naoki Sato   Dec. 5, 1999. Started initial project.
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.
	For Mac version, define the compiler as well.
	
	Integrated into SISEQ package. Dec. 8, 1999.
	Inclusion of Japanese messages. March 4, 2000.
	
	
		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"

#ifndef INTEGRATED
#ifdef MAC
extern int ccommand(char ***);
#endif
#endif

/* Prototype declaration */
static void print_usage(void);
void help_noncod(void);

#ifdef INTEGRATED
int noncod(int c, char (*p)[S_2]);
#endif

/* External functions */

#include "libSeq.h"

/* Global variables  */
extern Boolean circular;
extern char *data_type_name[];
extern long num_seq;
extern FILE *fileptr;	/* defined in excdfn.c */


/***************************************************************/
static void print_usage(void)
/***************************************************************/
{
	help_noncod();
	return;
}

/***************************************************************/
void help_noncod(void)
/***************************************************************/
{
#ifdef JAPANESE
#ifdef UNIX
#include "EUC/noncodE.h"
#else
#ifdef MAC
#include "Mac/SJIS/noncodS.h"
#else
#include "Dos/SJIS/noncodS.h"
#endif
#endif
#else
        printf("\n********************* NONCOD help ***********************\n");
        printf("This program extracts non-coding regions from an annotated \n");
        printf("file in the GenBank or EMBL format.\n\n");
        printf("Usage: noncod infile outfile flag.\n");
	printf("\n");
        printf("	infile: 	The name of input file containing sequence data.\n");
        printf("	outfile:	The name of output file.\n");
        printf("	flag:		1 if introns should be included in the output. Otherwise 0.\n");
        printf("\n");
        printf("Other options include:\n");
	printf("	 -help (-h)	This help.\n");
        printf("*********************************************************\n\n");
#endif
	return;
}


/***************************************************************/
/******************** Main *************************************/
/***************************************************************/
#ifdef INTEGRATED
int noncod(int cc, char (*p)[S_2])

#else
#ifdef MAC 
main(int argc, char **argv)
#else 
int main(int argc, char **argv)
#endif

#endif
/***************************************************************/

{
	char fname1[100] = "infile";
	char fname2[100] = "outfile";

#include        "filenam.h"     /* definition of char *str1 and char *str2 */

	char str[MAXLINE]="";
	char *buffer;
	/*char allrights[100];*/
	Boolean flag = FALSE;	/* TRUE if introns are to be included in the final output. */
	Boolean need_connect=FALSE;	/* TRUE if the first and the last non-coding regions should be connected
					in the circular DNA. */
	int c = 0;
	int c1 = '0';	/* character to fill in the coding regions. */
	char bb[10];
	long fileloc=0L;
	long fileloc1=0L;
	Boolean exit_flag=FALSE;
	Boolean no_cds=FALSE;
	Boolean no_rna=FALSE;
	long i = 0L;
	FILE *fin;
	Seqfile *seq_in;	/* sequence file for the input sequence */
	Seqfile *seq_inR;	/* sequence file for the input sequence for RNA genes, just for compatibility */
	Seqfile *pp;	/* for swapping */
	Seqfile **seqM;		/* for headers */
	Seqfile **seqR;		/* for RNA genes */
	Seqfile **seq_out;	/* for output sequences. */
	long N=1L;	/* number of sequences in seqM */
	long Nr=1L;	/* number of sequences in seqR */
	long N2=1L;
	long Nout=1L;	/* number of sequences in seq_out */
	long newsize=0L;	/* connected sequence size */

	Boolean circular_bak;

#include	"parsarg.h"
	
/******************** File names and options ******************/

	
	circular_bak=circular;
	circular=FALSE;

	switch(cc){
	
		case 1:	
			strcpy(fname1,"infile");
			if(filename_input(fname1,str1)!=1){
#include        "exit1.h"
			}
				
			strcpy(fname2,fname1);
			strcat(fname2,".out");
			if(filename_input(fname2,str2)!=1){
#include        "exit1.h"
			}
			printf("Do you want introns included in the output?\n");
			printf("If yes, press 1, otherwise 0.\n\n");
			scanf("%s",str);
			if(!strcmp(str,"1")) flag=TRUE;
			break;
			
		case 2:
			strcpy(bb,p[1]);
			if((strcmp(bb,"-h")==0) || (strcmp(bb,"-help")==0)){
				help_noncod();
#include        "exit1.h"
			}
			break;

		case 3:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			flag=FALSE;
			break;

		case 4:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			if(!strcmp(p[3],"1") || !strcmp(p[3],"true") || !strcmp(p[3],"TRUE")) flag=TRUE;
			break;

		default:
			print_usage();
#include        "exit1.h"
	}


/*********************** Main part ****************************/

/* Read sequence */
	printf("Reading file: %s.\n",fname1);
	
#ifdef MAC
#ifdef CODEWARRIOR
		SIOUXSetTitle("\pnoncod");
#endif
#endif

	c=check_filetype(fname1);
		
	if(c!=GENBANK_SEQ && c!=EMBL_SEQ){
		fprintf(stderr,"This is not a GenBank or EMBL file.\n");
#include 	"exit1.h"
	}

/* Start of processing */
	if((seqM=(Seqfile **)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory\n");
#include "exit1.h"
	}
	if((seqR=(Seqfile **)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory\n");
#include "exit1.h"
	}
	if((seq_out=(Seqfile**)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory.\n");
#include "exit1.h"
	}
	if((seq_in=(Seqfile *)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory\n");
#include "exit1.h"
	}
	if((seq_in->ID=(char *)calloc(MINMEMSIZE,sizeof(char)))==NULL){ 
		fprintf(stderr,"Out of memory\n");
#include "exit1.h"
	}
	if((seq_in->header=(char *)calloc(2,sizeof(char)))==NULL){ 
		fprintf(stderr,"Out of memory\n");
#include "exit1.h"
	}
	if((seq_inR=(Seqfile *)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory\n");
#include "exit1.h"
	}
	if((seq_inR->ID=(char *)calloc(MINMEMSIZE,sizeof(char)))==NULL){ 
		fprintf(stderr,"Out of memory\n");
#include "exit1.h"
	}
	if((seq_inR->header=(char *)calloc(2,sizeof(char)))==NULL){ 
		fprintf(stderr,"Out of memory\n");
#include "exit1.h"
	}
	if((fin=fopen(fname1,"r"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n\n",fname1);
#include "exit1.h"
	}

#ifdef MAC
#ifdef CODEWARRIOR
        _fcreator=CREATOR_NAME;
        _ftype=FILETYPE_NAME;
#endif
#endif
	if(ftouch(fname2)){
		fprintf(stderr,"Cannot open output file: %s\n\n",fname2);
#include "exit1.h"
	}

/******************** start of main loop ****************************************/

	while(!feof(fin)){
		no_cds=FALSE;
		no_rna=FALSE;
		need_connect=FALSE;
		Nout=0;
		circular=FALSE;
		fileloc1=fileloc;
		if((seqM=extract_cds2(fin,seq_in,&fileloc,c,unknown))==NULL){
			if(feof(fin)) break;
			fprintf(stderr,"Error in extracting CDS\n");
			no_cds=TRUE;
		}
		N=num_seq;
#ifdef VERBOUS
	printf("N= %ld.\n",N);
#endif
		if(N<1) no_cds=TRUE;
		if(seq_in->seq==NULL || strlen(seq_in->seq)==0) {
			if(feof(fin)) break;
			else {
				putc(8,stdout);
				putc('*',stdout);
				fflush(stdout);
				no_cds=TRUE;
			}
		}
		num_seq=1;
		fileloc=fileloc1;
		if((seqR=extract_cds2(fin,seq_inR,&fileloc,c,RNA))==NULL){
			if(feof(fin)) break;
			fprintf(stderr,"Error in extracting RNA\n");
			no_rna=TRUE;
		}
		Nr=num_seq;
#ifdef VERBOUS
	printf("Nr= %ld.\n",Nr);
#endif
		if(Nr<1) no_rna=TRUE;
		if(seq_inR->seq==NULL || strlen(seq_inR->seq)==0) {
			if(feof(fin)) break;
			else {
				putc(8,stdout);
				putc('/',stdout);
				fflush(stdout);
				no_rna=TRUE;
			}
		}
		if(no_cds && no_rna) continue;
		if(no_cds){
			pp=seq_in;
			seq_in=seq_inR;
			seq_inR=pp;
		}

	/************ fill 0 in the coding regions ******************/
/*		exit_flag=FALSE;
		fileptr=fin;
*/
		circular=seq_in->circular;
		if(circular) printf("Circular DNA: %s.\n",seq_in->ID);

		if(!no_cds){
			num_seq=N;
			if(EraseCodingRegion(seq_in,seqM,c1,flag)){
				fprintf(stderr,"Unable to erase coding regions in sequence %ld.\n",i);
				continue;
			}
		}
		if(!no_rna){
			num_seq=Nr;
			if(EraseCodingRegion(seq_in,seqR,c1,flag)){
				fprintf(stderr,"Unable to erase RNA genes in sequence %ld.\n",i);
				continue;
			}
		}
		

/* debug */
#ifdef VERBOUS
		fwrite_fasta(stdout,seq_in,1,'_');
#endif		
		if(circular){
			if(seq_in->seq[0]!=c1 && seq_in->seq[seq_in->seqlen-1]!=c1){
				need_connect=TRUE;
			}
		}

		if(CountNonCoding(seq_in,&Nout,c1)){
			fprintf(stderr,"Unable to select coding regions in sequence %ld.\n",N);
			continue;
		}

		if(Nout<1) continue;

		if((seq_out=(Seqfile**)calloc(Nout+1,sizeof(Seqfile)))==NULL){
			fprintf(stderr,"Out of memory\n");
#include        "exit1.h"
		}

		for(i=0;i<Nout;i++){
			if((seq_out[i]=(Seqfile*)calloc(1,sizeof(Seqfile)))==NULL){
				fprintf(stderr,"Out of memory\n");
#include        "exit1.h"
			}
			if((seq_out[i]->ID=(char*)calloc(MINMEMSIZE,sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory\n");
#include        "exit1.h"
			}
			if((seq_out[i]->header=(char*)calloc(MINMEMSIZE,sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory\n");
#include        "exit1.h"
			}
			strcpy(seq_out[i]->ID,seq_in->ID);
			sprintf(seq_out[i]->header,"NonCoding_%ld",i+1);
		}


		if(SelectNonCoding(seq_in,seq_out,&Nout,c1)){
			fprintf(stderr,"Unable to select coding regions in sequence %ld.\n",i);
			continue;
		}

/* debug */
#ifdef VERBOUS
	printf("Nout=%ld\n",Nout);
#endif
		if(Nout<1) continue;

		for(i=0;i<Nout;i++){
			seq_out[i]->seqlen=strlen(seq_out[i]->seq);
/* debug */
#ifdef VERBOUS
	printf("i= %ld, seqlen=%ld.\n",i, seq_out[i]->seqlen);
#endif
		}

#ifdef VERBOUS
		printf("Nout=%ld, need_connect= %d.\n",Nout,need_connect);
#endif

	/* connect the first and last sequences if circular */

		if(need_connect){
			newsize=seq_out[0]->seqlen + seq_out[Nout-1]->seqlen;

/*	printf("newsize=%ld",newsize);
*/
			if((buffer=(char*)calloc(newsize+2,sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory\n");
#include 	"exit1.h"
			}
			strcpy(buffer,seq_out[0]->seq);
			strcat(buffer,seq_out[Nout-1]->seq);
			if((seq_out[0]->seq=(char*)calloc(newsize+2,sizeof(char)))==NULL){
				fprintf(stderr,"Out of memory\n");
#include 	"exit1.h"
			}
			strcpy(seq_out[0]->seq,buffer);
			seq_out[0]->seqlen=strlen(seq_out[0]->seq);
			Nout-=1;
		}

		N2=num_seq;
		num_seq=Nout;
		if(Mwrite(fname2,seq_out)){
			fprintf(stderr,"Unable to write to file: %s.\n",fname2);
			continue;
		}
		num_seq=N2;

		num_seq=N;

		strcpy(seq_in->ID,"");
		strcpy(seq_in->header,"");
		if(exit_flag) break;
	}

	fclose(fin);

	circular=circular_bak;
	printf("Terminated.\n");
#include 	"exit0.h"
}

/******************* end of file noncod.c **********************************/

