/***********************************************************************
Nucleic acid alignment (nucaln) 
main file : nucaln.c 

 	Creates a nucleic acid alignment that matches an amino acid 
	sequence alignment.

  Usage: nucaln aa.aln nuc.fa out.aln
  
  Source files are:	nucaln.c
			gtsqfn.c
  			utility.c
  			Dmc.h
  			getseq2.h
  			cmp_tbl.h
			exit0.h
			exit1.h
  	
 	Naoki Sato   Feb. 22, 2000.
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.
	For Mac version, define the compiler as well.

	Inclusion of japanese files. March 2, 2000.
	Bug fix. May 4, 2000.

		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"

#ifndef INTEGRATED
#ifdef MAC
extern int ccommand(char ***);
#endif
#endif

/* Prototype declaration */
static void print_usage(void);
#ifdef INTEGRATED
int nucaln(int c, char (*p)[S_2]);
#endif
void help_nucaln(void);
Seqfile *CreateNucAln(Seqfile *seqfile1,char *seq2,int frame,long start);

/* External functions */

#include "libSeq.h"

/* Global variables */
extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];
extern char univ_code[];
extern char code200[];
extern char code201[];
extern char mt_code[];
Boolean editing = FALSE;

/***************************************************************/
static void print_usage(void)
/***************************************************************/
{
	help_nucaln();
	return;
}
/***************************************************************/

/***************************************************************/
void help_nucaln(void)
/***************************************************************/
{
#ifdef JAPANESE

#ifdef UNIX
#include "EUC/nalnE1.h"
#else
#ifdef MAC
#include "Mac/SJIS/nalnS1.h"
#else
#include "Dos/SJIS/nalnS1.h"
#endif
#endif

#else
	printf("***************** NUCALN HELP ***********************************\n");
	printf("Usage: nucaln aa.aln nuc.fa out.aln <editing>.\n\n");
	printf("       This program creates a nucleic acid sequence alignment\n");
	printf("       that matches exactly an amino acid sequence alignment.\n");
	printf("       aa.aln should be an amino acid sequence alignment in Clustal format.\n");
	printf("       nuc.fa should be a multi-fasta file of nucleic acid sequences.\n");
	printf("       Output file is a nucleic acid alignment file in Clustal format.\n");
	printf("       All names of sequences should be identical between aa.aln and nuc.fa. \n");
	printf("       <editing> switch: if 1, editing of C to T is allowed.\n");
	printf("*****************************************************************\n");
#endif
	return;
}
/***************************************************************/

/***********************************************************************/
Seqfile *CreateNucAln(Seqfile *seqfile1,char *seq2,int frame,long start)
/***********************************************************************
This function returns a Seqfile of nucleic acid sequence
corresponding to the amino acid sequence that is part of the 
alignment file. 
************************************************************************/
{
	long i;
	char *p,*q;
	Seqfile *seq_out;
	int g='-';
	int a;
	char codon[3];
	char *codon_table;
	char *codon_table2;
	char *codon_table3;
	char *pp;
	Boolean flag=FALSE;	/* flag for no more sequence */

	if(start<1 || start>strlen(seq2)){
		fprintf(stderr,"Strange number of start value %ld in CreateNucAln.\n",start);
		return NULL;
	}
	if((seq_out=copy_seq_h(seqfile1))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return NULL;
	}
	if((seq_out->seq=(char*)calloc(3*seqfile1->seqlen+2,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return NULL;
	}
	if((codon_table=(char*)calloc(100,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return NULL;
	}
	if((codon_table2=(char*)calloc(100,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return NULL;
	}
	if((codon_table3=(char*)calloc(100,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		return NULL;
	}
	strcpy(codon_table,univ_code);
	strcpy(codon_table2,code200);
	strcpy(codon_table3,code201);

	i=frame+3*(start-1);
	q=seq_out->seq;
	for(p=seqfile1->seq;*p!='\0';p++){
		if(*p==g){
			if(i + 2 > strlen(seq2)) flag = TRUE;
			else {
				pp=extract_string(seq2,i+1,i+3);
				strcpy(codon,pp);
				free(pp);
			}

/* for debug 
	printf("codon= %s\n",codon);
*/
			if(flag) {
				*q++=g;
				*q++=g;
				*q++=g;
			} 
			else if((a=codon_translate(codon,codon_table))=='*'){

				if(editing && (*codon=='C' || *codon+1=='C')){
					if((a=codon_translate(codon,codon_table2))=='*'){
						*q++=seq2[i++];
						*q++=seq2[i++];
						*q++=seq2[i++];
					}
					else {
						*q++=g;
						*q++=g;
						*q++=g;
					}
				}
					
			}
			else {
				*q++=g;
				*q++=g;
				*q++=g;
			}
/* for debug 
	printf("codon= %c\n",a);
*/
			continue;
		}
		else if(isalpha(*p)){
			pp=extract_string(seq2,i+1,i+3);
			strcpy(codon,pp);
			free(pp);
			if(codon_translate(codon,codon_table)==*p){
				*q++=seq2[i++];
				*q++=seq2[i++];
				*q++=seq2[i++];
			}
			else {
				if(editing && (*codon=='C' || *codon+1=='C')){
					if((a=codon_translate(codon,codon_table2))==*p || (a=codon_translate(codon,codon_table3))==*p){
						*q++=seq2[i++];
						*q++=seq2[i++];
						*q++=seq2[i++];
					} else {
						*q++=tolower(seq2[i++]);
						*q++=tolower(seq2[i++]);
						*q++=tolower(seq2[i++]);
					}
				} else {
					*q++=tolower(seq2[i++]);
					*q++=tolower(seq2[i++]);
					*q++=tolower(seq2[i++]);
				}
			}
			continue;
		}
		else {
			fprintf(stderr,"Strange character %c in the amino acid sequence file.\n",*p);	
			return NULL;
		}
	}
	*q='\0';
	seq_out->seqlen=strlen(seq_out->seq);
	
	return seq_out;
}
/***********************************************************************/



/***************************************************************
****************** Main program *******************************/
#ifdef INTEGRATED
int nucaln(int cc, char (*p)[S_2])

#else
#ifdef MAC
main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif
#endif
/***************************************************************/
{
	char fname1[MAXLINE]="aa.aln";
	char fname2[MAXLINE]="nuc.fa";
	char fname3[MAXLINE]="out.aln";
	char fname3_bak[MAXLINE]="out.aln";
	char *codon_table;
	int c1=0;
	int c2=0;
	int partial_len;
	long toadd;

#ifdef JAPANESE

#ifdef UNIX
#include "EUC/nalnE2.h"
#else
#ifdef MAC
#include "Mac/SJIS/nalnS2.h"
#else
#include "Dos/SJIS/nalnS2.h"
#endif
#endif

#else
	char *str1="File name for aa alignment";
	char *str2="File name for nucleic acid seq. file";
	char *str3="Output file name";
#endif
	enum data_type dtype1=unknown;
	enum data_type dtype2=unknown;
	long numseq1=0L;
	long numseq2=0L;
	long i,j;
	long seqlen1;	/* sequence length */
	long seqlen_out;	/* sequence length for output */

	/* sequence variables */
	Seqfile **mseq_in1;
	Seqfile **mseq_in2;
	Seqfile **mseq_out;
	char *s,*pp;	/* temporary sequence container */
	char seqname[AC_LENGTH];	/* temporary sequence name container */
	char seqname2[AC_LENGTH];	/* temporary sequence name container */
	char *sn;	/* temporary nucleic acid sequence container */
	char *old_s;
	double cutoff = 0.9;

#include	"parsarg.h"

/*********************** File names ******************/
/*
printf("argc=%d, cc=%d",argc,cc);
*/
	switch(cc){
		case 1:	
			strcpy(fname1,"file1");
			if(filename_input(fname1,str1)!=1){
#include	"exit1.h" 
			}
			strcpy(fname2,"file2");
			if(filename_input(fname2,str2)!=1){
#include	"exit1.h" 
			}
			strcpy(fname3,"out.aln");
			if(filename_input(fname3,str3)!=1){
#include	"exit1.h" 
			}
			break;
		
		case 3:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			strcpy(fname3,"out.aln");
			if(filename_input(fname3,str3)!=1){
#include	"exit1.h" 
			}
			break;
			
		case 4:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			strcpy(fname3,p[3]);
			break;

		case 5:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			strcpy(fname3,p[3]);
			if(!strcmp(p[4],"1") || !strcmp(p[4],"true") || !strcmp(p[4],"TRUE")){
				editing = TRUE;
			}
			break;

		default:
			print_usage();
#include        "exit1.h"
	}

/*********************** main part ****************************/

#ifdef MAC
#ifdef CODEWARRIOR
	SIOUXSetTitle("\pnucaln");
#endif
#endif
	c1=check_filetype(fname1);
	c2=check_filetype(fname2);
	if(c1==99 || c2==99){
		fprintf(stderr,"Unknown file format.\n");
		exit(1);
	}

	if(ftouch(fname3)){
		fprintf(stderr,"Error in opening output file.\n");
#include 	"exit1.h"
	}
	strcpy(fname3_bak, fname3);
	if(c1!=CLUSTAL_SEQ || c2!=FASTA_SEQ){
		fprintf(stderr,"File formats are not correct.\n");
#include        "exit1.h"
	}
	fprintf(stdout,"Reading aa alignment ...\n");
	if((mseq_in1=read_clustal(fname1,dtype1))==NULL){
		fprintf(stderr,"Error in reading file: %s\n",fname1);
#include        "exit1.h"
               }
	numseq1=num_seq;
	dtype1=check_data_type(mseq_in1[0]);
	if(dtype1!=protein){
		fprintf(stderr,"This is not an amino acid alignment.\n");
#include        "exit1.h"
	}

	fprintf(stdout,"Reading nucleic acid sequences...\n");
	num_seq=0L;
	if((mseq_in2=Mread_fasta(fname2,dtype2))==NULL){
                       fprintf(stderr,"Error in reading file: %s\n",fname2);
#include        "exit1.h"
	}
	numseq2=num_seq;
	dtype2=check_data_type(mseq_in2[0]);
	if(dtype2!=DNA){
		fprintf(stderr,"These are not DNA sequences.\n");
#include        "exit1.h"
	}

	if(numseq1>numseq2){
		fprintf(stderr,"Not enough number of sequences in the nucleic acid sequence file.\n");
#include        "exit1.h"
	}
		
	seqlen1=mseq_in1[0]->seqlen;
	seqlen_out = 3 * seqlen1;
/*
	if((s=(char*)calloc(seqlen1+2,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory in nucaln 1.\n");
#include        "exit1.h"
	}
*/
	if((mseq_out=(Seqfile**)calloc(numseq1+2,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory in nucaln 2.\n");
#include        "exit1.h"
	}
	if((codon_table=(char*)calloc(100,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory in nucaln 3.\n");
#include        "exit1.h"
	}
	strcpy(codon_table,univ_code);

	for(i=0;i<numseq1;i++){

/*		strcpy(s,mseq_in1[i]->seq);
*/
		s = mseq_in1[i]->seq;
		strcpy(seqname,mseq_in1[i]->ID);
		subst(seqname,'=','\0');
/*
		if(StringClearGaps(s)){
			fprintf(stderr,"Error in clearing gaps.\n");
#include        "exit1.h"
		}
*/

		/* 130.51 */
		for(j=0;j<numseq2;j++){
			strcpy(seqname2,mseq_in2[j]->ID);
			subst(seqname2,'=','\0');
			
			if(!strcmp(seqname,seqname2)) break;
		}
		if(j>=numseq2){
			fprintf(stderr,"Cannot find a correct nucleic acid sequence for %s.\n",seqname);
#include        "exit1.h"
		}
		
		if((mseq_out[i]=copy_seq_h(mseq_in1[i]))==NULL){
			fprintf(stderr,"Error in copying sequence in nucaln.\n");
#include        "exit1.h"
		}
		if((mseq_out[i]->seq=(char*)calloc(seqlen_out + 20,sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory in nucaln 2.\n");
#include        "exit1.h"
		}

		sn = mseq_in2[j]->seq;	/* sn is used as a pointer to the new position to be aligned. */
		cutoff = 0.90;
		old_s = s;

		/* repeat */

		while(TRUE){

/* for debug */
#ifdef VERBOUS
printf("i= %ld, s= %s, sn= %s\n",i,s,sn);
#endif

			if(MatchSeqN(&s,&sn,mseq_out[i]->seq,seqlen_out,10,cutoff,codon_table)==0){
				
				/* check if all characters in seq1 are gaps */
				for(pp = s; *pp != '\0'; pp++){
					if(*pp != '-') break;
				}
				if(*pp=='\0') break;
				
				else if(strlen(mseq_out[i]->seq) < seqlen_out){
					fprintf(stderr,"Unable to find matching position for %s.\n",seqname);
					fprintf(stderr,"Length of mseq_out is %ld, while 3 x seqlen1 = %ld.\n",\
					strlen(mseq_out[i]->seq), seqlen_out);
					if((partial_len=strlen(mseq_out[i]->seq)) > 8) {
						fprintf(stderr,"Last point is: %s.\n",mseq_out[i]->seq+(partial_len - 9));
						fprintf(stderr,"Next amino acids are: %s.\n",s);
					}
#include        "exit1.h"
				} else {
					break;
				}
			}
			if(old_s == s) cutoff -= 0.1;
			else cutoff = 0.90;
			old_s = s;
		}
		mseq_out[i]->seqlen = strlen(mseq_out[i]->seq);
		
		/* 200805 */
		if((toadd = seqlen_out - mseq_out[i]->seqlen) > 0){
			strcat(mseq_out[i]->seq,space_line(toadd,'-'));
			mseq_out[i]->seqlen = strlen(mseq_out[i]->seq);
		}	
	
	}

	fprintf(stdout,"Writing sequences...\n");
	num_seq=numseq1;
	if(strcmp(fname3,"")==0) strcpy(fname3,fname3_bak);
	if(write_clustal(fname3,mseq_out)){
		fprintf(stderr,"Error in writing sequence.\n");
#include        "exit1.h"
	}

#ifdef VERBOUS
        fprintf(stdout,"Sequence written to file: %s\n",fname3);
#else
	putc(8,stdout);
	putc('w',stdout);
	fflush(stdout);
#endif

	printf("\nTerminated.\n");

#include "exit0.h"

}

/************************************************************
		End of file nucaln.c
************************************************************/


