/*****************************************************
File name : pretty.c (pretty print)
	
	Functions : 	
			write_pretty
			fwrite_pretty
				
	Naoki Sato Aug. 23, 2000.

	Integrated Mac, DOS, and UNIX version.

	Modified according to the change in exon_list. Oct. 25, 2000.
	Use of version_info. Dec. 2, 2001.

	--- Do not delete this comment.---

******************************************************/

#include "Dmc.h"
#include "getseq2.h"

/* Prototype declaration */

int make_header(Seqfile *seqfile1,char *exon_file);
int fwrite_pretty(FILE *fout,Seqfile *seqfile1,char *exon_file,char *codon_file);
int write_pretty(char *fname,Seqfile *seqfile1,char *exon_file,char *codon_file);
static void show_help(void);

/* External functions */
#ifndef __PRETTY
#define __PRETTY
#endif
#include "libSeq.h"

/* Global variables are defined here. */

extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];
extern char aaname[20][2][10];
extern Boolean circular;
extern int print_line;
extern char version_string[20];

/**********************************************************************************/
static void show_help(void)
/**********************************************************************************/
{
#ifdef JAPANESE
#ifdef UNIX
#include "EUC/prettyE.h"
#else
#ifdef MAC
#include "Mac/SJIS/prettyS.h"
#else
#include "Dos/SJIS/prettyS.h"
#endif
#endif
#else
        printf("********************** PRETTY PRINT HELP ************************\n");
        printf("Usage: toprot infile outfile p<exon file> codon_table.\n\n");
        printf("       <exon file> is a name of a file that describes position of \n");
        printf("       exons. This file should contain a single line plus RETURN.\n");
        printf("  Example line of exon file. Do not include comments after --.\n");
        printf("       ntcA 1 91  762  -- a single ORF without introns.\n");
        printf("       nad7 3 join 828 967  1999 2067  3066 4038  -- 2 introns.\n");
        printf("                   <exon 1> <exon 2>   <exon 3>\n");
        printf("       Codon_table is optional. Use siseq help.\n");
        printf("*****************************************************************\n");

#endif
	return;
}
/**********************************************************************************/



/**********************************************************************************/
int make_header(Seqfile *seqfile1,char *exon_file)
/**********************************************************************************/
{
	FILE *fin;
	char string[MAXLINE+2];
	char string2[MAXLINE+2];	/* gene name */
	char string3[MAXLINE+2];	/* exon number character */
	char str[MAXLINE+2];
	int i;
	int exons=0;
	char *tmp;
	long seqpos=0L;
	long seqlen=seqfile1->seqlen;

	if(exon_file==NULL){
		fprintf(stderr,"Exon file name empty in make_header\n");
		return TRUE;
	}
	
	if(!strcmp(exon_file,"")){
		show_help();
		printf("Do you wish to input exon data interactively?  Answer y or n. ");
		scanf("%s",str);
		if (!strcmp(str,"y")){
			printf("What is the name of gene?  ");
			fflush(stdin);
			while(fgets(str,MAXLINE,stdin)!=NULL){
				tailchop(str);
				if(strcmp(str,"")){
					strcpy(string,str);
					break;
				}
				fflush(stdin);
			}
			strcpy(str,"");
			printf("Number of exons?  Use 0 to abort.");
			fflush(stdin);
			while(fgets(str,MAXLINE,stdin)!=NULL){
				tailchop(str);
				if(!int_check(str)){
					exons=atoi(str);
					if(exons<=0) break;
					sprintf(string2," %s",str);
					strcat(string,string2);
					strcpy(string2,"");
					if(exons>1) strcat(string," join"); 
					strcpy(str,"");
					break;
				}
				fflush(stdin);
			}
			if(exons<=0){
				fprintf(stderr,"Input aborted.\n");
#include 	"exit1.h"
			}

			for(i=1;i<=exons;i++){

				printf("Start position of exon %d? Use 0 to abort. ",i);
				fflush(stdin);
				while(fgets(str,MAXLINE,stdin)!=NULL){
					tailchop(str);
					if(strcmp(str,"")){
						if(!int_check(str)){
							if((seqpos=atol(str))==0L){
#include 	"exit1.h"
							} 
							if(seqpos>seqlen-10) {
								printf("This value exceeds sequence length. Type new value. ");
								fflush(stdin);
								continue;
							} 
							tmp=extract_string(seqfile1->seq,seqpos,seqpos+10);
							printf("\n   %s\n   >\n",tmp);
							free(tmp); 
							printf("Is this OK? y or n.");
							scanf("%s",string3);
							if(strcmp(string3,"y") && strcmp(string3,"")){
								printf("Type new value. ");
								fflush(stdin);
								continue;
							} 
							sprintf(string2,"  %s",str);
							strcat(string,string2);
							strcpy(string2,"");
							strcpy(str,"");
							break;
						}
					}
					fflush(stdin);
				}
				printf("End position of exon %d? Use 0 to abort. ",i);
				fflush(stdin);
				while(fgets(str,MAXLINE,stdin)!=NULL){
					tailchop(str);
					if(strcmp(str,"")){
						if(!int_check(str)){
							if((seqpos=atol(str))==0L){
#include 	"exit1.h"
							} 
							if(seqpos>seqlen-1) {
								printf("This value exceeds sequence length. Type new value. ");
								fflush(stdin);
								continue;
							} 
							tmp=extract_string(seqfile1->seq,seqpos-10,seqpos);
							printf("\n   %s\n             >\n",tmp);
							free(tmp); 
							printf("Is this OK? y or n.");
							scanf("%s",string3);
							if(strcmp(string3,"y") && strcmp(string3,"")){
								printf("Type new value. ");
								fflush(stdin);
								continue;
							} 
							sprintf(string2," %s",str);
							strcat(string,string2);
							strcpy(string2,"");
							strcpy(str,"");
							break;
						}
					}
					fflush(stdin);
				}

			}
			sprintf(string2," ");
			strcat(string,string2);
			strcpy(string2,"");
		}
		else return TRUE;
	}

	else {
		if((fin=fopen(exon_file,"r"))==NULL){
			fprintf(stderr,"Cannot open file: %s\n",exon_file);
			return TRUE;
		}

		if(fgets(string,MAXLINE,fin)==NULL) return FALSE;	/* This might be a while loop */
		sscanf(string,"%s %s",string2,string3);
		if(num_check(string3)) {
			fprintf(stderr,"No exon number is indicated.\n");
			return TRUE;
		}
		fclose(fin);
	}


	if((seqfile1->header=(char*)calloc(strlen(string)+2,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory in make_header.\n");
		return TRUE;
	}
	tailchop(string);
	strcpy(seqfile1->header,string);

	return FALSE;
}


/**********************************************************************************/
int fwrite_pretty(FILE *fout,Seqfile *seqfile1,char *exon_file,char *codon_file)
/**********************************************************************************/
{
/*	long code_count[30];
	enum data_type dtype1;*/
	Seqfile *seq_exons;
	Seqfile *aa_seq;
	char *blank;
	char *seq[2];
	long seqlen=seqfile1->seqlen;
	long aaseqlen;
	long j,k,m;
	long m1=0L;
	int i;
	char buffer[100];
	int linelen=60;
	char *p;
	Exon_record **exon;
	int exons=0;
	char str1[100],str2[100];
	char comment[MAXLINE];

	if(seqfile1==NULL){
		fprintf(stderr,"Empty sequence.\n");
		return TRUE;
	}

	if(print_line>=21 && print_line<=91){ 
		linelen=3*(print_line/3);
	}

	if(make_header(seqfile1,exon_file)) {
		fprintf(stderr,"Unable to make header in fwrite_pretty.\n");
		return TRUE;
	}

	if((seq_exons=ext_exons(seqfile1))==NULL) {
		fprintf(stderr,"Unable to extract exons in fwrite_pretty.\n");
		return TRUE;
	}

	if((aa_seq=seq_translate(seq_exons,1,codon_file))==NULL){
		fprintf(stderr,"Error of translation in sequence: %s.\n",seqfile1->ID);
		return TRUE;
	}
	aaseqlen=aa_seq->seqlen;

	blank=space_line((long)strlen(seq_exons->seq),' ');
	strcpy(seq_exons->seq,blank);
	free(blank);

/* analyzing exon structure */

	sscanf(seqfile1->header,"%s %s",str1,str2);
	if(int_check(str2)){ 
		fprintf(stderr,"Error of exon number in fwrite_pretty.\n");
		return TRUE;
	}
	else sscanf(str2,"%d",&exons);

	if((exon=(Exon_record**)calloc(exons+1,sizeof(Exon_record)))==NULL){
		fprintf(stderr,"Out of memory\n");
		return 1;
	}
	for(i=0;i<exons;i++){
		if((exon[i]=(Exon_record*)calloc(1,sizeof(Exon_record)))==NULL){
			fprintf(stderr,"Out of memory\n");
			return TRUE;
		}
	}

	strcpy(comment,"                                              ");
	if(exon_list(seqfile1->header,exon,comment)){
		fprintf(stderr,"Error of making exon list in fwrite_pretty from seq: %s.\n",seqfile1->ID);
		return TRUE;
	}


	if(TransSplicing(exon,exons)){
		fprintf(stderr,"This might be trans-spliced. Abort to write pretty sequence.\n");
		return TRUE;
	}


/* making output sequence */

	for(i=0;i<2;i++){
		if((seq[i]=(char*)calloc(seqlen+3,sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory in fwrite_pretty.\n");
#include        "exit1.h"
		}
	}

	strcpy(seq[0],seqfile1->seq);
	for(j=0;j<seqlen;j++){
		seq[0][j]=tolower(seq[0][j]);
		seq[1][j]=' ';
	}


	k=0;
	for(i=0;i<exons;i++){
		if(exon[i]->nc){
			for(j=exon[i]->end-1;j>=exon[i]->begin-1;j--){
				seq[0][j]=toupper(seq[0][j]);
				if(k%3==0 && k<aaseqlen*3-2) seq[1][j]=aa_seq->seq[(k++)/3];
				else {
					seq[1][j]=' ';
					k++;
				}
				if(k>seqlen) break;
			}
		}
		else{
			for(j=exon[i]->begin-1;j<exon[i]->end;j++){
				seq[0][j]=toupper(seq[0][j]);
				if(k%3==0 && k<aaseqlen*3-2) seq[1][j]=aa_seq->seq[(k++)/3];
				else {
					seq[1][j]=' ';
					k++;
				}
				if(k>seqlen) break;
			}
		}
	}

/* printing */

	version_info(fout);
	fprintf(fout,"%s%s\n",FASTA_INIT,seqfile1->ID);
	fprintf(fout,"%s\n",seqfile1->header);
	fprintf(fout,"  Codon table file: %s\n\n",codon_file);

	m=0;	/* amino acid count */
	for(j=0;j<seqlen;j+=linelen){
		for(i=0;i<2;i++){
			if(j+linelen-1>seqlen-1) k=seqlen-1;
			else k=j+linelen-1;
			if(i==1 && k+1>seqlen) strcpy(buffer," ");
			else{
				p=extract_string(seq[i],j+1,k+1);
				strcpy(buffer,p);
				free(p);
				if(k-j<linelen) {
					p=space_line(linelen+j-k,' ');
					strcat(buffer,p);
					free(p);
				}
			}
			if(i==0) fprintf(fout,"%s%6ld\n",buffer,k+1);
			else {
				m1=count_alnum(buffer);
				if(m1>0) {
					m+=m1;
					fprintf(fout,"%s%6ld\n",buffer,m);
				}
			}
		}
		if(m1>0) fprintf(fout,"\n");
	}

	fputc('\n',fout);
	fputc('\n',fout);
	fflush(fout);	
	return 0;
}



/**********************************************************************************/
int write_pretty(char *fname,Seqfile *seqfile1,char *exon_file,char *codon_file)
/**********************************************************************************/
{
	FILE *fout;

	if(seqfile1==NULL){
		fprintf(stderr,"Empty sequence.\n");
		return TRUE;
	}

#ifdef MAC
#ifdef CODEWARRIOR
	_fcreator=CREATOR_NAME;
	_ftype=FILETYPE_NAME;
#endif
#endif

	if((fout=fopen(fname,"a+"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n",fname);
		return TRUE;
	}

	if(fwrite_pretty(fout,seqfile1,exon_file,codon_file)){
		fprintf(stderr,"Trouble in writing pretty sequence: %s\n",fname);
		fclose(fout);
		return TRUE;
	}

#ifdef VERBOUS
	fprintf(stdout,"Sequence written to file: %s\n",fname);
#else
/*	printf("w");*/
	putc(8,stdout);
	putc('w',stdout);
	fflush(stdout);
#endif

	fclose(fout);
	return FALSE;
}


/**********************************************************************************/

/*********************** end of file pretty.c *************************************/

