/***********************************************************************
Sequence catenate (seqcat) 
main file : seqcat.c 

 	Adds sequences in file2 to the sequences in file1.

  Usage: seqcat file1 file2 outfile
  
  Source files are:	seqcat.c
			gtsqfn.c
  			utility.c
  			Dmc.h
  			getseq2.h
  			cmp_tbl.h
			exit0.h
			exit1.h
  	
 	Naoki Sato   Oct. 14, 1999.
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.
	For Mac version, define the compiler as well.
	siseq_exit0 or 1 changed to exit0 or 1. Nov. 13, 1999.
	Japanese messages. Feb. 4, 2000.

		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"

#ifndef INTEGRATED
#ifdef MAC
extern int ccommand(char ***);
#endif
#endif

/* Prototype declaration */
static void print_usage(void);
#ifdef INTEGRATED
int seqcat(int c, char (*p)[S_2]);
#endif
void help_seqcat(void);

/* External functions */

#include "libSeq.h"

/* Global variables */
extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];

/***************************************************************/
static void print_usage(void)
/***************************************************************/
{
	help_seqcat();
	return;
}
/***************************************************************/

/***************************************************************/
void help_seqcat(void)
/***************************************************************/
{
#ifdef JAPANESE
#ifdef UNIX
#include "EUC/seqcatE.h"
#else
#ifdef MAC
#include "Mac/SJIS/seqcatS.h"
#else
#include "Dos/SJIS/seqcatS.h"
#endif
#endif
#else
        printf("***************** SEQCAT HELP ***********************************\n");
        printf("Usage: seqcat file1 file2 outfile.\n\n");
        printf("       This program adds sequences in file2 to\n");
	printf("       the sequences in file1 and writes the combined\n");
	printf("       sequences in outfile.\n");
        printf("       All file formats that can be read in the siseq package\n");
	printf("       are allowed as a single sequence file.\n");
	printf("       If file1 and file2 are in the CLUSTAL format, the output\n");
        printf("       will be in the CLUSTAL format. Otherwise, output will be \n");
        printf("       in the FASTA format.\n");
        printf("*****************************************************************\n");
#endif
	return;
}
/***************************************************************/

/***************************************************************
****************** Main program *******************************/
#ifdef INTEGRATED
int seqcat(int cc, char (*p)[S_2])

#else
#ifdef MAC
main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif
#endif
/***************************************************************/
{
	char fname1[MAXLINE]="file1";
	char fname2[MAXLINE]="file2";
	char fname3[MAXLINE]="outfile";
	int c1=0;
	int c2=0;
#ifdef JAPANESE
	char *str1="ե̾ 1";
	char *str2="ե̾ 2";
	char *str3="ϥե̾";
#else
	char *str1="File name 1";
	char *str2="File name 2";
	char *str3="Output file name";
#endif
	enum data_type dtype1=unknown;
	enum data_type dtype2=unknown;
	enum data_type dtype_out=unknown;
	long numseq1=0L;
	long numseq2=0L;

	/* sequence variables */
	Seqfile *seq_in1;
	Seqfile *seq_in2;
	Seqfile **mseq_in1;
	Seqfile **mseq_in2;

#include	"parsarg.h"

/*********************** File names ******************/
/*
printf("argc=%d, cc=%d",argc,cc);
*/
	switch(cc){
		case 1:	
			strcpy(fname1,"file1");
			if(filename_input(fname1,str1)!=1){
#include	"exit1.h" 
			}
			strcpy(fname2,"file2");
			if(filename_input(fname2,str2)!=1){
#include	"exit1.h" 
			}
			strcpy(fname3,"outfile");
			if(filename_input(fname3,str3)!=1){
#include	"exit1.h" 
			}
			break;
		
		case 3:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			strcpy(fname3,"outfile");
			if(filename_input(fname3,str3)!=1){
#include	"exit1.h" 
			}
			break;
			
		case 4:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			strcpy(fname3,p[3]);
			break;

		default:
			print_usage();
#include        "exit1.h"
	}

/*********************** main part ****************************/

	if((seq_in1=(Seqfile*)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		exit(1);
	}
	if((seq_in2=(Seqfile*)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		exit(1);
	}

#ifdef MAC
#ifdef CODEWARRIOR
	SIOUXSetTitle("\pseqcat");
#endif
#endif
	c1=check_filetype(fname1);
	c2=check_filetype(fname2);
	if(c1==99 || c2==99){
		fprintf(stderr,"Unknown file format.\n");
		exit(1);
	}

	if(ftouch(fname3)){
		fprintf(stderr,"Error in opening output file.\n");
#include 	"exit1.h"
	}
	if(c1==CLUSTAL_SEQ && c2==CLUSTAL_SEQ){
		fprintf(stdout,"These are CLUSTAL files.\n");
		fprintf(stdout,"Reading sequence 1...\n");
		if((mseq_in1=read_clustal(fname1,dtype1))==NULL){
			fprintf(stderr,"Error in reading file: %s\n",fname1);
                        exit(1);
                }
		numseq1=num_seq;
		dtype1=check_data_type(mseq_in1[0]);

		fprintf(stdout,"Reading sequence 2...\n");
		num_seq=0L;
		if((mseq_in2=read_clustal(fname2,dtype2))==NULL){
                        fprintf(stderr,"Error in reading file: %s\n",fname2);
                        exit(1);
		}
		numseq2=num_seq;
		dtype2=check_data_type(mseq_in2[0]);

		if(numseq1!=numseq2){
			fprintf(stderr,"Numbers of sequences are not equal.\n");
#include        "exit1.h"
		}
/*		if(dtype1!=dtype2){ 
			fprintf(stderr,"Types of sequences are not identical.\n");
#include        "exit1.h"
		}
*/
		if(cat_aln(mseq_in1,mseq_in2)){
			fprintf(stderr,"Unable to combine sequences.\n");
#include        "exit1.h"
		}

		fprintf(stdout,"Writing sequences...\n");
		if(write_clustal(fname3,mseq_in1)){
                        fprintf(stderr,"Error in writing sequence.\n");
#include        "exit1.h"
                }
	}

	else {
		fprintf(stdout,"Reading sequence 1...\n");
		if(c1==GENBANK_SEQ) printf("This is a GenBank file.\n");
		else if(c1==GBPROT_SEQ) printf("This is a GenBank protein file.\n");
		else if(c1==EMBL_SEQ) printf("This is an EMBL file.\n");
		else if(c1==SW_SEQ) printf("This is a SwissProt file.\n");
		else if(c1==PIR_SEQ) printf("This is a PIR file.\n");
		else if(c1==FASTA_SEQ) printf("This is a FASTA file.\n");
		if((seq_in1=read_seq(fname1,c1,dtype1))==NULL){
                       	fprintf(stderr,"Error in reading sequence 1.\n");
#include        "exit1.h"
               	}
		if(seq_in1==NULL) {
                        fprintf(stderr,"Error in reading sequence 1.\n");
#include        "exit1.h"
                }
		if(strlen(seq_in1->seq)==0) {
                        fprintf(stderr,"Error in reading sequence 1.\n");
#include        "exit1.h"
                }

		if(seq_in1->dtype==protein || seq_in1->dtype==DNA ){
			dtype1=seq_in1->dtype;
		}
		else {
			dtype1=check_data_type(seq_in1);
			seq_in1->dtype=dtype1;
		}

		fprintf(stdout,"Reading sequence 2...\n");
		if(c2==GENBANK_SEQ) printf("This is a GenBank file.\n");
		else if(c2==GBPROT_SEQ) printf("This is a GenBank protein file.\n");
		else if(c2==EMBL_SEQ) printf("This is an EMBL file.\n");
		else if(c2==SW_SEQ) printf("This is a SwissProt file.\n");
		else if(c2==PIR_SEQ) printf("This is a PIR file.\n");
		else if(c2==FASTA_SEQ) printf("This is a FASTA file.\n");
		if((seq_in2=read_seq(fname2,c2,dtype2))==NULL){
                       	fprintf(stderr,"Error in reading sequence 2.\n");
#include        "exit1.h"
               	}
		if(seq_in2==NULL) {
                        fprintf(stderr,"Error in reading sequence 2.\n");
#include        "exit1.h"
                }
		if(strlen(seq_in2->seq)==0) {
                        fprintf(stderr,"Error in reading sequence 2.\n");
#include        "exit1.h"
                }

		if(seq_in2->dtype==protein || seq_in2->dtype==DNA ){
			dtype2=seq_in2->dtype;
		}
		else {
			dtype2=check_data_type(seq_in2);
			seq_in2->dtype=dtype2;
		}
		
		if(dtype1!=dtype2){
			fprintf(stderr,"The two sequences are of different types.\n");
#include        "exit1.h"
		}
	
		printf("Data type is %s\n",data_type_name[dtype1]);
		dtype_out=dtype1;
		if(cat_seq(seq_in1,seq_in2)){
			fprintf(stderr,"Unable to combine sequences.\n");
#include        "exit1.h"
		}

#ifdef MAC
#ifdef CODEWARRIOR
        _fcreator=CREATOR_NAME;
        _ftype=FILETYPE_NAME;
#endif
#endif
		if(write_fasta(fname3,seq_in1,0,' ')){
			fprintf(stderr,"Error in writing sequence.\n");
			exit(1);
		}
	}

#ifdef VERBOUS
        fprintf(stdout,"Sequence written to file: %s\n",fname3);
#else
	putc(8,stdout);
	putc('w',stdout);
	fflush(stdout);
#endif

	printf("\nTerminated.\n");

#include "exit0.h"

}

/************************************************************
		End of file seqcat.c
************************************************************/


