/***********************************************************************
Print similarity table (simtbl) 
main file : simtbl.c

 	Calculates similarity score between each pair of sequences in a
	FASTA or CLUSTAL sequence file.

  Usage: simtbl infile outfile 
  
  Default outfile name is "infile"+".out"
  
  Source files are:	simtbl.c
  			gtsqfn.c
  			utility.c
  			Dmc.h
  			getseq2.h
  			cmptbl.h
  	
 	Naoki Sato   Dec. 15, 1999.  
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.
	For Mac version, define the compiler as well.
        Inclusion of Japanese messages. March 4, 2000.

	
		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"

#ifndef INTEGRATED
#ifdef MAC
extern int ccommand(char ***);
#endif
#endif

/* Prototype declaration */
static void print_usage(void);
#ifdef INTEGRATED
int simtbl(int c, char (*p)[S_2]);
#endif
void help_simtbl(void);

#include "libSeq.h"

/* Global variables */
extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];

/***************************************************************/
static void print_usage(void)
/***************************************************************/
{
	help_simtbl();
	return;
}
/***************************************************************/

/***************************************************************/
void help_simtbl(void)
/***************************************************************/
{
#ifdef JAPANESE
#ifdef UNIX
#include "EUC/simtblE.h"
#else
#ifdef MAC
#include "Mac/SJIS/simtblS.h"
#else
#include "Dos/SJIS/simtblS.h"
#endif
#endif
#else
	printf("****************** SIMTBL HELP ****************************\n");
	printf("Usage: simtbl infile outfile \n");
	printf("       This program calculates similarity score between \n");
	printf("       each pair of sequences in a FASTA or CLUSTAL files. \n");
	printf("       The sequence length of all sequences in a FASTA \n");
	printf("       file should be identical.\n");
	printf("Usage: simtbl infile outfile list\n");
	printf("       List of identical sequences.\n");
	printf("Usage: simtbl infile outfile nr\n");
	printf("       Identical sequences are removed.\n");
	printf("***********************************************************\n");
#endif
	return;
}
/***************************************************************/


/***************************************************************
****************** Main program *******************************/
#ifdef INTEGRATED
int simtbl(int cc, char (*p)[S_2])

#else
#ifdef MAC
main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif

#endif
/***************************************************************/

{
	char fname1[100]="infile";
	char fname2[100]="outfile";
	FILE *fout;
	int c=0;
	int i1,i2;
	long k1,k2;	/* counter for sequences */
	Boolean list_mode=FALSE;
	Boolean nr_mode=FALSE;
	Boolean table_mode=TRUE;

#include        "filenam.h"     /* definition of char *str1 and char *str2 */

	long N0;	/* number of sequences */
	enum data_type dtype1=unknown;
	Seqfile **seq_in;
	SimilarityCounts **sc;	/* container of similarity score */ 
	double scoring_table[30][30];	/* table of scoring coefficients */

#include	"parsarg.h"
	
/*********************** File names ******************/

	switch(cc){
		case 1:	
			strcpy(fname1,"infile");
			if(filename_input(fname1,str1)!=1){
#include	"exit1.h"
			}
			strcpy(fname2,fname1);
			strcat(fname2,".out");
			if(filename_input(fname2,str2)!=1){
#include	"exit1.h"
			}
			break;
		
		case 3:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			break;

		case 4:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			if(!strcmp(p[3],"list")){
				list_mode=TRUE;
				table_mode=FALSE;
			} else if(!strcmp(p[3],"nr")){
				nr_mode=TRUE;
				table_mode=FALSE;
			}
			break;
			
		default:
			print_usage();
#include	"exit1.h"
	}
		
/***********************Main part****************************/

	printf("Reading sequence...\n");

#ifdef MAC
#ifdef CODEWARRIOR
	SIOUXSetTitle("\psimtbl");
#endif
#endif
	c=check_filetype(fname1);
	if(c!=CLUSTAL_SEQ && c!=FASTA_SEQ) {
		fprintf(stderr,"This is not a FASTA or CLUSTAL file.\n");
#include 	"exit1.h"
	}
	
	if(c==CLUSTAL_SEQ && !table_mode){
		fprintf(stderr,"Only FASTA sequences are accepted in the list or nr mode.\n");
#include 	"exit1.h"
	}

	if(c==CLUSTAL_SEQ){	
		if((seq_in=read_clustal(fname1,dtype1))==NULL){
			fprintf(stderr,"Error in reading file: %s\n",fname1);
#include 	"exit1.h"
		}
	} else {
		if((seq_in=Mread_fasta(fname1,dtype1))==NULL){
			fprintf(stderr,"Error in reading file: %s\n",fname1);
#include 	"exit1.h"
		}
		if(table_mode && UnequalLength(seq_in)){
			fprintf(stderr,"Unequal length of sequences in file: %s\n",fname1);
#include 	"exit1.h"
		}
	}

	N0=num_seq;
	if(dtype1==unknown){
		dtype1=check_data_type(seq_in[0]);
		if(dtype1!=DNA && dtype1!=RNA && dtype1!=protein){
			fprintf(stderr,"Sequences should be DNA, RNA or protein in file: %s. \n",fname1);
#include 	"exit1.h"
		}
	}
	printf("data type is %s.\n",data_type_name[dtype1]);
	for(k1=1;k1<N0;k1++){
		if(dtype1!=check_data_type(seq_in[k1])){
/* April 12, 2010 */
			fprintf(stderr,"All sequences should be of the same data type, either DNA or RNA in %s.\n",seq_in[k1]->ID);
			fprintf(stderr,"But we continue ...\n");
		}
	}


/*********** initializing scoring table **************************/
	for(i1=0;i1<30;i1++){
		for(i2=0;i2<30;i2++){
			scoring_table[i1][i2]=0.0;
		}
	}

	if(dtype1==DNA || dtype1==RNA){
/************ making scoring table for nucleic acid ****************/
		scoring_table['A'-'A']['A'-'A']=1.0;
		scoring_table['T'-'A']['T'-'A']=1.0;
		scoring_table['G'-'A']['G'-'A']=1.0;
		scoring_table['C'-'A']['C'-'A']=1.0;
		scoring_table['U'-'A']['U'-'A']=1.0;
		scoring_table['U'-'A']['T'-'A']=1.0;
		scoring_table['T'-'A']['U'-'A']=1.0;
	}
	else if(dtype1==protein){
/************ making scoring table for protein ****************/
		scoring_table['A'-'A']['A'-'A']=1.0;
		scoring_table['C'-'A']['C'-'A']=1.0;
		scoring_table['D'-'A']['D'-'A']=1.0;
		scoring_table['E'-'A']['E'-'A']=1.0;
		scoring_table['F'-'A']['F'-'A']=1.0;
		scoring_table['G'-'A']['G'-'A']=1.0;
		scoring_table['H'-'A']['H'-'A']=1.0;
		scoring_table['I'-'A']['I'-'A']=1.0;
		scoring_table['K'-'A']['K'-'A']=1.0;
		scoring_table['L'-'A']['L'-'A']=1.0;
		scoring_table['M'-'A']['M'-'A']=1.0;
		scoring_table['N'-'A']['N'-'A']=1.0;
		scoring_table['P'-'A']['P'-'A']=1.0;
		scoring_table['Q'-'A']['Q'-'A']=1.0;
		scoring_table['R'-'A']['R'-'A']=1.0;
		scoring_table['S'-'A']['S'-'A']=1.0;
		scoring_table['T'-'A']['T'-'A']=1.0;
		scoring_table['U'-'A']['U'-'A']=1.0;
		scoring_table['V'-'A']['V'-'A']=1.0;
		scoring_table['W'-'A']['W'-'A']=1.0;
		scoring_table['Y'-'A']['Y'-'A']=1.0;
	}
	else {
		fprintf(stderr,"All sequences should be of the same data type, either DNA or RNA in %s.\n",seq_in[k1]->ID);
		fprintf(stderr,"But we continue ...\n");
	}


/************* initializing similarity table **********************/
	if((sc=(SimilarityCounts **)calloc(N0*N0+1,sizeof(SimilarityCounts)))==NULL){
		fprintf(stderr,"Error in allocating memory to sc table.\n");
#include 	"exit1.h"
	}
	for(k1=0;k1<N0*N0+1;k1++){
		if((sc[k1]=(SimilarityCounts *)calloc(1,sizeof(SimilarityCounts)))==NULL){
			fprintf(stderr,"Error in allocating memory to sc table.\n");
#include 	"exit1.h"
		}
	}

/************ calculation  *******************/
	for(k1=1;k1<N0;k1++){
		for(k2=0;k2<k1;k2++){	
			if(CountSimilarity(seq_in[k1],seq_in[k2],N0,scoring_table,sc,k1*N0+k2)){
			fprintf(stderr,"Error in making similarity table.\n");
#include 	"exit1.h"
			}
		}
	}

/********* write table **********************/

#ifdef MAC
#ifdef CODEWARRIOR
        _fcreator=CREATOR_NAME;
        _ftype=FILETYPE_NAME;
#endif
#endif

	if(ftouch(fname2)){
		fprintf(stderr,"Cannot open file %s.\n",fname2);
#include 	"exit1.h"
	}	
	if((fout=fopen(fname2,"a"))==NULL){
		fprintf(stderr,"Cannot open file %s.\n",fname2);
#include 	"exit1.h"
	}	


	if(table_mode){
		fprintf(fout,"\nSimilarity table\n");
		for(k1=0;k1<N0;k1++){
			fprintf(fout,"%6ld: %s.\n",k1+1,seq_in[k1]->ID);
		}
		fprintf(fout,"\n\n");
	
		PrintScTable(fout,N0,sc,3);

	} else if(list_mode){
		for(k1=1;k1<N0;k1++){
			for(k2=0;k2<k1;k2++){
//				if(sc[k1*N0+k2]->total != 0.0 && sc[k1*N0+k2]->match == sc[k1*N0+k2]->total){
				if(sc[k1*N0+k2]->total != 0L && sc[k1*N0+k2]->match == sc[k1*N0+k2]->total){
					fprintf(fout,"Sequence %ld: %s is identical to sequence %ld: %s.\n",k1,seq_in[k1]->ID,k2,seq_in[k2]->ID);
				}
			}
		}
		fprintf(fout,"\n");

	} else if(nr_mode){
		for(k1=1;k1<N0;k1++){
			if(seq_in[k1]->seqlen ==0) continue;
			for(k2=0;k2<k1;k2++){
//				if(sc[k1*N0+k2]->total != 0.0 && sc[k1*N0+k2]->match == sc[k1*N0+k2]->total){
				if(sc[k1*N0+k2]->total != 0L && sc[k1*N0+k2]->match == sc[k1*N0+k2]->total){
					seq_in[k1]->seq = "";
					seq_in[k1]->seqlen = 0;
					break;
				}
			}
		}
		if(Mfwrite2(fout,seq_in,0L,0L)){
			fprintf(stderr,"Error in writing sequence.\n");
			exit(1);
		}
		printf("\nTerminated.\n");
		exit(0);
	}
	
	fclose(fout);

	for(k1=0;k1<N0;k1++){
		free_seq(seq_in[k1]);
	}
	
	printf("\nTerminated.\n");

#include 	"exit0.h"

}

/************************************************************
		End of file simtbl.c
************************************************************/
