/***********************************************************************
Siseq (Simple sequence manipulation tool, an integrated version)
main file : siseq.c

        This is a front end for the various tools of 
        the siseq package.

  Usage: Double click on the siseq icon, and then input 
  	commands according to the menu.
  
  Source files are:

extcds.c   getclu.c   siseq.c     
cdncfn.c    extrfn.c   getseq2.c  sprem.c    tofast.c   trnsfn.c
cdsnuc.c   excdfn.c   extrna.c   gtsqfn.c   srfunc.c   toprot.c  utility.c
mrdra.c	genlist.c   getent.c	chname.c   seqcat.c	noncod.c
extint.c	calcseq.c	simtbl.c	

Dmc.h      cmptbl.h   getseq2.h  glovar.h   version.h	exit1.h	exit0.h
 
 	Naoki Sato   Nov. 23, 1998.

 	Siseq is being developed as Mac, Unix, and Windows(DOS) 
 	integrated version. But this is a Macintosh version.
	To compile, edit Dmc.h to define the platform.
	For Mac version, define the compiler as well.
	
	Help menus added. Nov. 24, 1998.
	mrdfa added. Dec. 28, 1998.
	Modified. Jan. 10, 1999.
	Toprot1 and toprot6 are combined to toprot. Feb. 14, 1999.
	Menu modified. Feb. 21, 1999.
	Menu modified. March 13, 1999.
	Added comple. April 22, 1999.
	Comple is adapted to multiple sequence file. April 25, 1999.
	Added genlist. April 25, 1999.
	Added print_line. April 29, 1999.
	Added command input from siseq.cf. May 2, 1999.
	Added shell commands. May 3, 1999.
	Removed clrgap, which is included in tofast. May 17, 1999.
	Removed toRNA and comple, which are integrated into tofast. May 19, 1999.
	Added setvar command. May 20, 1999.
	Removed clu2fa, which is integrated into tofast. May 21, 1999.
	Added dual modes of cdsnuc and extrna. ADDSEQG in setvar. May 30, 1999.
	Added getent. May 31, 1999.
	Added fcat. June 1, 1999.
	Added sequence import facility in cdsnuc. Removed analcds. June 9, 1999.
	Changes in the location of declaration of global variables, i.e.,
	within the libSeq.a. libSeq.h was added. June 15, 1999.
	Menus modified. Aug. 11, 1999.
	Added command_scan to allow reading of quoted arguments. Aug. 25, 1999.
	Autocloseonexit for Mac. Modification of menus. Aug. 30, 1999.
	Added chname. Oct. 11, 1999.
	Added seqcat. Oct. 13, 1999.
	Version 1.2.0. Oct. 18, 1999.
	Size of s enlarged. Nov. 11, 1999.
	siseq_exit0 or 1 changed to exit0 or 1. Nov. 13, 1999.
	Bug fix in command input. Nov. 15, 1999.
	Added noncod and extint. Version 1.2.1. Dec. 8, 1999.
	Added simtbl. Version 1.2.2. Dec. 19, 1999.
	Bug fix in show_all_help. Jan. 21, 2000.
	Classified menu. Feb. 4, 2000.
	Inclusion of Japanese messages. March 3, 2000.

		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"

#ifdef MAC
extern int ccommand(char ***);
#endif

/* Prototype declaration */
static void print_welcome(void);
static void print_menu(void);
static void classified_menu(void);
void show_all_help(void);
void show_help(char *string);
void print_command_list(void);
static int call_sub(int c, char s[S_1][S_2]);
int setvar(int c, char (*p)[S_2]);
int system_call(int c, char (*p)[S_2]);
int command_scan(char *line, char s[S_1][S_2]);

/* external functions */
extern int cdsnuc(int c, char (*p)[S_2]);
extern int extcds(int c, char (*p)[S_2]);
extern int extrna(int c, char (*p)[S_2]);
extern int getseq2(int c, char (*p)[S_2]);
extern int toprot(int c, char (*p)[S_2]);
extern int tofast(int c, char (*p)[S_2]);
extern int txtr(int c, char (*p)[S_2]);
extern int getclu(int c, char (*p)[S_2]);
extern int genlist(int c, char (*p)[S_2]);
extern int getent(int c, char (*p)[S_2]);
extern int chname(int c, char (*p)[S_2]);
extern int seqcat(int c, char (*p)[S_2]);
extern int noncod(int c, char (*p)[S_2]);
extern int extint(int c, char (*p)[S_2]);
extern int simtbl(int c, char (*p)[S_2]);
extern int nucaln(int c, char (*p)[S_2]);
extern int sites(int c, char (*p)[S_2]);
extern int splcod(int c, char (*p)[S_2]);

/* help functions */

extern void help_cdsnuc(void);
extern void help_extcds(void);
extern void help_extrna(void);
extern void help_getseq2(void);
extern void help_toprot(void);
void help_toRNA(void);
extern void help_tofast(void);
extern void help_txtr(void);
extern void help_getclu(void);
extern void help_getent(void);
extern void help_chname(void);
extern void help_seqcat(void);
extern void help_noncod(void);
extern void help_extint(void);
extern void help_simtbl(void);
extern void help_nucaln(void);
extern void help_sites(void);
extern void help_splcod(void);
extern void help_codon_table(void);
void help_comple(void);
extern void help_genlist(void);
void help_commands(void);

/* utility.c */

extern void pause_screen(void);
extern void tailchop(char *string);
extern int siseq_rm(int c, char (*p)[S_2]);
extern int siseq_cp(int c, char (*p)[S_2]);
extern int siseq_cat(int c, char (*p)[S_2]);
extern int num_check(char *string);
extern int copy_stream(FILE *fin,FILE *fout);
extern int ftouch(char *filename);
extern int fcat(int c, char (*p)[S_2]);
extern void erasequote(char *string);

/* Global variables */

/*#include "version.h"
*/
extern char version_string[20];
extern Boolean ADDSEQG;
extern Boolean SEQ_IMPORT;	/* defined in excdfn.c */
#ifdef MAC
extern Boolean CMDQ;
#endif
extern int print_line;
extern Boolean circular;
extern long num_seq;
extern Boolean GI_out;

#ifdef JAPANESE
#ifdef UNIX
#include "EUC/siseqE1.h"
#else
#ifdef MAC
#include "Mac/SJIS/siseqS1.h"
#else
#include "Dos/SJIS/siseqS1.h"
#endif
#endif
#endif


/***************************************************************/
static void print_welcome(void)
/***************************************************************/
{
	printf("\n\n");
	printf("**************************************************\n");
#ifdef JAPANESE
	me1(version_string);
#else
	printf("      Welcome to SISEQ   Version %s \n",version_string);
	printf("      Type 'menu' to use classified menu.\n");
#endif
	printf("**************************************************\n");
	printf("\n");
	return;
}
/***************************************************************/

/***************************************************************/
static void print_menu(void)
/***************************************************************/
{
#ifdef JAPANESE
	me2();
#else
	printf("******************** MAIN MENU ******************************\n");
	printf("*  1. txtr    2. getseq2  3. toprot   4. tofast  5. seqcat  *\n");
	printf("* 21. extcds 22. cdsnuc  23. extrna  24. genlist            *\n");
	printf("* 25. getent 26. extint  27. noncod   system  setvar        *\n");
	printf("* 31. getclu 32. chname  33. simtbl   help  exit  menu      *\n");
	printf("*<<<<<< Select a command name or number.>>>>>>>>>>>>>>>>>>>>*\n");
	printf("*  If you are familiar with the SISEQ commands,             *\n"); 
	printf("*  you can also type all file names and options.            *\n");
	printf("*************************************************************\n");
#endif
	printf("\n");
	return;
}
/***************************************************************/

/***************************************************************/
static void classified_menu(void)
/***************************************************************/
{
#ifdef JAPANESE
	me3();
#else
	printf("*************** CLASSIFIED MENU ************************\n");
	printf("* Database extraction                                  *\n");
	printf("*       amino acid sequences       extcds              *\n");
	printf("*       coding sequences           cdsnuc              *\n");
	printf("*       RNAs                       extrna              *\n");
	printf("*       introns                    extint              *\n");
	printf("*       non-coding sequences       noncod              *\n");
	printf("*       entire entries             getent              *\n");
	printf("*       list of gene names         genlist             *\n");
	printf("*       conversion to FASTA        tofast              *\n");
	printf("* Sequence operation <can work also on database files> *\n");
	printf("*       partial sequence           getseq2             *\n");
	printf("*       combined sequence          seqcat              *\n");
	printf("*       conversion to FASTA        tofast              *\n");
	printf("*       translation to protein     toprot              *\n");
	printf("*       change sequence name       chname              *\n");
	printf("*       motif search               sites               *\n");
#ifdef DOS
	pause_screen();
#endif
	printf("* Clustal file operation                               *\n"); 
	printf("*       partial alignment          getclu              *\n");
	printf("*       conversion to multi-FASTA  tofast              *\n");
	printf("*       partial alignment          getclu              *\n");
	printf("*       combined alignment         seqcat              *\n");
	printf("*       change sequence names      chname              *\n");
	printf("*       similarity table           simtbl              *\n");
	printf("*       separate codon positions   splcod              *\n");
	printf("*       make nucleic alignment from amino acid         *\n");
	printf("*       alignment                  nucaln              *\n");
	printf("* Text operation                   txtr                *\n");
	printf("* Others    help  menu  exit  system  setvar           *\n");
	printf("* Type 'help' to get info for the input method.        *\n");
	printf("********************************************************\n");
#endif
	printf("\n");
	return;
}
/***************************************************************/

/**************************************************************/
void show_all_help(void)
/**************************************************************/
{
	char string[MAXLINE]="";
	char word[MAXLINE];

	while(TRUE){
		print_command_list();
#ifdef 	JAPANESE
		me4();
#else
		printf("Select command name for which you need help.\n");
		printf("To return to the main menu, type \"exit\" or \"e\".\n\n"); 
#endif
		fgets(string,10,stdin);
		if(strcmp(string,"")==0) continue;
		tailchop(string);
		if(strcmp(string,"")==0) continue;
		if((strcmp(string,"exit")==0) || (strcmp(string,"e")==0)) break;
		sscanf(string,"%s",word);
		if(!strcmp(word,"")) continue;
		else {
			show_help(word);
			pause_screen(); 
		}
		strcpy(string,"");
		strcpy(word,"");
		continue;
	}
	return;
}
/**************************************************************/

/***************************************************************/
void help_comple(void)
/***************************************************************/
{
        printf("Usage: comple infile outfile.\n\n");
        printf("       This program converts a sequence file in the raw,\n");
        printf("       GenBank and EMBL formats into a complementary sequence\n");
        printf("       in a fasta file .\n");
        printf("       Now accepts multiple file in FASTA, GenBank and EMBL\n");
        printf("       formats. Comments are removed from FASTA file.\n\n");
        return;
}
/***************************************************************/

/***************************************************************/
void help_toRNA(void)
/***************************************************************/
{
        printf("Usage: toRNA infile outfile.\n\n");
        printf("       This program converts a DNA sequence into \n");
        printf("       an RNA sequence in a fasta file.\n\n");
        return;
}
/***************************************************************/


/***************************************************************/
void help_commands(void)
/***************************************************************/
{
#ifdef JAPANESE
	me5();
#else
	printf("***************** OTHER COMMANDS HELP ***********************\n");
	printf("Other commands. \n\n");
	printf("   Test commands. \n");
	printf("      splcod  separate codon positions (aln file).\n");
	printf("      nucaln  make nucleic alignment from amino acid \n");
	printf("              alignment. \n");
	printf("      sites   motif search. \n");
	printf("   File operations. \n");
	printf("      copy(cp) <file1> <file2> \n");
	printf("             copies <file1> to <file2> \n");
	printf("      remove(rm,delete,del) <file1> <file2> ... \n");
	printf("             removes all files <file1> <file2> ... \n");
	printf("      fcat <file1> <file2>\n");
	printf("             Add contents of <file2> at the end of <file1>. \n");
	printf("   Operation of variables. \n");
	printf("      setvar form [circular/default].\n");
	printf("             circular: forces circular DNA.\n");
	printf("             default: use setting of data file.\n");
	printf("      setvar printline(line) [<number>/default].\n");
	printf("             <number>: sets line length to the number .\n");
	printf("             default: sets line length to 75 characters.\n");
	printf("      setvar addseqg [true,TRUE,1/false].\n");
	printf("             true: include introns in cdsnuc and extrna.\n");
	printf("      setvar seq_import [true,TRUE,1/false].\n");
	printf("             true: enables import of external sequences.\n");
	printf("      setvar GI_out [true,TRUE,1/false].\n");
	printf("             true: enables output of GI number.\n");
	printf("   Use of system commands (Unix and DOS). \n");
	printf("      system <system command>.\n");
	printf("*************************************************************\n");
#endif
        return;
}
/***************************************************************/

/**************************************************************/
void print_command_list(void)
/**************************************************************/
{
#ifdef JAPANESE
	me6();
#else
	printf("**********************************************************\n");
	printf("              Available commands are:\n\n");
	printf(" 1. txtr     2. getseq2   3. toprot    4. tofast\n");
	printf(" 5. seqcat  21. extcds   22. cdsnuc   23. extrna\n");
	printf("24. genlist 25. getent   26. extint   27. noncod\n");
	printf("31. getclu  32. chname   33. simtbl   \n");
	printf("91. codon (explanation)  99. Other commands.\n");
	printf("**********************************************************\n");
#endif
	printf("\n");
	return;
}
/**************************************************************/


/**************************************************************/
void show_help(char *string)
/**************************************************************/
{
        if(!strcmp(string,"txtr") || !strcmp(string,"1")) help_txtr();
        else if(!strcmp(string,"getseq2") || !strcmp(string,"2")) help_getseq2();
        else if(!strcmp(string,"toprot") || !strcmp(string,"3")) help_toprot();
        else if(!strcmp(string,"tofast") || !strcmp(string,"4")) help_tofast();
        else if(!strcmp(string,"complement")) help_comple();
        else if(!strcmp(string,"toRNA")) help_toRNA();
        else if(!strcmp(string,"seqcat") || !strcmp(string,"5")) help_seqcat();
        else if(!strcmp(string,"extcds") || !strcmp(string,"21")) help_extcds();
        else if(!strcmp(string,"cdsnuc") || !strcmp(string,"22")) help_cdsnuc();
        else if(!strcmp(string,"extrna") || !strcmp(string,"23")) help_extrna();
        else if(!strcmp(string,"genlist") || !strcmp(string,"24")) help_genlist();
        else if(!strcmp(string,"getent") || !strcmp(string,"25")) help_getent();
        else if(!strcmp(string,"extint") || !strcmp(string,"26")) help_extint();
        else if(!strcmp(string,"noncod") || !strcmp(string,"27")) help_noncod();

        else if(!strcmp(string,"getclu") || !strcmp(string,"31")) help_getclu();
        else if(!strcmp(string,"chname") || !strcmp(string,"32")) help_chname();
        else if(!strcmp(string,"simtbl") || !strcmp(string,"33")) help_simtbl();
        else if(!strcmp(string,"other") || !strcmp(string,"99")) help_commands();
        else if(!strcmp(string,"system") || !strcmp(string,"setvar")) help_commands();
        else if(!strcmp(string,"codon") || !strcmp(string,"91")) help_codon_table();
        else if(!strcmp(string,"nucaln")) help_nucaln();
        else if(!strcmp(string,"sites")) help_sites();
        else if(!strcmp(string,"splcod")) help_splcod();
	else 
#ifdef JAPANESE
		me7(string);
#else
		printf("No help menu for %s available.\n",string);
#endif
	return;
}
/**************************************************************/


/**************************************************************/
int system_call(int c, char (*p)[S_2])
/* system command.*/
/**************************************************************/
{
	char *str4="system";
	char str5[MAXLINE]="";
	int i;
	int j=0;

#ifdef MAC
	fprintf(stderr,"This command is not supported on MacOS.\n");
	return 1;
#endif

	if(c<2 || c>10) return 1;
	if(!strcmp(p[0],str4)){
		strcpy(str5,"");
		for(i=1;i<c;i++){
			strcat(str5,p[i]);
			strcat(str5," ");
		}
		j=system(str5);
		if(j) {
#ifdef JAPANESE
			me8(str5);
#else
			fprintf(stderr,"System command failed: %s.\n",str5);
#endif
		}
		else {
#ifdef 	JAPANESE
			me9(str5);
#else
			fprintf(stderr,"System command successful: %s.\n",str5);
#endif
		}
	}
	return 0;
}

/**************************************************************/


/**************************************************************/
int setvar(int c, char (*p)[S_2])
/* sets global variables. Only works in integrated mode.*/
/**************************************************************/
{
	char *str1="printline";
	char *str11="line";
	char *str2="form";
	char *str3="default";
	char *str4="system";
	char *str6="addseqg";
	char *str7="seq_import";
	char *str8="GI_out";
	char str5[MAXLINE]="";

	int i;
	int j=0;


	if(c<2 || c>10) return 1;

	if((!strcmp(p[1],str1) || !strcmp(p[1],str11)) && c==3){
		if(!num_check(p[2])) print_line=atoi(p[2]);
		else if(!strcmp(p[2],str3)) print_line=PRINTLINE;
	}

	else if(!strcmp(p[1],str2) && c==3){
		if(!strcmp(p[2],"circular")) {
			circular=TRUE;
#ifdef JAPANESE
			me10("circular","TRUE");
#else
			printf("circular has been set TRUE.\n");
#endif
		}
		else if(!strcmp(p[2],str3)) circular=FALSE;
	}

	else if(!strcmp(p[1],str4) && c>=3){
		strcpy(str5,"");
		for(i=2;i<c;i++){
			if(!strcmp(p[i],">>")) strcpy(p[i]," >> ");
			strcat(str5,p[i]);
			strcat(str5," ");
		}
		j=system(str5);
		if(j) {
#ifdef JAPANESE
			me8(str5);
#else
			fprintf(stderr,"System command failed: %s.\n",str5);
#endif
		}
		else {
#ifdef 	JAPANESE
			me9(str5);
#else
			fprintf(stderr,"System command successful: %s.\n",str5);
#endif
		}
	}
	else if(!strcmp(p[1],str6) && c==3){
		if(!strcmp(p[2],"true") || !strcmp(p[2],"TRUE") || !strcmp(p[2],"1")){
			ADDSEQG=TRUE;
#ifdef JAPANESE
			me11();
#else
			printf("ADDSEQG has been set TRUE.\n");
			printf("This will cause cdsnuc and extrna extract sequences including introns.\n");
#endif
		}
		else {
			ADDSEQG=FALSE;
#ifdef JAPANESE
			me10("ADDSEQG","FALSE");
#else
			printf("ADDSEQG has been set FALSE.\n");
#endif
		}
	}
	else if(!strcmp(p[1],str7) && c==3){
		if(!strcmp(p[2],"true") || !strcmp(p[2],"TRUE") || !strcmp(p[2],"1") || !strcmp(p[2],str3)){
			SEQ_IMPORT=TRUE;
#ifdef JAPANESE
			me12();
#else
			printf("SEQ_IMPORT has been set TRUE.\n");
			printf("This will enable cdsnuc and extrna to import sequences \n");
			printf("from external database entries.\n");
#endif
		}
		else {
			SEQ_IMPORT=FALSE;
#ifdef JAPANESE
			me10("SEQ_IMPORT","FALSE");
#else
			printf("SEQ_IMPORT has been set FALSE.\n");
#endif
		}
	}
	else if(!strcmp(p[1],str8) && c<=3){
		if(!strncmp(p[2],"true",4) || !strncmp(p[2],"TRUE",4) || !strncmp(p[2],"1",1) || !strcmp(p[2],str8)){
			GI_out=TRUE;
#ifdef JAPANESE
			me12();
#else
			printf("GI_out has been set TRUE.\n");
			printf("This will enable cdsnuc and extcds to output \n");
			printf("GI number.\n");
#endif
		}
		else {
			GI_out=FALSE;
#ifdef JAPANESE
			me10("GI_out","FALSE");
#else
			printf("GI_out has been set FALSE.\n");
#endif
		}
	}

	return 0;
		
}
/**************************************************************/


/********************** Main part *****************************/
#ifdef MAC
main(void)
#else
int main(int argc, char **argv)
#endif
/***************************************************************/

{
	char line[2*MAXLINE];
	char s[S_1][S_2]={"","","","","","","","","",""};
	char (*p)[S_2]=s;
	FILE *file_cf;
	char cf_name[20]="siseq.cf";
	char *str;	
	int n=0; /* line number in siseq.cf */
	int c=0;
	int i=0;
	int result=0;
#ifdef JAPANESE
	short fontNum;
#endif
		
#ifdef MAC
#ifdef CODEWARRIOR
/*	SIOUXSettings.autocloseonquit = TRUE; */
	SIOUXSettings.asktosaveonclose = FALSE;
	SIOUXSettings.showstatusline = TRUE;
#ifdef JAPANESE
	GetFNum( "\pOsaka)B", &fontNum );
	SIOUXSettings.fontid = fontNum;
	SIOUXSettings.fontsize = 12;
#endif
#endif
#endif
	
/*********************** Input **********************/
	print_welcome();
#ifdef MAC
#ifdef CODEWARRIOR
	SIOUXSetTitle("\pSISEQ");
#endif
#endif

/* setting print_line and other global variables */
	str=getenv("PRINTLINE");
	if(str!=NULL){
		if(!num_check(str)) print_line=atoi(str);
	}
	str=NULL;
	ADDSEQG=FALSE;
	if((str=getenv("ADDSEQG"))!=NULL){
		if(!strcmp(str,"TRUE") || !strcmp(str,"true") || !strcmp(str,"1")) ADDSEQG=TRUE;
	}
	str=NULL;
	SEQ_IMPORT=TRUE;
	if((str=getenv("SEQ_IMPORT"))!=NULL){
		if(!strcmp(str,"FALSE") || !strcmp(str,"false") || !strcmp(str,"0")) SEQ_IMPORT=FALSE;
	}
	str=NULL;
	GI_out=FALSE;
	if((str=getenv("GI_out"))!=NULL){
		if(*str=='T' || *str=='t' || *str=='1'){
			GI_out=TRUE;
			printf("GI_out is set.\n");
		}
	}

#ifndef MAC
/* Command line input is available on UNIX and DOS. */
/* instead of this, just 

	p=argv++;
	c=argc-1;

will be good,if p is defined as char **p.
After all, I decided not to do this, because this does not change the essential point.
Namely, the routine of file names has the restriction of string length. 
*/

	if(argc > 0){
		for(i=1;i<argc;i++){
			strcpy(s[i-1],argv[i]);
			if(i>=9) break;
		}
		c=argc-1;
	}
#endif

/* Use of an input file called siseq.cf that lists commands for siseq, 
one command per line. In this way, siseq can process many files 
according to the commands in siseq.cf. New in version 1.1.1*/

	if(c==0){
		if((file_cf=fopen(cf_name,"r"))!=NULL){
			while(fgets(line,MAXLINE,file_cf)!=NULL){
				n++;
				tailchop(line);
#ifdef JAPANESE
				me14(line);
#else
				printf("\nLine %d: %s.\n",n,line);
#endif
				if(strcmp(line,"")==0) continue;
				if(strlen(line)+2*S_1+2>2*MAXLINE){
#ifdef JAPANESE
					me13();
#else
					fprintf(stderr,"Commands too long.\n");
#endif
#include	"exit1.h"
				}
				for(i=0;i<S_1;i++) strcat(line," #");

				if(command_scan(line,s)){
					fprintf(stderr,"Error in reading line from %s.\n",cf_name);
#include	"exit1.h"
				}

				for(i=0;i<S_1;i++){
					if(strcmp(s[i],"#")==0) break;
				}
				c=i;
				for(i=c;i<S_1;i++){
					strcpy(s[i],"");
				}

				result=call_sub(c,s);
				if(result) {
					fprintf(stderr,"Error in line %d with command: %s %s. Skipping...\n",n,s[0],s[1]);
				}
			}
#include	"exit0.h"
		}


/*		else {
			pause_screen();
		}
Usually this is not necessary.*/


	}
	else result=call_sub(c,s);

	while(c==0){
		if(strcmp(s[0],"menu")) print_menu();
		
		while(TRUE){
			fgets(line,MAXLINE,stdin);
			tailchop(line);
#ifdef JAPANESE
			me14(line);
#else
			printf("\nYour input: %s.\n",line);
#endif
			if(strcmp(line,"")!=0) break;
		}
		
		if(strlen(line)+2*S_1+2>2*MAXLINE){
#ifdef JAPANESE
			me13();
#else
			fprintf(stderr,"Commands too long.\n");
#endif
#include	"exit1.h"
		}
		for(i=0;i<S_1;i++) strcat(line," #");
/*		for(i=0;i<S_1;i++) sscanf(line,"%s",s[i]);
*/
		if(command_scan(line,s)){
			fprintf(stderr,"Error in reading line.\n");
#include	"exit1.h"
		}

		for(i=0;i<S_1;i++){
			if(strcmp(s[i],"#")==0) break;
		}
		c=i;
		for(i=c;i<S_1;i++){
			strcpy(s[i],"");
		}
		if(!strcmp(s[0],"h") || !strcmp(s[0],"help") || !strcmp(s[0],"-help") || !strcmp(s[0],"-h")){
			if(strcmp(s[1],"")==0) show_all_help();
			else show_help(s[1]);
			c=0;
		} 
		else if(!strcmp(s[0],"menu")){
			if(strcmp(s[1],"")==0) classified_menu();
			c=0;
		} 
		else if(!strcmp(s[0],"e") || !strcmp(s[0],"exit")) break;
		else if(!strcmp(s[0],"setvar")){
			result=setvar(c,p);
			pause_screen();
			c=0;
		}
		else if(!strcmp(s[0],"system")){
			result=system_call(c,p);
			pause_screen();
			c=0;
		}
		else result=call_sub(c,s);
	}
	
	if(result==0){
/*		if(strcmp(s[0],"rm") && strcmp(s[0],"remove") && strcmp(s[0],"del") && strcmp(s[0],"delete")\
			&& strcmp(s[0],"cp") && strcmp(s[0],"copy")\
			&& strcmp(s[0],"fcat") && strcmp(s[0],"cat") && strcmp(s[0],"h") && \
			strcmp(s[0],"help") && strcmp(s[0],"-h") strcmp(s[0],"-help")){
	
			printf("Line length was %d characters.\n",print_line);
			printf("To change line length, set environmental variable PRINTLINE.\n");
		}
*/
#ifdef JAPANESE
		me15();
#else
		printf("All done.\n");
#endif
	}
	else {
#ifdef JAPANESE
		me16();
#else
		printf("Exiting..\n");
#endif
	}
	
	if(result){
#include	"exit1.h"
	} else {
#include	"exit0.h"
	}
}
/*******************************************/

/*******************************************/
static int call_sub(int c, char s[S_1][S_2])
/*******************************************/
{
	char (*p)[S_2]=s;
	int result=0;
	int i=0;

/*	printf("command = %s ",s[0]);
*/
	for(i=1;i<c;i++){ printf("%s ",s[i]); }
	printf("\n");

	/* Call subprogram */

	if((strcmp(s[0],"txtr")==0) || (strcmp(s[0],"1")==0)) result=txtr(c,p);
	else if((strcmp(s[0],"getseq2")==0) || (strcmp(s[0],"2")==0)) result=getseq2(c,p);
	else if((strcmp(s[0],"toprot")==0) || (strcmp(s[0],"3")==0)) result=toprot(c,p);
	else if((strcmp(s[0],"tofast")==0) || (strcmp(s[0],"4")==0)) result=tofast(c,p);
	else if(!strcmp(s[0],"complement") || !strcmp(s[0],"comple")) {
		if(strcmp(s[1],"") && strcmp(s[2],"")){
			c=4;
			strcpy(s[3],"c");
			result=tofast(c,p);
		}
		else result=tofast(c,p);
	}
	else if(!strcmp(s[0],"toRNA") || !strcmp(s[0],"torna")) {
		if(strcmp(s[1],"") && strcmp(s[2],"")){
			c=4;
			strcpy(s[3],"r");
			result=tofast(c,p);
		}
		else result=tofast(c,p);
	}
	else if((strcmp(s[0],"seqcat")==0) || (strcmp(s[0],"5")==0)) result=seqcat(c,p);

	else if((strcmp(s[0],"extcds")==0) || (strcmp(s[0],"21")==0)) result=extcds(c,p);
	else if((strcmp(s[0],"cdsnuc")==0) || (strcmp(s[0],"22")==0)) result=cdsnuc(c,p);
	else if((strcmp(s[0],"extrna")==0) || (strcmp(s[0],"23")==0)) result=extrna(c,p);
	else if((strcmp(s[0],"genlist")==0) || (strcmp(s[0],"24")==0)) result=genlist(c,p);
	else if((strcmp(s[0],"getent")==0) || (strcmp(s[0],"25")==0)) result=getent(c,p);
	else if((strcmp(s[0],"extint")==0) || (strcmp(s[0],"26")==0)) result=extint(c,p);
	else if((strcmp(s[0],"noncod")==0) || (strcmp(s[0],"27")==0)) result=noncod(c,p);

	else if((strcmp(s[0],"getclu")==0) || (strcmp(s[0],"31")==0)) result=getclu(c,p);
	else if((strcmp(s[0],"chname")==0) || (strcmp(s[0],"32")==0)) result=chname(c,p);
	else if((strcmp(s[0],"simtbl")==0) || (strcmp(s[0],"33")==0)) result=simtbl(c,p);
	else if(!strcmp(s[0],"clu2fa")) result=tofast(c,p);
	else if(!strcmp(s[0],"nucaln")) result=nucaln(c,p);
	else if(!strcmp(s[0],"sites")) result=sites(c,p);
	else if(!strcmp(s[0],"splcod")) result=splcod(c,p);
	else if(!strcmp(s[0],"h") || !strcmp(s[0],"help") || !strcmp(s[0],"-h") || !strcmp(s[0],"-help")){
		if(strcmp(s[1],"")==0) show_all_help();
		else show_help(s[1]);
	}

/* Additional commands for shell mode. */

	else if((strcmp(s[0],"rm")==0) || (strcmp(s[0],"remove")==0) || (strcmp(s[0],"del")==0) || (strcmp(s[0],"delete")==0)){
		result=siseq_rm(c,p);
	}
	else if(!strcmp(s[0],"cp") || !strcmp(s[0],"copy")) result=siseq_cp(c,p);
	else if(!strcmp(s[0],"setvar")) result=setvar(c,p);
	else if(!strcmp(s[0],"system")) result=system_call(c,p);
	else if(!strcmp(s[0],"fcat") || !strcmp(s[0],"cat")) result=fcat(c,p);
	else result=1;

	return result;
}
/****************************************/

/********************************************/
int command_scan(char *line, char s[S_1][S_2])
/********************************************/
{
	int i=0;
	int j=0;
	char *p;
	int flag=0;	/* true if within a quotation. */
	int space_flag=0;	/* true after a space character. */

	for(p=line;*p!='\0' && i<S_1 && j<S_2;p++){

		if(!flag && *p==' '){
			if(!space_flag){
				s[i][j]='\0';
				if(!strcmp(s[i],"#")) break;
#ifdef VERBOUS
				printf("%s\n",s[i]);
#endif
				i++;
				j=0;
				space_flag=1;
			}
		} 
		else {
			space_flag=0;
			if(*p==34){
				s[i][j++]=*p;
				flag=1-flag;
			}
			else {
				s[i][j++]=*p;
			}
		}
	}

	if(i>=S_1 || j>=S_2) return 1;

	for(i=0;i<S_1;i++){
		if(s[i][0]==34 && s[i][strlen(s[i])-1]==34){
			erasequote(s[i]);
		}
	}

	return 0;	
}
/********************************************/


/************************************************************
		End of file siseq.c
************************************************************/

