/***********************************************************************
Show sites (sites) 
main file : sites.c

 	Show location of a sequence motif.
 	At first stage, input file formats are limited to
 	raw, FASTA, GenBank, and EMBL.

  Usage: sites infile outfile motif 
  
  Default outfile name is "infile"+".out"
  
  Source files are:	sites.c
			gtsqfn.c
  			utility.c
  			Dmc.h
  			getseq2.h
  			cmptbl.h
  	
 	Naoki Sato   Aug. 6, 1999.
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.
	For Mac version, define the compiler as well.
	
	Integrated in the siseq main program. Feb. 22, 2000.
	siseq_exit0 or 1 changed to exit0 or 1. Nov. 13, 1999.
	Bug fix. May 4, 2000.
	Multiple sequence file. Oct. 15, 2000.
	Estimation of frequency. Oct. 17, 2000.
	Accept circular. Oct. 18, 2000.
	Removed redundancy. Oct. 19, 2000.
	Added automatic search for palindromes. Jan. 5, 2001.
	Added non-palindromic comparison. Dec. 12, 2001.
	Bug fix in search_motif. Sept. 5, 2003.
	
		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"

#ifndef INTEGRATED
#ifdef MAC
extern int ccommand(char ***);
#endif
#endif

/* Prototype declaration */
static void print_usage(void);
double estimate_freq(double comp[30],char *motif,enum data_type dtype1);
long search_match(char *seq,char *motif,char *compmot,long position,double precision,enum data_type dtype);
long write_sites(FILE *fout,Seqfile *seq_in,enum data_type dtype1,long motlen,char *motif,char *compmot,double match_level,int mode);
void write_comp(FILE *fout,long code_count[30],double comp[30],long len,enum data_type dtype1,int mode);
void generate_motif(char *string,int i);
void generate_motifNP(char *string,int i);
long search_match3(Seqfile *seq_in,char *motif,long position);
long write_sites3(Seqfile *seq_in,long motlen,char *motif);

#ifdef INTEGRATED
int sites(int c, char (*p)[S_2]);
#endif
void help_sites(void);

/* External functions. */

#include "libSeq.h"
#include <math.h>

/* Global variables */
extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];
extern long num_seq;
extern Boolean circular;

#define DEFAULT_MATCHING_LEVEL 0.99

/***************************************************************/
static void print_usage(void)
/***************************************************************/
{
	help_sites();
	return;
}
/***************************************************************/

/***************************************************************/
void help_sites(void)
/***************************************************************/
{
#ifdef JAPANESE

#ifdef UNIX
#include "EUC/sitesE.h"
#else
#ifdef MAC
#include "Mac/SJIS/sitesS.h"
#else
#include "Dos/SJIS/sitesS.h"
#endif
#endif

#else
	printf("****************************** SITES HELP *************************************\n");
	printf("Usage: sites infile outfile mode motif <matching level>.\n\n");
	printf("       This program shows location of a sequence\n");
	printf("       motif within a database entry\n");
	printf("Input file formats should be: raw, Fasta, GenBank, EMBL, SwissProt, or PRF. \n");
	printf("mode 0, short summary only; mode 1, detailed output (default); mode 2, table output.\n");
	printf("mode 4, as mode 1, but only selected data are printed.\n");
	printf("mode 3, automatic search for palindrome sequences. Min and max lengths should \n");
	printf("       be given as motif. These two numbers should be separated by a dot. \n");
	printf("<matching level> is an option.\n");
	printf("*******************************************************************************\n");
#endif
	return;
}
/***************************************************************/

/************************************************************************/
double estimate_freq(double comp[30],char *motif,enum data_type dtype1)
/* returns estimated frequency of the motif. returns negative value 
as a sign of error.*/
/************************************************************************/
{
	double freq1=1.0;
	double freq2=1.0;
	double freq3=1.0;
	int motlen=1;
	int i=0;
	char *compmot;
	Boolean palindrome=FALSE;

	if(motif==NULL) return -1.0;
	motlen=strlen(motif);
	if(motlen<3) return -1.0;

	/* make complementary motif */

	if((compmot=(char*)calloc(motlen+2,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		exit(1);
	}
	if(dtype1==DNA){
		for(i=0L;i<motlen;i++)  compmot[motlen-i-1]=to_compD(motif[i]);
	}
	if(!strcmp(motif,compmot)) palindrome=TRUE;

	if(dtype1==RNA){
		for(i=0;i<motlen;i++){
			if(motif[i]=='A') freq2=comp[A];
			else if(motif[i]=='G') freq2=comp[G];
			else if(motif[i]=='C') freq2=comp[C];
			else if(motif[i]=='U') freq2=comp[U];
			else if(motif[i]=='N') freq2=1.0;
			else if(motif[i]=='R') freq2=comp[A]+comp[G];
			else if(motif[i]=='Y') freq2=comp[U]+comp[C];
			else return -1.0;	/* error */
			freq1*=freq2;
		}
	}
	else if(dtype1==DNA){
		for(i=0;i<motlen;i++){
			if(motif[i]=='A') freq2=comp[A];
			else if(motif[i]=='G') freq2=comp[G];
			else if(motif[i]=='C') freq2=comp[C];
			else if(motif[i]=='T') freq2=comp[T];
			else if(motif[i]=='N') freq2=1.0;
			else if(motif[i]=='R') freq2=comp[A]+comp[G];
			else if(motif[i]=='Y') freq2=comp[T]+comp[C];
			else if(motif[i]=='W') freq2=comp[T]+comp[A];
			else if(motif[i]=='S') freq2=comp[C]+comp[G];
			else if(motif[i]=='M') freq2=comp[C]+comp[A];
			else if(motif[i]=='K') freq2=comp[T]+comp[G];
			else return -1.0;	/* error */
			freq1*=freq2;
		}
		for(i=0;i<motlen;i++){
			if(motif[i]=='A') freq2=comp[T];
			else if(motif[i]=='G') freq2=comp[C];
			else if(motif[i]=='C') freq2=comp[G];
			else if(motif[i]=='T') freq2=comp[A];
			else if(motif[i]=='N') freq2=1.0;
			else if(motif[i]=='R') freq2=comp[T]+comp[C];
			else if(motif[i]=='Y') freq2=comp[A]+comp[G];
			else if(motif[i]=='W') freq2=comp[T]+comp[A];
			else if(motif[i]=='S') freq2=comp[C]+comp[G];
			else if(motif[i]=='M') freq2=comp[C]+comp[A];
			else if(motif[i]=='K') freq2=comp[T]+comp[G];
			else return -1.0;	/* error */
			freq3*=freq2;
		}
		if(!palindrome) freq1+=freq3;
	}
	else if(dtype1==protein){
		for(i=0;i<motlen;i++){
			if(motif[i]=='A') freq2=comp[A];
			else if(motif[i]=='C') freq2=comp[C];
			else if(motif[i]=='D') freq2=comp[D];
			else if(motif[i]=='E') freq2=comp[E];
			else if(motif[i]=='F') freq2=comp[F];
			else if(motif[i]=='G') freq2=comp[G];
			else if(motif[i]=='H') freq2=comp[H];
			else if(motif[i]=='I') freq2=comp[I];
			else if(motif[i]=='K') freq2=comp[K];
			else if(motif[i]=='L') freq2=comp[L];
			else if(motif[i]=='M') freq2=comp[M];
			else if(motif[i]=='N') freq2=comp[N];
			else if(motif[i]=='P') freq2=comp[P];
			else if(motif[i]=='Q') freq2=comp[Q];
			else if(motif[i]=='R') freq2=comp[R];
			else if(motif[i]=='S') freq2=comp[S];
			else if(motif[i]=='T') freq2=comp[T];
			else if(motif[i]=='V') freq2=comp[V];
			else if(motif[i]=='W') freq2=comp[W];
			else if(motif[i]=='Y') freq2=comp[Y];
			else return -1.0;	/* error */
			freq1*=freq2;
		}
	}
	else return -1.0;

	return freq1;

}
/************************************************************************/


/******************************************************************************************************/
void write_comp(FILE *fout,long code_count[30],double comp[30],long len,enum data_type dtype1,int mode)
/******************************************************************************************************/
{
	double AT=0L;
	double GC=0L;

	if(dtype1==DNA || dtype1==RNA){
		comp[A]=(double)code_count[A]/(double)len;
		comp[C]=(double)code_count[C]/(double)len;
		comp[G]=(double)code_count[G]/(double)len;
		if(mode!=2 && mode!=3){
			fprintf(fout,"   A = %6ld, %6.2f %%.",code_count[A],100*comp[A]);
			fprintf(fout,"   C = %6ld, %6.2f %%.\n",code_count[C],100*comp[C]);
			fprintf(fout,"   G = %6ld, %6.2f %%.",code_count[G],100*comp[G]);
		}
		if(dtype1==DNA){
			comp[T]=(double)code_count[T]/(double)len;
			if(mode!=2 && mode!=3) fprintf(fout,"   T = %6ld, %6.2f %%.\n",code_count[T],100*comp[T]);
		}
		else{
			comp[U]=(double)code_count[U]/(double)len;
			if(mode!=2) fprintf(fout,"   U = %6ld, %6.2f %%.\n",code_count[U],100*comp[U]);
		}
		if(dtype1==DNA){
			AT=comp[A]+comp[T];
			GC=comp[G]+comp[C];
			if(mode!=2 && mode!=3){
				fprintf(fout,"\n");
				fprintf(fout,"    AT = %6.2f %%.  ",100*AT);
			} 
			if(mode==2 || mode==3) fprintf(fout,"%6.2f %%",100*GC);
			else fprintf(fout,"  GC = %6.2f %%.\n",100*GC);
		}

	}
	else {
		comp[A]=(double)code_count[A]/(double)len;
		fprintf(fout,"   A = %6ld, %6.2f %%.",code_count[A],100*comp[A]);
		comp[C]=(double)code_count[C]/(double)len;
		fprintf(fout,"   C = %6ld, %6.2f %%.\n",code_count[C],100*comp[C]);
		comp[D]=(double)code_count[D]/(double)len;
		fprintf(fout,"   D = %6ld, %6.2f %%.",code_count[D],100*comp[D]);
		comp[E]=(double)code_count[E]/(double)len;
		fprintf(fout,"   E = %6ld, %6.2f %%.\n",code_count[E],100*comp[E]);
		comp[F]=(double)code_count[F]/(double)len;
		fprintf(fout,"   F = %6ld, %6.2f %%.",code_count[F],100*comp[F]);
		comp[G]=(double)code_count[G]/(double)len;
		fprintf(fout,"   G = %6ld, %6.2f %%.\n",code_count[G],100*comp[G]);
		comp[H]=(double)code_count[H]/(double)len;
		fprintf(fout,"   H = %6ld, %6.2f %%.",code_count[H],100*comp[H]);
		comp[I]=(double)code_count[I]/(double)len;
		fprintf(fout,"   I = %6ld, %6.2f %%.\n",code_count[I],100*comp[I]);
		comp[K]=(double)code_count[K]/(double)len;
		fprintf(fout,"   K = %6ld, %6.2f %%.",code_count[K],100*comp[K]);
		comp[L]=(double)code_count[L]/(double)len;
		fprintf(fout,"   L = %6ld, %6.2f %%.\n",code_count[L],100*comp[L]);
		comp[M]=(double)code_count[M]/(double)len;
		fprintf(fout,"   M = %6ld, %6.2f %%.",code_count[M],100*comp[M]);
		comp[N]=(double)code_count[N]/(double)len;
		fprintf(fout,"   N = %6ld, %6.2f %%.\n",code_count[N],100*comp[N]);
		comp[P]=(double)code_count[P]/(double)len;
		fprintf(fout,"   P = %6ld, %6.2f %%.",code_count[P],100*comp[P]);
		comp[Q]=(double)code_count[Q]/(double)len;
		fprintf(fout,"   Q = %6ld, %6.2f %%.\n",code_count[Q],100*comp[Q]);
		comp[R]=(double)code_count[R]/(double)len;
		fprintf(fout,"   R = %6ld, %6.2f %%.",code_count[R],100*comp[R]);
		comp[S]=(double)code_count[S]/(double)len;
		fprintf(fout,"   S = %6ld, %6.2f %%.\n",code_count[S],100*comp[S]);
		comp[T]=(double)code_count[T]/(double)len;
		fprintf(fout,"   T = %6ld, %6.2f %%.",code_count[T],100*comp[T]);
		comp[V]=(double)code_count[V]/(double)len;
		fprintf(fout,"   V = %6ld, %6.2f %%.\n",code_count[V],100*comp[V]);
		comp[W]=(double)code_count[W]/(double)len;
		fprintf(fout,"   W = %6ld, %6.2f %%.",code_count[W],100*comp[W]);
		comp[Y]=(double)code_count[Y]/(double)len;
		fprintf(fout,"   Y = %6ld, %6.2f %%.\n",code_count[Y],100*comp[Y]);
		fprintf(fout,"\n");	
	}	
	return;
}
/*****************************************************************************/

/***********************************************************************************************************/
long search_match3(Seqfile *seq_in,char *motif,long position)
/* Mode 3 only. Does not really use compmot and precision. Allows R,Y,M,K,N for DNA.*/
/***********************************************************************************************************/
{
	long i,j;
	long length=seq_in->seqlen;
	long motlen=(long)strlen(motif);
	char *seq=seq_in->seq;

        if(length==0L || motlen==0L){
                fprintf(stderr,"Sequence lengh zero.\n" );
                return 0;
        }

	/* search motif -- rapid version*/
	
	for(i=position;i<length-motlen+1;i++){
		for(j=0;j<motlen;j++){
			if(motif[j]=='N') continue;
			else if(seq[i+j]==motif[j]) continue;
			else if(motif[j]=='R'){
				if(seq[i+j]=='A' || seq[i+j]=='G') continue;
			} 
			else if(motif[j]=='Y'){
				if(seq[i+j]=='T' || seq[i+j]=='C') continue;
			} 
			else if(motif[j]=='W'){
				if(seq[i+j]=='A' || seq[i+j]=='T') continue;
			} 
			else if(motif[j]=='S'){
				if(seq[i+j]=='G' || seq[i+j]=='C') continue;
			} 
			else if(motif[j]=='M'){
				if(seq[i+j]=='A' || seq[i+j]=='C') continue;
			} 
			else if(motif[j]=='K'){
				if(seq[i+j]=='G' || seq[i+j]=='T') continue;
			} 
			else break;
		}
		if(j==motlen) return i+1;
	}
	return 0;
}
/***************************************************************/


/***********************************************************************************************************/
long search_match(char *seq,char *motif,char *compmot,long position,double precision,enum data_type dtype)
/* allows R,Y,M,K,N for DNA, and R,Y,N for RNA.*/
/***********************************************************************************************************/
{
	long i,j;
	double score;
	long iscore;
	long length=(long)strlen(seq);
	long motlen=(long)strlen(motif);

	if(length==0L || motlen==0L){
		fprintf(stderr,"Sequence lengh zero.\n" );
		return 0;
	}
	if(precision>1.0 || precision<0.5){
		fprintf(stderr,"Precision should be between 0.5 and 1.0.\n" );
		return 0;
	}

/************************************************/

	/* search motif */
	
	for(i=position;i<length-motlen+1;i++){
		score=0.0;
		iscore=0L;
		for(j=0;j<motlen;j++){
			if(seq[i+j]==motif[j]) iscore+=1L;
			else if(dtype==protein) {
				if(motif[j]=='X') iscore+=1L;
			} else if(dtype==DNA) {
				if(motif[j]=='N') iscore+=1L;
				else if(motif[j]=='R'){
					if(seq[i+j]=='A' || seq[i+j]=='G') iscore+=1L;
				} 
				else if(motif[j]=='Y'){
					if(seq[i+j]=='T' || seq[i+j]=='C') iscore+=1L;
				} 
				else if(motif[j]=='W'){
					if(seq[i+j]=='A' || seq[i+j]=='T') iscore+=1L;
				} 
				else if(motif[j]=='S'){
					if(seq[i+j]=='G' || seq[i+j]=='C') iscore+=1L;
				} 
				else if(motif[j]=='M'){
					if(seq[i+j]=='A' || seq[i+j]=='C') iscore+=1L;
				} 
				else if(motif[j]=='K'){
					if(seq[i+j]=='G' || seq[i+j]=='T') iscore+=1L;
				} 
				else if(precision>0.999) break;
			} else if(dtype==RNA) {
				if(motif[j]=='N') iscore+=1L;
				else if(motif[j]=='R'){
					if(seq[i+j]=='A' || seq[i+j]=='G') iscore+=1L;
				} 
				else if(motif[j]=='Y'){
					if(seq[i+j]=='U' || seq[i+j]=='C') iscore+=1L;
				} 
				else if(precision>0.999) break;
			}
		}
		score=(double)iscore;
		if(score>=motlen*precision) return i+1;
		if(dtype!=DNA) continue;
		score=0.0;
		iscore=0L;
		for(j=0;j<motlen;j++){
			if(motif[j]=='N') iscore+=1L;
			else if(seq[i+j]==compmot[j]) iscore+=1L;
			else if(compmot[j]=='R'){
				if(seq[i+j]=='A' || seq[i+j]=='G') iscore+=1L;
			} 
			else if(compmot[j]=='Y'){
				if(seq[i+j]=='T' || seq[i+j]=='C') iscore+=1L;
			} 
			else if(motif[j]=='W'){
				if(seq[i+j]=='A' || seq[i+j]=='T') iscore+=1L;
			} 
			else if(motif[j]=='S'){
				if(seq[i+j]=='G' || seq[i+j]=='C') iscore+=1L;
			} 
			else if(motif[j]=='M'){
				if(seq[i+j]=='A' || seq[i+j]=='C') iscore+=1L;
			} 
			else if(motif[j]=='K'){
				if(seq[i+j]=='G' || seq[i+j]=='T') iscore+=1L;
			} 
			else if(precision>0.999) break;
		}
		score=(double)iscore;
		if(score>=motlen*precision) return (-1)*(i+1);
	}
	return 0;
}
/***************************************************************/

/**************************************************************************************************************************/
long write_sites3(Seqfile *seq_in,long motlen,char *motif)
/*** Mode 3 only */
/**************************************************************************************************************************/
{
	long count=0L;	/* matching count */
	int c=0;	/* data type */
	long position0=0L;
	long position1=0L;
	long motlen1=0L;


	if(seq_in==NULL) {
		fprintf(stderr,"Sequence is empty.\n");
		return -1;
	}
	motlen1=strlen(motif);
	if(motlen1!=motlen) motlen=motlen1;
	if(motlen<3) {
		fprintf(stderr,"Motif too short.\n");
		return -1;
	}
	if(motlen>MAXLINE-2) {
		fprintf(stderr,"Motif too long.\n");
		return -1;
	}

	c=check_data_type(seq_in);
	if(c!=DNA) {
		fprintf(stderr,"Mode 3 is only compatible with DNA. Stop processing...\n");
		return -1;
	}


	count=1L;
	while(TRUE){
		position1=search_match3(seq_in,motif,position0);
		if(position1==0) break;
		count+=1L;
		position0=position1+1;
		position1=0;
	}

	return count;
}
/*******************************************************************************************************/


/**************************************************************************************************************************/
long write_sites(FILE *fout,Seqfile *seq_in,enum data_type dtype1,long motlen,char *motif,char *compmot,double match_level,int mode)
/*** mode 0, no output; mode 1, verbous output. mode 4, like mode 1, but outputs only found sites. */
/**************************************************************************************************************************/
{
	long count=0L;	/* matching count */
	int c=0;	/* data type */
	Boolean circular_bak=FALSE;
	static char str5[MINMEMSIZE];
	long position0=0L;
	long position1=0L;
	double level=1.0; 
	long motlen1=0L;
	Boolean flag_header_printed=0;	/* mode 4 only */

	if(seq_in==NULL) {
		fprintf(stderr,"Sequence is empty.\n");
		return -1;
	}

	motlen1=strlen(motif);
	if(motlen1!=motlen) motlen=motlen1;
	if(motlen<3) {
		fprintf(stderr,"Motif too short.\n");
		return -1;
	}
	if(motlen>MAXLINE-2) {
		fprintf(stderr,"Motif too long.\n");
		return -1;
	}

	c=check_data_type(seq_in);
	if(dtype1!=c) {
/*		fprintf(stderr,"Data types seems mixed. Stop processing...\n");
		return -1;
*/
		fprintf(stderr,"Data types seems mixed. Skip this one...\n");
		return 0;
	}

	circular_bak=circular;
	circular=seq_in->circular;

	if(mode==1){
		fprintf(fout,"  Sequence name: %s.   ",seq_in->ID);
		fprintf(fout,"    Length  : %ld. ",seq_in->seqlen);
		if(circular) fprintf(fout,"circular");
		fprintf(fout,"    Data type: %s.\n",data_type_name[dtype1]);
		fprintf(fout,"  Key motif    : %s.",motif);
		if(dtype1==DNA) fprintf(fout,"  Complementary: %s.",compmot);
		fprintf(fout,"\n");
	}

	/* for circular */
	
	if(circular){
		printf("\tcircular genome: %s.\n",seq_in->ID);
		if((seq_in->seq=(char*)realloc(seq_in->seq,(seq_in->seqlen+motlen+2)*sizeof(char)))==NULL){
			fprintf(stderr,"Out of memory.\n");
			exit(1);
		}
	/* bug fix Jan 2006 */
		strncpy(str5,seq_in->seq,motlen-1);
		strcat(seq_in->seq,str5);
/*
		pp=extract_string(seq_in->seq,1,motlen-1); 
		strcat(seq_in->seq,pp); 
		free(pp);
*/
		seq_in->seqlen=(long)strlen(seq_in->seq);
	}

	flag_header_printed = 0;
	for(level=1.0;level>=match_level-0.0000001;level-=0.1){
		if(mode==1) fprintf(fout,"\nMatching level= %4.2f\n",level);
		count=1L;
		while(TRUE){
			position1=search_match(seq_in->seq,motif,compmot,position0,level,c);
			if(position1==0) break;
			else{
				if(mode!=3){
					if(mode==4){
						if(flag_header_printed==0){
							fprintf(fout,"Sequence name: %s\t",seq_in->ID);
							fprintf(fout,"Length  : %ld\t",seq_in->seqlen);
							if(circular) fprintf(fout,"circular\t");
							fprintf(fout,"Data type: %s",data_type_name[dtype1]);
							if(dtype1==DNA) fprintf(fout,"\tComplementary: %s",compmot);
							fprintf(fout,"\n");
						
							fprintf(fout,"    Matching level= %4.2f\n",level);

							flag_header_printed = 1;
						}
					}
	
					if(position1>0){
						strncpy(str5,seq_in->seq+position1-1,motlen);
						str5[motlen]='\0';
						if(mode==1 || mode==4) fprintf(fout,"%7ld position=%8ld,  %s\n",count,position1,str5);
					}
					else{
						position1=0-position1;
					/* bug fix Jan 2006 */
						strncpy(str5,seq_in->seq+position1-1,motlen);
						str5[motlen]='\0';
						if(mode==1 || mode==4) fprintf(fout,"%7ld position=%8ld,  %s complementary\n",count,position1,str5);
					}
					if(mode==4) fprintf(fout,"\n");
				}
				count+=1L;
			}
			position0=position1+1;
			position1=0;
		}
		position1=0L;
		position0=0L;
	}

	/* seq_in-> seq reverted to original. */
	if(circular) {
		seq_in->seq[seq_in->seqlen+1-motlen]='\0';
		seq_in->seqlen=strlen(seq_in->seq);
	}
		
	circular=circular_bak;
	
	return count;

}
/*******************************************************************************************************/

/**************************************************************/
void generate_motif(char *string,int i)
/* returns next palindromic string of the same size. */
/**************************************************************/
{
	char a[5]="TCAG";
	char b[5]="AGTC";
	int j=0;
	char last_string[MAXLINE];

	if(i<1){
		strcpy(string,"");
		 return;
	}
	strcpy(last_string,space_line((long)i,a[3]));
	if(i%2==1) last_string[(i-1)/2]='N';
	for(j=(i+1)/2;j<i;j++) last_string[j]=b[3];

	if(!strcmp(string,last_string)){
		strcpy(string,"");
		return;
	}

	if(!strcmp(string,"") || strlen(string)<i){
		strcpy(string,space_line((long)i,a[0]));
		if(i%2==1) string[(i-1)/2]='N';
		for(j=(i+1)/2;j<i;j++) string[j]=b[0];
		return;
	} else {
		for(j=0;j<i/2;j++){
			if(string[j]==a[0]){
				string[j]=a[1];
				string[i-j-1]=b[1];
				break;
			}
			else if(string[j]==a[1]){
				string[j]=a[2];
				string[i-j-1]=b[2];
				break;
			}
			else if(string[j]==a[2]){
				string[j]=a[3];
				string[i-j-1]=b[3];
				break;
			}
			else if(string[j]==a[3]){
				string[j]=a[0];
				string[i-j-1]=b[0];
				continue;
			}
		}	
	}
	return;
}
/**************************************************************/


/**************************************************************/
void generate_motifNP(char *string,int i)
/* returns next non-palindromic string of the same size. */
/**************************************************************/
{
	char a[5]="TCAG";
	int j=0;
	char last_string[MAXLINE];

	if(i<1){
		strcpy(string,"");
		 return;
	}
	strcpy(last_string,space_line((long)i,a[3]));

	if(!strcmp(string,last_string)){
		strcpy(string,"");
		return;
	}

	if(!strcmp(string,"") || strlen(string)<i){
		strcpy(string,space_line((long)i,a[0]));
		return;
	} else {
		for(j=0;j<i;j++){
			if(string[j]==a[0]){
				string[j]=a[1];
				break;
			}
			else if(string[j]==a[1]){
				string[j]=a[2];
				break;
			}
			else if(string[j]==a[2]){
				string[j]=a[3];
				break;
			}
			else if(string[j]==a[3]){
				string[j]=a[0];
				continue;
			}
		}	
	}
	return;
}
/**************************************************************/


/***************************************************************
****************** Main program *******************************/
#ifdef INTEGRATED
int sites(int cc, char (*p)[S_2])

#else
#ifdef MAC
main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif

#endif
/***************************************************************/

{
	FILE *fin;
	FILE *fout;
	char fname1[100]="infile";
	char fname2[100]="outfile";
	int c=0;
	Boolean circular_bak=FALSE;
	char *str1="Input file name";
	char *str2="Output file name";
	char str3[MINMEMSIZE]="";	/* motif */
	char str4[MINMEMSIZE]="";	/* compmot */
	char *str;
	char *pp;	/* temporary use */
	long motlen=0L;
	long i=0L;
	long len1=0L;
	enum data_type dtype1=unknown;
   /* sequence variables */
/*	Seqfile *seq_in; */
	Seqfile **mseq_in;
	long seqfile_size=0L;
	double level=DEFAULT_MATCHING_LEVEL;	/* matching level */
	int mode=1;	/* mode 0, no detailed output; mode 1, verbous output; mode 2, table output. */
	long total_count=0L;	/* total matching count */
	long count=0L;	/* matching count */
	long total_seqlen=0L;	/* total sequence length */
	double frequency=0.0;
	double e_freq=0.0;
	Boolean made_comp=FALSE;
	int ul=3;	/* upper limit of search in the auto mode */
	int ll=3;	/* lower limit of search in the auto mode */
	int c1=1; 	/* counter for ul */
	int c2=1; 	/* counter for motif sequences */
	int c3=1; 	/* number of possible motifs */

   /* for base composition */
	int j=0;
	long code_count[30];
	long total_code[30];
	double comp[30];

   /* statistics */
	double x;	/* deviation */
	double avr;	/* average */
	double vari;	/* variance */
	double signif;	/* significance level */

	long fileloc=0L;

#include	"parsarg.h"

/*********************** File names ******************/

	switch(cc){

		case 1:	
			strcpy(fname1,"infile");
			if(filename_input(fname1,str1)!=1){
#include	"exit1.h"	
			}
			strcpy(fname2,fname1);
			strcat(fname2,".out");
			if(filename_input(fname2,str2)!=1){
#include	"exit1.h"	
			}
			printf("Mode? 0, short summary only; 1, detailed output; 2, table output; 3, auto. \n");
			scanf("%s",str3);
			if(!strcmp(str3,"0")) mode=0;
			else if(!strcmp(str3,"t") || !strcmp(str3,"2")) mode=2;
			else if(!strcmp(str3,"3")) mode=3;
			else {
			fprintf(stderr,"No valid mode. Try again.");
#include	"exit1.h"	
			}
			if(mode==3){
				printf("Type minimum and maximum lengths of search.\n");
				printf("The two numbers should be separated by a dot.\n");
				scanf("%d.%d",&ll,&ul);
			} else {
				printf("Type a sequence motif.\n\n");
				scanf("%s",str3);
			}
			break;
			
		case 3:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			printf("Mode? 0, short summary only; 1, detailed output; 2, table output; 3, auto. \n");
			scanf("%s",str3);
			if(!strcmp(str3,"0")) mode=0;
			else if(!strcmp(str3,"t") || !strcmp(str3,"2")) mode=2;
			else if(!strcmp(str3,"3")) mode=3;
			else {
				fprintf(stderr,"No valid mode. Try again.");
#include	"exit1.h"	
			}
			if(mode==3){
				printf("Type minimum and maximum lengths of search.\n");
				printf("The two numbers should be separated by a dot.\n");
				scanf("%d.%d",&ll,&ul);
			} else {
				printf("Type a sequence motif.\n\n");
				scanf("%s",str3);
			}
			break;
			
		case 4:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			mode=atoi(p[3]);
			if(mode==3){
				printf("Type minimum and maximum lengths of search.\n");
				printf("The two numbers should be separated by a dot.\n");
				scanf("%d.%d",&ll,&ul);
			} else {
				printf("Type a sequence motif.\n\n");
				scanf("%s",str3);
			}
			break;
			
		case 5:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			mode=atoi(p[3]);
			if(mode==3) sscanf(p[4],"%d.%d",&ll,&ul);
			else strcpy(str3,p[4]);
			break;
			
		case 6:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			mode=atoi(p[3]);
			if(mode==3) sscanf(p[4],"%d.%d",&ll,&ul);
			else strcpy(str3,p[4]);
			sscanf(p[5],"%lf",&level);
			break;
			
		default:
			print_usage();
#include	"exit1.h"	
	}

/***********************Main part****************************/

	if((str=getenv("CIRCULAR"))!=NULL){
		if(!strcmp(str,"TRUE") || !strcmp(str,"true") || !strcmp(str,"1")) circular=TRUE;
	}
	else circular=FALSE;
	str=NULL;

	motlen=strlen(str3);
	if(mode !=3 && motlen<3) {
		fprintf(stderr,"Motif too short.\n");
#include 	"exit1.h"
	}
	if(motlen>MAXLINE-2) {
		fprintf(stderr,"Motif too long.\n");
#include 	"exit1.h"
	}

	c=check_filetype(fname1);
	if(c==99){
		fprintf(stderr,"Unknown file format.\n");
#include 	"exit1.h"
	}
	printf("Reading sequence...\n");

#ifdef MAC
#ifdef CODEWARRIOR
	SIOUXSetTitle("\psites");
#endif
#endif

	if(ftouch(fname2)){
		fprintf(stderr,"Error in opening output file.\n");
#include 	"exit1.h"
	}

#ifdef MAC
#ifdef CODEWARRIOR
        _fcreator=CREATOR_NAME;
        _ftype=FILETYPE_NAME;
#endif
#endif

	if((fout=fopen(fname2,"a+"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n",fname2);
#include 	"exit1.h"
	}


/* motif sequence converted to upper case */
	for(i=0;i<motlen;i++) str3[i]=toupper(str3[i]);
	

	printf("mode: %d.\n",mode);

	if(mode==2){
		fprintf(fout,"%s\t",fname1);
		fprintf(fout,"%s\t",str3);
	} else if (mode==3){
		fprintf(stdout,"%s\t",fname1);
		fprintf(stdout,"Searching palindromic sequences.\n");
		fprintf(stdout,"Minimum length is %d.\n",ll);
		fprintf(stdout,"Maximum length is %d.\n",ul);
		if(ll>ul || ll<3 || ul<3){
			fprintf(stderr,"Use correct minimum and maximum lengths.\n");
#include	"exit1.h"
		}
	} else {
	
		fprintf(fout,"************************************************************\n");
		fprintf(fout,"Search matching\n\n");
		fprintf(fout,"  Input file: %s.\n",fname1);
		fprintf(fout,"  Target motif: %s.\n",str3);
		fprintf(fout,"************************************************************\n");
		fprintf(fout,"\n");
	}
	fflush(fout);

	initialize_comp(code_count);
	initialize_comp(total_code);

	if(c==FASTA_SEQ || c==CLUSTAL_SEQ){
		if(c==FASTA_SEQ) fprintf(stdout,"This is a FASTA file.\n");
		else fprintf(stdout,"This is a CLUSTAL file.\n");

		if(c==FASTA_SEQ){
			if((mseq_in=Mread_fasta(fname1,dtype1))==NULL){
				fprintf(stderr,"Error in reading file: %s\n",fname1);
#include 	"exit1.h"
                	}
		}
                else {
			if((mseq_in=read_clustal(fname1,dtype1))==NULL){
				fprintf(stderr,"Error in reading file: %s\n",fname1);
				exit(1);
			}
		}
	}

	else if(c==GENBANK_SEQ || c==GBPROT_SEQ || c==EMBL_SEQ || c==SW_SEQ || c==PIR_SEQ){
		if(c==GENBANK_SEQ) printf("This is a GenBank file.\n");
		else if(c==GBPROT_SEQ) printf("This is a GenBank protein file.\n");
		else if(c==EMBL_SEQ) printf("This is an EMBL file.\n");
		else if(c==SW_SEQ) printf("This is a SwissProt file.\n");
		else if(c==PIR_SEQ) printf("This is a PIR file.\n");
		if((fin=fopen(fname1,"r"))==NULL){
			fprintf(stderr,"Cannot open file: %s\n\n",fname1);
#include 	"exit1.h"
		}		
		if((mseq_in=(Seqfile**)calloc(1,sizeof(Seqfile)))==0){
			fprintf(stderr,"Out of memory\n");
#include 	"exit1.h"
		}

		i=0L;
		while(TRUE){

			seqfile_size=(i + 2)*sizeof(Seqfile);   
			if((mseq_in=(Seqfile**)realloc(mseq_in,seqfile_size))==0){
				fprintf(stderr,"Out of memory\n");
#include 	"exit1.h"
			}
			if((mseq_in[i]=(Seqfile*)calloc(1,sizeof(Seqfile)))==0){
				fprintf(stderr,"Out of memory\n");
#include 	"exit1.h"
			}

			if(c==GENBANK_SEQ || c==GBPROT_SEQ) mseq_in[i]=read_genbank(fin,&fileloc,dtype1);
			else if(c==EMBL_SEQ || c==SW_SEQ) mseq_in[i]=read_embl(fin,&fileloc,dtype1);
			else if(c==PIR_SEQ) mseq_in[i]=read_pir(fin,&fileloc,dtype1);
			if(mseq_in[i]==NULL) break;
			i++;
		}
		num_seq=i;
		fclose(fin);
	}

	else if(c==RAW_SEQ){
		if((mseq_in=(Seqfile**)calloc(1,sizeof(Seqfile)))==0){
			fprintf(stderr,"Out of memory\n");
#include 	"exit1.h"
		}
		if((mseq_in[0]=(Seqfile*)calloc(1,sizeof(Seqfile)))==0){
			fprintf(stderr,"Out of memory\n");
#include 	"exit1.h"
		}
		if((mseq_in[0]=read_seq(fname1,c,dtype1))==NULL){
			fprintf(stderr,"Error in reading file: %s\n",fname1);
#include 	"exit1.h"
		}
		num_seq=1;
	}

	else {
		fprintf(stderr,"This file is not compatible with this program: %s\n",fname2);
#include 	"exit1.h"
	}


	/* environmental variable CIRCULAR is used if defined. */
	if(circular) {
		for(i=0;i<num_seq;i++) mseq_in[i]->circular=TRUE;
	}

	if(clear_gapsM(mseq_in)){
#include 	"exit1.h"
	}

	dtype1=check_data_type(mseq_in[0]);
	if(dtype1!=DNA && mode==3){
		fprintf(stderr,"Mode 3 is valid for DNA sequences.\n");
#include 	"exit1.h"
	}

	/******** Mode 3, automatic generation of palindromes ****************/

	if(mode==3){
		if(ul<3){
			fprintf(stderr,"Maximum length must be larger than 2.\n");
#include	"exit1.h"
		}
		initialize_comp(code_count);
		for(i=0;i<num_seq;i++){	
			if(c_comp(mseq_in[i],code_count)){
				fprintf(stderr,"Error in counting bases.\nBut we continue.\n");
			}
			for(j=0;j<30;j++) total_code[j]+=code_count[j];
			initialize_comp(code_count);
		}


		total_seqlen=0L;
		for(i=0;i<num_seq;i++){	
			total_seqlen+=mseq_in[i]->seqlen;
		}

		/* circular DNA is modified here */
		for(i=0;i<num_seq;i++){	
			if(mseq_in[i]->circular){
				printf("\tcircular genome: %s sequence no.%ld.\n",mseq_in[i]->ID,i+1);
				if((mseq_in[i]->seq=(char*)realloc(mseq_in[i]->seq,(mseq_in[i]->seqlen+ul+2)*sizeof(char)))==NULL){
					fprintf(stderr,"Out of memory.\n");
					exit(1);
				}
				pp=extract_string(mseq_in[i]->seq,1,ul-1); 
				strcat(mseq_in[i]->seq,pp); 		/* Here, the original seq is modified. */
				free(pp);

				mseq_in[i]->seqlen+=ll-2;
			}
		}


		for(c1=ll;c1<=ul;c1++){
			strcpy(str3,"");
			c2=0;
			c3=ipow(4,(int)c1/2);
			printf("Searching %d-bases motifs...\n",c1);
			for(c2=0;c2<c3;c2++){
				generate_motif(str3,c1);
				if(!strcmp(str3,"")) break;
				printf("   %s\n",str3);
				fflush(stdout);
				total_count=0L;
				for(i=0;i<num_seq;i++){	
					if(c2==0 && mseq_in[i]->circular) mseq_in[i]->seqlen+=1;
					if((count=write_sites3(mseq_in[i],motlen,str3))<0){ 
						fprintf(stderr,"Error in writing motif sites in file: %s\n",fname2);
#include 	"exit1.h"
					}
					if(count>1) total_count+=(count-1);
				}

			/* print summary */

				frequency=(double)total_count/(double)total_seqlen;
				fprintf(fout,"%s\t",fname1);
				fprintf(fout,"%ld\t",total_seqlen);
				write_comp(fout,total_code,comp,total_seqlen,dtype1,mode);
				fprintf(fout,"\t%d\t",c1);
				fprintf(fout,"%s\t",str3);
				fprintf(fout,"%ld\t",total_count);
				fprintf(fout,"%10.7f\t",frequency);

				e_freq=estimate_freq(comp,str3,dtype1);
				if(e_freq>=0.0){
					fprintf(fout,"%10.7f\t",e_freq);
				}
				if(e_freq < 1e-15) {
					fprintf(stdout,"Estimation of probability aborted because estimated frequency is too small.\n");
					fprintf(fout,"0.0");
				} 
				else { 
/*
					avr=(double)total_seqlen * e_freq;
					vari=avr * (1 - e_freq);
					x=abs((double)total_count - avr)/sqrt(vari);
					signif=get_signif(x);
					if(signif<=0.000026){
						fprintf(fout,"0.0");
					} else {
						fprintf(fout,"%7.1e",signif);
					}
*/
				fprintf(fout,"%10.7f",frequency/e_freq);
				}
				fprintf(fout,"\n");
				fflush(fout);
			}
			if(!strcmp(str3,"")) break;
		}

		/* reverting sequences if circular, may be omitted. */
		for(i=0;i<num_seq;i++){	
			if(mseq_in[i]->circular){
				len1=mseq_in[i]->seqlen-ul+1;
				mseq_in[i]->seq[len1]='\0';
				mseq_in[i]->seqlen=strlen(mseq_in[i]->seq);
			}
		}

	}
 
	/********** Other modes, single motif ******************************/
	else {
	/* make complementary */
		if(dtype1==DNA) {
			for(i=0;i<motlen;i++) str4[motlen-i-1]=to_compD(str3[i]);
			str4[motlen]=0;
			made_comp=TRUE;
		}

		total_seqlen=0L;
		total_count=0L;

		for(i=0;i<num_seq;i++){	
			if(mode==1 && i>0) fprintf(fout,"*************************************\n");
			if((count=write_sites(fout,mseq_in[i],dtype1,motlen,str3,str4,level,mode))<0){ 
				fprintf(stderr,"Error in writing motif sites in file: %s\n",fname2);
#include 	"exit1.h"
			}
			if(count>1) total_count+=(count-1);
		}

		for(i=0;i<num_seq;i++){	
			total_seqlen+=mseq_in[i]->seqlen;
		}

		for(i=0;i<num_seq;i++){	
			if(c_comp(mseq_in[i],code_count)){
				fprintf(stderr,"Error in counting bases.\nBut we continue.\n");
			}
			for(j=0;j<30;j++) total_code[j]+=code_count[j];
			initialize_comp(code_count);
		}

	/* print summary */

		frequency=(double)total_count/(double)total_seqlen;

   		if(mode!=2){
			fprintf(fout,"\n");
			fprintf(fout,"----- Summary of motif search -----\n");
			fprintf(fout,"  Total length of sequence: %ld.\n",total_seqlen);
			fprintf(fout,"  Number of match:          %ld.\n",total_count);
			fprintf(fout,"    Frequency:              %10.7f.\n",frequency);
			fprintf(fout,"\n");
   		} else {
			fprintf(fout,"%ld\t",total_seqlen);
			fprintf(fout,"%ld\t",total_count);
			fprintf(fout,"%10.7f\t",frequency);
   		}


		write_comp(fout,total_code,comp,total_seqlen,dtype1,mode);
		if(mode!=2) fprintf(fout,"\n");

		e_freq=estimate_freq(comp,str3,dtype1);
		if(e_freq>=0.0){
		   if(mode==2){
			fprintf(fout,"%10.7f",e_freq);
		   } else {
			fprintf(fout,"    Estimated frequency:    %10.7f.\n",e_freq);
		   }
		}

		if(e_freq < 1e-15) {
			fprintf(fout,"Estimation of probability aborted because estimated frequency is too small.\n");
		} else if(mode!=2){
			avr=(double)total_seqlen * e_freq;
			vari=avr * (1 - e_freq);
			x=fabs((double)total_count - avr)/sqrt(vari);
			signif=get_signif(x);
			if(signif<=0.000026){
				fprintf(fout,"    Probability: < %7.1e.\n",signif);
			} else {
				fprintf(fout,"    Probability:   %7.1e.\n",signif);
			}
		}
		
		fprintf(fout,"\n");
   		if(mode!=2){
			fprintf(fout,"************************************************************\n");
   		}
	}

	fclose(fout);
	circular=circular_bak;
	for(i=0;i<num_seq;i++) free_seq(mseq_in[i]);

	printf("\nTerminated.\n");

#include	"exit0.h"

}

/************************************************************
		End of file sites.c
************************************************************/

