/********************************************************
Text transformer (txtr) main file : Sprem.c
 	Transforms text file by removing spaces, numbers etc.
	Also converts DOS, Mac or UNIX file formats.
	Normally, any of the three formats are converted to 
	the format of the current system, without specifying
	original file format.
	
	To make the program 'txtr' (single command program),
	undefine INTEGRATED in Dmc.h, and
	add the following files to a single project.
		Dmc.h
		exit0.h
		exit1.h
		getseq2.h
		parsarg.h
		glovar.h
		gtsqfn.c
		sprem.c
		srfunc.c
		utility.c

  Usage: txtr infile outfile option

  Default outfile name is "infile"+"1"
  	
 	Naoki Sato   11 Sept. 1996.
	Modified DOS, Mac, Unix integrated version
	21 Aug. 1997

	Added integrated mode. Nov. 23, 1998.
	Use of parsarg.h. May 31, 1999.
	siseq_exit0 or 1 changed to exit0 or 1. Nov. 13, 1999.
	Bug fix for Mac version. Nov. 16, 1999.
	Japanese messages. March. 4, 2000.
	Justify. Oct. 23, 2000.
	Extract lines. July 5, 2005.

		--- Do not delete this comment. ---
 ********************************************************/

#include "Dmc.h"
#include "getseq2.h"

#ifndef INTEGRATED
#ifdef MAC
extern int ccommand(char ***);
#endif
#endif

/* Prototype declaration */
static void print_usage(void);
#ifdef INTEGRATED
int txtr(int c, char (*p)[S_2]);
#endif
void help_txtr(void);

/* External functions */

/* srfunc.c */
extern void delete_character(char* fnameA, char* fnameB, char x);
extern int select_from_menu(char *a);
extern int select_command(char *a, char *str);
extern void delete_num(char *fnameA, char *fnameB);
extern void cr_lf2lf(char *fnameA, char *fnameB);	/* for Unix */
extern void cr_lf2cr(char *fnameA, char *fnameB);	/* for Mac */
extern void lf2cr_lf(char *fnameA, char *fnameB);	/* for DOS */
extern void to_lower(char *fnameA, char *fnameB);
extern void to_upper(char *fnameA, char *fnameB);
extern void delete_empty_line(char *fnameA, char *fnameB);
extern void replace_character(char* fnameA, char* fnameB, char x, char y);
extern void justify(char *fnameA, char *fnameB, int len);

/* utility.c */
extern int filename_input(char* fname,char * message);
extern void copy_file(char *fnameA, char *fnameB);
extern int copy_file2(char *fnameA, char *fnameB, int start, int end);

#ifdef MAC
extern Boolean CMDQ;
#endif

/***************************************************************/
static void print_usage(void)
/***************************************************************/
{
	help_txtr();
	return;
}
/***************************************************************/

/***************************************************************/
void help_txtr(void)
/***************************************************************/
{
#ifdef JAPANESE
#ifdef UNIX
#include "EUC/spremE.h"
#else
#ifdef MAC
#include "Mac/SJIS/spremS.h"
#else
#include "Dos/SJIS/spremS.h"
#endif
#endif
#else
        printf("************* TXTR HELP ************************\n");
        printf("Usage: txtr infile outfile.\n\n");
        printf("       This program modifies a text file\n");
        printf("       by deleting spaces, characters, numbers\n");
        printf("       or by converting file formats.\n");
        printf("************************************************\n");
#endif
	return;
}
/***************************************************************/


/***************************************************************
****************** Main program *******************************/
#ifdef INTEGRATED
int txtr(int cc, char (*p)[S_2])

#else
#ifdef MAC
main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif

#endif
/***************************************************************/

{
	char fname3[100]="tmpfil.nao";
	char fname1[100]="infile";
	char fname2[100]="outfile";
	int c=0;
	char a;
	char d=' ';
	char b[50];
	int start,end;
	Boolean replace_mode=FALSE;

#include	"filenam.h"	/* definition of char *str1 and char *str2 */
	
#include	"parsarg.h"

/*********************** File names ******************/

	switch(cc){
		case 1:	
			strcpy(fname1,"infile");
			if(filename_input(fname1,str1)!=1){
#include	"exit1.h"
			}
			strcpy(fname2,fname1);
			strcat(fname2,"1");
			if(filename_input(fname2,str2)!=1){
#include	"exit1.h"
			}
			c=select_from_menu(b);
			break;
			
		case 2:	
			strcpy(fname1,p[1]);
			if(filename_input(fname2,str2)!=1){
#include	"exit1.h"
			}
			c=select_from_menu(b);
			break;
				
		case 3:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			c=select_from_menu(b);
			break;
			
		case 4:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			c=select_command(b,p[3]);
			break;
			
		default:
			print_usage();
#include	"exit1.h"
	}


/***********************Menu****************************/
	switch(c){
		case 0:
			printf("Exiting...\n");
#include	"exit1.h"
			break;

		case 1:	/* delete spaces */		
			printf("Deleting spaces from file : %s\n",fname1);
			delete_character(fname1,fname2,' ');

			printf("and writing into a temporary file : %s\n",fname3);
			copy_file(fname2,fname3);

			printf("Deleting empty lines and writing to file : %s\n",fname2);
			delete_empty_line(fname3,fname2);
			break;

		case 2:	/* delete spaces and isolated numbers */
			printf("Deleting numbers from file : %s\n",fname1);
			printf("and writing into a temporary file : %s\n",fname3);
			delete_num(fname1,fname3);
			
			printf("\nDeleting spaces from file : %s\n",fname3);
			delete_character(fname3,fname2,' ');

			printf("and writing into a temporary file : %s\n",fname3);
			copy_file(fname2,fname3);

			printf("Deleting empty lines and writing to file : %s\n",fname2);
			delete_empty_line(fname3,fname2);
			break;

		case 3:
			a=b[0];
			if(b[1]!='\0') {
				d=b[1];
				replace_mode=TRUE;
			}
			if(replace_mode) {
				replace_character(fname1,fname2,a,d);
			} 
			else delete_character(fname1,fname2,a);
			break;
			
		case 4:
			to_lower(fname1,fname2);
			break;
			
		case 5:
			to_upper(fname1,fname2);
			break;
			
		case 6:
			delete_empty_line(fname1,fname2);
			break;
			
		case 7:
			copy_file(fname1,fname2);
			break;

		case 71:
			sscanf(b,"%d-%d",&start,&end);
			copy_file2(fname1,fname2,start,end);
			break;

		case 8:
			justify(fname1,fname2,atoi(b));
			break;
			
		case 91:	/* for Mac */
			cr_lf2cr(fname1,fname2);
			break;

		case 92:	/* for DOS */
			lf2cr_lf(fname1,fname2);
			break;

		case 93:	/* for UNIX */
			cr_lf2lf(fname1,fname2);
			break;

		case 99:
			printf("This option has not been implemented.\n");
			break;

		default:
			printf("This program will terminate without any file operation.\n");
	}
	
	if((c==1) || (c==2)){
		printf("Trying to remove temporary file...\n");
		if((c=remove(fname3))!=0){
			fprintf(stderr,"Unable to remove temporary file: %s.\n",fname3);
		}
		
	}
		printf("Terminated.\n");

#include	"exit0.h"
}

/********************** end of file sprem.c **********************************/

