/***********************************************************************
stat.c

	Statistics utility for siseq programs 
  
 	Naoki Sato   Dec. 28, 1999. Started initial project.
	
	
		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"
#include <math.h>

/* Prototype declaration */
double get_kai2_level(double KI2,int n);
double Kai2(int n, double P);
double LogComb(long a, long n);
double get_signif(double x);

/* External functions */
#ifndef __STAT
#define __STAT
#endif

#include "libSeq.h"
#include "kai2.h"


/* Global variables  */

extern Boolean circular;
extern char *data_type_name[];
extern long num_seq;

/**************************************/
double get_kai2_level(double KI2,int n)
/**************************************/
{
	int PP;

	if(n<1 || KI2 <0.0) return -1.0;
	for(PP=1;PP<6;PP++){
		if(KI2<Kai2(n,kai2[0][PP])) break;
	}
	if(PP==0) return 1.0;
	else return kai2[0][PP-1];
}



/***********************************/
double Kai2(int n, double P)
/***********************************/
{
	double i=0.0;
	int nn,PP;

	if(n<1 || P<=0.0 || P>0.2) return 0.0;
	if(P>=kai2[0][1]) PP=1;
	else if(P>=kai2[0][2]) PP=2;
	else if(P>=kai2[0][3]) PP=3;
	else if(P>=kai2[0][4]) PP=4;
	else  PP=5;

/*	printf("Input P for Kai2 %8f defaults to %8f.\n",P,kai2[0][PP]);
*/
	if(n<=40) i=kai2[n][PP];
	else if(n<=100) {
		nn=(n/10)*10;
		if(n==nn) i=kai2[(n-40)/10+40][PP];
		else i=0.1*( (nn+10-n)*kai2[(nn-40)/10+40][PP] + (n-nn)*kai2[(nn-40)/10+41][PP] );
	}
	else	i=0.5*(kai2[47][PP] + sqrt(2*n-2))*(kai2[47][PP] + sqrt(2*n-2));
		
	return i;	
}


/*********************************/
double LogComb(long a, long n)
/********************************/
{
        long i;
        double j=0.0;

        if( a<=n || a<1 || n<1 ) return 0.0;
        for(i=1;i<=n;++i){
                j += log( ( (double)a-(double)n+(double)i ) /(double)i );
        }
        return j;
}

/*********************************/
double get_signif(double x)
/********************************/
{
#include "normal.h"

        double i=0.0;
        int j = 0;

        if(x<0) x = -x;
        j = (int)(10 * x);
        if(j > 40) j = 40;
        i = 0.50000 - normal[j];
        return i;
}


/*********************** end of file stat.c **********************************/

