/***********************************************************************
To fasta (tofast) 
main file : tofast.c : New version that includes tofast, comple,
			and toprot1.

 	Read input file in the raw, GenBank, and EMBL
 	formats, convert to complementary and write to a fasta file.

  Usage: tofast infile outfile
  
  Default outfile name is "infile"+".out"
  
  Source files are:	tofast.c
			gtsqfn.c
			trnsfn.c
  			utility.c
  			Dmc.h
  			getseq2.h
  			cmp_tbl.h
			exit0.h
			exit1.h
  
 	Naoki Sato   April 25, 1999.
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.
	For Mac version, define the compiler as well.

	Created as a copy of tofast.c. April 25, 1999.
	Modified. April 29, 1999.
	New version with various options. May 17-19, 1999.
	Verbous output suppressed. Aug. 25, 1999.
	Added compatibility with PIR. Aug. 31, 1999.
	Changed output format. Inclusion of short header. Oct. 16, 1999.
	siseq_exit0 or 1 changed to exit0 or 1. Nov. 13, 1999.
	Bug fix with dtype_out. Dec. 8, 1999.
        Inclusion of Japanese messages. March 4, 2000.
	Added info mode. May 9, 2000.
	Added basecomp mode. Nov. 29, 2000.
        Non-alphabetical characters are allowed in reading GenBank
                        and EMBL files. Dec. 21, 2000.
	Added options min and max. Jan. 13, 2002.
	Added stdout. Jan. 7, 2003.
	Added codon usage. Jan. 15, 2003.
	Added sequence number output. Dec. 25, 2005.
	Added MS analysis. March 7, 2007.
	Added mask option. June 29, 2007.
	Added rev option. Feb. 4, 2008.

		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"

#ifndef INTEGRATED
#ifdef MAC
extern int ccommand(char ***);
#endif
#endif

/* Prototype declaration */
static void print_usage(void);
#ifdef INTEGRATED
int tofast(int c, char (*p)[S_2]);
#endif
void help_tofast(void);

/* External functions */

#include "libSeq.h"

extern char version_string[20];

/* Global variables */
extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];
extern Boolean to_stdout;
extern int print_line;

/***************************************************************/
static void print_usage(void)
/***************************************************************/
{
	help_tofast();
	return;
}
/***************************************************************/

/***************************************************************/
void help_tofast(void)
/***************************************************************/
{
#ifdef JAPANESE
#ifdef UNIX
#include "EUC/tofastE.h"
#else
#ifdef MAC
#include "Mac/SJIS/tofastS.h"
#else
#include "Dos/SJIS/tofastS.h"
#endif
#endif
#else
	printf("*************************** TOFAST HELP ***********************************\n");
	printf("Usage: tofast infile outfile [d/r/p/c/i/b/s/min/max/ms/o/mask/rev/-] <options>.\n\n");
	printf("       This program converts a multiple (catenated) \n");
	printf("       sequence file in the raw (single sequence only),\n");
	printf("       GenBank and EMBL formats into a DNA,RNA or protein\n");
	printf("       sequence file in the complementary or normal sequences\n");
	printf("       in a FASTA file. A CLUSTAL alignment file is also\n");
	printf("       converted to a FASTA file. Also compatible with a \n");
	printf("       catenated or single PIR and SwissProt file.\n");
	printf("       d [n,c], DNA; r [n,c], RNA; p [frame] [codon file(option)], protein;\n");
	printf("       In [n,c], n: normal strand  c: complementary strand.\n");
	printf("       c, complementary (no change of data type).\n");
	printf("       i, print sequence information. Optionally, sequence [s], length [l],\n");
	printf("          name [n], number [#], or codon usage [c] may be specified.\n");
	printf("       b, print information about variation in base composition etc.\n");
	printf("       s, sort multiple sequences according to sequence name.\n");
	printf("       min, select sequences longer than the minimum value.\n");
	printf("       max, select sequences shorter than the maximum value.\n");
	printf("       ms, MS analysis.\n");
	printf("       mask, mask single letter repeats.\n");
	printf("       In options d and r, 'n' can be omitted.\n");
	printf("       In option p, codon table file can be specified.\n");
	printf("       In option b, window size and slide size should be added.\n");
	printf("       In option ms, name of definition file should be added\n");
	printf("       If no options are specified, just converts to FASTA <clear mode>.\n");
	printf("   Special option in TOFAST command\n");
	printf("       outfile may be stdout. In this case, all verbous output is directed\n");
	printf("       to stderr, while data output is directed to stdout. Useful in UNIX.\n");
	printf("***************************************************************************\n");
#endif
	return;
}
/***************************************************************/

/***************************************************************
****************** Main program *******************************/
#ifdef INTEGRATED
int tofast(int cc, char (*p)[S_2])

#else
#ifdef MAC
main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif
#endif
/***************************************************************/
{
	FILE *fin;
	FILE *fout;
	char fname1[MAXLINE]="infile";
	char fname2[MAXLINE]="outfile";
	int c=0;
	long k=0;
	long window=0L;
	long slide=0L;
	long min=1L;
	long max=0L;
	long *code_count;
	long *code_count_total;
	long aalen=0L;
	char codon_table[100];
	int j;

#include        "filenam.h"     /* definition of char *str1 and char *str2 */

	char str3[MINMEMSIZE]="";
	enum data_type dtype1=unknown;
	enum data_type dtype2=unknown;
	enum data_type dtype_out=unknown;
	enum data_type dtype_out_bak=unknown;
	Boolean iscompl=FALSE;
	Boolean need_input=FALSE;
	Boolean cleaning_mode=FALSE;
	Boolean error_writing=FALSE;
	char codon_file[50]="default";
	int fr=1;
	Boolean info_mode=FALSE;
	char info_mode_opt=0;
	Boolean bcomp_mode=FALSE;
	Boolean sort_mode=FALSE;
	Boolean limit_mode=FALSE; 	/* min or max */
	Boolean mask_mode=FALSE;
	Boolean rev_mode=FALSE;

	/* sequence variables */
	Seqfile *seq_in;
	Seqfile *seq_tr;
	Seqfile **mseq_in;
	Seqfile **mseq_tr;
	long fileloc=0L;

	/* ms analysis */
	char msdef[100]="";		/* name of MS definition file */
	Boolean ms_mode=FALSE;

#include	"parsarg.h"

/*********************** File names ******************/
	switch(cc){
		case 1:	
			strcpy(fname1,"infile");
			if(filename_input(fname1,str1)!=1){
#include	"exit1.h" 
			}
			if(filename_input(fname2,str2)!=1){
#include	"exit1.h" 
			}
			need_input=TRUE;
			break;
		
		case 3:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			cleaning_mode=TRUE;
			break;
			
		case 4:
		case 5:
		case 6:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			p[3][0]=tolower((int)p[3][0]);
			if(p[3][0]=='d') dtype_out=DNA;
			else if(p[3][0]=='r') {
				if(p[3][1]=='e') rev_mode=TRUE;
				else dtype_out=RNA;
			}
			else if(p[3][0]=='p') dtype_out=protein;
			else if(p[3][0]=='c') iscompl=TRUE;
			else if(p[3][0]=='b') bcomp_mode=TRUE;
			else if(p[3][0]=='m') {
				if(p[3][1]=='s') ms_mode=TRUE;
				else if(!strcmp(p[3],"mask")) mask_mode=TRUE;
				else limit_mode=TRUE;
			}
			else if(p[3][0]=='s') sort_mode=TRUE;
			else if(p[3][0]=='i') {
				info_mode=TRUE;
				if(cc==5 || cc==6){
					if(p[4][0]=='s' || p[4][0]=='l' || p[4][0]=='n'\
					 || p[4][0]=='c' || p[4][0]=='#' ) info_mode_opt=p[4][0];
					else {
						fprintf(stderr,"Unknown argument 4. Exiting...\n");
#include        "exit1.h"
					}
				} 
				if(cc==6) strcpy(codon_file,p[5]);
				break;
			}
			else {
				fprintf(stderr,"Unknown argument 4. Exiting...\n");
#include        "exit1.h"
			}
			if(cc==5 || cc==6){
				if(cc==6 && bcomp_mode){
					if(!num_check(p[4])) window=atol(p[4]);
					else {
						fprintf(stderr,"Unknown argument 4. Exiting...\n");
#include        "exit1.h"
					}
					if(!num_check(p[5])) slide=atol(p[5]);
					else {
						fprintf(stderr,"Unknown argument 4. Exiting...\n");
#include        "exit1.h"
					}
				}
				else if(cc==5 && ms_mode){
					strcpy(msdef,p[4]);
				}
				else if(cc==5 && limit_mode){
					p[3][1]=tolower((int)p[3][1]);
					p[3][2]=tolower((int)p[3][2]);
					if(!num_check(p[4])) min=atol(p[4]);
					else {
						fprintf(stderr,"Unknown argument 4. Exiting...\n");
#include        "exit1.h"
					}
					if(!strcmp("max",p[3]) && min>20L){
						max=min;
						min=0L;
					} 
					else if(!strcmp("min",p[3]) && min>10L){
						max=0L;
					}
					else {
						fprintf(stderr,"Specify min or max. Proceed without this argumet.\n");
						limit_mode=FALSE;
					}	
				}
				else if(!strcmp(p[4],"c")) iscompl=TRUE;
				else if(dtype_out==protein) {
					if(!strcmp(p[4],"1")) fr=1;
					else if(!strcmp(p[4],"2")) fr=2;
					else if(!strcmp(p[4],"3")) fr=3;
					else if(!strcmp(p[4],"4") || !strcmp(p[4],"-1")) fr=-1;
					else if(!strcmp(p[4],"5") || !strcmp(p[4],"-2")) fr=-2;
					else if(!strcmp(p[4],"6") || !strcmp(p[4],"-3")) fr=-3;
					else {
						fprintf(stderr,"Set correct frame number 1 - 3 or -1 - -3.\n");
#include        "exit1.h"
					}
				}
				else if(!strcmp(p[4],"n")) iscompl=FALSE;
/* for flexibility, c is allowed to precede d or r. */
				else if(!strcmp(p[4],"d")) dtype_out=DNA; 
				else if(!strcmp(p[4],"r")) dtype_out=RNA; 
				else {
				fprintf(stderr,"Unknown argument 5. Exiting...\n");
#include        "exit1.h"
				}
			}
			if(dtype_out!=protein) break;
			if(cc==6 && dtype_out==protein && !bcomp_mode) strcpy(codon_file,p[5]);
			break;

		default:
			print_usage();
#include        "exit1.h"
	}

/*************** input if argc=1 ********************/

	if(need_input){
		while(TRUE){
			printf("Output data type? d <DNA>, r <RNA>, p <protein>, n <no change>.\n");
			printf("Or special output? i <info>, b <compositional variation>.\n");
			scanf("%s",str3);
			str3[0]=tolower((int)str3[0]);
			if(str3[0]=='d') dtype_out=DNA;
			else if(str3[0]=='r') {
				if(p[3][1]=='e') rev_mode=TRUE;
				else dtype_out=RNA;
			}
			else if(str3[0]=='p') dtype_out=protein;
			else if(str3[0]=='i') info_mode=TRUE; 
			else if(str3[0]=='b') bcomp_mode=TRUE; 
			else if(str3[0]=='s') sort_mode=TRUE; 
			else if(str3[0]=='m'){
				if(!strcmp(str3,"ms")){
					ms_mode=TRUE;
				} else if(!strcmp(str3,"mask")){
					mask_mode=TRUE;
				} else {
				/*	limit_mode=TRUE; */
					fprintf(stderr,"Limit mode is not available in dialog mode.\n");
					fprintf(stderr,"Type tofast help for usage.\n");
#include        "exit1.h"
				}
			}
			else if(str3[0]!='n') continue;
			if(dtype_out!=unknown) break;
		}
		if(info_mode){
		}
		else if(ms_mode){
			while(TRUE){
				strcpy(str3,"");
				printf("Name of the file for the definition of MS analysis?\n");
				scanf("%s",str3);
				if(!strcmp(str3,"")) continue;
				if(!strcmp(str3,"exit")){
#include        "exit1.h"
				}
				strcpy(msdef,str3);
				break;
			}
		}
		else if(bcomp_mode){
			while(TRUE){
				strcpy(str3,"");
				printf("Window size in bp?\n");
				scanf("%s",str3);
				if(!strcmp(str3,"")) continue;
				if(!strcmp(str3,"exit")){
#include        "exit1.h"
				}
				if(!num_check(str3)) window=atol(str3);
				else continue;
				break;
			}
			while(TRUE){
				strcpy(str3,"");
				printf("Slide size in bp?\n");
				scanf("%s",str3);
				if(!strcmp(str3,"")) continue;
				if(!strcmp(str3,"exit")){
#include        "exit1.h"
				}
				if(!num_check(str3)) slide=atol(str3);
				else continue;
				break;
			}
		}
		else if(dtype_out==DNA || dtype_out==RNA){
			while(TRUE){
				strcpy(str3,"");
				printf("c <complementary> or n <normal> strand?\n");
				scanf("%s",str3);
				if(!strcmp(str3,"")) continue;
				str3[0]=tolower(str3[0]);
				if(str3[0]=='c') iscompl=TRUE;
				else iscompl=FALSE;
				break;
			}
		}
		else if(dtype_out==protein){
			while(TRUE){
				strcpy(str3,"");
				printf("Translation frame? Use 1 - 3 for normal strand.\n");
				printf("   Use -1 - -3 (or 4 - 6) for complementary.\n");
				scanf("%s",str3);
				if(!strcmp(str3,"")) continue;
				if(!strcmp(str3,"1")) fr=1;
				else if(!strcmp(str3,"2")) fr=2;
				else if(!strcmp(str3,"3")) fr=3;
				else if(!strcmp(str3,"4") || !strcmp(str3,"-1")) fr=-1;
				else if(!strcmp(str3,"5") || !strcmp(str3,"-2")) fr=-2;
				else if(!strcmp(str3,"6") || !strcmp(str3,"-3")) fr=-3;
				else {
					fprintf(stderr,"This is not a number.\n");
					continue;
				}
				break;
			}
			printf("Codon table? Input a name of codon table file.\n");
			printf("Type <uni> to use universal table.\n");
			printf("Type <mt> to use mitochondrial table.\n");
			scanf("%s",str3);
			if(strcmp(str3,"")){
				if(!strcmp(str3,"default") || !strcmp(str3,"universal") \
				|| !strcmp(str3,"def") || !strcmp(str3,"uni")){
					strcpy(codon_file,"default");
				}
				else if(!strcmp(str3,"mt") || !strcmp(str3,"mito")){
					strcpy(codon_file,"mt");
				}
				else if(strlen(str3)<40) strcpy(codon_file,str3); 
			}
		}
		need_input=FALSE;
	} else {
		if(!strcmp(fname2,"stdout") || !strcmp(fname2,"STDOUT")) to_stdout=TRUE;
	}

/*********************** main part ****************************/

	if((!strcmp(fname2,"stdout") || !strcmp(fname2,"STDOUT")) && !to_stdout){
		fprintf(stderr,"Specify output file name. Exiting...\n");
		exit(1);
	}

	if((seq_in=(Seqfile*)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		exit(1);
	}
	if((seq_tr=(Seqfile*)calloc(1,sizeof(Seqfile)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		exit(1);
	}
	fprintf(stderr,"Reading sequence...\n");

#ifdef MAC
#ifdef CODEWARRIOR
	SIOUXSetTitle("\ptofast");
#endif
#endif
	c=check_filetype(fname1);
	if(c==99){
		fprintf(stderr,"Unknown file format.\n");
		exit(1);
	}
	if(c==FASTA_SEQ || c==CLUSTAL_SEQ){
		if(c==FASTA_SEQ) fprintf(stderr,"This is a FASTA file.\n");
		else fprintf(stderr,"This is a CLUSTAL file.\n");

		fprintf(stderr,"Reading sequences...\n");

		if(c==FASTA_SEQ){
			if((mseq_in=Mread_fasta(fname1,dtype1))==NULL){
				fprintf(stderr,"Error in reading file: %s\n",fname1);
				exit(1);
			}
			if(cleaning_mode){
				if(clear_gapsM(mseq_in)){
					exit(1);
				}
			}
		}
		else {
			if((mseq_in=read_clustal(fname1,dtype1))==NULL){
				fprintf(stderr,"Error in reading file: %s\n",fname1);
				exit(1);
			}
		}

		dtype1=check_data_type(mseq_in[0]);
		if(dtype1==protein) {
			if(dtype_out!=protein){
				fprintf(stderr,"This is a protein sequence.\n");
				fprintf(stderr,"Output is set to protein sequence.\n");
				dtype_out=protein;
			}
			if(iscompl==TRUE){
				fprintf(stderr,"Your option <complementary> is neglected.\n");
				iscompl=FALSE;
			}
		}
/* for debug
printf("dtype1=%d, dtype_out=%d, iscompl=%d.\n",dtype1,dtype_out,iscompl);
*/

		if(iscompl){
			if(dtype1!=DNA && dtype1!=RNA) {
				fprintf(stderr,"This is not a nucleic acid sequence file: %s\n",fname1);
				exit(1);
			}
			else{
				fprintf(stderr,"Converting to complementary sequence...\n");
				if(comp_seqM(mseq_in)){
					fprintf(stderr,"Error in converting to complementary sequence.\n");
					exit(1);
				}
			}
		}

		if(dtype1==DNA && dtype_out==RNA){
			if(to_rnaM(mseq_in)){
				fprintf(stderr,"Error in converting to RNA.\n");
				exit(1);
			}
		}
		else if(dtype1==RNA && dtype_out==DNA){
			if(to_dnaM(mseq_in)){
				fprintf(stderr,"Error in converting to DNA.\n");
				exit(1);
			}
		}

		if(rev_mode){
			if(to_revM(mseq_in)){
				fprintf(stderr,"Error in sequence reversion.\n");
				exit(1);
			}
		}

		if(!to_stdout){
			if(ftouch(fname2)){
				fprintf(stderr,"Error in opening output file.\n");
#include 	"exit1.h"
			}
		}

		if(ms_mode){
			if(dtype1 != protein){
				fprintf(stderr,"Only protein sequence can be analyzed in the ms mode.\n");
#include 	"exit1.h"
			}
			if(MSanal(msdef,fname2,mseq_in)){
				fprintf(stderr,"Error in MS analysis.\n");
#include 	"exit1.h"
			}
			fprintf(stderr,"Terminated.\n");
#include 	"exit0.h"
		}

		if(info_mode){
			fprintf(stderr,"Writing sequence info...\n");
			if(info_mode_opt=='c'){
				if(dtype1!=DNA) {
					fprintf(stderr,"Sequence must be DNA.\n");
#include 	"exit1.h"
				}
				if(Mwrite_info3(fname2,mseq_in,codon_file)){
					fprintf(stderr,"Error in writing info.\n");
#include 	"exit1.h"
				}
			} else if(info_mode_opt){
				if(Mwrite_info2(fname2,mseq_in,info_mode_opt)){
					fprintf(stderr,"Error in writing info.\n");
#include 	"exit1.h"
				}
			} else {
				if(Mwrite_info(fname2,mseq_in)){
					fprintf(stderr,"Error in writing info.\n");
#include 	"exit1.h"
				}
			}
			fprintf(stderr,"Terminated.\n");
#include 	"exit0.h"
		}

		if(bcomp_mode){
			if(window<=0L || slide<=0L){
				fprintf(stderr,"Window and slide sizes are not correct.\n");
#include 	"exit1.h"
			}
			fprintf(stderr,"Writing compositional variation...\n");
			if(Mwrite_Dinfo(fname2,mseq_in,window,slide)){
				fprintf(stderr,"Error in writing DNA info.\n");
#include 	"exit1.h"
			}
			fprintf(stderr,"Terminated.\n");
#include 	"exit0.h"
		}

		if(sort_mode){
			sort_seq(mseq_in);
		}

		fprintf(stderr,"Writing sequences...\n");
		if((dtype1==DNA || dtype1==RNA) && dtype_out==protein){
			mseq_tr=seq_translateM(mseq_in,fr,codon_file);
			if(Mwrite(fname2,mseq_tr)){
				fprintf(stderr,"Error in writing sequence.\n");
				exit(1);
			}
		}
		else {
			if(cleaning_mode){
				for(k=0;k<num_seq;k++){
					strcpy(mseq_in[k]->header,"");
				
					dtype2=check_data_type(mseq_in[k]);
					if(dtype2!=dtype1){
						strcpy(mseq_in[k]->seq,"");
						mseq_in[k]->seqlen=0;
					}
				}
			}
			if(mask_mode){
				if(Mwrite2(fname2,mseq_in,-10,-10)){
					fprintf(stderr,"Error in writing sequence.\n");
					exit(1);
				}
			}
			else if(limit_mode){
				if(Mwrite2(fname2,mseq_in,min,max)){
					fprintf(stderr,"Error in writing sequence.\n");
					exit(1);
				}
			}else{
				if(Mwrite(fname2,mseq_in)){
					fprintf(stderr,"Error in writing sequence.\n");
					exit(1);
				}
			}
		}
	}

	else if(c==GENBANK_SEQ || c==GBPROT_SEQ || c==EMBL_SEQ || c==SW_SEQ || c==UNIPROT_SEQ || c==PIR_SEQ){
		if(c==GENBANK_SEQ) fprintf(stderr,"This is a GenBank file.\n");
		else if(c==GBPROT_SEQ) fprintf(stderr,"This is a GenBank protein file.\n");
		else if(c==EMBL_SEQ) fprintf(stderr,"This is an EMBL file.\n");
		else if(c==SW_SEQ) fprintf(stderr,"This is a SwissProt file.\n");
		else if(c==UNIPROT_SEQ) fprintf(stderr,"This is a UniProt file.\n");
		else if(c==PIR_SEQ) fprintf(stderr,"This is a PIR file.\n");
		if((fin=fopen(fname1,"r"))==NULL){
			fprintf(stderr,"Cannot open file: %s\n\n",fname1);
			exit(1);
		}
#ifdef MAC
#ifdef CODEWARRIOR
        _fcreator=CREATOR_NAME;
        _ftype=FILETYPE_NAME;
#endif
#endif
		if(to_stdout){
			fout=stdout;
		} else {
			if((fout=fopen(fname2,"w"))==NULL){
				fprintf(stderr,"Cannot open file: %s\n",fname2);
				exit(1);
			}
		}
		dtype_out_bak=dtype_out;

		if(info_mode && info_mode_opt=='c') {
			if((code_count=(long*)calloc(65,sizeof(long)))==NULL){
				fprintf(stderr,"Out of memory.\n");
				return TRUE;
			}
			if((code_count_total=(long*)calloc(65,sizeof(long)))==NULL){
				fprintf(stderr,"Out of memory.\n");
				return TRUE;
			}
			strcpy(codon_table,CodonTable(codon_file));
		}


		while(TRUE){
			if(c==GENBANK_SEQ || c==GBPROT_SEQ) seq_in=read_genbank(fin,&fileloc,dtype1);
			else if(c==EMBL_SEQ || c==SW_SEQ || c==UNIPROT_SEQ) seq_in=read_embl(fin,&fileloc,dtype1);
			else if(c==PIR_SEQ) seq_in=read_pir(fin,&fileloc,dtype1);
			if(seq_in==NULL) break;
			
			dtype_out=dtype_out_bak;
				/* dtype_out might be changed during the processing.
					dtype_out is always corrected. */

			if(seq_in->dtype==protein || seq_in->dtype==DNA ){
				dtype1=seq_in->dtype;
			}
			else {
				dtype1=check_data_type(seq_in);
				seq_in->dtype=dtype1;
			}
			if(dtype1==protein) {
				if(dtype_out!=protein){
					fprintf(stderr,"This is a protein sequence.\n");
					fprintf(stderr,"Output is set to protein sequence.\n");
					dtype_out=protein;
				}
				if(iscompl==TRUE){
					fprintf(stderr,"Your option <complementary> is neglected.\n");
					iscompl=FALSE;
				}
			}

			fprintf(stderr,"%s, ",seq_in->ID);
			fflush(stderr);
/*			fprintf(stderr,"Data type is %s\n",data_type_name[dtype1]);
			if(clear_gaps(seq_in)){
				fprintf(stderr,"Error in clearing gaps in file: %s\n",seq_in->ID);
				exit(1);
			}
  Dec. 21, 2000*/
			if(iscompl){
				if(dtype1!=DNA && dtype1!=RNA){
					fprintf(stderr,"This is not a nucleic acid sequence: %s\n",seq_in->ID);
					free_seq(seq_in);
					continue;
				}
				else{
					if(comp_seq4(seq_in)){
						fprintf(stderr,"Error in converting to complementary in file: %s\n",seq_in->ID);
						free_seq(seq_in);
						continue;
					}
				}
			}

			if(dtype1==DNA && dtype_out==RNA){
				if(to_rna(seq_in)){
					fprintf(stderr,"Error in converting to RNA.\n");
					exit(1);
				}
			}
			else if(dtype1==RNA && dtype_out==DNA){
				if(to_dna(seq_in)){
					fprintf(stderr,"Error in converting to DNA.\n");
					exit(1);
				}
			}

			if(rev_mode){
				if(rev_seq(seq_in)){
					fprintf(stderr,"Error in sequence reversion.\n");
					exit(1);
				}
			}

			if(ms_mode){
				fprintf(stderr,"Only a FASTA file can be analyzed in the ms mode.\n");
#include 	"exit1.h"
			}

			if(info_mode){
				fprintf(stderr,"Writing sequence info...\n");
				if(info_mode_opt=='c'){
					if(fwrite_info3(fout,seq_in,codon_table,code_count)){
						fprintf(stderr,"Error in writing info3.\n");
#include 	"exit1.h"
					}
					aalen += seq_in->seqlen / 3L;
					for(j=0;j<64;j++){
						code_count_total[j] += code_count[j];
					}
					initialize_comp2(code_count,65);
					
				} else if(info_mode_opt){
					if(fwrite_info2(fout,seq_in,info_mode_opt)){
						fprintf(stderr,"Error in getting code_count.\n");
#include 	"exit1.h"
					}

				} else {
					if(fwrite_info(fout,seq_in,1,' ')){
						fprintf(stderr,"Error in writing info.\n");
#include 	"exit1.h"
					}
				}
				free_seq(seq_in);
				continue;
			}

			if(bcomp_mode){
				if(window<=0L || slide<=0L){
					fprintf(stderr,"Window and slide sizes are not correct.\n");
#include 	"exit1.h"
				}
				fprintf(stderr,"Writing compositional variation...\n");
				if(fwrite_Dinfo(fout,seq_in,window,slide)){
					fprintf(stderr,"Error in writing DNA info.\n");
#include 	"exit1.h"
				}
				free_seq(seq_in);
				continue;
			}

			if((dtype1==DNA || dtype1==RNA) && dtype_out==protein){
				seq_tr=seq_translate(seq_in,fr,codon_file);
				sprintf(seq_tr->header,"Frame %d ",fr);
				if(fwrite_fasta(fout,seq_tr,1,' ')){
					fprintf(stderr,"Trouble in writing file: %s\n",fname2);
					fclose(fout);
					fclose(fin);
					free_seq(seq_in);
					free_seq(seq_tr);
#include 	"exit1.h"
				}
				free_seq(seq_tr);
			}
			else {
				if(cleaning_mode){
					if(fwrite_fasta(fout,seq_in,0,' ')) error_writing=TRUE;
				} else {
					/* May 11, 2007 */
					pack_multispace(seq_in->header);
					if(strlen(seq_in->header)> (2*print_line)) seq_in->header[2*print_line]='\0';
					if(fwrite_fasta(fout,seq_in,1,' ')) error_writing=TRUE;
				}
				if(error_writing){
					fprintf(stderr,"Trouble in writing file: %s\n",fname2);
					if(!to_stdout) fclose(fout);
					fclose(fin);
					free_seq(seq_in);
#include 	"exit1.h"
				}
			}

			free_seq(seq_in);
		}
		fprintf(stderr,"\n");	
		fclose(fin);

		if(info_mode && info_mode_opt=='c'){
			if(write_codon_summary(fout,codon_table,code_count_total,aalen)) {
				fprintf(stderr,"Trouble in writing codon summary.\n");
#include 	"exit1.h"
			}
		}   


#ifdef VERBOUS
		fprintf(stderr,"Sequence written to file: %s\n",fname2);
#else
		putc(8,stderr);
		putc('w',stderr);
		fflush(stderr);
#endif

		if(!to_stdout) fclose(fout);
	}else{			/* raw sequence */
		
		if(to_stdout){
			fout=stdout;
		} else {
			if((fout=fopen(fname2,"w"))==NULL){
				fprintf(stderr,"Cannot open file: %s\n",fname2);
				exit(1);
			}
		}
		if((seq_in=read_seq(fname1,c,dtype1))==NULL){
			fprintf(stderr,"Error in reading file: %s\n",fname1);
			exit(1);
		}
		dtype1=check_data_type(seq_in);
		if(dtype1==protein) {
			if(dtype_out!=protein){
				fprintf(stderr,"This is a protein sequence.\n");
				fprintf(stderr,"Output is set to protein sequence.\n");
				dtype_out=protein;
			}
			if(iscompl==TRUE){
				fprintf(stderr,"Your option <complementary> is neglected.\n");
				iscompl=FALSE;
			}
		}
		fprintf(stderr,"Data type is %s\n",data_type_name[dtype1]);

		if((c=clear_gaps(seq_in))!=0){
			fprintf(stderr,"Error in clearing gaps in file: %s\n",seq_in->ID);
			exit(1);
		}
		fprintf(stderr,"Gaps were cleared.\n");
		if(iscompl){
			if(dtype1!=DNA && dtype1!=RNA){
				fprintf(stderr,"This is not a nucleic acid sequence: %s\n",seq_in->ID);
				free_seq(seq_in);
				exit(1);
			}
			else{
				if(comp_seq4(seq_in)){
					fprintf(stderr,"Error in converting to complementary in file: %s\n",seq_in->ID);
					exit(1);
				}
			}
		}

		if(dtype1==DNA && dtype_out==RNA){
			if(to_rna(seq_in)){
				fprintf(stderr,"Error in converting to RNA.\n");
				exit(1);
			}
		}
		else if(dtype1==RNA && dtype_out==DNA){
			if(to_dna(seq_in)){
				fprintf(stderr,"Error in converting to DNA.\n");
				exit(1);
			}
		}

		if(rev_mode){
			if(rev_seq(seq_in)){
				fprintf(stderr,"Error in sequence reversion.\n");
				exit(1);
			}
		}
	
		if(info_mode){
			fprintf(stderr,"Writing sequence info...\n");
			if(info_mode_opt=='c') {
				if((code_count=(long*)calloc(65,sizeof(long)))==NULL){
					fprintf(stderr,"Out of memory.\n");
					return TRUE;
				}
				strcpy(codon_table,CodonTable(codon_file));
				if(fwrite_info3(fout,seq_in,codon_table,code_count)){
					fprintf(stderr,"Error in writing info3.\n");
#include 	"exit1.h"
				}

			} else if(info_mode_opt){
				if(fwrite_info2(fout,seq_in,info_mode_opt)){
					fprintf(stderr,"Error in writing info.\n");
#include 	"exit1.h"
				}
			} else {
				if(fwrite_info(fout,seq_in,1,' ')){
					fprintf(stderr,"Error in writing info.\n");
#include 	"exit1.h"
				}
			}
			fprintf(stderr,"Terminated.\n");
#include 	"exit0.h"
		}

		if(bcomp_mode){
			if(window<=0L || slide<=0L){
				fprintf(stderr,"Window and slide sizes are not correct.\n");
#include 	"exit1.h"
			}
			fprintf(stderr,"Writing compositional variation...\n");
			if(fwrite_Dinfo(fout,seq_in,window,slide)){
				fprintf(stderr,"Error in writing DNA info.\n");
#include 	"exit1.h"
			}
			fprintf(stderr,"Terminated.\n");
#include 	"exit0.h"
		}


		if((dtype1==DNA || dtype1==RNA) && dtype_out==protein){
			seq_tr=seq_translate(seq_in,fr,codon_file);
			sprintf(seq_tr->header,"Frame %d ",fr);
			if((c=fwrite_fasta(fout,seq_tr,1,' '))!=0){
				fprintf(stderr,"Trouble in writing file: %s\n",fname2);
				fprintf(stderr,"Error code: %d.\n",c);
				fclose(fout);
				free_seq(seq_in);
				free_seq(seq_tr);
				exit(1);
			}
			free_seq(seq_tr);
		}
		else{
			if(fwrite_fasta(fout,seq_in,0,' ')){
				fprintf(stderr,"Error in writing sequence.\n");
				exit(1);
			}
		}
		free_seq(seq_in);
		if(!to_stdout) fclose(fout);
	}

	to_stdout=FALSE;	
	fprintf(stderr,"\nTerminated.\n");

#include "exit0.h"

}

/************************************************************
		End of file tofast.c
************************************************************/

