/***********************************************************************
To protein (toprot) 
main file : toprot.c

 	This program translates a DNA sequence to protein sequences.
 	The output is either a single-frame translation,
	a 6-frame translation in the fasta format, or a
	composite translation with nucleic acid sequence.

  Usage: toprot infile outfile switch codon_file
  
  Default outfile name is "infile"+".out"
  
  Source files are:	toprot.c
			gtsqfn.c
  			trnsfn.c
  			excdfn.c
  			utility.c
  			Dmc.h
  			getseq2.h
  			cmptbl.h
  	
 	Naoki Sato   Feb. 1, 1998.
	Mac, Unix, and Windows(DOS) integrated version
	To compile, edit Dmc.h to define the platform.
	For Mac version, define the compiler as well.

	Originally writted as "toprot6 and toprot1".
	Added integrated mode. Nov. 23, 1998.
	Single program "toprot" with switch option and selection of
	codon_file. Feb. 14, 1999.
	option "composite". Feb. 15, 1999.
	Use of parsarg.h. May 31, 1999.
	siseq_exit0 or 1 changed to exit0 or 1. Nov. 13, 1999.
	Included 3 frame composite output (n). Nov. 13, 1999.
	Japanese messages. March, 4, 2000.
	Japanese messages addition. March, 17, 2000.
	Option p added. Aug. 22, 2000.
	Option a added. automatic extraction of ORFs. Nov. 30, 2001.
	
		--- Do not delete this comment. ---
 ***********************************************************************/

#include "Dmc.h"
#include "getseq2.h"

#ifndef INTEGRATED
#ifdef MAC
extern int ccommand(char ***);
#endif
#endif

/* Prototype declaration */
static void print_usage(void);
#ifdef INTEGRATED
int toprot(int c, char (*p)[S_2]);
#endif
void help_toprot(void);

/* External functions */

#include "libSeq.h"

/* Global variables */
extern char *filetype_name[];
extern char *residue_code_name;
extern char *data_type_name[];

/***************************************************************/
static void print_usage(void)
/***************************************************************/
{
	help_toprot();
	return;
}
/***************************************************************/

/***************************************************************/
void help_toprot(void)
/***************************************************************/
{
#ifdef JAPANESE
#ifdef UNIX
#include "EUC/toprotE.h"
#else
#ifdef MAC
#include "Mac/SJIS/toprotS.h"
#else
#include "Dos/SJIS/toprotS.h"
#endif
#endif
#else
	printf("************************* TOPROT HELP ***************************\n");
	printf("Usage: toprot infile outfile switch codon_table.\n\n");
	printf("       This program converts a DNA sequence into \n");
	printf("       either a single-frame translation (1),\n");
	printf("       a 6-frame translation (6) in the fasta file,\n");
	printf("       a composite of 6-frame translation and DNA sequence (c),\n");
	printf("       a composite of 3-frame translation and DNA sequence (n),\n");
	printf("       a composite of exon translation and DNA sequence (p),\n");
	printf("       a list of possible open reading frames (a),\n");
	printf("       or a set of possible open reading frames (s).\n");
	printf("       If switch p is selected, add exon_file after the p character\n");
	printf("       without space.\n");
	printf("       If switch a or s is selected, add MinLimit after the a character\n");
	printf("       without space. max can be used to extract the longest ORF.\n");
	printf("       longests followed by number (without space) can be used to \n");
	printf("       extract the longest of overlapping ORFs. \n");
	printf("*****************************************************************\n");
#endif
	return;
}
/***************************************************************/


/***************************************************************
****************** Main program *******************************/
#ifdef INTEGRATED
int toprot(int cc, char (*p)[S_2])

#else
#ifdef MAC
main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif

#endif
/***************************************************************/

{
	char fname1[100]="infile";
	char fname2[100]="outfile";
	char sw[100]="6";
	char *exon_file;
	char codon_file[50]="default";
	char *dummy="";
	int c=0;
	int i;
	ORF **multiorf;
	int NumOrf=1;
	long MinLimit=MINLIMIT;		/* 0 for max */
	char *limstring;	/* string for MinLimit */

#include        "filenam.h"     /* definition of char *str1 and char *str2 */

#ifdef JAPANESE
#ifdef UNIX
#include "EUC/toprotE2.h"
#else
#ifdef MAC
#include "Mac/SJIS/toprotS2.h"
#else
#include "Dos/SJIS/toprotS2.h"
#endif
#endif

#else
	char *str3="Codon table file";
#endif
	enum data_type dtype1=unknown;
	/* sequence variables */
	Seqfile *seq_in;
	Seqfile *seq_comp;
	Seqfile **seq_out6;
	
#include	"parsarg.h"

/*********************** File names ******************/

	switch(cc){
		case 1:	
			strcpy(fname1,"infile");
			if(filename_input(fname1,str1)!=1){
#include	"exit1.h"
			}
			strcpy(fname2,fname1);
			strcat(fname2,".out");
			if(filename_input(fname2,str2)!=1){
#include	"exit1.h"
			}
			printf("Switch --  1, 6, n, c, p, a, or s\n");
			scanf("%s",sw);
			if(strcmp(sw,"1")!=0 && strcmp(sw,"6")!=0 && strcmp(sw,"c")!=0 &&\
			 strcmp(sw,"n")!=0 && sw[0]!='p' && sw[0]!='a' && sw[0]!='s'){
				fprintf(stderr,"Switch should be either 1, 6, c, n, p, a or s.\n");
#include	"exit1.h"
			}	
			if((c=filename_input(codon_file,str3))==0) exit(1);
			if(c!=1){
				fprintf(stderr,"Error in file name input.\n");
#include	"exit1.h"
			}
			break;

		case 3:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			break;
			
		case 4:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			strcpy(sw,p[3]);
			if(strcmp(sw,"1")!=0 && strcmp(sw,"6")!=0 && strcmp(sw,"c")!=0 &&\
			 strcmp(sw,"n")!=0 && sw[0]!='p' && sw[0]!='a' && sw[0]!='s'){
				fprintf(stderr,"Switch should be either 1, 6, c, n, p, a or s.\n");
#include	"exit1.h"
			}	
			break;
		
		case 5:
			strcpy(fname1,p[1]);
			strcpy(fname2,p[2]);
			strcpy(sw,p[3]);
			if(strcmp(sw,"1")!=0 && strcmp(sw,"6")!=0 && strcmp(sw,"c")!=0 &&\
			 strcmp(sw,"n")!=0 && sw[0]!='p' && sw[0]!='a' && sw[0]!='s'){
				fprintf(stderr,"Switch should be either 1, 6, c, n, p, a or s.\n");
#include	"exit1.h"
			}
			strcpy(codon_file,p[4]);	
			break;

		default:
			print_usage();
#include	"exit1.h"
	}

/*********************** Main part ****************************/

		c=check_filetype(fname1);
		if(c==99){
			fprintf(stderr,"Unknown file format.\n");
			exit(1);
		}
		printf("Reading sequence...\n");

#ifdef MAC
#ifdef CODEWARRIOR
		SIOUXSetTitle("\ptoprot");
#endif
#endif
		
		if((seq_in=read_seq(fname1,c,dtype1))==NULL){
			fprintf(stderr,"Error in reading file: %s\n",fname1);
			exit(1);
		}
		c=check_data_type(seq_in);
		printf("Data type is %s\n",data_type_name[c]);
		if(c!=1 && c!=2){
			fprintf(stderr,"This is not a DNA sequence: %s\n",fname1);
			exit(1);
		}
		if((c=clear_gaps(seq_in))!=0){
			fprintf(stderr,"Error in clearing gaps in file: %s\n",seq_in->ID);
			exit(1);
		}
		printf("Gaps were cleared.\n");

		if((seq_out6=(Seqfile **)calloc(1,sizeof(Seqfile)))==NULL){
			fprintf(stderr,"Out of memory\n");
			exit(1);
		}


		if(sw[0]=='6' || sw[0]=='c' || sw[0]=='n'){
			for(i=1;i<4;i++){
				seq_out6[i-1]=seq_translate(seq_in,i,codon_file);
				sprintf(seq_out6[i-1]->header,"Frame %d.",i);
			}
			for(i=1;i<4;i++){
				seq_out6[i+2]=seq_translate(seq_in,(-1)*i,codon_file);
				sprintf(seq_out6[i+2]->header,"Frame %d.",(-1)*i);
			}
			num_seq=6;

			if(sw[0]=='c'){
				if((c=write_composite(fname2,seq_in,seq_out6,codon_file,0))!=0){
					fprintf(stderr,"Unable to write file.\n");
					exit(1);
				}
			}
			else if(sw[0]=='n'){
				if((c=write_composite(fname2,seq_in,seq_out6,codon_file,1))!=0){
					fprintf(stderr,"Unable to write file.\n");
					exit(1);
				}
			}
			else{	
				if(ftouch(fname2)){
					fprintf(stderr,"Error in opening output file.\n");
#include 	"exit1.h"
				}	
				if((c=Mwrite(fname2,seq_out6))!=0){
					fprintf(stderr,"Unable to write file.\n");
					exit(1);
				}
			}
		}

		else if(sw[0]=='1'){
			seq_out6[0]=seq_translate(seq_in,1,codon_file);
			sprintf(seq_out6[0]->header,"Frame 1. ");
			num_seq=1;
			if(ftouch(fname2)){
				fprintf(stderr,"Error in opening output file.\n");
#include 	"exit1.h"
			}	
			if(write_fasta(fname2,seq_out6[0],0,' ')){
				fprintf(stderr,"Unable to write file.\n");
				exit(1);
			}
		}

		else if(sw[0]=='p'){
			num_seq=1;
			if(ftouch(fname2)){
				fprintf(stderr,"Error in opening output file.\n");
#include 	"exit1.h"
			}

			if(strlen(sw)>1){
				exon_file = sw + 1;
			}
			else {
				exon_file=dummy;
			}

			if(write_pretty(fname2,seq_in,exon_file,codon_file)){
				fprintf(stderr,"Unable to write file.\n");
#include 	"exit1.h"
			}
/* 			if(exon_file!=dummy) free(exon_file);
*/
		}

		else if(sw[0]=='a' || sw[0]=='s'){		/* automatic extraction of possible open reading frames. */
			if(strlen(sw)>1){
				limstring=sw+1;

/*
				if((limstring=extract_string(sw,2,strlen(sw)))==NULL){
					fprintf(stderr,"Unable to extract string.\n");
#include 	"exit1.h"
				}
*/

				if(!strcmp(limstring,"max")) MinLimit=0L;
				else if(strstr(limstring,"long")==limstring){
					for(;limstring!='\0';limstring++){
						if(isdigit(*limstring)) break;
					}
					if(!num_check(limstring)){
						sscanf(limstring,"%ld",&MinLimit);
						MinLimit=0L-MinLimit;
					} else {
						MinLimit=MINLIMIT;
					}
				}
				else if(!num_check(limstring)){
					sscanf(limstring,"%ld",&MinLimit);
				}
				else MinLimit=MINLIMIT;
			}

			if((multiorf=InitOrfInfo(NumOrf))==NULL){
				fprintf(stderr,"Unable to initialize multiorf.\n");
#include 	"exit1.h"
			}
			if((multiorf=AddOrfInfo(seq_in,multiorf,&NumOrf,codon_file,1))==NULL){
				fprintf(stderr,"Unable to add orf infos to multiorf.\n");
#include 	"exit1.h"
			}
			if((seq_comp=copy_seq(seq_in))==NULL){
				fprintf(stderr,"Error in copying sequence.\n");
#include 	"exit1.h"
			}
			if(comp_seq4(seq_comp)){
				fprintf(stderr,"Error in making complementary.\n");
#include 	"exit1.h"
			}
			if((multiorf=AddOrfInfo(seq_comp,multiorf,&NumOrf,codon_file,0))==NULL){
				fprintf(stderr,"Unable to add orf infos to multiorf.\n");
#include 	"exit1.h"
			}
			if(sw[0]=='a'){
				if(PrintOrfInfo(fname2,seq_in,multiorf,NumOrf,MinLimit)){
					fprintf(stderr,"Unable to print orf infos in multiorf.\n");
#include 	"exit1.h"
				}
			} else {
				if(PrintOrfSeq(fname2,seq_in,multiorf,NumOrf,MinLimit,codon_file)){
					fprintf(stderr,"Unable to print orf seq.\n");
#include 	"exit1.h"
				}
			}
		}

		if(seq_in!=NULL) free_seq(seq_in);
		printf("\nTerminated.\n");

#include	"exit0.h"

}

/************************************************************
		End of file toprot.c
************************************************************/

