/*****************************************************
General utility program

File name : utility.c
	
	Functions : filename_input
		copy_file
		copy_stream
		fcat
		ftouch
		StringClearGaps
		linetype
		pause_screen
		extract_string
		pick_string
		string_alnum
		pack_seq
		pack_space
		count_char
		count_characters
		count_alnum
		num_check
		num_check2
		int_check
		tail_string
		last_line
		space_line
		subst
		tailtype
		tailchop
		str_shift
		blank_line
		erasequote
		siseq_rm
		siseq_cp
		ipow
		version_info
		catch_word
	
	Naoki Sato April 15, 1998.
	
	To use this file, include Dmc.h, which 
	has been properly edited to define the platform.

	Integrated Mac, DOS, and UNIX version.

	Bugs fixed. Nov. 20, 1998.
	Added space_line. Jan. 14, 1999.
	Added subst. March 11, 1999.
	Added tailtype. March 15, 1999.
	Added str_shift. March 16, 1999.
	Bug fix in extract_string. April 26, 1999.
	Added erasequote. April 29, 1999.
	Modified string_alnum. April 30, 1999.
	Added shell commands. May 3, 1999.
	Modified string_alnum. May 6-8, 1999.
	Added num_check2 to allow <>. May 8, 1999.
	Modified erasequote. May 11, 1999.
	Bug fix in space_line. May 23, 1999.
	Added ftouch. May 24, 1999.
	Added pick_string and fill_both_sides. May 26, 1999.
	Added fcat. June 1, 1999.
	Simplified menus. Aug. 11, 1999.
	Modified pause_screen. Aug. 19, 1999.
	Bug fix in pause_screen. Jan. 21, 2000.
	Added StringClearGaps. Feb. 20, 2000.
	Inclusion of japanese files. March 2, 2000.
	Bug fix. May 4, 2000.
	Added count_characters and count_alnum. Aug. 23, 2000.
	Modified str_shift. Oct. 14, 2000.
	Added ipow. Jan. 4, 2001.
	Added pack_seq. Jan. 31, 2001.
	Added blank_line. Mar. 7, 2001.
	Added version_info. Dec. 2, 2001.
	Added catch_word. Dec. 10, 2001.
	Bug fix version of extract_string. Jan. 16, 2003.
	Added headchop. Feb. 8, 2003.
	Added pack_multispace. May 11, 2007.
	Added remove_character. April 11, 2010.
	
	--- Do not delete this comment.---

******************************************************/

#include "Dmc.h"	/* This is necessary for all the platforms.*/
#include "def.h"

/* Prototype declaration */

int filename_input(char* fname, char* message);
int copy_file(char *fnameA, char *fnameB);
int copy_file2(char *fnameA, char *fnameB, int start, int end);
int copy_stream(FILE *fin,FILE *fout);
int ftouch(char *filename);
int StringClearGaps(char *s);
Boolean linetype(char *line,char *code);
char *extract_string(char *str_name, long int startpos, long int endpos);
void extract_string2(char *str, long startpos, long endpos,char *out);
int pick_string(char *str_name, long int startpos, long int endpos);
void string_alnum(char *string,char *list);
void pack_seq(char *string,char *list);
void pack_space(char *string);
void remove_character(char *string,char c1);
void pack_multispace(char *string);
int count_char(char *string,int s);
int count_characters(char *string);
int count_alnum(char *string);
int num_check(char *string);
int num_check2(char *string);
int int_check(char *string);
void tail_string(char *string,int s);
Boolean last_line(char *string);
void pause_screen(void);
char *space_line(long size,int c);
void subst(char *string, char c1, char c2);
Boolean tailtype(char *line, char *code);
void tailchop(char *string);
void headchop(char *string);
char *str_shift(char *string,char delimit);
Boolean blank_line(char *str);
void erasequote(char *string);
void fill_both_sides(char *string,long a,long b,int s);
int siseq_rm(int c, char (*p)[S_2]);
int siseq_cp(int c, char (*p)[S_2]);
int fcat(int c, char (*p)[S_2]);
int ipow(int a,int c);
void version_info(FILE *stream);
int catch_word(char *s,char *last);

/* Definition of the code for the end of line (LF).*/

#ifndef END_OF_LINE
#define END_OF_LINE 10	/* CR will be ignored in the program. */
#endif
#ifndef END_OF_LINE2
#define END_OF_LINE2 13		/* for DOS or Mac->UNIX converted files */
#endif

/* Other definitions */

#ifndef MINMEMSIZE
#define MINMEMSIZE 100	/* minimum memory size (bytes) allocated to char string */
#endif
#ifndef MAXLINE 
#define MAXLINE 512
#endif
extern char version_string[20];

/***************************************************************/
int filename_input(char *fname, char *message)
/***************************************************************/

{
	char str[30];
	int state=0;
	
	while(TRUE){
		printf("     -------------------------------------------\n");
#ifdef JAPANESE

#ifdef UNIX
#include "EUC/utilE1.h"
#else
#ifdef MAC
#include "Mac/SJIS/utilS1.h"
#else
#include "Dos/SJIS/utilS1.h"
#endif
#endif

#else
                printf("         Type %s, or <exit>.\n",message);
#endif
		printf("     -------------------------------------------\n\n");

		scanf("%s",str);
		if ((strcmp(str,"exit"))==0){
			printf("You selected EXIT.\n");
			state=0;
			break;
		}
		else if((strcmp(str,"y"))==0){
			state=1;
			break;
		}
		else{ 
			strcpy(fname,str);
			state=1;
			break;
		}
	}
	return state;
}


/******************************************************/
int copy_stream(FILE *fin,FILE *fout)
/******************************************************/
{
	int c=1;
	int xl=MAXLINE;
	char s[MAXLINE+2];

	while(fgets(s,xl,fin)!=NULL){
		fputs(s,fout);
		if (((c++)%10)==0)
			printf(".");
			fflush(stdout);
	}
	printf("\n");
	return 0;
}



/******************************************************/
int copy_file(char *fnameA, char *fnameB)
/******************************************************/
{
	FILE *fin, *fout;
	int code=0;

	printf("Copying text file...\n");

	if((fin=fopen(fnameA,"r"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n\n",fnameA);
		return 1;
	}

#ifdef MAC
#ifdef CODEWARRIOR
	_fcreator=CREATOR_NAME;
	_ftype=FILETYPE_NAME;
#endif
#endif

	if((fout=fopen(fnameB,"w"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n\n",fnameB);
		fclose(fin);
		return 1;
	}

	copy_stream(fin,fout); 

	fclose(fin);
	fclose(fout);
	printf("\n");
	return code;
}


/******************************************************/
int copy_file2(char *fnameA, char *fnameB, int start, int end)
/******************************************************/
{
	FILE *fin, *fout;
	int code=0;
	char buffer[MAXLINE];
	int counter;

	printf("Copying text file from line %d to %d ...\n",start,end);

	if((fin=fopen(fnameA,"r"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n\n",fnameA);
		return 1;
	}

#ifdef MAC
#ifdef CODEWARRIOR
	_fcreator=CREATOR_NAME;
	_ftype=FILETYPE_NAME;
#endif
#endif

	if((fout=fopen(fnameB,"w"))==NULL){
		fprintf(stderr,"Cannot open file: %s\n\n",fnameB);
		fclose(fin);
		return 1;
	}

	counter = 1;
	while(fgets(buffer,MAXLINE,fin)!=NULL){
		if(counter >= start && counter <= end) fprintf(fout,"%s",buffer);
		counter++;
		if(counter > end) break;
	}
	if(start > counter || end > counter){
		fprintf(stderr,"There are %d lines in the file %s. Check start and end parameters.\n",counter-1,fnameB);
	}
	fclose(fin);
	fclose(fout);
	printf("\n");
	return code;
}


/******************************************************/
/******************************************************/
int fcat(int c, char (*p)[S_2])
/******************************************************/
{

        FILE *fin, *fout;

        if(c!=3) return 1;
        printf("Adding text file: %s after the file: %s\n",p[2],p[1]);

        if((fout=fopen(p[1],"a"))==NULL){
                fprintf(stderr,"Cannot open file: %s\n",p[1]);
                return 1;
        }

#ifdef MAC
#ifdef CODEWARRIOR
        _fcreator=CREATOR_NAME;
        _ftype=FILETYPE_NAME;
#endif
#endif

        if((fin=fopen(p[2],"r"))==NULL){
                fprintf(stderr,"Cannot open file: %s\n",p[2]);
                fclose(fout);
                return 1;
        }

        copy_stream(fin,fout);
        fclose(fin);
        fclose(fout);
        printf("\n");
        return 0;
}



/**************************************************************/
int ftouch(char *filename)
/**************************************************************/
{
        FILE *fout;

#ifdef MAC
#ifdef CODEWARRIOR
        _fcreator=CREATOR_NAME;
        _ftype=FILETYPE_NAME;
#endif
#endif
        if((fout=fopen(filename,"w"))==NULL){
                fprintf(stderr,"Cannot open file: %s\n\n",filename);
                return 1;
        }
        fclose(fout);

	return 0;
}

/*******************************************************************/
int StringClearGaps(char *s)
/*******************************************************************
This function removes '-' from the string s.
*******************************************************************/
{
        char *p,*q;
       
	if(s==NULL) return 1;
 
        for(p=s,q=s;*p!='\0';p++){
                if(*p!='-'){
                        if(islower(*p)) *q=toupper(*p);
			else *q=*p;
                        q++;
                }
        }
	*q='\0';
	return 0;
}
/********************************************************************/


/***************************************/
int catch_word(char *s,char *last)
/***************************************/
{
	char *p;
	long a;

	if(s==NULL) return FALSE;
	if(last==NULL) return FALSE;
	if(strlen(s)<strlen(last)) return FALSE;
	a=(long)strlen(last);

	if((p=strstr(s,last))!=NULL){
		for(;*(p+a)!='\0';p++){
			*p=*(p+a);
		}
		*p='\0';
		return TRUE;
	}
	return FALSE;
}
/***************************************/


/***************************************/
Boolean linetype(char *line,char *code)
/****** From utils.c in clustalx ********
returns 1 if the string "code" is located 
at the top of the string "line".
*****************************************/
{
	return( strncmp(line,code,strlen(code)) == 0 );
}

/****************************************/


/****************************************/
Boolean tailtype(char *line, char *code)
/* returns 1 if the string "code" is located
at the end of the string "line". Neglects
CR code at the end of the line.*/
/****************************************/
{
	int s;
/*	char *p;
*/
	if(strlen(code)==0 || strlen(line)==0) return FALSE;
	tailchop(line);
	if((s=strlen(line))<1) return FALSE;
/*
	p=line+s-strlen(code);
	if((p=extract_string(line,s-strlen(code)+1,s))==0) return TRUE;
*/
	if(!strcmp(code,line+s-strlen(code))) {
		return TRUE;
	}
	else {
		return FALSE;
	}
}

/****************************************/

/****************************************/
Boolean blank_line(char *str)
/* returns 1 if the string contains no 
alphabet and number. */
/****************************************/
{
	char *p;

	if(strlen(str)==0) return TRUE;
	for(p=str;*p!='\0';p++){
		if(isalnum((int)(*p))) return FALSE;
	}
	return TRUE;
}

/****************************************/

/***********************************************/
void erasequote(char *string)
/* removes quotation or space from both sides of 
the string.  34 is double quote. */
/***********************************************/
/* old version 
{
	int i,s;
	
	for(i=strlen(string)-1;i>=0;i--){
		s=(int)string[i];
		if(s==34 || s==' ') string[i]=0;
		else break;
	}
	return;
}
*/

{
	char *p1,*p2;
	int flag=0;
	
	p2=string;
	for(p1=string;*p1!='\0';p1++){
		if(!flag){
		 	if(*p1==34 || *p1==' ') continue;
			else flag=1;
		}
		*p2++=*p1;
	}
	*p2='\0';

	for(p2--;p2>=string;p2--){
		if(*p2==34 || *p2==' ') *p2='\0';
		else break;
	}
		
	return;
}
/****************************************/
void tailchop(char *string)
/* removes CR, LF, or space from the 
end of the string. */
/****************************************/
{
	int i,s;
	
	for(i=strlen(string)-1;i>=0;i--){
		s=(int)string[i];
		if(s==END_OF_LINE || s==END_OF_LINE2 || s==' ') string[i]=0;
		else break;
	}
	return;
}
/****************************************/

/****************************************/
void headchop(char *string)
/* removes spaces from the 
head of the string. */
/****************************************/
{
	char *p1,*p2;
	
	for(p1=string;*p1!='\0';p1++){
		if(*p1!=' ') break;
	}
	if(p1==string) return;
	
	for(p2=string;*p2!='\0';p2++){
		*p2=*(p1++);
		if(*p2=='\0') break;
	}

	return;
}
/****************************************/

/**************************************************/
void pause_screen(void)
/**************************************************/
{
        char c[100];
#ifdef JAPANESE

#ifdef UNIX
#include "EUC/utilE2.h"
#else
#ifdef MAC
#include "Mac/SJIS/utilS2.h"
#else
#include "Dos/SJIS/utilS2.h"
#endif
#endif

#else
        printf("Press y + RETURN to continue.\n");
#endif
        while(TRUE){
                fgets(c,5,stdin);
		if(!strcmp(c,"")) continue;
		tailchop(c);
                if(!strcmp(c,"y")) {
			fflush(stdin);
			return;
		}
        }
	return;
}
/**************************************************/

/***********************************************************************/
int pick_string(char *str_name, long int startpos, long int endpos)
/* This is different from extract_string in that this function
changes the original string, and returns TRUE on error . */
/***********************************************************************/
{
	long string_length;
	char *p1,*p2;
	if((string_length=strlen(str_name))<1) {
		fprintf(stderr,"\nString null.\n");
		return TRUE;
	}
	if(startpos<1 || startpos>string_length || endpos>string_length || endpos<startpos){
		fprintf(stderr,"\nError of extraction positions: %ld, %ld with string length %ld.\n",startpos, endpos, string_length);
		fprintf(stderr,"in string %s.\n",str_name);
		return TRUE;
	}
	for(p1=str_name,p2=str_name+startpos-1;p2<str_name+endpos;p2++) *p1++=*p2;
	*p1='\0';
	return FALSE;
}
/*****************************************************/

/***********************************************************************/
void extract_string2(char *str, long startpos, long endpos,char *out)
/* A bug-fix version of extract_string */
/***********************************************************************/
{
	long i=0;
	long j=0;
	long length;
	if((length=strlen(str))<1) {
		fprintf(stderr,"\nString null.\n");
		strcpy(out,"");
		return;
	}
	if(startpos==1 && endpos==0) {
		strcpy(out,"");
		return;
	}
	if(startpos>length || endpos>length || endpos<startpos){
		fprintf(stderr,"\nError of extraction positions: %ld, %ld with string length %ld.\n",startpos, endpos, length);
		fprintf(stderr,"in string %s.\n",str);
		strcpy(out,"");
		return;
	}
	for(i=startpos-1;i<endpos;++i){
		out[j++]=str[i];
	}
	out[j]='\0';
	return;
}

/***********************************************************************/
char *extract_string(char *str_name, long int startpos, long int endpos)
/***********************************************************************/
{
	char *str_tmp;
	long int i=0;
	long int j=0;
	long int string_length;
	if((string_length=strlen(str_name))<1) {
		fprintf(stderr,"\nString null.\n");
		return "";
	}
	if(startpos==1 && endpos==0) return "";
	if(startpos>string_length || endpos>string_length || endpos<startpos){
		fprintf(stderr,"\nError of extraction positions: %ld, %ld with string length %ld.\n",startpos, endpos, string_length);
		fprintf(stderr,"in string %s.\n",str_name);
		return "";
	}
	if((str_tmp=(char*)calloc(string_length+2,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory.\n");
		exit(1);
	}
	for(i=startpos-1;i<endpos;++i){
		str_tmp[j++]=str_name[i];
	}
	str_tmp[j]=0;
	return str_tmp;
}


/*****************************************************/
void pack_space(char *string)
/* This removes spaces from the string. */
/*****************************************************/
{
	char *p1,*p2;
	for(p1=string,p2=string;*p1!='\0';p1++){
		if(*p1!=' ') *p2++=*p1;
	}
	*p2='\0';
	return;
}

/*****************************************************/
void remove_character(char *string,char c1)
/* This removes character c1 from the string. */
/*****************************************************/
{
	char *p1,*p2;
	for(p1=string,p2=string;*p1!=c1;p1++){
		if(*p1=='\0' || *p1=='\n') break;
		else if(*p1!=' ') *(p2++)=*p1;
	}
	if(*p1==c1) *p2='\0';
	return;
}

/*****************************************************/
void pack_multispace(char *string)
/* This removes extra spaces within the string. 
One space is left for each continuous spaces */
/*****************************************************/
{
	char *p1,*p2;
	int flag = 0;

	for(p1=string,p2=string;*p1!='\0';p1++){
		if(*p1==' '){
			if(flag){
				continue;
			} else {
				flag = 1;
				*p2++=*p1;
			}	
		} else {
			*p2++=*p1;
			flag = 0;
		}
	}
	*p2='\0';
	return;
}

/*****************************************************/
void pack_seq(char *string,char *list)
/* This function removes spaces and non-alphabetical
characters from string, except characters in the list. */ 
/*****************************************************/
{
	char *p1,*p2,*p3;

	for(p1=string,p2=string;*p1!='\0';p1++){
		if(isalpha(*p1)) *p2++=*p1;
		else if(isprint(*p1) && *p1!=' '){ 
			for(p3=list;*p3!='\0';p3++){
				if(*p1==*p3){
					*p2++=*p1;
					break;
				}
			}
		}
	}
	*p2='\0';
	return;
}
/*****************************************************/

/******************************************************/
void string_alnum(char *string,char *list)
/* Changes characters in the string to spaces,
except number, alphabet or characters in the list. 
Two consecutive periods are also changed to two spaces,
whereas single periods are retained.*/ 
/******************************************************/
{
	char *p1,*p2;
	int flag=0;
	
	for(p1=string;*p1!='\0';p1++){
		if(*p1=='.'){
			if(flag==1) {
				*p1=' ';
				*(p1-1)=' ';
			}
			else flag=1;
			continue;
		}	
		else {
			flag=0;
			if(isalnum((int)*p1)) continue;
			else {
				for(p2=list;*p2!='\0';p2++){
					if(*p1==*p2) break;
				}
				if(*p1!=*p2) *p1=' ';
			}
		}
	}
	return;
}
/**************************************************************/
int count_char(char *string,int s)
/**************************************************************/
{
	long int i;
	int j=0;
	
	for(i=0;i<(long)strlen(string);i++){
		if(string[i]==s) j++;	
	}
	return j;
}

	
/******************************************************/
int count_characters(char *string)
/* This function returns the number of characters
that are visible and printable, excluding control 
and space characters. */
/******************************************************/
{
	char *p;
	int j=0;
	
	for(p=string;*p!='\0';p++){
		if(isgraph(*p) && !isspace(*p)) j++;	
	}
	return j;
}

/******************************************************/
int count_alnum(char *string)
/* This function returns the number of alphabets
and numbers. */
/******************************************************/
{
	char *p;
	int j=0;
	
	for(p=string;*p!='\0';p++){
		if(isalnum(*p)) j++;	
	}
	return j;
}


/******************************************************/
int num_check(char *string)
/* returns TRUE if string is not a number. 
*/
/******************************************************/
{
	register long i;
	
	for(i=0;i<(long)strlen(string);i++){
		if(isdigit((int)string[i])==FALSE) return 1;
	}
	return 0;
}
/******************************************************/
int num_check2(char *string)
/* returns TRUE if string is not a number.
   The first character may be < or >. */
/******************************************************/
{
	register long int i;
	
	for(i=0;i<(long)strlen(string);i++){
		if(i==0){
			if(!isdigit((int)string[i]) && string[i]!='>' && string[i]!='<') return 1;
		}
		else {	
			if(!isdigit((int)string[i])) return 1;
		}
	}
	return 0;
}
/******************************************************/
int int_check(char *string)
/* returns TRUE if string is not an integer.
   The first character may be + or -.
   removes the first + from the string. */
/******************************************************/
{
	char *p1;
	int plus_flag=0;
	
	for(p1=string;*p1!='\0';p1++){
		if(p1==string){
			if(!isdigit((int)*p1) && *p1!='-' && *p1!='+') return 1;
			if(*p1=='+') plus_flag=1;
		}
		else {	
			if(!isdigit((int)*p1)) return 1;
		}
	}
	if(plus_flag){
		for(p1=string;*p1!='\0';p1++) *p1=*(p1+1); 
	}
	return 0;
}
/******************************************************/
void tail_string(char *string,int s)
/* Takes the last s characters of the string. */
/******************************************************/
{	
	if(s<(int)strlen(string)){
		if(pick_string(string,strlen(string)-s+1,strlen(string))){
			fprintf(stderr,"Error in extracting string in tail_string.\n");
			exit(1);
		}
	}
	return;
}
/******************************************************/
Boolean last_line(char *string)
/* True if the string ends with a CR code. */
/******************************************************/
{
	register long i;
	
	for (i=0;i<(long)strlen(string);i++){
		if(string[i]=='\n') return TRUE;	
	}
	return FALSE;
}
/*****************************************************/
char *space_line(long size,int c)
/*****************************************************
This function returns a string of characters c with 
size specified.
*****************************************************/
{
	char *string;
	long i;
	if(! isprint(c)) c=' ';
	if((string=(char*)calloc(size+2,sizeof(char)))==NULL){
		fprintf(stderr,"Out of memory in space_line.\n");
		exit(1);
	}
	if(size>0L){
		for(i=0;i<size;i++) string[i]=(char)c;
	}
	return string;
}
/******************************************************/
void subst(char *string, char c1, char c2)
/* This function replaces c1 with c2 in string. */
/******************************************************/
{
	char *p;
	for(p=string;*p!='\0';p++) {
		if(*p==c1) *p=c2;
	}
	
	return;
}
/*****************************************************/
void fill_both_sides(char *string,long a,long b,int s)
/* Adds a characters of s to the left end, and
   b characters of s to the right end of the string. */
/*****************************************************/
{
	char *p1;
	long length;
	if(string==NULL || !isprint(s)) return;
	length=strlen(string);
	*(string+a+length+b)='\0';
	for(p1=string+a+length+b-1;p1>=string+a+length;p1--) *p1=s;
	for(p1=string+a+length-1;p1>=string+a;p1--) *p1=*(p1-a);
	for(p1=string+a-1;p1>=string;p1--) *p1=s;
	return;
	
}
/*****************************************************/

/*****************************************/
char *str_shift(char *string,char delimit)
/* splits the contents of string with delimiter,
then returns the remainder of string.  
string contains the first word.
Removes delimiters before the first word.
*/
/*****************************************/
{
	char *p,*q;
	int flag=0;
	int begin=0;
	
	if(delimit=='\0') return NULL;
	q=string;

	for(p=string;*p!='\0';p++){
		if(!begin){			/* Oct. 14, 2000 */
			q=p+1;
			if(*p!=delimit) begin=1;
			continue;
		}
		if(*p==delimit){
			*p='\0';
			q= p+1;
			flag=1;
		}
		else if(flag==1){
			return q;
		}
	}
	return "";	
}
/*****************************************/
/*****************************************/
int siseq_rm(int c, char (*p)[S_2])
/*****************************************/
{
	int i;
	int err_no=0;
	
	if(strcmp(p[0],"rm") && strcmp(p[0],"remove") && strcmp(p[0],"del") && strcmp(p[0],"delete")){
		return 1;
	}
	for(i=1;i<c;i++){
		if(!strcmp(p[i],"")) return err_no;
		else if(remove(p[i])){
			fprintf(stderr,"Unable to remove file: %s.\n",p[i]);
			err_no=1;
		}
		else printf("File: %s removed.\n",p[i]);
	}
	return err_no;
}

/*****************************************/
int siseq_cp(int c, char (*p)[S_2])
/*****************************************/
{
	printf("Copying file: %s to %s.\n",p[1],p[2]);

	if(c!=3) return 1;
	if(strcmp(p[0],"cp") && strcmp(p[0],"copy")) return 1;
	if(!strcmp(p[1],"") || !strcmp(p[2],"")) return 1;

	return (copy_file(p[1],p[2]));
}
/*****************************************/

/******************************************/
int ipow(int a,int c)
/* computes c powers of a.*/
/******************************************/
{
	int d=1;
	int i=0;

	if(a<=0 || c<=0) return 0;
	for(i=0;i<c;i++) d*=a;
	
	return d;
}
/******************************************/

/******************************************/
void version_info(FILE *stream)
/******************************************/
{
        fprintf(stream,"Output of SISEQ version %s. All rights reserved. Naoki Sato 2001-3.\n",version_string);
	return;
}
/******************************************/

/******** end of file: utility.c *********************/

