#!/bin/sh
# Test shell for testing SISEQ programs.
#
LOG="test.log"
DIFF="diff"

if [ -f $LOG ]
then
	echo "Removing existing log file..."
	rm -f $LOG
fi

BINDIR="../bin"
EXE=${BINDIR}/siseq

if [ ! -x ${EXE} ] 
then
	echo "Program siseq is not found or executable."
	exit 1
fi

echo "**************************************************"
echo "Testing reading functions..."
echo "**************************************************"
echo ""
	echo "Reading test." > $LOG
	echo "  Bare sequence." >> $LOG
if ${EXE} getseq2 bare.seq outT11 101 300 c
then
	$DIFF outT11 Out11 >> $LOG 
	echo "" >> $LOG
	echo "Since the program adds time of execution to bare sequence as a header," >> $LOG
	echo "the output is normally different from the standard output file." >> $LOG
	echo "Examine the two files closely, and check if there is important difference." >> $LOG
	echo "" >> $LOG
fi

	echo "  FASTA sequence." >> $LOG 
if ${EXE} getseq2 fasta.seq outT12 51 100 c
then
	$DIFF outT12 Out12 >> $LOG
fi

	echo "  SWISS PROT sequence." >> $LOG

if ${EXE} getseq2 sw.seq outT13 1 60 n
then
	$DIFF outT13 Out13 >> $LOG
fi
	echo "" >> $LOG

echo ""
echo "**************************************************"
echo "File conversion utilities..."
echo "**************************************************"
echo ""
	echo "File conversion utilities." >> $LOG
	echo "  tofast -- single sequence file" >> $LOG
if ${EXE} tofast gb.seq outT21
then
	$DIFF outT21 Out21 >> $LOG
fi
	echo "  tofast -- catenated sequence file" >> $LOG
if ${EXE} tofast Anabaena.gbk outT211
then
	$DIFF outT211 Out211 >> $LOG
fi
	echo "  complement -- complementary for GB catenated file" >> $LOG
if ${EXE} complement Anabaena.gbk outT212
then
	$DIFF outT212 Out212 >> $LOG
fi
	echo "  complement -- complementary for EMBL catenated file" >> $LOG
if ${EXE} complement emblcat.seq outT213
then
	$DIFF outT213 Out213 >> $LOG
fi
	echo "  single frame translation" >> $LOG
if ${EXE} toprot gb.seq outT23 1 codontable.mt
then
	$DIFF outT23 Out23 >> $LOG
fi
	echo "  six-frame translation" >> $LOG
if ${EXE} toprot gb.seq outT24 6
then
	$DIFF outT24 Out24 >> $LOG
fi
	echo "  combined translation" >> $LOG
if ${EXE} toprot gb.seq outT25 c
then
	$DIFF outT25 Out25 >> $LOG
fi
	echo "  conversion to RNA" >> $LOG
if ${EXE} toRNA gb.seq outT26
then
	$DIFF outT26 Out26  >> $LOG
fi
	echo "" >> $LOG


echo "**************************************************"
echo "GenBank/EMBL extractors..."
echo "**************************************************"
	echo "GenBank/EMBL extractors..." >> $LOG
	echo "Extracting CDS field." >> $LOG
	echo "  GenBank single genome file." >> $LOG
if ${EXE} extcds ppumitoG outT31
then
	$DIFF outT31 Out31 >> $LOG
fi
	echo "  GenBank catenated sequence file." >> $LOG
if ${EXE} extcds Anabaena.gbk outT32
then
	$DIFF outT32 Out32 >> $LOG
fi
	echo "  EMBL single genome file." >> $LOG
if ${EXE} extcds ppumitoE outT33
then
	$DIFF outT33 Out33 >> $LOG
fi
echo "  EMBL catenated sequence file." >> $LOG
if ${EXE} extcds emblcat.seq outT34
then
	$DIFF outT34 Out34 >> $LOG
fi
echo "  genlist for EMBL seq" >> $LOG
if ${EXE} genlist emblcat.seq outT35
then
	$DIFF outT35 Out35 >> $LOG
fi
	echo "" >> $LOG

echo "**************************************************"
echo "Extracting DNA for CDS..."
echo "**************************************************"

	echo "Extracting DNA for CDS..." >> $LOG
	echo "  GenBank single genome file." >> $LOG
if ${EXE} cdsnuc ppumitoG outT41 s -10 s 10 
then
	$DIFF outT41 Out41 >> $LOG
fi
	echo "  GenBank catenated sequence file." >> $LOG
if ${EXE} cdsnuc Anabaena.gbk outT42 s 0 e 0
then
	$DIFF outT42 Out42 >> $LOG
fi
	echo "  EMBL single genome file." >> $LOG
if ${EXE} cdsnuc ppumitoE outT43 s -10 s 10
then
	$DIFF outT43 Out43 >> $LOG
fi 
	echo "  EMBL catenated sequence file." >> $LOG
if ${EXE} cdsnuc emblcat.seq outT44 s 0 e 0
then
	$DIFF outT44 Out44 >> $LOG
fi
	echo "  Import of external sequences" >> $LOG
cp siseq.cf.test siseq.cf
if ${EXE}
then
	if [ -f outT47 ]
	then
	$DIFF outT47 Out47 >> $LOG
	fi
	if [ -f outT48 ]
	then
	$DIFF outT48 Out48 >> $LOG
	fi
fi
rm siseq.cf
	echo "" >> $LOG

echo "**************************************************"
echo "Extracting RNA genes..."
echo "**************************************************"
	echo "Extracting RNA genes..." >> $LOG
	echo "  GenBank single genome file." >> $LOG
if ${EXE} extrna ppumitoG outT51 S 0 E 0
then
	$DIFF outT51 Out51 >> $LOG
fi
	echo "  EMBL single genome file." >> $LOG
if ${EXE} extrna ppumitoE outT52 S 0 E 0
then
	$DIFF outT52 Out52 >> $LOG
fi
	echo "" >> $LOG

echo "**************************************************"
echo "Extracting other sequences..."
echo "**************************************************"
	echo "Extracting noncoding regions..." >> $LOG
if ${EXE} noncod ppumitoG outT53
then
	$DIFF outT53 Out53 >> $LOG
fi
	echo "Extracting introns..." >> $LOG
if ${EXE} extint emblcat.seq outT54 S -2 E 2
then
	$DIFF outT54 Out54 >> $LOG
fi
	echo "" >> $LOG

echo "**************************************************"
echo "Clustal files..."
echo "**************************************************"
	echo "Clustal files..." >> $LOG
	echo "  clu2fa" >> $LOG
if ${EXE} clu2fa pltbzip.aln outT61
then
	$DIFF outT61 Out61 >> $LOG
fi
	echo "  getclu" >> $LOG
if ${EXE} getclu pltbzip.aln outT62 440 570
then
	$DIFF outT62 Out62 >> $LOG
fi
	echo "  simtbl" >> $LOG
if ${EXE} simtbl pltbzip.aln outT63
then
	$DIFF outT63 Out63 >> $LOG
fi
	echo " " >> $LOG


echo "**************************************************"
echo "Other commands..."
echo "**************************************************"
	echo "Other commands..." >> $LOG
	echo "  sites table format" >> $LOG
if ${EXE} sites ppumitoE outT711 3 5.7
then
	$DIFF outT711 Out711 >> $LOG
fi

	echo "  sites mode 1" >> $LOG
if ${EXE} sites Anabaena.gbk outT712 1 CYCGRG 0.7
then
	$DIFF outT712 Out712 >> $LOG
fi

	echo "  tofast table info" >> $LOG
if ${EXE} tofast ppumitoG outT721 b 1000 500
then
	$DIFF outT721 Out721 >> $LOG
fi

	echo "  tofast info" >> $LOG
if ${EXE} tofast emblcat.seq outT722 i
then
	$DIFF outT722 Out722 >> $LOG
fi

	echo "  nucaln" >> $LOG
cd NucAln ; sh nucaln_test.sh >> ../$LOG ; cd ..



	echo " " >> $LOG



echo "**************************************************"
echo "This is the result of comparison with standard output files."
echo ""
cat $LOG
echo ""
echo "End of test."

echo ""
echo "If you want to proceed to test txtr program,"
echo "press 'y'. If you want to stop testing now,"
echo "press any other characters. Then hit return."
echo ""

read answer
if [ "$answer" != "y" ]
then
	exit 0
else
	if [ -x ./testtxtr.sh ]
	then
		./testtxtr.sh
	fi
fi

if [ -f ./core ] 
then
	echo "Attention! At least one core file has been generated." >> $LOG
	echo
fi
echo " End of all test."
echo ""
exit 0

